@@ -6,9 +6,10 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @author Hatem Ltaief
+ * @author Dulceneia Becker
  * @date 2010-11-15
  * @precisions normal z -> s d c
  *
@@ -28,9 +29,9 @@ void plasma_pzgeqrfrh_quark(PLASMA_desc A, PLASMA_desc T, int BS,
     Quark_Task_Flags task_flags = Quark_Task_Flags_Initializer;
 
     int k, m, n;
-    int K, M, RD;
+    int M, RD;
     int ldaM, ldam, ldaMRD;
-    int tempkn, tempMm, tempnn, tempmm, tempMRDm;
+    int tempkmin, tempkn, tempMm, tempnn, tempmm, tempMRDm;
     int ib;
 
     plasma = plasma_context_self();
@@ -39,33 +40,29 @@ void plasma_pzgeqrfrh_quark(PLASMA_desc A, PLASMA_desc T, int BS,
     QUARK_Task_Flag_Set(&task_flags, TASK_SEQUENCE, (intptr_t)sequence->quark_sequence);
 
     ib = PLASMA_IB;
-    K = min(A.mt, A.nt);
-    for (k = 0; k < K; k++) {
+
+    for (k = 0; k < min(A.mt, A.nt); k++) {
         tempkn = k == A.nt-1 ? A.n-k*A.nb : A.nb;
-        for (M = k;
-             M < A.mt-1 || M == k;  /* No bottom single-row subdomain */
-             M += BS) {
+        for (M = k; M < A.mt; M += BS) {
             tempMm = M == A.mt-1 ? A.m-M*A.mb : A.mb;
+            tempkmin = min(tempMm, tempkn);
             ldaM = BLKLDD(A, M);
             QUARK_CORE_zgeqrt(
                 plasma->quark, &task_flags,
                 tempMm, tempkn, ib, T.nb,
                 A(M, k), ldaM,
                 T(M, k), T.mb);
-
             for (n = k+1; n < A.nt; n++) {
                 tempnn = n == A.nt-1 ? A.n-n*A.nb : A.nb;
                 QUARK_CORE_zunmqr(
                     plasma->quark, &task_flags,
                     PlasmaLeft, PlasmaConjTrans,
-                    tempMm, tempnn, tempMm, ib, T.nb,
+                    tempMm, tempnn, tempkmin, ib, T.nb,
                     A(M, k), ldaM,
                     T(M, k), T.mb,
                     A(M, n), ldaM);
             }
-            for (m = M+1;
-                 (m < M+BS && m < A.mt) || m == A.mt-1; /* Suck in bottom single-row domain */
-                 m++) {
+            for (m = M+1; m < min(M+BS, A.mt); m++) {
                 tempmm = m == A.mt-1 ? A.m-m*A.mb : A.mb;
                 ldam = BLKLDD(A, m);
                 QUARK_CORE_ztsqrt(
@@ -89,9 +86,7 @@ void plasma_pzgeqrfrh_quark(PLASMA_desc A, PLASMA_desc T, int BS,
             }
         }
         for (RD = BS; RD < A.mt-k; RD *= 2) {
-            for (M = k;
-                 M+RD < A.mt-1; /* No reduction with bottom single-row subdomain */
-                 M += 2*RD) {
+            for (M = k; M+RD < A.mt; M += 2*RD) {
                 tempMRDm = M+RD == A.mt-1 ? A.m-(M+RD)*A.mb : A.mb;
                 ldaM   = BLKLDD(A, M   );
                 ldaMRD = BLKLDD(A, M+RD);
