@@ -8,7 +8,7 @@
  *
  * LU with Partial pivoting.
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @author Hatem Ltaief
  * @date 2009-11-15
@@ -39,7 +39,7 @@ void CORE_zgetrf_reclap_init(void);
  **/
 void plasma_pzgetrf_reclap_rl_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    int k, m, n;
+    int k, m, n, minmnt;
     plasma_context_t *plasma;
     int tempkm, tempkn, tempmm, tempnn;
     int tempm, tempk;
@@ -68,7 +68,8 @@ void plasma_pzgetrf_reclap_rl_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *s
 
     CORE_zgetrf_reclap_init();
 
-    for (k = 0; k < min(A.mt, A.nt); k++)
+    minmnt = min(A.mt, A.nt);
+    for (k = 0; k < minmnt; k++)
     {
         tempk  = k * A.mb;
         tempm  = A.m - k * A.mb;
@@ -90,7 +91,7 @@ void plasma_pzgetrf_reclap_rl_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *s
                 plasma->quark, &task_flagsP,
                 tempm, tempkn, A.nb,
                 A(k, k), ldak, IPIV(k),
-                sequence, request, 1, A.mb*k,
+                sequence, request, 1, tempk,
                 panel_thread_count );
         }
         else {
@@ -98,14 +99,14 @@ void plasma_pzgetrf_reclap_rl_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *s
                 plasma->quark, &task_flagsU,
                 tempm, tempkn, A.mb,
                 A(k, k), ldak, IPIV(k),
-                sequence, request, 1, A.mb*k );
+                sequence, request, 1, tempk );
         }
 #else
         QUARK_CORE_zgetrf(
                 plasma->quark, &task_flagsU,
                 tempm, tempkn, A.mb,
                 A(k, k), ldak, IPIV(k),
-                sequence, request, 1, A.mb*k );
+                sequence, request, 1, tempk );
 #endif
 
         fakedep = (void *)(intptr_t)(k+1);
@@ -162,15 +163,14 @@ void plasma_pzgetrf_reclap_rl_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *s
         }
     }
 
-    for (k = 0; k < min(A.mt, A.nt); k++)
+    for (k = 0; k < minmnt; k++)
     {
-        tempk  = k * A.mb;
-        tempm  = A.m - k * A.mb;
         tempkm = k == A.mt-1 ? A.m-k*A.mb : A.mb;
         tempkn = k == A.nt-1 ? A.n-k*A.nb : A.nb;
+        tempk  = min(tempkn, tempkm);
         ldak = BLKLDD(A, k);
 
-      /*
+        /*
          * Apply row interchange behind the panel (work on the panel)
          */
         QUARK_Task_Flag_Set(&task_flagsU, TASK_PRIORITY, QUARK_TASK_MIN_PRIORITY );
@@ -179,7 +179,7 @@ void plasma_pzgetrf_reclap_rl_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *s
         {
             QUARK_CORE_zlaswp_f2(
                 plasma->quark, &task_flagsU,
-                A.nb, A(k, n), A.lm, 1, tempkm, IPIV(k), 1,
+                A.nb, A(k, n), A.lm, 1, tempk, IPIV(k), 1,
                 /* Dependency on previous swapb */
                 A(k-1,n), A.lm*A.nb, INPUT,
                 /* Dependency on all GEMM from previous step */
