@@ -8,7 +8,7 @@
  *
  * LU with Partial pivoting.
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @author Hatem Ltaief
  * @date 2009-11-15
@@ -73,15 +73,19 @@ void plasma_pzgetrf_rectil_rl_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *s
           panel_thread_count--;
           QUARK_Task_Flag_Set(&task_flagsP, TASK_THREAD_COUNT, panel_thread_count );
         }
-          
+
         QUARK_Task_Flag_Set(&task_flagsP, TASK_PRIORITY, QUARK_TASK_MAX_PRIORITY - k );
-        
+
         QUARK_CORE_zgetrf_rectil(
             plasma->quark, &task_flagsP,
-            plasma_desc_submatrix(A, tempk, k*A.nb, tempm, tempkn), A(k, k), A.mb*A.nb,
-            IPIV(k), sequence, request, 
-            1, A.mb*k, panel_thread_count );
-        
+            plasma_desc_submatrix(A, tempk, k*A.nb, tempm, tempkn),
+            A(k, k), A.mb*A.nb, IPIV(k),
+            sequence, request, 1, tempk,
+            panel_thread_count );
+
+        /*
+         * Update the trailing submatrix
+         */
         fakedep = (void *)(intptr_t)(k+1);
         for (n = k+1; n < A.nt; n++)
         {
@@ -100,7 +104,7 @@ void plasma_pzgetrf_rectil_rl_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *s
             if ( m < A.mt ) {
                 tempmm = m == A.mt-1 ? A.m-m*A.mb : A.mb;
                 ldam = BLKLDD(A, m);
-    
+
                 QUARK_CORE_zgemm2(
                     plasma->quark, &task_flagsU,
                     PlasmaNoTrans, PlasmaNoTrans,
@@ -108,12 +112,12 @@ void plasma_pzgetrf_rectil_rl_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *s
                     mzone, A(m, k), ldam,
                            A(k, n), ldak,
                     zone,  A(m, n), ldam);
-    
+
                 for (m = k+2; m < A.mt; m++)
                 {
                     tempmm = m == A.mt-1 ? A.m-m*A.mb : A.mb;
                     ldam = BLKLDD(A, m);
-    
+
                     QUARK_CORE_zgemm_f2(
                         plasma->quark, &task_flagsU,
                         PlasmaNoTrans, PlasmaNoTrans,
@@ -130,17 +134,20 @@ void plasma_pzgetrf_rectil_rl_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *s
         }
     }
 
+    QUARK_Task_Flag_Set(&task_flagsU, TASK_PRIORITY, QUARK_TASK_MIN_PRIORITY );
     for (k = 0; k < min(A.mt, A.nt); k++)
     {
+        int mintmp;
         tempk  = k * A.mb;
         tempm  = A.m - tempk;
         tempkm = k == A.mt-1 ? tempm : A.mb;
+        tempkn = k == A.nt-1 ? A.n - k * A.nb : A.nb;
+        mintmp = min(tempkm, tempkn);
         ldak = BLKLDD(A, k);
 
         /*
          * Apply row interchange behind the panel (work on the panel)
          */
-        QUARK_Task_Flag_Set(&task_flagsU, TASK_PRIORITY, QUARK_TASK_MIN_PRIORITY );
         fakedep = (void*)(intptr_t)k;
         for (n = 0; n < k; n++)
         {
@@ -148,7 +155,7 @@ void plasma_pzgetrf_rectil_rl_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *s
             QUARK_CORE_zlaswp_ontile_f2(
                 plasma->quark, &task_flagsU,
                 plasma_desc_submatrix(A, tempk, n*A.nb, tempm, tempnn),
-                A(k, n), 1, tempkm, IPIV(k), 1, 
+                A(k, n), 1, mintmp, IPIV(k), 1,
                 /* Dependency on previous swapb */
                 A(k-1,n), A.lm*A.nb, INPUT,
                 /* Dependency on all GEMM from previous step */
@@ -204,11 +211,11 @@ void plasma_pzgetrf_tile_ll_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *seq
             tempm  = A.m - tempk;
             tempkm = k == A.mt-1 ? tempm : A.mb;
             ldak = BLKLDD(A, k);
-            
+
             QUARK_CORE_zswptr_ontile(
                 plasma->quark, &task_flagsU,
-                plasma_desc_submatrix(A, tempk, n*A.nb, tempm, tempnn), 
-                A(k, n), 1, tempkm, IPIV(k), 1, 
+                plasma_desc_submatrix(A, tempk, n*A.nb, tempm, tempnn),
+                A(k, n), 1, tempkm, IPIV(k), 1,
                 A(k, k), ldak);
 
             if (k < A.mt-1) {
@@ -229,7 +236,7 @@ void plasma_pzgetrf_tile_ll_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *seq
                 {
                     tempmm = m == A.mt-1 ? A.m-m*A.mb : A.mb;
                     ldam = BLKLDD(A, m);
-                    
+
                     QUARK_CORE_zgemm_f2(
                         plasma->quark, &task_flagsU,
                         PlasmaNoTrans, PlasmaNoTrans,
@@ -256,11 +263,11 @@ void plasma_pzgetrf_tile_ll_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *seq
                 panel_thread_count--;
                 QUARK_Task_Flag_Set(&task_flagsP, TASK_THREAD_COUNT, panel_thread_count );
             }
-          
+
             QUARK_CORE_zgetrf_rectil(
                 plasma->quark, &task_flagsP,
                 plasma_desc_submatrix(A, k*A.mb, k*A.nb, tempm, tempkn), A(k, k), A.mb*A.nb,
-                IPIV(k), sequence, request, 
+                IPIV(k), sequence, request,
                 1, A.mb*k, panel_thread_count );
         }
     }
@@ -272,7 +279,7 @@ void plasma_pzgetrf_tile_ll_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *seq
         tempm  = A.m - tempk;
         tempkm = k == A.mt-1 ? tempm : A.mb;
         ldak = BLKLDD(A, k);
-        
+
         fakedep = (void*)(intptr_t)k;
         for (n = 0; n < k; n++)
         {
@@ -283,7 +290,7 @@ void plasma_pzgetrf_tile_ll_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *seq
             QUARK_CORE_zlaswp_ontile_f2(
                 plasma->quark, &task_flagsU,
                 plasma_desc_submatrix(A, tempk, n*A.nb, tempm, tempnn),
-                A(k, n), 1, tempkm, IPIV(k), 1, 
+                A(k, n), 1, tempkm, IPIV(k), 1,
                 /* Dependency on previous swapb */
                 A(k-1,n), A.lm*A.nb, INPUT,
                 /* Dependency on all GEMM from previous step */
@@ -291,4 +298,3 @@ void plasma_pzgetrf_tile_ll_quark(PLASMA_desc A, int *IPIV, PLASMA_sequence *seq
         }
     }
 }
-
