@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Emmanuel Agullo
  * @author Mathieu Faverge
  * @date 2010-11-15
@@ -39,12 +39,18 @@ void plasma_pzlange(plasma_context_t *plasma)
 
     double* lwork;
     double normtmp, normtmp2;
+    double *scale, *sumsq;
 
     plasma_unpack_args_6(norm, A, work, result, sequence, request);
     *result = 0.0;
 
-    if (PLASMA_RANK == 0)
-      memset(work, 0, PLASMA_SIZE*sizeof(double));
+    if (PLASMA_RANK == 0) {
+        if ( norm == PlasmaFrobeniusNorm ) {
+            memset(work, 0, 2*PLASMA_SIZE*sizeof(double));
+        } else {
+            memset(work, 0,   PLASMA_SIZE*sizeof(double));
+        }
+    }
     ss_init(PLASMA_SIZE, 1, 0);
 
     switch (norm) {
@@ -161,9 +167,49 @@ void plasma_pzlange(plasma_context_t *plasma)
         plasma_private_free(plasma, lwork);
         break;
     /*
-     *  PlasmaFrobeniusNorm - not implemented
+     *  PlasmaFrobeniusNorm
      */
     case PlasmaFrobeniusNorm:
+        n = 0;
+        m = PLASMA_RANK;
+        
+        scale = work + 2 * PLASMA_RANK;
+        sumsq = work + 2 * PLASMA_RANK + 1;
+
+        *scale = 0.;
+        *sumsq = 1.;
+
+        while (m >= A.mt && n < A.nt) {
+            n++;
+            m = m-A.mt;
+        }
+
+        while (n < A.nt) {
+            next_m = m;
+            next_n = n;
+
+            next_m += PLASMA_SIZE;
+            while (next_m >= A.mt && next_n < A.nt) {
+                next_n++;
+                next_m = next_m-A.mt;
+            }
+
+            X1 = m == 0      ?  A.i       %A.mb   : 0;
+            X2 = m == A.mt-1 ? (A.i+A.m-1)%A.mb+1 : A.mb;
+            X = X2 - X1;
+
+            Y1 = n == 0      ?  A.j       %A.nb   : 0;
+            Y2 = n == A.nt-1 ? (A.j+A.n-1)%A.nb+1 : A.nb;
+            Y = Y2 - Y1;
+
+            ldam = BLKLDD(A, m);
+            CORE_zgessq( X, Y, A(m, n, X1, Y1, ldam), ldam, scale, sumsq );
+
+            m = next_m;
+            n = next_n;
+        }
+        ss_cond_set(PLASMA_RANK, 0, 1);
+        break;
     default:;
     }
 
@@ -192,6 +238,48 @@ void plasma_pzlange(plasma_context_t *plasma)
 
         if (PLASMA_RANK == 0)
             *result = work[0];
+    } 
+    else {
+        step = 1;
+        lrank = PLASMA_RANK;
+        while ( (lrank%2 == 0) && (PLASMA_RANK+step < PLASMA_SIZE) ) {
+            double scale1, scale2;
+            double sumsq1, sumsq2;
+
+            ss_cond_wait(PLASMA_RANK+step, 0, step);
+
+            scale1 = work[ 2 * PLASMA_RANK ];
+            sumsq1 = work[ 2 * PLASMA_RANK + 1 ];
+            scale2 = work[ 2 * (PLASMA_RANK+step) ];
+            sumsq2 = work[ 2 * (PLASMA_RANK+step) + 1 ];
+
+            if ( scale2 != 0. ){
+                if( scale1 < scale2 ) {
+                    work[2 * PLASMA_RANK+1] = sumsq2 + (sumsq1 * (( scale1 / scale2 ) * ( scale1 / scale2 )));
+                    work[2 * PLASMA_RANK  ] = scale2;
+                } else {
+                    work[2 * PLASMA_RANK+1] = sumsq1 + (sumsq2 * (( scale2 / scale1 ) * ( scale2 / scale1 )));
+                }
+            }
+            lrank = lrank >> 1;
+            step  = step << 1;
+            ss_cond_set(PLASMA_RANK, 0, step);
+        }
+        if (PLASMA_RANK > 0) {
+            while( lrank != 0 ) {
+                if (lrank%2 == 1) {
+                    ss_cond_set(PLASMA_RANK, 0, step);
+                    lrank = 0;
+                } else {
+                    lrank = lrank >> 1;
+                    step  = step << 1;
+                    ss_cond_set(PLASMA_RANK, 0, step);
+                }
+            }
+        }
+
+        if (PLASMA_RANK == 0)
+            *result = work[0] * sqrt( work[1] );
     }
     ss_finalize();
 }
@@ -205,11 +293,12 @@ void plasma_pzlange_quark(PLASMA_enum norm, PLASMA_desc A, double *work, double
     plasma_context_t *plasma;
     Quark_Task_Flags task_flags = Quark_Task_Flags_Initializer;
 
+    double* lwork;
     int X, X1, X2, Y, Y1, Y2;
     int ldam;
-    int m, n;
+    int m, n, k;
     int szeW;
-    double* lwork;
+    int nbworker = 1;
 
     plasma = plasma_context_self();
     if (sequence->status != PLASMA_SUCCESS)
@@ -314,9 +403,44 @@ void plasma_pzlange_quark(PLASMA_enum norm, PLASMA_desc A, double *work, double
         QUARK_CORE_free(plasma->quark, &task_flags, lwork, (A.m+1)*sizeof(PLASMA_Complex64_t));
         break;
     /*
-     *  PlasmaFrobeniusNorm - not implemented
+     *  PlasmaFrobeniusNorm
      */
     case PlasmaFrobeniusNorm:
+        szeW = 2*(PLASMA_SIZE+1);
+        lwork = (double*)plasma_shared_alloc(plasma, szeW, PlasmaRealDouble);
+
+        for(m = 0; m < PLASMA_SIZE+1; m++) {
+            lwork[2*m  ] = 0.;
+            lwork[2*m+1] = 1.;
+        }
+
+        k = 0;
+        for(m = 0; m < A.mt; m++) {
+            X1 = m == 0      ?  A.i       %A.mb   : 0;
+            X2 = m == A.mt-1 ? (A.i+A.m-1)%A.mb+1 : A.mb;
+            X = X2 - X1;
+            ldam = BLKLDD(A, m);
+            for(n = 0; n < A.nt; n++) {
+                Y1 = n == 0      ?  A.j       %A.nb   : 0;
+                Y2 = n == A.nt-1 ? (A.j+A.n-1)%A.nb+1 : A.nb;
+                Y = Y2 - Y1;
+                k++; nbworker++;
+                QUARK_CORE_zgessq_f1(
+                    plasma->quark, &task_flags,
+                    X, Y,
+                    A(m, n, X1, Y1, ldam), ldam,
+                    lwork + 2*k, 
+                    lwork + 2*k + 1,
+                    lwork, szeW, OUTPUT | GATHERV );
+                k = k % PLASMA_SIZE;
+            }
+        }
+        QUARK_CORE_dplssq(
+            plasma->quark, &task_flags,
+            min(nbworker, PLASMA_SIZE+1), lwork, result );
+
+        QUARK_CORE_free(plasma->quark, &task_flags, lwork, szeW*sizeof(double));
+        break;
     default:;
     }
 }
