@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -18,9 +18,9 @@
 #define IPIV(k) &(IPIV[(int64_t)B.mb*(int64_t)(k)])
 
 /***************************************************************************//**
- *  Parallel tile matrix-matrix multiplication - dynamic scheduling
+ *  Parallel tile row interchanges - dynamic scheduling
  **/
-void plasma_pzlaswp_quark(PLASMA_desc B, int *IPIV, int inc,
+void plasma_pzlaswp_quark(PLASMA_desc B, const int *IPIV, int inc,
                           PLASMA_sequence *sequence, PLASMA_request *request)
 {
     plasma_context_t *plasma;
@@ -34,7 +34,7 @@ void plasma_pzlaswp_quark(PLASMA_desc B, int *IPIV, int inc,
         return;
     QUARK_Task_Flag_Set(&task_flags, TASK_SEQUENCE, (intptr_t)sequence->quark_sequence);
 
-    if ( inc > 0 ) 
+    if ( inc > 0 )
     {
         for (m = 0; m < B.mt; m++) {
             tempi = m * B.mb;
@@ -43,29 +43,29 @@ void plasma_pzlaswp_quark(PLASMA_desc B, int *IPIV, int inc,
 
             for (n = 0; n < B.nt; n++) {
                 tempnn = n == B.nt-1 ? B.n - n * B.nb : B.nb;
-                
+
                 QUARK_CORE_zlaswp_ontile(
                     plasma->quark, &task_flags,
                     plasma_desc_submatrix(B, tempi, n*B.nb, tempm, tempnn),
                     B(m, n), 1, tempmm, IPIV(m), inc, B(B.mt-1, n) );
             }
         }
-    } 
-    else 
+    }
+    else
     {
         for (m = B.mt-1; m > -1; m--) {
             tempi = m * B.mb;
             tempm = B.m - tempi;
-            tempmm = m == B.mt-1 ? B.m - m * B.mb : B.mb;
+            tempmm = m == B.mt-1 ? tempm : B.mb;
 
             for (n = 0; n < B.nt; n++) {
                 tempnn = n == B.nt-1 ? B.n - n * B.nb : B.nb;
-                
+
                 QUARK_CORE_zlaswp_ontile(
                     plasma->quark, &task_flags,
                     plasma_desc_submatrix(B, tempi, n*B.nb, tempm, tempnn),
                     B(m, n), 1, tempmm, IPIV(m), inc, B(0, n) );
             }
         }
-    } 
+    }
 }
