@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @author Hatem Ltaief
  * @author Mathieu Faverge
@@ -28,7 +28,7 @@ void plasma_pztrsmpl(plasma_context_t *plasma)
     PLASMA_desc A;
     PLASMA_desc B;
     PLASMA_desc L;
-    int *IPIV;
+    const int *IPIV;
     PLASMA_sequence *sequence;
     PLASMA_request *request;
 
@@ -110,7 +110,7 @@ void plasma_pztrsmpl(plasma_context_t *plasma)
 /***************************************************************************//**
  *  Parallel forward substitution for tile LU - dynamic scheduling
  **/
-void plasma_pztrsmpl_quark(PLASMA_desc A, PLASMA_desc B, PLASMA_desc L, int *IPIV,
+void plasma_pztrsmpl_quark(PLASMA_desc A, PLASMA_desc B, PLASMA_desc L, const int *IPIV,
                            PLASMA_sequence *sequence, PLASMA_request *request)
 {
     plasma_context_t *plasma;
