@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Dulceneia Becker
  * @date 2011-05-24
  * @precisions normal z -> s d c
@@ -22,8 +22,9 @@
  *  Parallel construction of Q using tile V (application to identity;
  *  reduction Householder) - dynamic scheduling
  **/
-void plasma_pzunglqrh_quark(PLASMA_desc A, PLASMA_desc Q, PLASMA_desc T, int BS,
-                            PLASMA_sequence *sequence, PLASMA_request *request)
+void plasma_pzunglqrh_quark(PLASMA_desc A, PLASMA_desc Q,
+        PLASMA_desc T, int BS,
+        PLASMA_sequence *sequence, PLASMA_request *request)
 {
     plasma_context_t *plasma;
     Quark_Task_Flags task_flags = Quark_Task_Flags_Initializer;
@@ -32,7 +33,7 @@ void plasma_pzunglqrh_quark(PLASMA_desc A, PLASMA_desc Q, PLASMA_desc T, int BS,
     int K, N, RD, lastRD;
     int ldak;
     int ldqm;
-    int tempkm, tempNn, tempnn, tempmm, tempNRDn, tempkmin;
+    int tempkm, tempkmin, tempNn, tempnn, tempmm, tempNRDn;
     int ib;
 
     plasma = plasma_context_self();
@@ -50,9 +51,7 @@ void plasma_pzunglqrh_quark(PLASMA_desc A, PLASMA_desc Q, PLASMA_desc T, int BS,
         for (RD = BS; RD < A.nt-k; RD *= 2)
             lastRD = RD;
         for (RD = lastRD; RD >= BS; RD /= 2) {
-            for (N = k;
-                 N+RD < A.nt-1; /* No reduction with rightmost single-column subdomain */
-                 N += 2*RD) {
+            for (N = k; N+RD < A.nt; N += 2*RD) {
                 tempNRDn = N+RD == A.nt-1 ? A.n-(N+RD)*A.nb : A.nb;
                 for (m = 0; m < Q.mt; m++) {
                     tempmm = m == Q.mt-1 ? Q.m-m*Q.mb : Q.mb;
@@ -69,14 +68,10 @@ void plasma_pzunglqrh_quark(PLASMA_desc A, PLASMA_desc Q, PLASMA_desc T, int BS,
                 }
             }
         }
-        for (N = k;
-             N < A.nt-1 || N == k;  /* No rightmost single-column */
-             N += BS) {
+        for (N = k; N < A.nt; N += BS) {
             tempNn = N == A.nt-1 ? A.n-N*A.nb : A.nb;
             tempkmin = min(tempkm, tempNn);
-            for (n = N+BS-1 == A.nt-2 ? A.nt-1 : min(N+BS-1, A.nt-1); /* Suck in rightmost single-column domain */
-                 n >= N+1;
-                 n--) {
+            for (n = min(N+BS, A.nt)-1; n > N; n--) {
                 tempnn = n == Q.nt-1 ? Q.n-n*Q.nb : Q.nb;
                 
                 for (m = 0; m < Q.mt; m++) {
