@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Jakub Kurzak
  * @author Dulceneia Becker
@@ -21,10 +21,11 @@
 #define T(m,n)  BLKADDR(T, PLASMA_Complex64_t, (m), (n))
 #define T2(m,n) BLKADDR(T, PLASMA_Complex64_t, (m), (n)+(A.nt))
 /***************************************************************************//**
- *  Parallel construction of Q using tile V (application to identity; reduction Householder)
- *  - dynamic scheduling
+ *  Parallel construction of Q using tile V (application to identity;
+ * reduction Householder) - dynamic scheduling
  **/
-void plasma_pzungqrrh_quark(PLASMA_desc A, PLASMA_desc Q, PLASMA_desc T, int BS,
+void plasma_pzungqrrh_quark(PLASMA_desc A, PLASMA_desc Q,
+                            PLASMA_desc T, int BS,
                             PLASMA_sequence *sequence, PLASMA_request *request)
 {
     plasma_context_t *plasma;
@@ -50,9 +51,7 @@ void plasma_pzungqrrh_quark(PLASMA_desc A, PLASMA_desc Q, PLASMA_desc T, int BS,
         for (RD = BS; RD < A.mt-k; RD *= 2)
             lastRD = RD;
         for (RD = lastRD; RD >= BS; RD /= 2) {
-            for (M = k;
-                 M+RD < A.mt-1; /* No reduction with bottom single-row subdomain */
-                 M += 2*RD) {
+            for (M = k; M+RD < A.mt; M += 2*RD) {
                 tempMRDm = M+RD == A.mt-1 ? A.m-(M+RD)*A.mb : A.mb;
                 ldbM   = BLKLDD(Q, M   );
                 ldbMRD = BLKLDD(Q, M+RD);
@@ -71,16 +70,12 @@ void plasma_pzungqrrh_quark(PLASMA_desc A, PLASMA_desc Q, PLASMA_desc T, int BS,
                 }
             }
         }
-        for (M = k;
-             M < A.mt-1 || M == k;  /* No bottom single-row subdomain */
-             M += BS) {
+        for (M = k; M < A.mt; M += BS) {
             tempMm   = M == A.mt-1 ? A.m-M*A.mb : A.mb;
             tempkmin = min(tempMm, tempkn);
             ldaM = BLKLDD(A, M);
             ldbM = BLKLDD(Q, M);
-            for (m = M+BS-1 == A.mt-2 ? A.mt-1 : min(M+BS-1, A.mt-1); /* Suck in bottom single-row domain */
-                 m >= M+1;
-                 m--) {
+            for (m = min(M+BS, A.mt)-1; m > M; m--) {
                 tempmm = m == A.mt-1 ? A.m-m*A.mb : A.mb;
                 ldbm = BLKLDD(Q, m);
                 ldam = BLKLDD(A, m);
