@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Jakub Kurzak
  * @author Mathieu Faverge
@@ -140,7 +140,7 @@ void plasma_pzunmlq_quark(PLASMA_enum side, PLASMA_enum trans,
     Quark_Task_Flags task_flags = Quark_Task_Flags_Initializer;
 
     int k, m, n;
-    int ldak, ldan, ldbk, ldbm;
+    int ldak, ldbk, ldbm;
     int tempmm, tempnn, tempkn, tempkm, tempkmin;
     int ib, minMT, minM;
 
@@ -243,7 +243,6 @@ void plasma_pzunmlq_quark(PLASMA_enum side, PLASMA_enum trans,
                 ldak = BLKLDD(A, k);
                 for (n = B.nt-1; n > k; n--) {
                     tempnn = n == B.nt-1 ? B.n-n*B.nb : B.nb;
-                    ldan = BLKLDD(A, n);
                     for (m = 0; m < B.mt; m++) {
                         tempmm = m == B.mt-1 ? B.m-m*B.mb : B.mb;
                         ldbm = BLKLDD(B, m);
