@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Jakub Kurzak
  * @author Dulceneia Becker
@@ -21,8 +21,8 @@
 #define T(m,n)  BLKADDR(T, PLASMA_Complex64_t, (m), (n))
 #define T2(m,n) BLKADDR(T, PLASMA_Complex64_t, (m), (n)+A.nt)
 /***************************************************************************//**
- *  Parallel application of Q using tile V - QR factorization (reduction Householder)
- *  - dynamic scheduling
+ *  Parallel application of Q using tile V - LQ factorization (reduction
+ *  Householder) - dynamic scheduling
  **/
 void plasma_pzunmlqrh_quark(PLASMA_enum side, PLASMA_enum trans,
         PLASMA_desc A, PLASMA_desc B, PLASMA_desc T, int BS,
@@ -33,7 +33,7 @@ void plasma_pzunmlqrh_quark(PLASMA_enum side, PLASMA_enum trans,
 
     int k, m, n;
     int K, N, RD, lastRD;
-    int ldaN, ldam, ldak;
+    int ldaN, ldak;
     int ldbN, ldbm, ldbNRD;
     int tempNn, tempkm, tempnn, tempmm, tempNRDn, tempkmin;
     int ib;
@@ -54,9 +54,7 @@ void plasma_pzunmlqrh_quark(PLASMA_enum side, PLASMA_enum trans,
             for (k = 0; k < K; k++) {
                 tempkm = k == A.mt-1 ? A.m-k*A.mb : A.mb;
                 ldak = BLKLDD(A, k);
-                for (N = k;
-                     N < A.nt-1 || N == k;  /* No rightmost single-column subdomain */
-                     N += BS) {
+                for (N = k; N < A.nt; N += BS) {
                     tempNn   = N == A.nt-1 ? A.n-N*A.nb : A.nb;
                     tempkmin = min(tempkm,tempNn);
                     ldaN = BLKLDD(A, N);
@@ -72,12 +70,9 @@ void plasma_pzunmlqrh_quark(PLASMA_enum side, PLASMA_enum trans,
                                 T(k, N), T.mb,
                                 B(N, n), ldbN);
                     }
-                    for (m = N+1;
-                           (m < N+BS && m < A.nt) || m == A.nt-1; /* Suck in rightmost single-column domain */
-                           m++) {
+                    for (m = N+1; m < min(N+BS, A.nt); m++) {
                         tempmm = m == B.mt-1 ? B.m-m*B.mb : B.mb;
                         ldbm = BLKLDD(B, m);
-                        ldam = BLKLDD(A, m);
                         for (n = 0; n < B.nt; n++) {
                             tempnn = n == B.nt-1 ? B.n-n*B.nb : B.nb;
                             QUARK_CORE_ztsmlq(
@@ -93,9 +88,7 @@ void plasma_pzunmlqrh_quark(PLASMA_enum side, PLASMA_enum trans,
                     }
                 }
                 for (RD = BS; RD < A.nt-k; RD *= 2) {
-                    for (N = k;
-                         N+RD < A.nt-1; /* No reduction with rightmost single-column subdomain */
-                         N += 2*RD) {
+                    for (N = k; N+RD < A.nt; N += 2*RD) {
                         tempNRDn = N+RD == A.nt-1 ? A.n-(N+RD)*A.nb : A.nb;
                         ldbN   = BLKLDD(B, N   );
                         ldbNRD = BLKLDD(B, N+RD);
@@ -125,9 +118,7 @@ void plasma_pzunmlqrh_quark(PLASMA_enum side, PLASMA_enum trans,
                 for (RD = BS; RD < A.nt-k; RD *= 2)
                     lastRD = RD;
                 for (RD = lastRD; RD >= BS; RD /= 2) {
-                    for (N = k;
-                         N+RD < A.nt-1; /* No reduction with rightmost single-column subdomain */
-                         N += 2*RD) {
+                    for (N = k; N+RD < A.nt; N += 2*RD) {
                         tempNRDn = N+RD == A.nt-1 ? A.n-(N+RD)*A.nb : A.nb;
                         ldbN   = BLKLDD(B, N   );
                         ldbNRD = BLKLDD(B, N+RD);
@@ -145,19 +136,14 @@ void plasma_pzunmlqrh_quark(PLASMA_enum side, PLASMA_enum trans,
                         }
                     }
                 }
-                for (N = k;
-                     N < A.nt-1 || N == k;  /* No rightmost single-column subdomain */
-                     N += BS) {
+                for (N = k; N < A.nt; N += BS) {
                     tempNn   = N == A.nt-1 ? A.n-N*A.nb : A.nb;
                     tempkmin = min(tempkm,tempNn);
                     ldaN = BLKLDD(A, N);
                     ldbN = BLKLDD(B, N);
-                    for (m = N+BS-1 == A.nt-2 ? A.nt-1 : min(N+BS-1, A.nt-1); /* Suck in rightmost single-column domain */
-                         m >= N+1;
-                         m--) {
+                    for (m = min(N+BS, A.nt)-1; m > N; m--) {
                         tempmm = m == B.mt-1 ? B.m-m*B.mb : B.mb;
                         ldbm = BLKLDD(B, m);
-                        ldam = BLKLDD(A, m);
                         for (n = 0; n < B.nt; n++) {
                             tempnn = n == B.nt-1 ? B.n-n*B.nb : B.nb;
                             QUARK_CORE_ztsmlq(
@@ -198,9 +184,7 @@ void plasma_pzunmlqrh_quark(PLASMA_enum side, PLASMA_enum trans,
                   for (RD = BS; RD < A.nt-k; RD *= 2)
                       lastRD = RD;
                   for (RD = lastRD; RD >= BS; RD /= 2) {
-                      for (N = k;
-                           N+RD < A.nt-1; /* No reduction with rightmost single-column subdomain */
-                           N += 2*RD) {
+                      for (N = k; N+RD < A.nt; N += 2*RD) {
                           tempNRDn = N+RD == A.nt-1 ? A.n-(N+RD)*A.nb : A.nb;
                           for (m = 0; m < B.mt; m++) {
                               ldbm   = BLKLDD(B, m);
@@ -217,14 +201,10 @@ void plasma_pzunmlqrh_quark(PLASMA_enum side, PLASMA_enum trans,
                           }
                       }
                   }
-                  for (N = k;
-                       N < A.nt-1 || N == k;  /* No rightmost single-column subdomain */
-                       N += BS) {
+                  for (N = k; N < A.nt; N += BS) {
                       tempNn   = N == A.nt-1 ? A.n-N*A.nb : A.nb;
                       tempkmin = min(tempkm,tempNn);
-                      for (n = N+BS-1 == A.nt-2 ? A.nt-1 : min(N+BS-1, A.nt-1); /* Suck in rightmost single-column domain */
-                              n >= N+1;
-                              n--) {
+                      for (n = min(N+BS, A.nt)-1; n > N; n--) {
                           tempnn = n == B.nt-1 ? B.n-n*B.nb : B.nb;
                           for (m = 0; m < B.mt; m++) {
                               tempmm = m == B.mt-1 ? B.m-m*B.mb : B.mb;
@@ -261,9 +241,7 @@ void plasma_pzunmlqrh_quark(PLASMA_enum side, PLASMA_enum trans,
             for (k = 0; k < K; k++) {
                 tempkm = k == A.mt-1 ? A.m-k*A.mb : A.mb;
                 ldak = BLKLDD(A, k);
-                for (N = k;
-                     N < A.nt-1 || N == k;  /* No rightmost single-column subdomain */
-                     N += BS) {
+                for (N = k; N < A.nt; N += BS) {
                     tempNn = N == A.nt-1 ? A.n-N*A.nb : A.nb;
                     tempkmin = min(tempkm,tempNn);
                     ldaN = BLKLDD(A, N);
@@ -279,9 +257,7 @@ void plasma_pzunmlqrh_quark(PLASMA_enum side, PLASMA_enum trans,
                                 T(k, N), T.mb,
                                 B(m, N), ldbm);
                     }
-                    for (n = N+1;
-                            (n < N+BS && n < A.nt) || n == A.nt-1; /* Suck in rightmost single-column domain */
-                            n++) {
+                    for (n = N+1; n < min(N+BS, A.nt); n++) {
                         tempnn = n == B.nt-1 ? B.n-n*B.nb : B.nb;
                         for (m = 0; m < B.mt; m++) {
                             tempmm = m == B.mt-1 ? B.m-m*B.mb : B.mb;
@@ -299,9 +275,7 @@ void plasma_pzunmlqrh_quark(PLASMA_enum side, PLASMA_enum trans,
                     }
                 }
                 for (RD = BS; RD < A.nt-k; RD *= 2) {
-                    for (N = k;
-                            N+RD < A.nt-1; /* No reduction with rightmost single-column subdomain */
-                            N += 2*RD) {
+                    for (N = k; N+RD < A.nt; N += 2*RD) {
                         tempNRDn = N+RD == A.nt-1 ? A.n-(N+RD)*A.nb : A.nb;
                         for (m = 0; m < B.mt; m++) {
                             tempmm = m == B.mt-1 ? B.m-m*B.mb : B.mb;
