@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Jakub Kurzak
  * @author Dulceneia Becker
@@ -21,8 +21,8 @@
 #define T(m,n)  BLKADDR(T, PLASMA_Complex64_t, (m), (n))
 #define T2(m,n) BLKADDR(T, PLASMA_Complex64_t, (m), (n)+A.nt)
 /***************************************************************************//**
- *  Parallel application of Q using tile V - QR factorization (reduction Householder)
- *  - dynamic scheduling
+ *  Parallel application of Q using tile V - QR factorization (reduction
+ *  Householder) - dynamic scheduling
  **/
 void plasma_pzunmqrrh_quark(PLASMA_enum side, PLASMA_enum trans,
         PLASMA_desc A, PLASMA_desc B, PLASMA_desc T, int BS,
@@ -53,9 +53,7 @@ void plasma_pzunmqrrh_quark(PLASMA_enum side, PLASMA_enum trans,
              */
             for (k = 0; k < K; k++) {
                 tempkn = k == A.nt-1 ? A.n-k*A.nb : A.nb;
-                for (M = k;
-                        M < A.mt-1 || M == k;  /*  No bottom single-row subdomain */
-                        M += BS) {
+                for (M = k; M < A.mt; M += BS) {
                     tempMm   = M == A.mt-1 ? A.m-M*A.mb : A.mb;
                     tempkmin = min(tempMm, tempkn);
                     ldaM = BLKLDD(A, M);
@@ -71,9 +69,7 @@ void plasma_pzunmqrrh_quark(PLASMA_enum side, PLASMA_enum trans,
                                 T(M, k), T.mb,
                                 B(M, n), ldbM);
                     }
-                    for (m = M+1;
-                            (m < M+BS && m < A.mt) || m == A.mt-1; /*  Suck in bottom single-row domain */
-                            m++) {
+                    for (m = M+1; m < min(M+BS, A.mt); m++) {
                         tempmm = m == A.mt-1 ? A.m-m*A.mb : A.mb;
                         ldbm = BLKLDD(B, m);
                         ldam = BLKLDD(A, m);
@@ -92,9 +88,7 @@ void plasma_pzunmqrrh_quark(PLASMA_enum side, PLASMA_enum trans,
                     }
                 }
                 for (RD = BS; RD < A.mt-k; RD *= 2) {
-                    for (M = k;
-                            M+RD < A.mt-1; /*  No reduction with bottom single-row subdomain */
-                            M += 2*RD) {
+                    for (M = k; M+RD < A.mt; M += 2*RD) {
                         tempMRDm = M+RD == A.mt-1 ? A.m-(M+RD)*A.mb : A.mb;
                         ldbM   = BLKLDD(B, M   );
                         ldbMRD = BLKLDD(B, M+RD);
@@ -124,9 +118,7 @@ void plasma_pzunmqrrh_quark(PLASMA_enum side, PLASMA_enum trans,
                 for (RD = BS; RD < A.mt-k; RD *= 2)
                     lastRD = RD;
                 for (RD = lastRD; RD >= BS; RD /= 2) {
-                    for (M = k;
-                            M+RD < A.mt-1; /*  No reduction with bottom single-row subdomain */
-                            M += 2*RD) {
+                    for (M = k; M+RD < A.mt; M += 2*RD) {
                         tempMRDm = M+RD == A.mt-1 ? A.m-(M+RD)*A.mb : A.mb;
                         ldbM   = BLKLDD(B, M   );
                         ldbMRD = BLKLDD(B, M+RD);
@@ -145,16 +137,12 @@ void plasma_pzunmqrrh_quark(PLASMA_enum side, PLASMA_enum trans,
                         }
                     }
                 }
-                for (M = k;
-                        M < A.mt-1 || M == k;  /*  No bottom single-row subdomain */
-                        M += BS) {
+                for (M = k; M < A.mt; M += BS) {
                     tempMm   = M == A.mt-1 ? A.m-M*A.mb : A.mb;
                     tempkmin = min(tempMm, tempkn);
                     ldaM = BLKLDD(A, M);
                     ldbM = BLKLDD(B, M);
-                    for (m = M+BS-1 == A.mt-2 ? A.mt-1 : min(M+BS-1, A.mt-1); /*  Suck in bottom single-row domain */
-                            m >= M+1;
-                            m--) {
+                    for (m = min(M+BS, A.mt)-1; m > M; m--) {
                         tempmm = m == A.mt-1 ? A.m-m*A.mb : A.mb;
                         ldbm = BLKLDD(B, m);
                         ldam = BLKLDD(A, m);
@@ -196,9 +184,7 @@ void plasma_pzunmqrrh_quark(PLASMA_enum side, PLASMA_enum trans,
                   for (RD = BS; RD < A.mt-k; RD *= 2)
                       lastRD = RD;
                   for (RD = lastRD; RD >= BS; RD /= 2) {
-                      for (M = k;
-                              M+RD < A.mt-1; /*  No reduction with bottom single-row subdomain */
-                              M += 2*RD) {
+                      for (M = k; M+RD < A.mt; M += 2*RD) {
                           tempMRDm = M+RD == A.mt-1 ? A.m-(M+RD)*A.mb : A.mb;
                           ldaMRD = BLKLDD(A, M+RD);
                           for (m = 0; m < B.mt; m++) {
@@ -216,16 +202,12 @@ void plasma_pzunmqrrh_quark(PLASMA_enum side, PLASMA_enum trans,
                           }
                       }
                   }
-                  for (M = k;
-                          M < A.mt-1 || M == k;  /*  No bottom single-row subdomain */
-                          M += BS) {
+                  for (M = k; M < A.mt; M += BS) {
                       tempMm   = M == A.mt-1 ? A.m-M*A.mb : A.mb;
                       tempkmin = min(tempMm, tempkn);
                       ldaM = BLKLDD(A, M);
                       ldbM = BLKLDD(B, M);
-                      for (n = M+BS-1 == A.mt-2 ? A.mt-1 : min(M+BS-1, A.mt-1); /*  Suck in bottom single-row domain */
-                              n >= M+1;
-                              n--) {
+                      for (n = min(M+BS, A.mt)-1; n > M; n--) {
                           ldan = BLKLDD(A, n);
                           tempnn = n == B.nt-1 ? B.n-n*B.nb : B.nb;
                           for (m = 0; m < B.mt; m++) {
@@ -262,9 +244,7 @@ void plasma_pzunmqrrh_quark(PLASMA_enum side, PLASMA_enum trans,
              */
             for (k = 0; k < K; k++) {
                 tempkn = k == A.nt-1 ? A.n-k*A.nb : A.nb;
-                for (M = k;
-                        M < A.mt-1 || M == k;  /*  No bottom single-row subdomain */
-                        M += BS) {
+                for (M = k; M < A.mt; M += BS) {
                     tempMm   = M == A.mt-1 ? A.m-M*A.mb : A.mb;
                     tempkmin = min(tempMm, tempkn);
                     ldaM = BLKLDD(A, M);
@@ -280,9 +260,7 @@ void plasma_pzunmqrrh_quark(PLASMA_enum side, PLASMA_enum trans,
                                 T(M, k), T.mb,
                                 B(m, M), ldbm);
                     }
-                    for (n = M+1;
-                            (n < M+BS && n < A.mt) || n == A.mt-1; /*  Suck in bottom single-row domain */
-                            n++) {
+                    for (n = M+1; n < min(M+BS,  A.mt); n++) {
                         tempnn = n == B.nt-1 ? B.n-n*B.nb : B.nb;
                         ldan = BLKLDD(A, n);
                         for (m = 0; m < B.mt; m++) {
@@ -301,9 +279,7 @@ void plasma_pzunmqrrh_quark(PLASMA_enum side, PLASMA_enum trans,
                     }
                 }
                 for (RD = BS; RD < A.mt-k; RD *= 2) {
-                    for (M = k;
-                            M+RD < A.mt-1; /*  No reduction with bottom single-row subdomain */
-                            M += 2*RD) {
+                    for (M = k; M+RD < A.mt; M += 2*RD) {
                         tempMRDm = M+RD == A.mt-1 ? A.m-(M+RD)*A.mb : A.mb;
                         ldaMRD = BLKLDD(A, M+RD);
                         for (m = 0; m < B.mt; m++) {
