@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Emmanuel Agullo
  * @date 2010-11-15
  * @precisions mixed zc -> ds
@@ -18,14 +18,14 @@
 #include <lapacke.h>
 #include "common.h"
 
-#define PLASMA_zlag2c(_descA, _descSB)		      \
-  plasma_parallel_call_4(plasma_pzlag2c,	      \
+#define PLASMA_zlag2c(_descA, _descSB)                \
+  plasma_parallel_call_4(plasma_pzlag2c,              \
                          PLASMA_desc,      (_descA),  \
                          PLASMA_desc,      (_descSB), \
                          PLASMA_sequence*, sequence,  \
                          PLASMA_request*,  request)
 
-#define PLASMA_clag2z(_descSA, _descB)		      \
+#define PLASMA_clag2z(_descSA, _descB)                \
   plasma_parallel_call_4(plasma_pclag2z,              \
                          PLASMA_desc,      (_descSA), \
                          PLASMA_desc,      (_descB),  \
@@ -50,8 +50,8 @@
                          PLASMA_sequence*, sequence,         \
                          PLASMA_request*,  request)
 
-#define PLASMA_zaxpy(_alpha, _descA, _descB)           \
-  plasma_parallel_call_5(plasma_pzaxpy,                \
+#define PLASMA_zgeadd(_alpha, _descA, _descB)           \
+  plasma_parallel_call_5(plasma_pzgeadd,                \
                          PLASMA_Complex64_t, (_alpha), \
                          PLASMA_desc,        (_descA), \
                          PLASMA_desc,        (_descB), \
@@ -184,7 +184,17 @@ int PLASMA_zcgels(PLASMA_enum trans, int M, int N, int NRHS,
         plasma_fatal_error("PLASMA_zcgels", "PLASMA not initialized");
         return PLASMA_ERR_NOT_INITIALIZED;
     }
+
+    *ITER = 0;
+
     /* Check input arguments */
+    if (trans != PlasmaNoTrans   &&
+        trans != PlasmaConjTrans &&
+        trans != PlasmaTrans )
+    {
+        plasma_error("PLASMA_zcgels", "illegal value of trans");
+        return -1;
+    }
     if (trans != PlasmaNoTrans) {
         plasma_error("PLASMA_zcgels", "only PlasmaNoTrans supported");
         return PLASMA_ERR_NOT_SUPPORTED;
@@ -373,7 +383,7 @@ int PLASMA_zcgels(PLASMA_enum trans, int M, int N, int NRHS,
  * @param[out] ITER
  *          If > 0, ITER is the number of the current iteration in the iterative refinement process.
  *          -ITERMAX-1, if the refinment step failed and the double precision factorization has been used.
- *       
+ *
  *******************************************************************************
  *
  * @return
@@ -419,7 +429,7 @@ int PLASMA_zcgels_Tile(PLASMA_enum trans, PLASMA_desc *A, PLASMA_desc *T,
  *  mixed-precision iterative refinement.
  *  Non-blocking equivalent of PLASMA_zcgels_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -443,10 +453,10 @@ int PLASMA_zcgels_Tile_Async(PLASMA_enum trans, PLASMA_desc *A, PLASMA_desc *T,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
     int M, N, NRHS, NB, NBNB, MT, NT, NTRHS;
-    PLASMA_desc descA = *A;
-    PLASMA_desc descT = *T;
-    PLASMA_desc descB = *B;
-    PLASMA_desc descX = *X;
+    PLASMA_desc descA;
+    PLASMA_desc descT;
+    PLASMA_desc descB;
+    PLASMA_desc descX;
     plasma_context_t *plasma;
     double *work;
 
@@ -478,21 +488,29 @@ int PLASMA_zcgels_Tile_Async(PLASMA_enum trans, PLASMA_desc *A, PLASMA_desc *T,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zcgels_Tile", "invalid first descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descT) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(T) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zcgels_Tile", "invalid second descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descT = *T;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zcgels_Tile", "invalid third descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descB = *B;
     }
-    if (plasma_desc_check(&descX) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(X) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zcgels_Tile", "invalid fourth descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descX = *X;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb || descB.nb != descB.mb || descX.nb != descX.mb) {
@@ -744,7 +762,7 @@ int PLASMA_zcgels_Tile_Async(PLASMA_enum trans, PLASMA_desc *A, PLASMA_desc *T,
         /* Convert SX back to double precision and update the current
            iterate. */
         PLASMA_clag2z(descSX, descR);
-        PLASMA_zaxpy(one, descR, descX);
+        PLASMA_zgeadd(one, descR, descX);
 
         /* Compute R = B - AX. */
         PLASMA_zlacpy(descB,descR);
