@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Emmanuel Agullo
  * @author Mathieu Faverge
  * @date 2010-11-15
@@ -18,18 +18,18 @@
 #include <lapacke.h>
 #include "common.h"
 
-#define PLASMA_zlag2c(_descA, _descSB)			\
-  plasma_parallel_call_4(plasma_pzlag2c,		\
-                         PLASMA_desc,      (_descA),	\
-                         PLASMA_desc,      (_descSB),	\
-                         PLASMA_sequence*, sequence,	\
+#define PLASMA_zlag2c(_descA, _descSB)                  \
+  plasma_parallel_call_4(plasma_pzlag2c,                \
+                         PLASMA_desc,      (_descA),    \
+                         PLASMA_desc,      (_descSB),   \
+                         PLASMA_sequence*, sequence,    \
                          PLASMA_request*,  request)
 
-#define PLASMA_clag2z(_descSA, _descB)			\
-  plasma_parallel_call_4(plasma_pclag2z,        	\
+#define PLASMA_clag2z(_descSA, _descB)                  \
+  plasma_parallel_call_4(plasma_pclag2z,                \
                          PLASMA_desc,      (_descSA),   \
                          PLASMA_desc,      (_descB),    \
-                         PLASMA_sequence*, sequence,	\
+                         PLASMA_sequence*, sequence,    \
                          PLASMA_request*,  request)
 
 #define PLASMA_zlange(_norm, _descA, _result, _work)    \
@@ -46,16 +46,16 @@
   plasma_parallel_call_5(plasma_pzlacpy,                        \
                          PLASMA_enum,      PlasmaUpperLower,    \
                          PLASMA_desc,      (_descA),            \
-                         PLASMA_desc,      (_descB),        	\
+                         PLASMA_desc,      (_descB),            \
                          PLASMA_sequence*, sequence,            \
                          PLASMA_request*,  request)
 
-#define PLASMA_zaxpy(_alpha, _descA, _descB)        	\
-  plasma_parallel_call_5(plasma_pzaxpy,                	\
-                         PLASMA_Complex64_t, (_alpha),	\
-                         PLASMA_desc,        (_descA),	\
-                         PLASMA_desc,        (_descB),	\
-                         PLASMA_sequence*,   sequence,	\
+#define PLASMA_zgeadd(_alpha, _descA, _descB)           \
+  plasma_parallel_call_5(plasma_pzgeadd,                \
+                         PLASMA_Complex64_t, (_alpha),  \
+                         PLASMA_desc,        (_descA),  \
+                         PLASMA_desc,        (_descB),  \
+                         PLASMA_sequence*,   sequence,  \
                          PLASMA_request*,    request)
 
 
@@ -190,6 +190,9 @@
  * @param[in] LDA
  *          The leading dimension of the array A. LDA >= max(1,N).
  *
+ * @param[out] IPIV
+ *          On exit, the pivot indices that define the permutations.
+ *
  * @param[in] B
  *          The N-by-NRHS matrix of right hand side matrix B.
  *
@@ -240,6 +243,9 @@ int PLASMA_zcgesv(int N, int NRHS,
         plasma_fatal_error("PLASMA_zcgesv", "PLASMA not initialized");
         return PLASMA_ERR_NOT_INITIALIZED;
     }
+
+    *ITER = 0;
+
     /* Check input arguments */
     if (N < 0) {
         plasma_error("PLASMA_zcgesv", "illegal value of N");
@@ -255,11 +261,11 @@ int PLASMA_zcgesv(int N, int NRHS,
     }
     if (LDB < max(1, N)) {
         plasma_error("PLASMA_zcgesv", "illegal value of LDB");
-        return -8;
+        return -7;
     }
     if (LDX < max(1, N)) {
         plasma_error("PLASMA_zcgesv", "illegal value of LDX");
-        return -10;
+        return -9;
     }
     /* Quick return */
     if (min(N, NRHS) == 0)
@@ -273,20 +279,24 @@ int PLASMA_zcgesv(int N, int NRHS,
     }
 
     NB = PLASMA_NB;
-    
+
     plasma_sequence_create(plasma, &sequence);
 
     /* DOUBLE PRECISION INITIALIZATION */
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   , plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)) );
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,    sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, sequence, &request,
+                             plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)) );
         plasma_zdesc_alloc(  descX, NB, NB, N, NRHS, 0, 0, N, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)); plasma_desc_mat_free(&(descX)) );
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   );
-        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,
+                            sequence, &request);
+        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS,
+                            sequence, &request);
 
         descX = plasma_desc_init(
-            PlasmaComplexDouble, NB, NB, (NB*NB), 
+            PlasmaComplexDouble, NB, NB, (NB*NB),
             LDX, NRHS, 0, 0, N, NRHS);
         descX.mat = X;
     }
@@ -296,15 +306,15 @@ int PLASMA_zcgesv(int N, int NRHS,
 
     if (status == PLASMA_SUCCESS) {
         if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-            plasma_zooptile2lap( descX, X, NB, NB, LDX, NRHS );
+            plasma_zooptile2lap( descX, X, NB, NB, LDX, NRHS,  sequence, &request);
             plasma_dynamic_sync();
             plasma_desc_mat_free(&descA);
             plasma_desc_mat_free(&descB);
             plasma_desc_mat_free(&descX);
         } else {
-            plasma_ziptile2lap( descA, A, NB, NB, LDA, N    );
-            plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS );
-            plasma_ziptile2lap( descX, X, NB, NB, LDX, NRHS );
+            plasma_ziptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+            plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
+            plasma_ziptile2lap( descX, X, NB, NB, LDX, NRHS,  sequence, &request);
             plasma_dynamic_sync();
         }
     }
@@ -334,13 +344,6 @@ int PLASMA_zcgesv(int N, int NRHS,
  *          - otherwise, it fell back to double precision solution,
  *          and then A contains the tile L and U factors from the factorization (not equivalent to LAPACK).
  *
- * @param[out] L
- *          On exit:
- *          - if the iterative refinement converged, L is not modified;
- *          - otherwise, it fell back to double precision solution,
- *          and then L is an auxiliary factorization data, related to the tile L factor,
- *          necessary to solve the system of equations (not equivalent to LAPACK).
- *
  * @param[out] IPIV
  *          On exit, the pivot indices that define the permutations (not equivalent to LAPACK).
  *
@@ -399,7 +402,7 @@ int PLASMA_zcgesv_Tile(PLASMA_desc *A, int *IPIV,
  *  the tile LU factorization and mixed-precision iterative refinement.
  *  Non-blocking equivalent of PLASMA_zcgesv_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -423,9 +426,9 @@ int PLASMA_zcgesv_Tile_Async(PLASMA_desc *A, int *IPIV,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
     int N, NB;
-    PLASMA_desc descA = *A;
-    PLASMA_desc descB = *B;
-    PLASMA_desc descX = *X;
+    PLASMA_desc descA;
+    PLASMA_desc descB;
+    PLASMA_desc descX;
     PLASMA_desc descR, descSA, descSX;
     plasma_context_t *plasma;
     double *work;
@@ -458,17 +461,23 @@ int PLASMA_zcgesv_Tile_Async(PLASMA_desc *A, int *IPIV,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zcgesv_Tile", "invalid first descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zcgesv_Tile", "invalid third descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descB = *B;
     }
-    if (plasma_desc_check(&descX) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(X) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zcgesv_Tile", "invalid fourth descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descX = *X;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb || descB.nb != descB.mb || descX.nb != descX.mb) {
@@ -487,13 +496,13 @@ int PLASMA_zcgesv_Tile_Async(PLASMA_desc *A, int *IPIV,
         return PLASMA_ERR_OUT_OF_RESOURCES;
     }
 
-    plasma_zdesc_alloc( descR,  NB, NB, descB.m, descB.n, 0, 0, descB.m, descB.n, 
+    plasma_zdesc_alloc( descR,  NB, NB, descB.m, descB.n, 0, 0, descB.m, descB.n,
                         plasma_shared_free( plasma, work ); plasma_desc_mat_free(&descR) );
-    plasma_cdesc_alloc( descSA, NB, NB, descA.m, descA.n, 0, 0, descA.m, descA.n, 
-                        plasma_shared_free( plasma, work ); plasma_desc_mat_free(&descR); 
+    plasma_cdesc_alloc( descSA, NB, NB, descA.m, descA.n, 0, 0, descA.m, descA.n,
+                        plasma_shared_free( plasma, work ); plasma_desc_mat_free(&descR);
                         plasma_desc_mat_free(&descSA) );
-    plasma_cdesc_alloc( descSX, NB, NB, descX.m, descX.n, 0, 0, descX.m, descX.n, 
-                        plasma_shared_free( plasma, work ); plasma_desc_mat_free(&descR); 
+    plasma_cdesc_alloc( descSX, NB, NB, descX.m, descX.n, 0, 0, descX.m, descX.n,
+                        plasma_shared_free( plasma, work ); plasma_desc_mat_free(&descR);
                         plasma_desc_mat_free(&descSA); plasma_desc_mat_free(&descSX) );
 
     /* Compute some constants */
@@ -552,7 +561,7 @@ int PLASMA_zcgesv_Tile_Async(PLASMA_desc *A, int *IPIV,
     PLASMA_zlange(PlasmaMaxNorm, descX, Xnorm, work);
     PLASMA_zlange(PlasmaMaxNorm, descR, Rnorm, work);
 
-    /* Wait the end of Anorm, Xnorm and Bnorm computations */
+    /* Wait for the end of Anorm, Xnorm and Bnorm computations */
     plasma_dynamic_sync();
 
     cte = Anorm*eps*sqrt((double) N)*bwdmax;
@@ -579,7 +588,7 @@ int PLASMA_zcgesv_Tile_Async(PLASMA_desc *A, int *IPIV,
         /* Convert SX back to double precision and update the current
            iterate. */
         PLASMA_clag2z(descSX, descR);
-        PLASMA_zaxpy(one, descR, descX);
+        PLASMA_zgeadd(one, descR, descX);
 
         /* Compute R = B - AX. */
         PLASMA_zlacpy(descB,descR);
@@ -599,7 +608,7 @@ int PLASMA_zcgesv_Tile_Async(PLASMA_desc *A, int *IPIV,
         PLASMA_zlange(PlasmaInfNorm, descX, Xnorm, work);
         PLASMA_zlange(PlasmaInfNorm, descR, Rnorm, work);
 
-        /* Wait the end of Xnorm and Bnorm computations */
+        /* Wait for the end of Xnorm and Bnorm computations */
         plasma_dynamic_sync();
 
         if (Rnorm < Xnorm * cte){
@@ -640,6 +649,6 @@ int PLASMA_zcgesv_Tile_Async(PLASMA_desc *A, int *IPIV,
     PLASMA_zlacpy(descB, descX);
 
     PLASMA_zgetrs(descA, IPIV, descX);
-    
+
     return PLASMA_SUCCESS;
 }
