@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Emmanuel Agullo
  * @date 2010-11-15
  * @precisions mixed zc -> ds
@@ -17,8 +17,8 @@
 #include <lapacke.h>
 #include "common.h"
 
-#define PLASMA_zlag2c(_descA, _descSB)		      \
-  plasma_parallel_call_4(plasma_pzlag2c,	      \
+#define PLASMA_zlag2c(_descA, _descSB)                \
+  plasma_parallel_call_4(plasma_pzlag2c,              \
                          PLASMA_desc,      (_descA),  \
                          PLASMA_desc,      (_descSB), \
                          PLASMA_sequence*, sequence,  \
@@ -60,12 +60,12 @@
                          PLASMA_sequence*, sequence,         \
                          PLASMA_request*,  request)
 
-#define PLASMA_zaxpy(_alpha, _descA, _descB)        	\
-  plasma_parallel_call_5(plasma_pzaxpy,                	\
-                         PLASMA_Complex64_t, (_alpha),	\
-                         PLASMA_desc,        (_descA),	\
-                         PLASMA_desc,        (_descB),	\
-                         PLASMA_sequence*,   sequence,	\
+#define PLASMA_zgeadd(_alpha, _descA, _descB)           \
+  plasma_parallel_call_5(plasma_pzgeadd,                \
+                         PLASMA_Complex64_t, (_alpha),  \
+                         PLASMA_desc,        (_descA),  \
+                         PLASMA_desc,        (_descB),  \
+                         PLASMA_sequence*,   sequence,  \
                          PLASMA_request*,    request)
 
 /***************************************************************************//**
@@ -187,6 +187,9 @@ int PLASMA_zcposv(PLASMA_enum uplo, int N, int NRHS,
         plasma_fatal_error("PLASMA_zcposv", "PLASMA not initialized");
         return PLASMA_ERR_NOT_INITIALIZED;
     }
+
+    *ITER = 0;
+
     /* Check input arguments */
     if (uplo != PlasmaUpper && uplo != PlasmaLower) {
         plasma_error("PLASMA_zcposv", "illegal value of uplo");
@@ -210,7 +213,7 @@ int PLASMA_zcposv(PLASMA_enum uplo, int N, int NRHS,
     }
     if (LDX < max(1, N)) {
         plasma_error("PLASMA_zcposv", "illegal value of LDX");
-        return -10;
+        return -9;
     }
     /* Quick return - currently NOT equivalent to LAPACK's
      * LAPACK does not have such check for ZCPOSV */
@@ -230,12 +233,16 @@ int PLASMA_zcposv(PLASMA_enum uplo, int N, int NRHS,
 
     /* DOUBLE PRECISION INITIALIZATION */
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   , plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)) );
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,    sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, sequence, &request,
+                             plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)) );
         plasma_zdesc_alloc(  descX, NB, NB, N, NRHS, 0, 0, N, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)); plasma_desc_mat_free(&(descX)) );
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   );
-        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,   
+                            sequence, &request);
+        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS,
+                            sequence, &request);
 
         descX = plasma_desc_init(
             PlasmaComplexDouble, NB, NB, (NB*NB), 
@@ -248,15 +255,15 @@ int PLASMA_zcposv(PLASMA_enum uplo, int N, int NRHS,
 
     if (status == PLASMA_SUCCESS) {
         if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-            plasma_zooptile2lap( descX, X, NB, NB, LDX, NRHS );
+            plasma_zooptile2lap( descX, X, NB, NB, LDX, NRHS,  sequence, &request);
             plasma_dynamic_sync();
             plasma_desc_mat_free(&descA);
             plasma_desc_mat_free(&descB);
             plasma_desc_mat_free(&descX);
         } else {
-            plasma_ziptile2lap( descA, A, NB, NB, LDA, N    );
-            plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS );
-            plasma_ziptile2lap( descX, X, NB, NB, LDX, NRHS );
+            plasma_ziptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+            plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
+            plasma_ziptile2lap( descX, X, NB, NB, LDX, NRHS,  sequence, &request);
             plasma_dynamic_sync();
         }
     }
@@ -352,7 +359,7 @@ int PLASMA_zcposv_Tile(PLASMA_enum uplo, PLASMA_desc *A, PLASMA_desc *B,
  *  and mixed-precision iterative refinement.
  *  Non-blocking equivalent of PLASMA_zcposv_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -376,9 +383,9 @@ int PLASMA_zcposv_Tile_Async(PLASMA_enum uplo, PLASMA_desc *A, PLASMA_desc *B,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
     int N, NB;
-    PLASMA_desc descA = *A;
-    PLASMA_desc descB = *B;
-    PLASMA_desc descX = *X;
+    PLASMA_desc descA;
+    PLASMA_desc descB;
+    PLASMA_desc descX;
     plasma_context_t *plasma;
     double *work;
     PLASMA_desc descR, descSA, descSX;
@@ -393,15 +400,15 @@ int PLASMA_zcposv_Tile_Async(PLASMA_enum uplo, PLASMA_desc *A, PLASMA_desc *B,
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
-        plasma_fatal_error("PLASMA_zcposv_Tile", "PLASMA not initialized");
+        plasma_fatal_error("PLASMA_zcposv_Tile_Async", "PLASMA not initialized");
         return PLASMA_ERR_NOT_INITIALIZED;
     }
     if (sequence == NULL) {
-        plasma_fatal_error("PLASMA_zcposv_Tile", "NULL sequence");
+        plasma_fatal_error("PLASMA_zcposv_Tile_Async", "NULL sequence");
         return PLASMA_ERR_UNALLOCATED;
     }
     if (request == NULL) {
-        plasma_fatal_error("PLASMA_zcposv_Tile", "NULL request");
+        plasma_fatal_error("PLASMA_zcposv_Tile_Async", "NULL request");
         return PLASMA_ERR_UNALLOCATED;
     }
     /* Check sequence status */
@@ -411,26 +418,31 @@ int PLASMA_zcposv_Tile_Async(PLASMA_enum uplo, PLASMA_desc *A, PLASMA_desc *B,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
-        plasma_error("PLASMA_zcposv_Tile", "invalid first descriptor");
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
+        plasma_error("PLASMA_zcposv_Tile_Async", "invalid first descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
-        plasma_error("PLASMA_zcposv_Tile", "invalid second descriptor");
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
+        plasma_error("PLASMA_zcposv_Tile_Async", "invalid second descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descB = *B;
     }
-    if (plasma_desc_check(&descX) != PLASMA_SUCCESS) {
-        plasma_error("PLASMA_zcposv_Tile", "invalid third descriptor");
+    if (plasma_desc_check(X) != PLASMA_SUCCESS) {
+        plasma_error("PLASMA_zcposv_Tile_Async", "invalid third descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descX = *X;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb || descB.nb != descB.mb || descX.nb != descX.mb) {
-        plasma_error("PLASMA_zcposv_Tile", "only square tiles supported");
+        plasma_error("PLASMA_zcposv_Tile_Async", "only square tiles supported");
         return PLASMA_ERR_ILLEGAL_VALUE;
     }
-    // if (uplo != PlasmaUpper && uplo != PlasmaLower) { ONLY Lower for now
-    if (uplo != PlasmaLower) {
-        plasma_error("PLASMA_zcposv_Tile", "illegal value of uplo");
+    if (uplo != PlasmaUpper && uplo != PlasmaLower) {
+        plasma_error("PLASMA_zcposv_Tile_Async", "illegal value of uplo");
         return -1;
     }
     /* Quick return - currently NOT equivalent to LAPACK's
@@ -447,7 +459,7 @@ int PLASMA_zcposv_Tile_Async(PLASMA_enum uplo, PLASMA_desc *A, PLASMA_desc *B,
 
     work = (double *)plasma_shared_alloc(plasma, PLASMA_SIZE, PlasmaRealDouble);
     if (work == NULL) {
-        plasma_error("PLASMA_zcposv", "plasma_shared_alloc() failed");
+        plasma_error("PLASMA_zcposv_Tile_Async", "plasma_shared_alloc() failed");
         plasma_shared_free(plasma, work);
         return PLASMA_ERR_OUT_OF_RESOURCES;
     }
@@ -525,7 +537,7 @@ int PLASMA_zcposv_Tile_Async(PLASMA_enum uplo, PLASMA_desc *A, PLASMA_desc *B,
     PLASMA_zlange(PlasmaInfNorm, descX, Xnorm, work);
     PLASMA_zlange(PlasmaInfNorm, descR, Rnorm, work);
 
-    /* Wait the end of Anorm, Xnorm and Bnorm computations */
+    /* Wait for the end of Anorm, Xnorm and Bnorm computations */
     plasma_dynamic_sync();
 
     cte = Anorm*eps*((double) N)*bwdmax;
@@ -574,7 +586,7 @@ int PLASMA_zcposv_Tile_Async(PLASMA_enum uplo, PLASMA_desc *A, PLASMA_desc *B,
         /* Convert SX back to double precision and update the current
            iterate. */
         PLASMA_clag2z(descSX, descR);
-        PLASMA_zaxpy(one, descR, descX);
+        PLASMA_zgeadd(one, descR, descX);
 
         /* Compute R = B - AX. */
         PLASMA_zlacpy(descB,descR);
@@ -594,7 +606,7 @@ int PLASMA_zcposv_Tile_Async(PLASMA_enum uplo, PLASMA_desc *A, PLASMA_desc *B,
         PLASMA_zlange(PlasmaInfNorm, descX, Xnorm, work);
         PLASMA_zlange(PlasmaInfNorm, descR, Rnorm, work);
 
-        /* Wait the end of Xnorm and Bnorm computations */
+        /* Wait for the end of Xnorm and Bnorm computations */
         plasma_dynamic_sync();
 
         if (Rnorm < Xnorm * cte){
