@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Emmanuel Agullo
  * @date 2010-11-15
  * @precisions mixed zc -> ds
@@ -18,14 +18,14 @@
 #include <lapacke.h>
 #include "common.h"
 
-#define PLASMA_zlag2c(_descA, _descSB)		      \
-  plasma_parallel_call_4(plasma_pzlag2c,	      \
+#define PLASMA_zlag2c(_descA, _descSB)                \
+  plasma_parallel_call_4(plasma_pzlag2c,              \
                          PLASMA_desc,      (_descA),  \
                          PLASMA_desc,      (_descSB), \
                          PLASMA_sequence*, sequence,  \
                          PLASMA_request*,  request)
 
-#define PLASMA_clag2z(_descSA, _descB)		      \
+#define PLASMA_clag2z(_descSA, _descB)                \
   plasma_parallel_call_4(plasma_pclag2z,              \
                          PLASMA_desc,      (_descSA), \
                          PLASMA_desc,      (_descB),  \
@@ -50,12 +50,12 @@
                          PLASMA_sequence*, sequence,         \
                          PLASMA_request*,  request)
 
-#define PLASMA_zaxpy(_alpha, _descA, _descB)        	\
-  plasma_parallel_call_5(plasma_pzaxpy,                	\
-                         PLASMA_Complex64_t, (_alpha),	\
-                         PLASMA_desc,        (_descA),	\
-                         PLASMA_desc,        (_descB),	\
-                         PLASMA_sequence*,   sequence,	\
+#define PLASMA_zgeadd(_alpha, _descA, _descB)                 \
+  plasma_parallel_call_5(plasma_pzgeadd,                      \
+                         PLASMA_Complex64_t, (_alpha),        \
+                         PLASMA_desc,        (_descA),        \
+                         PLASMA_desc,        (_descB),        \
+                         PLASMA_sequence*,   sequence,        \
                          PLASMA_request*,    request)
 
 /***************************************************************************//**
@@ -180,7 +180,17 @@ int PLASMA_zcungesv(PLASMA_enum trans, int N, int NRHS,
         plasma_fatal_error("PLASMA_zcungesv", "PLASMA not initialized");
         return PLASMA_ERR_NOT_INITIALIZED;
     }
+
+    *ITER = 0;
+
     /* Check input arguments */
+    if (trans != PlasmaNoTrans   && 
+        trans != PlasmaConjTrans &&
+        trans != PlasmaTrans ) 
+    {
+        plasma_error("PLASMA_zcungesv", "illegal value of trans");
+        return -1;
+    }
     if (trans != PlasmaNoTrans) {
         plasma_error("PLASMA_zcungesv", "only PlasmaNoTrans supported");
         return PLASMA_ERR_NOT_SUPPORTED;
@@ -199,7 +209,7 @@ int PLASMA_zcungesv(PLASMA_enum trans, int N, int NRHS,
     }
     if (LDB < max(1, N)) {
         plasma_error("PLASMA_zcungesv", "illegal value of LDB");
-        return -8;
+        return -7;
     }
     if (LDX < max(1, N)) {
         plasma_error("PLASMA_zcungesv", "illegal value of LDX");
@@ -223,12 +233,16 @@ int PLASMA_zcungesv(PLASMA_enum trans, int N, int NRHS,
 
     /* DOUBLE PRECISION INITIALIZATION */
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   , plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)) );
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,    sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, sequence, &request,
+                             plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)) );
         plasma_zdesc_alloc(  descX, NB, NB, N, NRHS, 0, 0, N, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)); plasma_desc_mat_free(&(descX)) );
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   );
-        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,   
+                            sequence, &request);
+        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS,
+                            sequence, &request);
 
         descX = plasma_desc_init(
             PlasmaComplexDouble, NB, NB, (NB*NB), 
@@ -245,15 +259,15 @@ int PLASMA_zcungesv(PLASMA_enum trans, int N, int NRHS,
 
     if (status == PLASMA_SUCCESS) {
         if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-            plasma_zooptile2lap( descX, X, NB, NB, LDX, NRHS );
+            plasma_zooptile2lap( descX, X, NB, NB, LDX, NRHS,  sequence, &request);
             plasma_dynamic_sync();
             plasma_desc_mat_free(&descA);
             plasma_desc_mat_free(&descB);
             plasma_desc_mat_free(&descX);
         } else {
-            plasma_ziptile2lap( descA, A, NB, NB, LDA, N    );
-            plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS );
-            plasma_ziptile2lap( descX, X, NB, NB, LDX, NRHS );
+            plasma_ziptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+            plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
+            plasma_ziptile2lap( descX, X, NB, NB, LDX, NRHS,  sequence, &request);
             plasma_dynamic_sync();
         }
     }
@@ -356,7 +370,7 @@ int PLASMA_zcungesv_Tile(PLASMA_enum trans, PLASMA_desc *A, PLASMA_desc *T,
  *  the tile QR or the tile LQ factorization and mixed-precision iterative refinement.
  *  Non-blocking equivalent of PLASMA_zcungesv_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -380,10 +394,10 @@ int PLASMA_zcungesv_Tile_Async(PLASMA_enum trans, PLASMA_desc *A, PLASMA_desc *T
                                PLASMA_sequence *sequence, PLASMA_request *request)
 {
     int N, NB, IB;
-    PLASMA_desc descA = *A;
-    PLASMA_desc descT = *T;
-    PLASMA_desc descB = *B;
-    PLASMA_desc descX = *X;
+    PLASMA_desc descA;
+    PLASMA_desc descT;
+    PLASMA_desc descB;
+    PLASMA_desc descX;
     PLASMA_desc descR, descSA, descST, descSX;
     plasma_context_t *plasma;
     double *work;
@@ -415,21 +429,29 @@ int PLASMA_zcungesv_Tile_Async(PLASMA_enum trans, PLASMA_desc *A, PLASMA_desc *T
     else
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zcungesv_Tile", "invalid first descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descT) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(T) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zcungesv_Tile", "invalid second descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descT = *T;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zcungesv_Tile", "invalid third descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descB = *B;
     }
-    if (plasma_desc_check(&descX) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(X) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zcungesv_Tile", "invalid fourth descriptor");
         return PLASMA_ERR_ILLEGAL_VALUE;
+    } else {
+        descX = *X;
     }
     /* Check input arguments */
     if ( (descA.nb != descA.mb) || (descB.nb != descB.mb) || (descX.nb != descX.mb) ||
@@ -525,7 +547,7 @@ int PLASMA_zcungesv_Tile_Async(PLASMA_enum trans, PLASMA_desc *A, PLASMA_desc *T
     PLASMA_zlange(PlasmaInfNorm, descX, Xnorm, work);
     PLASMA_zlange(PlasmaInfNorm, descR, Rnorm, work);
 
-    /* Wait the end of Anorm, Xnorm and Bnorm computations */
+    /* Wait for the end of Anorm, Xnorm and Bnorm computations */
     plasma_dynamic_sync();
 
     cte = Anorm*eps*((double) N)*bwdmax;
@@ -570,7 +592,7 @@ int PLASMA_zcungesv_Tile_Async(PLASMA_enum trans, PLASMA_desc *A, PLASMA_desc *T
         /* Convert SX back to double precision and update the current
            iterate. */
         PLASMA_clag2z(descSX, descR);
-        PLASMA_zaxpy(one, descR, descX);
+        PLASMA_zgeadd(one, descR, descX);
 
         /* Compute R = B - AX. */
         PLASMA_zlacpy(descB,descR);
@@ -590,7 +612,7 @@ int PLASMA_zcungesv_Tile_Async(PLASMA_enum trans, PLASMA_desc *A, PLASMA_desc *T
         PLASMA_zlange(PlasmaInfNorm, descX, Xnorm, work);
         PLASMA_zlange(PlasmaInfNorm, descR, Rnorm, work);
 
-        /* Wait the end of Xnorm and Bnorm computations */
+        /* Wait for the end of Xnorm and Bnorm computations */
         plasma_dynamic_sync();
 
         if (Rnorm < Xnorm * cte){
