@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -38,7 +38,7 @@
  * @param[in] LDA
  *          The leading dimension of the array A. LDA >= M.
  *
- * @param[in] T
+ * @param[in] descT
  *          Auxiliary factorization data, computed by PLASMA_zgelqf.
  *
  * @param[in,out] B
@@ -66,21 +66,22 @@
  ******************************************************************************/
 int PLASMA_zgelqs(int M, int N, int NRHS,
                   PLASMA_Complex64_t *A, int LDA,
-                  PLASMA_Complex64_t *T,
+                  PLASMA_desc *descT,
                   PLASMA_Complex64_t *B, int LDB)
 {
-    int NB, IB, IBNB, MT, NT;
+    int NB;
     int status;
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
     PLASMA_request request = PLASMA_REQUEST_INITIALIZER;
-    PLASMA_desc descA, descB, descT;
+    PLASMA_desc descA, descB;
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
         plasma_fatal_error("PLASMA_zgelqs", "PLASMA not initialized");
         return PLASMA_ERR_NOT_INITIALIZED;
     }
+
     /* Check input arguments */
     if (M < 0) {
         plasma_error("PLASMA_zgelqs", "illegal value of M");
@@ -114,50 +115,35 @@ int PLASMA_zgelqs(int M, int N, int NRHS,
         return status;
     }
 
-    /* Set MT, NT & NTRHS */
-    NB    = PLASMA_NB;
-    IB    = PLASMA_IB;
-    IBNB  = IB*NB;
-    MT    = (M%NB==0) ? (M/NB) : (M/NB+1);
-    NT    = (N%NB==0) ? (N/NB) : (N/NB+1);
+    /* Set NT */
+    NB = PLASMA_NB;
 
     plasma_sequence_create(plasma, &sequence);
 
-    if (plasma->householder == PLASMA_FLAT_HOUSEHOLDER) {
-        descT = plasma_desc_init(
-            PlasmaComplexDouble,
-            IB, NB, IBNB,
-            MT*IB, NT*NB, 0, 0, MT*IB, NT*NB);
-    }
-    else {
-        /* Double the size of T to accomodate the tree reduction phase */
-        descT = plasma_desc_init(
-            PlasmaComplexDouble,
-            IB, NB, IBNB,
-            MT*IB, 2*NT*NB, 0, 0, MT*IB, 2*NT*NB);
-    }
-    descT.mat = T;
-
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, M, N   , plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, M, N,    sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, sequence, &request,
+                             plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, M, N   );
-        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, M, N,   
+                            sequence, &request);
+        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
-    PLASMA_zgelqs_Tile_Async(&descA, &descT, &descB, sequence, &request);
+    PLASMA_zgelqs_Tile_Async(&descA, descT, &descB, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descA, A, NB, NB, LDA, N    );
-        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_zooptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
     plasma_dynamic_sync();
     plasma_desc_mat_free(&descA);
     plasma_desc_mat_free(&descB);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N    );
-        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
     }
     
@@ -232,7 +218,7 @@ int PLASMA_zgelqs_Tile(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *B)
  *  computed LQ factorization.
  *  Non-blocking equivalent of PLASMA_zgelqs_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -256,9 +242,9 @@ int PLASMA_zgelqs_Tile(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *B)
 int PLASMA_zgelqs_Tile_Async(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *B,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
-    PLASMA_desc descT = *T;
-    PLASMA_desc descB = *B;
+    PLASMA_desc descA;
+    PLASMA_desc descT;
+    PLASMA_desc descB;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -281,17 +267,23 @@ int PLASMA_zgelqs_Tile_Async(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *B,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgelqs_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descT) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(T) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgelqs_Tile", "invalid second descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descT = *T;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgelqs_Tile", "invalid third descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descB = *B;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb || descB.nb != descB.mb) {
