@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -61,7 +61,7 @@
  * @param[in] LDA
  *          The leading dimension of the array A. LDA >= max(1,M).
  *
- * @param[out] T
+ * @param[out] descT
  *          On exit, auxiliary factorization data.
  *
  * @param[in,out] B
@@ -93,22 +93,23 @@
  ******************************************************************************/
 int PLASMA_zgels(PLASMA_enum trans, int M, int N, int NRHS,
                  PLASMA_Complex64_t *A, int LDA,
-                 PLASMA_Complex64_t *T,
+                 PLASMA_desc *descT,
                  PLASMA_Complex64_t *B, int LDB)
 {
     int i, j;
-    int NB, IB, IBNB, MT, NT;
+    int NB;
     int status;
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
     PLASMA_request request = PLASMA_REQUEST_INITIALIZER;
-    PLASMA_desc descA, descB, descT;
+    PLASMA_desc descA, descB;
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
         plasma_fatal_error("PLASMA_zgels", "PLASMA not initialized");
         return PLASMA_ERR_NOT_INITIALIZED;
     }
+
     /* Check input arguments */
     if (trans != PlasmaNoTrans) {
         plasma_error("PLASMA_zgels", "only PlasmaNoTrans supported");
@@ -149,60 +150,49 @@ int PLASMA_zgels(PLASMA_enum trans, int M, int N, int NRHS,
         return status;
     }
 
-    /* Set MT, NT & NTRHS */
-    NB    = PLASMA_NB;
-    IB    = PLASMA_IB;
-    IBNB  = IB*NB;
-    NT    = (N%NB==0) ? (N/NB) : (N/NB+1);
-    MT    = (M%NB==0) ? (M/NB) : (M/NB+1);
+    /* Set NT */
+    NB = PLASMA_NB;
 
     plasma_sequence_create(plasma, &sequence);
 
-    if (plasma->householder == PLASMA_FLAT_HOUSEHOLDER) {
-        descT = plasma_desc_init(
-            PlasmaComplexDouble,
-            IB, NB, IBNB,
-            MT*IB, NT*NB, 0, 0, MT*IB, NT*NB);
-    }
-    else {
-        /* Double the size of T to accomodate the tree reduction phase */
-        descT = plasma_desc_init(
-            PlasmaComplexDouble,
-            IB, NB, IBNB,
-            MT*IB, 2*NT*NB, 0, 0, MT*IB, 2*NT*NB);
-    }
-    descT.mat = T;
-
     if ( M >= N ) {
         if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-            plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, M, N   , plasma_desc_mat_free(&(descA)) );
-            plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, M, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
+            plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, M, N, sequence, &request,
+                                 plasma_desc_mat_free(&(descA)) );
+            plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, M, NRHS, sequence, &request,
+                                 plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
         } else {
-            plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, M, N   );
-            plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, M, NRHS);
+            plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, M, N,
+                                sequence, &request);
+            plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, M, NRHS,
+                                sequence, &request);
         }
     } else {
         if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-            plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, M, N   , plasma_desc_mat_free(&(descA)) );
-            plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
+            plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, M, N, sequence, &request,
+                                 plasma_desc_mat_free(&(descA)) );
+            plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, sequence, &request,
+                                 plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
         } else {
-            plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, M, N   );
-            plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS);
+            plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, M, N,
+                                sequence, &request);
+            plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS,
+                                sequence, &request);
         }
     }
 
     /* Call the tile interface */
-    PLASMA_zgels_Tile_Async(PlasmaNoTrans, &descA, &descT, &descB, sequence, &request);
+    PLASMA_zgels_Tile_Async(PlasmaNoTrans, &descA, descT, &descB, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descA, A, NB, NB, LDA, N    );
-        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_zooptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
         plasma_desc_mat_free(&descB);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N    );
-        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
     }
     
@@ -293,7 +283,7 @@ int PLASMA_zgels_Tile(PLASMA_enum trans, PLASMA_desc *A,
  *  system of equations using the tile QR or the tile LQ factorization.
  *  Non-blocking equivalent of PLASMA_zgels_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -317,9 +307,9 @@ int PLASMA_zgels_Tile_Async(PLASMA_enum trans, PLASMA_desc *A,
                             PLASMA_desc *T, PLASMA_desc *B,
                             PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
-    PLASMA_desc descT = *T;
-    PLASMA_desc descB = *B;
+    PLASMA_desc descA;
+    PLASMA_desc descT;
+    PLASMA_desc descB;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -342,17 +332,23 @@ int PLASMA_zgels_Tile_Async(PLASMA_enum trans, PLASMA_desc *A,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgels_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descT) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(T) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgels_Tile", "invalid second descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descT = *T;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgels_Tile", "invalid third descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descB = *B;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb || descB.nb != descB.mb) {
