@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -38,7 +38,7 @@
  * @param[in] LDA
  *          The leading dimension of the array A. LDA >= max(1,M).
  *
- * @param[out] T
+ * @param[out] descT
  *          On exit, auxiliary factorization data, required by PLASMA_zgeqrs to solve the system
  *          of equations.
  *
@@ -60,20 +60,21 @@
  ******************************************************************************/
 int PLASMA_zgeqrf(int M, int N,
                   PLASMA_Complex64_t *A, int LDA,
-                  PLASMA_Complex64_t *T)
+                  PLASMA_desc *descT)
 {
-    int NB, IB, IBNB, MT, NT;
+    int NB;
     int status;
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
     PLASMA_request request = PLASMA_REQUEST_INITIALIZER;
-    PLASMA_desc descA, descT;
+    PLASMA_desc descA;
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
         plasma_fatal_error("PLASMA_zgeqrf", "PLASMA not initialized");
         return PLASMA_ERR_NOT_INITIALIZED;
     }
+
     /* Check input arguments */
     if (M < 0) {
         plasma_error("PLASMA_zgeqrf", "illegal value of M");
@@ -87,6 +88,7 @@ int PLASMA_zgeqrf(int M, int N,
         plasma_error("PLASMA_zgeqrf", "illegal value of LDA");
         return -4;
     }
+
     /* Quick return */
     if (min(M, N) == 0)
         return PLASMA_SUCCESS;
@@ -98,45 +100,28 @@ int PLASMA_zgeqrf(int M, int N,
         return status;
     }
 
-    /* Set MT & NT */
-    NB   = PLASMA_NB;
-    IB   = PLASMA_IB;
-    IBNB = IB*NB;
-    MT   = (M%NB==0) ? (M/NB) : (M/NB+1);
-    NT   = (N%NB==0) ? (N/NB) : (N/NB+1);
+    /* Set NT */
+    NB = PLASMA_NB;
 
     plasma_sequence_create(plasma, &sequence);
  
-     if (plasma->householder == PLASMA_FLAT_HOUSEHOLDER) {
-        descT = plasma_desc_init(
-            PlasmaComplexDouble,
-            IB, NB, IBNB,
-            MT*IB, NT*NB, 0, 0, MT*IB, NT*NB);
-    }
-    else {
-        /* Double the size of T to accomodate the tree reduction phase */
-        descT = plasma_desc_init(
-            PlasmaComplexDouble,
-            IB, NB, IBNB,
-            MT*IB, 2*NT*NB, 0, 0, MT*IB, 2*NT*NB);
-    }
-    descT.mat = T;
-    
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, M, N, plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, M, N, sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, M, N);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, M, N,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
-    PLASMA_zgeqrf_Tile_Async(&descA, &descT, sequence, &request);
+    PLASMA_zgeqrf_Tile_Async(&descA, descT, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descA, A, NB, NB, LDA, N );
+        plasma_zooptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
         plasma_dynamic_sync();
     }
 
@@ -210,7 +195,7 @@ int PLASMA_zgeqrf_Tile(PLASMA_desc *A, PLASMA_desc *T)
  *  PLASMA_zgeqrf_Tile_Async - Computes the tile QR factorization of a matrix.
  *  Non-blocking equivalent of PLASMA_zgeqrf_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -234,8 +219,8 @@ int PLASMA_zgeqrf_Tile(PLASMA_desc *A, PLASMA_desc *T)
 int PLASMA_zgeqrf_Tile_Async(PLASMA_desc *A, PLASMA_desc *T,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
-    PLASMA_desc descT = *T;
+    PLASMA_desc descA;
+    PLASMA_desc descT;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -258,13 +243,17 @@ int PLASMA_zgeqrf_Tile_Async(PLASMA_desc *A, PLASMA_desc *T,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgeqrf_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descT) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(T) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgeqrf_Tile", "invalid second descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descT = *T;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb) {
