@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @author Mathieu Faverge
  * @date 2010-11-15
@@ -72,7 +72,7 @@ int PLASMA_zgesv(int N, int NRHS,
                  int *IPIV,
                  PLASMA_Complex64_t *B, int LDB)
 {
-    int NB, IB, IBNB, NT;
+    int NB;
     int status;
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
@@ -99,7 +99,7 @@ int PLASMA_zgesv(int N, int NRHS,
     }
     if (LDB < max(1, N)) {
         plasma_error("PLASMA_zgesv", "illegal value of LDB");
-        return -8;
+        return -7;
     }
     /* Quick return */
     if (min(N, NRHS) == 0)
@@ -113,36 +113,37 @@ int PLASMA_zgesv(int N, int NRHS,
     }
 
     /* Set NT & NTRHS */
-    NB    = PLASMA_NB;
-    IB    = PLASMA_IB;
-    IBNB  = IB*NB;
-    NT    = (N%NB==0) ? (N/NB) : (N/NB+1);
+    NB = PLASMA_NB;
 
     plasma_sequence_create(plasma, &sequence);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   , plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,    sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, sequence, &request,
+                             plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   );
-        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,
+                            sequence, &request);
+        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
     PLASMA_zgesv_Tile_Async(&descA, IPIV, &descB, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descA, A, NB, NB, LDA, N    );
-        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_zooptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
         plasma_desc_mat_free(&descB);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N    );
-        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
     }
-    
+
     status = sequence->status;
     plasma_sequence_destroy(plasma, sequence);
     return status;
@@ -152,7 +153,7 @@ int PLASMA_zgesv(int N, int NRHS,
  * @ingroup PLASMA_Complex64_t_Tile
  *
  *  PLASMA_zgesv_Tile - Solves a system of linear equations using the tile LU factorization.
- *  Tile equivalent of PLASMA_zgetrf().
+ *  Tile equivalent of PLASMA_zgesv().
  *  Operates on matrices stored by tiles.
  *  All matrices are passed through descriptors.
  *  All dimensions are taken from the descriptors.
@@ -215,7 +216,7 @@ int PLASMA_zgesv_Tile(PLASMA_desc *A, int *IPIV, PLASMA_desc *B)
  *  LU factorization.
  *  Non-blocking equivalent of PLASMA_zgesv_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -239,8 +240,8 @@ int PLASMA_zgesv_Tile(PLASMA_desc *A, int *IPIV, PLASMA_desc *B)
 int PLASMA_zgesv_Tile_Async(PLASMA_desc *A, int *IPIV, PLASMA_desc *B,
                             PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
-    PLASMA_desc descB = *B;
+    PLASMA_desc descA;
+    PLASMA_desc descB;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -263,13 +264,17 @@ int PLASMA_zgesv_Tile_Async(PLASMA_desc *A, int *IPIV, PLASMA_desc *B,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgesv_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgesv_Tile", "invalid third descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descB = *B;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb || descB.nb != descB.mb) {
@@ -287,7 +292,7 @@ int PLASMA_zgesv_Tile_Async(PLASMA_desc *A, int *IPIV, PLASMA_desc *B,
         PLASMA_desc, descA,
         PLASMA_sequence*, sequence,
         PLASMA_request*,  request);
-    
+
     plasma_dynamic_call_4(plasma_pzgetrf_rectil,
         PLASMA_desc, descA,
         int*, IPIV,
@@ -308,8 +313,8 @@ int PLASMA_zgesv_Tile_Async(PLASMA_desc *A, int *IPIV, PLASMA_desc *B,
         int,         1,
         PLASMA_sequence*, sequence,
         PLASMA_request*,  request);
-    
-    plasma_parallel_call_9(            
+
+    plasma_parallel_call_9(
         plasma_pztrsm,
         PLASMA_enum, PlasmaLeft,
         PLASMA_enum, PlasmaLower,
@@ -320,7 +325,7 @@ int PLASMA_zgesv_Tile_Async(PLASMA_desc *A, int *IPIV, PLASMA_desc *B,
         PLASMA_desc, descB,
         PLASMA_sequence*, sequence,
         PLASMA_request*, request);
-    
+
     plasma_parallel_call_9(
         plasma_pztrsm,
         PLASMA_enum, PlasmaLeft,
@@ -332,6 +337,6 @@ int PLASMA_zgesv_Tile_Async(PLASMA_desc *A, int *IPIV, PLASMA_desc *B,
         PLASMA_desc, descB,
         PLASMA_sequence*, sequence,
         PLASMA_request*, request);
-    
+
     return PLASMA_SUCCESS;
 }
