@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -39,7 +39,7 @@
  * @param[in] LDA
  *          The leading dimension of the array A. LDA >= max(1,N).
  *
- * @param[out] L
+ * @param[out] descL
  *          On exit, auxiliary factorization data, related to the tile L factor,
  *          necessary to solve the system of equations.
  *
@@ -72,15 +72,15 @@
  ******************************************************************************/
 int PLASMA_zgesv_incpiv(int N, int NRHS,
                  PLASMA_Complex64_t *A, int LDA,
-                 PLASMA_Complex64_t *L, int *IPIV,
+                 PLASMA_desc *descL, int *IPIV,
                  PLASMA_Complex64_t *B, int LDB)
 {
-    int NB, IB, IBNB, NT;
+    int NB;
     int status;
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
     PLASMA_request request = PLASMA_REQUEST_INITIALIZER;
-    PLASMA_desc descA, descB, descL;
+    PLASMA_desc descA, descB;
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
@@ -117,38 +117,33 @@ int PLASMA_zgesv_incpiv(int N, int NRHS,
 
     /* Set NT & NTRHS */
     NB    = PLASMA_NB;
-    IB    = PLASMA_IB;
-    IBNB  = IB*NB;
-    NT    = (N%NB==0) ? (N/NB) : (N/NB+1);
 
     plasma_sequence_create(plasma, &sequence);
 
-    descL = plasma_desc_init(
-        PlasmaComplexDouble,
-        IB, NB, IBNB,
-        NT*IB, NT*NB, 0, 0, NT*IB, NT*NB);
-    descL.mat = L;
-
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   , plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,    sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, sequence, &request,
+                             plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   );
-        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,   
+                            sequence, &request);
+        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
-    PLASMA_zgesv_incpiv_Tile_Async(&descA, &descL, IPIV, &descB, sequence, &request);
+    PLASMA_zgesv_incpiv_Tile_Async(&descA, descL, IPIV, &descB, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descA, A, NB, NB, LDA, N    );
-        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_zooptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
         plasma_desc_mat_free(&descB);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N    );
-        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
     }
     
@@ -228,7 +223,7 @@ int PLASMA_zgesv_incpiv_Tile(PLASMA_desc *A, PLASMA_desc *L, int *IPIV, PLASMA_d
  *  LU factorization.
  *  Non-blocking equivalent of PLASMA_zgesv_incpiv_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -252,9 +247,9 @@ int PLASMA_zgesv_incpiv_Tile(PLASMA_desc *A, PLASMA_desc *L, int *IPIV, PLASMA_d
 int PLASMA_zgesv_incpiv_Tile_Async(PLASMA_desc *A, PLASMA_desc *L, int *IPIV, PLASMA_desc *B,
                             PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
-    PLASMA_desc descL = *L;
-    PLASMA_desc descB = *B;
+    PLASMA_desc descA;
+    PLASMA_desc descL;
+    PLASMA_desc descB;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -277,17 +272,23 @@ int PLASMA_zgesv_incpiv_Tile_Async(PLASMA_desc *A, PLASMA_desc *L, int *IPIV, PL
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgesv_incpiv_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descL) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(L) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgesv_incpiv_Tile", "invalid second descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descL = *L;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgesv_incpiv_Tile", "invalid third descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descB = *B;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb || descB.nb != descB.mb) {
