@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @date 2009-11-15
@@ -63,7 +63,7 @@ int PLASMA_zgetrf(int M, int N,
                   PLASMA_Complex64_t *A, int LDA,
                   int *IPIV)
 {
-    int NB, NBNB;
+    int NB, NBNB, minMN;
     int status;
     PLASMA_desc descA ;
     plasma_context_t *plasma;
@@ -111,7 +111,8 @@ int PLASMA_zgetrf(int M, int N,
         LDA, N, 0, 0, M, N);
     descA.mat = A;
 
-    memset(IPIV, 0, M*sizeof(int));
+    minMN = min(M, N);
+    memset(IPIV, 0, minMN*sizeof(int));
 
     /* Call the tile interface */
     plasma_dynamic_call_4(plasma_pzgetrf_reclap,
@@ -123,13 +124,15 @@ int PLASMA_zgetrf(int M, int N,
     plasma_dynamic_sync();
 
     /*
-     * Generate the correct IPIV (Has to be move in a task)
+     * Generate the correct IPIV (Has to be moved in a task)
      */
     { 
         int i, inc, tmp, j;
         for(i=1; i<descA.mt; i++) {
             inc = i*descA.mb;
-            tmp = i == descA.mt-1 ? descA.m-i*descA.mb : descA.mb;
+            tmp = min( minMN - inc, descA.mb);
+            if ( tmp < 1 )
+              break;
             
             for (j=0; j<tmp; j++)
                 IPIV[inc+j] = IPIV[inc+j] + inc;
@@ -158,10 +161,6 @@ int PLASMA_zgetrf(int M, int N,
  *          On entry, the M-by-N matrix to be factored.
  *          On exit, the tile factors L and U from the factorization.
  *
- * @param[out] L
- *          On exit, auxiliary factorization data, related to the tile L factor,
- *          required by PLASMA_zgetrs to solve the system of equations.
- *
  * @param[out] IPIV
  *          The pivot indices that define the permutations (not equivalent to LAPACK).
  *
@@ -210,7 +209,7 @@ int PLASMA_zgetrf_Tile(PLASMA_desc *A, int *IPIV)
  *  PLASMA_zgetrf_Tile_Async - Computes the tile LU factorization of a matrix.
  *  Non-blocking equivalent of PLASMA_zgetrf_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -234,7 +233,7 @@ int PLASMA_zgetrf_Tile(PLASMA_desc *A, int *IPIV)
 int PLASMA_zgetrf_Tile_Async(PLASMA_desc *A, int *IPIV,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
+    PLASMA_desc descA;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -257,9 +256,11 @@ int PLASMA_zgetrf_Tile_Async(PLASMA_desc *A, int *IPIV,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgetrf_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb) {
@@ -285,7 +286,7 @@ int PLASMA_zgetrf_Tile_Async(PLASMA_desc *A, int *IPIV,
         PLASMA_request*, request);
 
     plasma_dynamic_call_3(
-        plasma_pzbarrier_tl2pnl,
+        plasma_pzbarrier_pnl2tl,
         PLASMA_desc, descA,
         PLASMA_sequence*, sequence,
         PLASMA_request*,  request);
