@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -36,7 +36,7 @@
  * @param[in] LDA
  *          The leading dimension of the array A. LDA >= max(1,M).
  *
- * @param[out] L
+ * @param[out] descL
  *          On exit, auxiliary factorization data, related to the tile L factor,
  *          required by PLASMA_zgetrs_incpiv to solve the system of equations.
  *
@@ -64,14 +64,14 @@
  ******************************************************************************/
 int PLASMA_zgetrf_incpiv(int M, int N,
                   PLASMA_Complex64_t *A, int LDA,
-                  PLASMA_Complex64_t *L, int *IPIV)
+                  PLASMA_desc *descL, int *IPIV)
 {
-    int NB, IB, IBNB, MT, NT;
+    int NB;
     int status;
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
     PLASMA_request request = PLASMA_REQUEST_INITIALIZER;
-    PLASMA_desc descA, descL;
+    PLASMA_desc descA;
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
@@ -104,34 +104,26 @@ int PLASMA_zgetrf_incpiv(int M, int N,
 
     /* Set NT & NTRHS */
     NB   = PLASMA_NB;
-    IB   = PLASMA_IB;
-    IBNB = IB*NB;
-    MT   = (M%NB==0) ? (M/NB) : (M/NB+1);
-    NT   = (N%NB==0) ? (N/NB) : (N/NB+1);
 
     plasma_sequence_create(plasma, &sequence);
 
-    descL = plasma_desc_init(
-        PlasmaComplexDouble,
-        IB, NB, IBNB,
-        MT*IB, NT*NB, 0, 0, MT*IB, NT*NB);
-    descL.mat = L;
-
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, M, N, plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, M, N, sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, M, N);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, M, N,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
-    PLASMA_zgetrf_incpiv_Tile_Async(&descA, &descL, IPIV, sequence, &request);
+    PLASMA_zgetrf_incpiv_Tile_Async(&descA, descL, IPIV, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descA, A, NB, NB, LDA, N );
+        plasma_zooptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
         plasma_dynamic_sync();
     }
 
@@ -208,7 +200,7 @@ int PLASMA_zgetrf_incpiv_Tile(PLASMA_desc *A, PLASMA_desc *L, int *IPIV)
  *  PLASMA_zgetrf_incpiv_Tile_Async - Computes the tile LU factorization of a matrix.
  *  Non-blocking equivalent of PLASMA_zgetrf_incpiv_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -232,8 +224,8 @@ int PLASMA_zgetrf_incpiv_Tile(PLASMA_desc *A, PLASMA_desc *L, int *IPIV)
 int PLASMA_zgetrf_incpiv_Tile_Async(PLASMA_desc *A, PLASMA_desc *L, int *IPIV,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
-    PLASMA_desc descL = *L;
+    PLASMA_desc descA;
+    PLASMA_desc descL;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -256,13 +248,17 @@ int PLASMA_zgetrf_incpiv_Tile_Async(PLASMA_desc *A, PLASMA_desc *L, int *IPIV,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgetrf_incpiv_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descL) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(L) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgetrf_incpiv_Tile", "invalid second descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descL = *L;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb) {
