@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @author Mathieu Faverge
  * @date 2010-11-15
@@ -71,10 +71,10 @@
  ******************************************************************************/
 int PLASMA_zgetrs(PLASMA_enum trans, int N, int NRHS,
                   PLASMA_Complex64_t *A, int LDA,
-                  int *IPIV,
+                  const int *IPIV,
                   PLASMA_Complex64_t *B, int LDB)
 {
-    int NB, IB, IBNB, NT;
+    int NB;
     int status;
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
@@ -87,7 +87,7 @@ int PLASMA_zgetrs(PLASMA_enum trans, int N, int NRHS,
         return PLASMA_ERR_NOT_INITIALIZED;
     }
     /* Check input arguments */
-    if ( (trans != PlasmaNoTrans) && 
+    if ( (trans != PlasmaNoTrans) &&
          (trans != PlasmaTrans)   &&
          (trans != PlasmaConjTrans)) {
         plasma_error("PLASMA_zgetrs", "illegal value of trans");
@@ -107,7 +107,7 @@ int PLASMA_zgetrs(PLASMA_enum trans, int N, int NRHS,
     }
     if (LDB < max(1, N)) {
         plasma_error("PLASMA_zgetrs", "illegal value of LDB");
-        return -8;
+        return -7;
     }
     /* Quick return */
     if (min(N, NRHS) == 0)
@@ -121,32 +121,33 @@ int PLASMA_zgetrs(PLASMA_enum trans, int N, int NRHS,
     }
 
     /* Set NT & NTRHS */
-    NB    = PLASMA_NB;
-    IB    = PLASMA_IB;
-    IBNB  = IB*NB;
-    NT    = (N%NB==0) ? (N/NB) : (N/NB+1);
+    NB = PLASMA_NB;
 
     plasma_sequence_create(plasma, &sequence);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   , plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,    sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, sequence, &request,
+                             plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   );
-        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,   
+                            sequence, &request);
+        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
     PLASMA_zgetrs_Tile_Async(trans, &descA, IPIV, &descB, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
         plasma_desc_mat_free(&descB);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N    );
-        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
     }
 
@@ -199,7 +200,7 @@ int PLASMA_zgetrs(PLASMA_enum trans, int N, int NRHS,
  * @sa PLASMA_zgetrf_Tile
  *
  ******************************************************************************/
-int PLASMA_zgetrs_Tile(PLASMA_enum trans, PLASMA_desc *A, int *IPIV, PLASMA_desc *B)
+int PLASMA_zgetrs_Tile(PLASMA_enum trans, PLASMA_desc *A, const int *IPIV, PLASMA_desc *B)
 {
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
@@ -227,7 +228,7 @@ int PLASMA_zgetrs_Tile(PLASMA_enum trans, PLASMA_desc *A, int *IPIV, PLASMA_desc
  *  computed LU factorization.
  *  Non-blocking equivalent of PLASMA_zgetrs_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -248,11 +249,11 @@ int PLASMA_zgetrs_Tile(PLASMA_enum trans, PLASMA_desc *A, int *IPIV, PLASMA_desc
  * @sa PLASMA_zgetrf_Tile_Async
  *
  ******************************************************************************/
-int PLASMA_zgetrs_Tile_Async(PLASMA_enum trans, PLASMA_desc *A, int *IPIV, PLASMA_desc *B,
+int PLASMA_zgetrs_Tile_Async(PLASMA_enum trans, PLASMA_desc *A, const int *IPIV, PLASMA_desc *B,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
-    PLASMA_desc descB = *B;
+    PLASMA_desc descA;
+    PLASMA_desc descB;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -275,13 +276,17 @@ int PLASMA_zgetrs_Tile_Async(PLASMA_enum trans, PLASMA_desc *A, int *IPIV, PLASM
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgetrs_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zgetrs_Tile", "invalid third descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descB = *B;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb || descB.nb != descB.mb) {
@@ -360,6 +365,12 @@ int PLASMA_zgetrs_Tile_Async(PLASMA_enum trans, PLASMA_desc *A, int *IPIV, PLASM
             PLASMA_sequence*, sequence,
             PLASMA_request*, request);
 
+        plasma_dynamic_call_3(
+            plasma_pzbarrier_tl2pnl,
+            PLASMA_desc, descB,
+            PLASMA_sequence*, sequence,
+            PLASMA_request*,  request);
+
         /* swap */
         plasma_dynamic_call_5(
             plasma_pzlaswp,
