@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -29,11 +29,11 @@
  *
  *******************************************************************************
  *
- * @param[in] PlasmaItype
+ * @param[in] itype
  *          Intended usage:
  *          = 1: A*x=(lambda)*B*x
- *          = 2: A*Bx=(lambda)*x 
- *          = 3: B*A*x=(lambda)*x 
+ *          = 2: A*Bx=(lambda)*x
+ *          = 3: B*A*x=(lambda)*x
  *
  * @param[in] uplo
  *          Specifies whether the matrix A is upper triangular or
@@ -86,7 +86,7 @@ int PLASMA_zhegst(PLASMA_enum itype, PLASMA_enum uplo, int N,
                   PLASMA_Complex64_t *A, int LDA,
                   PLASMA_Complex64_t *B, int LDB)
 {
-    int NB, IB, IBNB, NT;
+    int NB;
     int status;
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
@@ -131,33 +131,34 @@ int PLASMA_zhegst(PLASMA_enum itype, PLASMA_enum uplo, int N,
     }
 
     /* Set NT */
-    NB   = PLASMA_NB;
-    IB   = PLASMA_IB;
-    IBNB = IB*NB;
-    NT   = (N%NB==0) ? (N/NB) : (N/NB+1);
+    NB = PLASMA_NB;
 
     plasma_sequence_create(plasma, &sequence);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N, plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descB, B, NB, NB, LDB, N, 0, 0, N, N, plasma_desc_mat_free(&(descB)) );
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N, sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descB, B, NB, NB, LDB, N, 0, 0, N, N, sequence, &request,
+                             plasma_desc_mat_free(&(descB)) );
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N);
-        plasma_ziplap2tile( descB, B, NB, NB, LDB, N, 0, 0, N, N);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N,
+                            sequence, &request);
+        plasma_ziplap2tile( descB, B, NB, NB, LDB, N, 0, 0, N, N,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
     PLASMA_zhegst_Tile_Async(itype, uplo, &descA, &descB, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descA, A, NB, NB, LDA, N );
-        plasma_zooptile2lap( descB, B, NB, NB, LDB, N );
+        plasma_zooptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
+        plasma_zooptile2lap( descB, B, NB, NB, LDB, N,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
         plasma_desc_mat_free(&descB);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N );
-        plasma_ziptile2lap( descB, B, NB, NB, LDB, N );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
+        plasma_ziptile2lap( descB, B, NB, NB, LDB, N,  sequence, &request);
         plasma_dynamic_sync();
     }
 
@@ -169,7 +170,7 @@ int PLASMA_zhegst(PLASMA_enum itype, PLASMA_enum uplo, int N,
 /***************************************************************************//**
  *
  * @ingroup PLASMA_Complex64_t_Tile
- * 
+ *
  *  PLASMA_zhegst_Tile - reduces a complex Hermitian-definite
  *  generalized eigenproblem to standard form.
  *  If PlasmaItype == 1, the problem is A*x = lambda*B*x, and A is
@@ -186,11 +187,11 @@ int PLASMA_zhegst(PLASMA_enum itype, PLASMA_enum uplo, int N,
  *
  *******************************************************************************
  *
- * @param[in] PlasmaItype
+ * @param[in] itype
  *          Intended usage:
  *          = 1: A*x=(lambda)*B*x
- *          = 2: A*Bx=(lambda)*x 
- *          = 3: B*A*x=(lambda)*x 
+ *          = 2: A*Bx=(lambda)*x
+ *          = 3: B*A*x=(lambda)*x
  *          Currently only PlasmaItype == 1 is supported.
  *
  * @param[in] uplo
@@ -233,8 +234,8 @@ int PLASMA_zhegst(PLASMA_enum itype, PLASMA_enum uplo, int N,
  * @sa PLASMA_zhegst_Tile
  *
  ******************************************************************************/
-int PLASMA_zhegst_Tile(PLASMA_enum itype, PLASMA_enum uplo, 
-                       PLASMA_desc *A, 
+int PLASMA_zhegst_Tile(PLASMA_enum itype, PLASMA_enum uplo,
+                       PLASMA_desc *A,
                        PLASMA_desc *B)
 {
     plasma_context_t *plasma;
@@ -270,7 +271,7 @@ int PLASMA_zhegst_Tile(PLASMA_enum itype, PLASMA_enum uplo,
  *  ONLY PlasmaItype == 1 and PlasmaLower supported!
  *  Non-blocking equivalent of PLASMA_zhegst_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -291,13 +292,13 @@ int PLASMA_zhegst_Tile(PLASMA_enum itype, PLASMA_enum uplo,
  * @sa PLASMA_zhegv_Tile_Async
  *
  ******************************************************************************/
-int PLASMA_zhegst_Tile_Async(PLASMA_enum itype, PLASMA_enum uplo, 
-                             PLASMA_desc *A, 
+int PLASMA_zhegst_Tile_Async(PLASMA_enum itype, PLASMA_enum uplo,
+                             PLASMA_desc *A,
                              PLASMA_desc *B,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
-    PLASMA_desc descB = *B;
+    PLASMA_desc descA;
+    PLASMA_desc descB;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -320,13 +321,17 @@ int PLASMA_zhegst_Tile_Async(PLASMA_enum itype, PLASMA_enum uplo,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zhegst_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zhegst_Tile", "invalid second descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descB = *B;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb) {
@@ -334,9 +339,9 @@ int PLASMA_zhegst_Tile_Async(PLASMA_enum itype, PLASMA_enum uplo,
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
     }
 
-    
-    /* 
-     * Transform Hermitian-definite generalized eigenproblem 
+
+    /*
+     * Transform Hermitian-definite generalized eigenproblem
      * to standard form
      */
     plasma_dynamic_call_6(plasma_pzhegst,
