@@ -6,7 +6,8 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
+ * @author Azzam Haidar
  * @author Hatem Ltaief
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -25,21 +26,19 @@
  *  B*A*x=(lambda)*x.
  *  Here A and B are assumed to be Hermitian and B is also positive
  *  definite.
- *  Note: Only PlasmaNoVec supported!
  *
  *******************************************************************************
  *
- * @param[in] PlasmaItype
+ * @param[in] itype
  *          Intended usage:
  *          = 1: A*x=(lambda)*B*x
- *          = 2: A*Bx=(lambda)*x 
- *          = 3: B*A*x=(lambda)*x 
+ *          = 2: A*Bx=(lambda)*x
+ *          = 3: B*A*x=(lambda)*x
  *
  * @param[in] jobz
  *          Intended usage:
  *          = PlasmaNoVec: computes eigenvalues only;
  *          = PlasmaVec: computes eigenvalues and eigenvectors.
- *          Note: Only PlasmaNoVec supported!
  *
  * @param[in] uplo
  *          Specifies whether the matrix A is upper triangular or
@@ -132,7 +131,7 @@ int PLASMA_zhegv(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo, int N,
                  PLASMA_desc *descT,
                  PLASMA_Complex64_t *Q, int LDQ)
 {
-    int NB, IB, IBNB, NT;
+    int NB;
     int status;
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
@@ -145,19 +144,6 @@ int PLASMA_zhegv(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo, int N,
         return PLASMA_ERR_NOT_INITIALIZED;
     }
 
-    /* Tune NB & IB depending on N; Set NBNB */
-    status = plasma_tune(PLASMA_FUNC_ZHEGV, N, N, 0);
-    if (status != PLASMA_SUCCESS) {
-        plasma_error("PLASMA_zhegv", "plasma_tune() failed");
-        return status;
-    }
-
-    /* Set NT */
-    NB   = PLASMA_NB;
-    IB   = PLASMA_IB;
-    IBNB = IB*NB;
-    NT   = (N%NB==0) ? (N/NB) : (N/NB+1);
-
     /* Check input arguments */
     if (itype != 1 && itype != 2 && itype != 3) {
         plasma_error("PLASMA_zhegv", "Illegal value of itype");
@@ -183,74 +169,65 @@ int PLASMA_zhegv(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo, int N,
         plasma_error("PLASMA_zhegv", "illegal value of LDB");
         return -8;
     }
-    if ( (plasma_desc_check(descT) != PLASMA_SUCCESS) || 
-         ( descT->m != NT*IB ) || (descT->n != NT*NB) ) {
-        plasma_error("PLASMA_zhegv", "invalid T descriptor");
-        return -10;
-    }
     if (LDQ < max(1, N)) {
         plasma_error("PLASMA_zhegv", "illegal value of LDQ");
         return -12;
     }
-    
+
     /* Quick return */
     if (N == 0)
         return PLASMA_SUCCESS;
 
-    if (jobz == PlasmaVec) {
-        plasma_error("PLASMA_zhegv", "computing the eigenvectors is not supported in this version");
-        return -1;
+    /* Tune NB & IB depending on N; Set NBNB */
+    status = plasma_tune(PLASMA_FUNC_ZHEGV, N, N, 0);
+    if (status != PLASMA_SUCCESS) {
+        plasma_error("PLASMA_zhegv", "plasma_tune() failed");
+        return status;
     }
 
+    /* Set NT */
+    NB = PLASMA_NB;
+
     plasma_sequence_create(plasma, &sequence);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N, 
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N, sequence, &request,
                              plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descB, B, NB, NB, LDB, N, 0, 0, N, N, 
+        plasma_zooplap2tile( descB, B, NB, NB, LDB, N, 0, 0, N, N, sequence, &request,
                              plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)) );
-        if (jobz == PlasmaVec) {
-            /* No need for conversion, it's just output */
-            plasma_zdesc_alloc( descQ, NB, NB, LDQ, N, 0, 0, N, N, 
-                                plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)); plasma_desc_mat_free(&(descQ)) );
-        }
+        /* No conversion for Q */
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N );
-        plasma_ziplap2tile( descB, B, NB, NB, LDB, N, 0, 0, N, N );
-        if (jobz == PlasmaVec) {
-            /* No need for conversion, it's just output */
-            descQ = plasma_desc_init(
-                PlasmaComplexDouble, NB, NB, NB*NB,
-                LDQ, N, 0, 0, N, N);
-            descQ.mat = Q;
-        }
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N,
+                            sequence, &request);
+        plasma_ziplap2tile( descB, B, NB, NB, LDB, N, 0, 0, N, N,
+                            sequence, &request);
     }
 
+    /* Q is used only as an output */
+    descQ = plasma_desc_init(
+        PlasmaComplexDouble, NB, NB, NB*NB,
+        LDQ, N, 0, 0, N, N);
+    descQ.mat = Q;
+
     /* Call the tile interface */
-    PLASMA_zhegv_Tile_Async(itype, PlasmaNoVec, uplo, 
-                            &descA, &descB, W, 
-                            descT, &descQ, 
+    PLASMA_zhegv_Tile_Async(itype, jobz, uplo,
+                            &descA, &descB, W,
+                            descT, &descQ,
                             sequence, &request);
 
+    plasma_ziptile2lap( descQ, Q, NB, NB, LDQ, N,  sequence, &request);
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descA, A, NB, NB, LDA, N );
-        plasma_zooptile2lap( descB, B, NB, NB, LDB, N );
-        if (jobz == PlasmaVec) {
-           plasma_zooptile2lap( descQ, Q, NB, NB, LDQ, N );
-        }
+        plasma_zooptile2lap( descA, A, NB, NB, LDA, N, sequence, &request);
+        plasma_zooptile2lap( descB, B, NB, NB, LDB, N, sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
         plasma_desc_mat_free(&descB);
-        if (jobz == PlasmaVec)
-           plasma_desc_mat_free(&descQ);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N );
-        plasma_ziptile2lap( descB, B, NB, NB, LDB, N );
-        if (jobz == PlasmaVec)
-           plasma_ziptile2lap( descQ, Q, NB, NB, LDQ, N );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N, sequence, &request);
+        plasma_ziptile2lap( descB, B, NB, NB, LDB, N, sequence, &request);
         plasma_dynamic_sync();
     }
-    
+
     status = sequence->status;
     plasma_sequence_destroy(plasma, sequence);
     return status;
@@ -273,17 +250,16 @@ int PLASMA_zhegv(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo, int N,
  *
  *******************************************************************************
  *
- * @param[in] PlasmaItype
+ * @param[in] itype
  *          Intended usage:
  *          = 1: A*x=(lambda)*B*x
- *          = 2: A*Bx=(lambda)*x 
- *          = 3: B*A*x=(lambda)*x 
+ *          = 2: A*Bx=(lambda)*x
+ *          = 3: B*A*x=(lambda)*x
  *
  * @param[in] jobz
  *          Intended usage:
  *          = PlasmaNoVec: computes eigenvalues only;
  *          = PlasmaVec: computes eigenvalues and eigenvectors.
- *          Currently only PlasmaVec is NOT supported.
  *
  * @param[in] uplo
  *          Specifies whether the matrix A is upper triangular or
@@ -291,9 +267,6 @@ int PLASMA_zhegv(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo, int N,
  *          = PlasmaUpper: Upper triangle of A and B are stored;
  *          = PlasmaLower: Lower triangle of A and B are stored.
  *
- * @param[in] N
- *          The order of the matrix A. N >= 0.
- *
  * @param[in,out] A
  *          On entry, the symmetric (or Hermitian) matrix A.
  *          If uplo = PlasmaUpper, the leading N-by-N upper triangular
@@ -313,9 +286,6 @@ int PLASMA_zhegv(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo, int N,
  *          uplo = PlasmaLower) or the upper triangle (if uplo =
  *          PlasmaUpper) of A, including the diagonal, is destroyed.
  *
- * @param[in] LDA
- *          The leading dimension of the array A. LDA >= max(1,N).
- *
  * @param[in,out] B
  *          On entry, the symmetric (or Hermitian) positive definite
  *          matrix B.
@@ -331,9 +301,6 @@ int PLASMA_zhegv(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo, int N,
  *          the matrix is overwritten by the triangular factor U or L
  *          from the Cholesky factorization B = U**H*U or B = L*L**H.
  *
- * @param[in] LDB
- *          The leading dimension of the array B. LDA >= max(1,N).
- *
  * @param[in,out] T
  *          On entry, descriptor as return by
  *          PLASMA_Alloc_Workspace_zhegv
@@ -367,8 +334,8 @@ int PLASMA_zhegv(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo, int N,
  * @sa PLASMA_ssygv_Tile
  *
  ******************************************************************************/
-int PLASMA_zhegv_Tile(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo, 
-                      PLASMA_desc *A, PLASMA_desc *B, double *W, 
+int PLASMA_zhegv_Tile(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo,
+                      PLASMA_desc *A, PLASMA_desc *B, double *W,
                       PLASMA_desc *T, PLASMA_desc *Q)
 {
     plasma_context_t *plasma;
@@ -399,10 +366,10 @@ int PLASMA_zhegv_Tile(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo,
  *  B*A*x=(lambda)*x.
  *  Here A and B are assumed to be Hermitian and B is also
  *  positive definite.
- * 
+ *
  *  Non-blocking equivalent of PLASMA_zhegv_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -422,20 +389,16 @@ int PLASMA_zhegv_Tile(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo,
  * @sa PLASMA_ssygv_Tile_Async
  *
  ******************************************************************************/
-int PLASMA_zhegv_Tile_Async(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo, 
-                            PLASMA_desc *A,
-                            PLASMA_desc *B,
-                            double *W,
-                            PLASMA_desc *T,
-                            PLASMA_desc *Q,
+int PLASMA_zhegv_Tile_Async(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum uplo,
+                            PLASMA_desc *A, PLASMA_desc *B, double *W,
+                            PLASMA_desc *T, PLASMA_desc *Q,
                             PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    int NB, IB, IBNB, NT;
-    int status;
-    PLASMA_desc descA = *A;
-    PLASMA_desc descB = *B;
-    PLASMA_desc descT = *T;
-    double *E;
+    int neig, status;
+    PLASMA_desc descA;
+    PLASMA_desc descB;
+    PLASMA_desc descQ;
+    PLASMA_enum trans;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -457,31 +420,30 @@ int PLASMA_zhegv_Tile_Async(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum upl
     else
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
-    /* Set NT & NTRHS */
-    NB   = PLASMA_NB;
-    IB   = PLASMA_IB;
-    IBNB = IB*NB;
-    NT   = (descA.ln%NB==0) ? (descA.ln/NB) : (descA.ln/NB+1);
-
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zhegv_Tile_Async", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zhegv_Tile_Async", "invalid second descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descB = *B;
     }
-    if (plasma_desc_check(&descT) != PLASMA_SUCCESS) {
-        plasma_error("PLASMA_zhegv_Tile_Async", "invalid descriptor");
+    if (plasma_desc_check(T) != PLASMA_SUCCESS) {
+        plasma_error("PLASMA_zhegv_Tile_Async", "invalid T descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
     }
-    if (jobz == PlasmaVec){
-       if (plasma_desc_check(Q) != PLASMA_SUCCESS) {
-           plasma_error("PLASMA_zhegv_Tile_Async", "invalid descriptor");
-           return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
-       }
+    if (plasma_desc_check(Q) != PLASMA_SUCCESS) {
+        plasma_error("PLASMA_zhegv_Tile_Async", "invalid Q descriptor");
+        return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descQ = *Q;
     }
+
     /* Check input arguments */
     if (itype != 1 && itype != 2 && itype != 3) {
         plasma_error("PLASMA_zhegv_Tile_Async", "Illegal value of itype");
@@ -492,133 +454,109 @@ int PLASMA_zhegv_Tile_Async(PLASMA_enum itype, PLASMA_enum jobz, PLASMA_enum upl
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
     }
     if (uplo != PlasmaLower && uplo != PlasmaUpper) {
-        plasma_error("PLASMA_zheev_Tile_Async", "illegal value of uplo");
+        plasma_error("PLASMA_zhegv_Tile_Async", "illegal value of uplo");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
     }
     if (descA.nb != descA.mb) {
         plasma_error("PLASMA_zhegv_Tile_Async", "only square tiles supported");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
     }
-    if (jobz == PlasmaVec) {
-        plasma_error("PLASMA_zhegv_Tile_Async", "computing the eigenvectors is not supported in this version");
-        return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
-    }
-    if ( (jobz == PlasmaVec) && (Q->nb != Q->mb) ) {
+    if ( (jobz == PlasmaVec) && (descA.nb != descA.mb) ) {
         plasma_error("PLASMA_zhegv_Tile_Async", "only square tiles supported");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
     }
 
-    E = (double *)plasma_shared_alloc(plasma, descA.n-1, PlasmaRealDouble);
-
-    /* Currently NOT equivalent to LAPACK's
+    /*
+     * Form a Cholesky factorization of B.
      */
     plasma_parallel_call_4(plasma_pzpotrf,
-        PLASMA_enum, uplo,
-        PLASMA_desc, descB,
+        PLASMA_enum,      uplo,
+        PLASMA_desc,      descB,
         PLASMA_sequence*, sequence,
-        PLASMA_request*, request);
+        PLASMA_request*,  request);
 
     status = sequence->status;
     if (status != 0){
-       status = descA.ln + status;
+       status = descA.n + status;
        return status;
     }
 
-    /* 
-     * Transform problem to standard eigenvalue problem and solve
+    /*
+     * Transform problem to standard eigenvalue problem and solve.
      */
     plasma_dynamic_call_6(plasma_pzhegst,
-        PLASMA_enum, itype,
-        PLASMA_enum, uplo,
-        PLASMA_desc, descA,
-        PLASMA_desc, descB,
+        PLASMA_enum,      itype,
+        PLASMA_enum,      uplo,
+        PLASMA_desc,      descA,
+        PLASMA_desc,      descB,
         PLASMA_sequence*, sequence,
-        PLASMA_request*, request);
-
-    /* Reduction to tridiagonal form
-     * with a two-stage approach.
-     */
-
-    /* 
-     *Reduction to BAND tridiagonal form
-     */
-    plasma_dynamic_call_5(plasma_pzherbt,
-        PLASMA_enum, uplo,
-        PLASMA_desc, descA,
-        PLASMA_desc, descT,
-        PLASMA_sequence*, sequence,
-        PLASMA_request*, request);
+        PLASMA_request*,  request);
 
     /*
-     * Build the Q of the first stage
+     * Call Eigensolver
      */
-    /* if (jobz == PlasmaVec){ */
-    /*     /\* Initialize Q to Identity *\/ */
-    /*     plasma_dynamic_call_6(plasma_pzlaset, */
-    /*                           PLASMA_enum, PlasmaUpperLower, */
-    /*                           PLASMA_Complex64_t, 0.0, */
-    /*                           PLASMA_Complex64_t, 1.0, */
-    /*                           PLASMA_desc, descQ, */
-    /*                           PLASMA_sequence*, sequence, */
-    /*                           PLASMA_request*, request); */
+    status = PLASMA_zheev_Tile_Async( jobz, uplo,
+                                      A, W, T,
+                                      (PLASMA_Complex64_t*)(Q->mat), Q->lm,
+                                      sequence, request );
 
-    /*     /\* Accumulate the transformations from the first stage *\/ */
-    /*     plasma_dynamic_call_6(plasma_pzungtr, */
-    /*                           PLASMA_enum, uplo, */
-    /*                           PLASMA_desc, descA, */
-    /*                           PLASMA_desc, descQ, */
-    /*                           PLASMA_desc, descT, */
-    /*                           PLASMA_sequence*, sequence, */
-    /*                           PLASMA_request*, request); */
-    /* } */
-    
-    /* Set the V's to zero before the 2nd stage (bulge chasing) */
-    plasma_dynamic_call_5(
-        plasma_pzlaset2,
-        PLASMA_enum, uplo,
-        PLASMA_Complex64_t, 0.0,
-        PLASMA_desc, uplo == PlasmaLower ? plasma_desc_submatrix(descA, descA.mb, 0, descA.m-descA.mb, descA.n-descA.nb) : 
-                                           plasma_desc_submatrix(descA, 0, descA.nb, descA.m-descA.mb, descA.n-descA.nb),
-        PLASMA_sequence*, sequence,
-        PLASMA_request*, request);
-    /* 
-     * Reduction from BAND tridiagonal to the final condensed form
+    /*
+     * Backtransform eigenvectors to the original problem.
      */
-    plasma_dynamic_call_7(plasma_pzhbrdt,
-        PLASMA_enum, uplo,
-        PLASMA_desc, descA,
-        double*, W,
-        double*, E,
-        PLASMA_desc, descT,
-        PLASMA_sequence*, sequence,
-        PLASMA_request*, request);
+    if( jobz == PlasmaVec ) {
+        neig = descA.n;
+        if (status > 0)
+            neig = status-1;
 
-    /* For eigenvalues only, call DSTERF.  
-     * For eigenvectors, first call ZUNGTR to generate the unitary matrix, 
-     * then call ZSTEQR.
-     */
-
-    if (jobz == PlasmaNoVec)
-        status = LAPACKE_dsterf(descA.n, W, E);
-    else {
-        status = LAPACKE_dsterf(descA.n, W, E);
-        /* Accumulate the transformations from the second stage */
         /*
-        plasma_dynamic_call_6(plasma_pzungtr,
-            PLASMA_enum, uplo,
-            PLASMA_desc, descA,
-            PLASMA_desc, descQ,
-            PLASMA_desc, descT,
-            PLASMA_sequence*, sequence,
-            PLASMA_request*, request);
-        LAPACKE_zsteqr(jobz, descA.n, W, E, descQ.mat, descQ.lm);
-        */
+         * Convert back the lapack layout used in zheev to tile layout
+         */
+        plasma_ziplap2tile( descQ, (PLASMA_Complex64_t*)Q->mat, descQ.mb, descQ.nb, descQ.lm, descQ.ln,
+                            0, 0, descQ.m, neig, sequence, request );
+
+        if( (itype == 1) || (itype == 2) ) {
+            /*
+             * For A*x=(lambda)*B*x and A*B*x=(lambda)*x;
+             * backtransform eigenvectors: x = inv(L)**H *y or inv(U)*y
+             */
+             if( uplo == PlasmaUpper )
+                 trans = PlasmaNoTrans;
+             else
+                 trans = PlasmaConjTrans;
+
+             plasma_parallel_call_9(plasma_pztrsm,
+                 PLASMA_enum,        PlasmaLeft,
+                 PLASMA_enum,        uplo,
+                 PLASMA_enum,        trans,
+                 PLASMA_enum,        PlasmaNonUnit,
+                 PLASMA_Complex64_t, 1.0,
+                 PLASMA_desc,        descB,
+                 PLASMA_desc,        descQ,
+                 PLASMA_sequence*,   sequence,
+                 PLASMA_request*,    request);
+        }
+        else if( itype == 3 ) {
+            /*
+             * For B*A*x=(lambda)*x;
+             * backtransform eigenvectors: x = L*y or U**H *y
+             */
+             if( uplo == PlasmaUpper )
+                 trans = PlasmaConjTrans;
+             else
+                 trans = PlasmaNoTrans;
+
+             plasma_dynamic_call_9(plasma_pztrmm,
+                 PLASMA_enum,        PlasmaLeft,
+                 PLASMA_enum,        uplo,
+                 PLASMA_enum,        trans,
+                 PLASMA_enum,        PlasmaNonUnit,
+                 PLASMA_Complex64_t, 1.0,
+                 PLASMA_desc,        descB,
+                 PLASMA_desc,        descQ,
+                 PLASMA_sequence*,   sequence,
+                 PLASMA_request*,    request);
+        }
     }
 
-    /* If matrix was scaled, then rescale eigenvalues appropriately.
-     */
-
-    plasma_shared_free(plasma, E);
-
     return PLASMA_SUCCESS;
 }
