@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions normal z -> c
@@ -159,16 +159,19 @@ int PLASMA_zhemm(PLASMA_enum side, PLASMA_enum uplo, int M, int N,
     plasma_sequence_create(plasma, &sequence);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, Am, 0, 0, Am, Am, 
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, Am, 0, 0, Am, Am, sequence, &request,
                              plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descB, B, NB, NB, LDB, N,  0, 0, M,  N,
+        plasma_zooplap2tile( descB, B, NB, NB, LDB, N,  0, 0, M,  N, sequence, &request,
                              plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
-        plasma_zooplap2tile( descC, C, NB, NB, LDC, N,  0, 0, M,  N,
+        plasma_zooplap2tile( descC, C, NB, NB, LDC, N,  0, 0, M,  N, sequence, &request,
                              plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)); plasma_desc_mat_free(&(descC)));
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, Am, 0, 0, Am, Am );
-        plasma_ziplap2tile( descB, B, NB, NB, LDB, N,  0, 0, M,  N  );
-        plasma_ziplap2tile( descC, C, NB, NB, LDC, N,  0, 0, M,  N  );
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, Am, 0, 0, Am, Am,
+                            sequence, &request);
+        plasma_ziplap2tile( descB, B, NB, NB, LDB, N,  0, 0, M,  N,
+                            sequence, &request);
+        plasma_ziplap2tile( descC, C, NB, NB, LDC, N,  0, 0, M,  N,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
@@ -176,15 +179,15 @@ int PLASMA_zhemm(PLASMA_enum side, PLASMA_enum uplo, int M, int N,
         side, uplo, alpha, &descA, &descB, beta, &descC, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descC, C, NB, NB, LDC, N );
+        plasma_zooptile2lap( descC, C, NB, NB, LDC, N,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
         plasma_desc_mat_free(&descB);
         plasma_desc_mat_free(&descC);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, Am );
-        plasma_ziptile2lap( descB, B, NB, NB, LDB, N );
-        plasma_ziptile2lap( descC, C, NB, NB, LDC, N );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, Am,  sequence, &request);
+        plasma_ziptile2lap( descB, B, NB, NB, LDB, N,  sequence, &request);
+        plasma_ziptile2lap( descC, C, NB, NB, LDC, N,  sequence, &request);
         plasma_dynamic_sync();
     }
 
@@ -280,7 +283,7 @@ int PLASMA_zhemm_Tile(PLASMA_enum side, PLASMA_enum uplo,
  *  PLASMA_zhemm_Tile_Async - Performs Hermitian matrix multiplication.
  *  Non-blocking equivalent of PLASMA_zhemm_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -306,9 +309,9 @@ int PLASMA_zhemm_Tile_Async(PLASMA_enum side, PLASMA_enum uplo,
                             PLASMA_sequence *sequence, PLASMA_request *request)
 {
     plasma_context_t *plasma;
-    PLASMA_desc descA = *A;
-    PLASMA_desc descB = *B;
-    PLASMA_desc descC = *C;
+    PLASMA_desc descA;
+    PLASMA_desc descB;
+    PLASMA_desc descC;
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
@@ -330,17 +333,23 @@ int PLASMA_zhemm_Tile_Async(PLASMA_enum side, PLASMA_enum uplo,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zhemm_Tile_Async", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zhemm_Tile_Async", "invalid second descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descB = *B;
     }
-    if (plasma_desc_check(&descC) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(C) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zhemm_Tile_Async", "invalid third descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descC = *C;
     }
     /* Check input arguments */
     if ( (side != PlasmaLeft) && (side != PlasmaRight) ){
