@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions normal z -> c
@@ -56,7 +56,8 @@
  *
  * @param[in] LDA
  *          The leading dimension of the array A. LDA must be at least
- *          max( 1, N ), otherwise LDA must be at least max( 1, K ).
+ *          max( 1, N ) if trans == PlasmaNoTrans, otherwise LDA must
+ *          be at least max( 1, K ).
  *
  * @param[in] beta
  *          beta specifies the scalar beta
@@ -149,24 +150,28 @@ int PLASMA_zherk(PLASMA_enum uplo, PLASMA_enum trans, int N, int K,
     plasma_sequence_create(plasma, &sequence);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, An, 0, 0, Am, An, plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descC, C, NB, NB, LDC, N,  0, 0, N,  N,  plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descC)));
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, An, 0, 0, Am, An, sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descC, C, NB, NB, LDC, N,  0, 0, N,  N,   sequence, &request,
+                             plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descC)));
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, An, 0, 0, Am, An );
-        plasma_ziplap2tile( descC, C, NB, NB, LDC, N,  0, 0, N,  N );
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, An, 0, 0, Am, An, 
+                            sequence, &request);
+        plasma_ziplap2tile( descC, C, NB, NB, LDC, N,  0, 0, N,  N, 
+                            sequence, &request);
     }
 
     /* Call the tile interface */
     PLASMA_zherk_Tile_Async(uplo, trans, alpha, &descA, beta, &descC, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descC, C, NB, NB, LDC, N );
+        plasma_zooptile2lap( descC, C, NB, NB, LDC, N,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
         plasma_desc_mat_free(&descC);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, An );
-        plasma_ziptile2lap( descC, C, NB, NB, LDC, N );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, An,  sequence, &request);
+        plasma_ziptile2lap( descC, C, NB, NB, LDC, N,  sequence, &request);
         plasma_dynamic_sync();
     }
 
@@ -253,7 +258,7 @@ int PLASMA_zherk_Tile(PLASMA_enum uplo, PLASMA_enum trans,
  *  PLASMA_zherk_Tile_Async - Performs Hermitian rank-k update.
  *  Non-blocking equivalent of PLASMA_zherk_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -279,10 +284,10 @@ int PLASMA_zherk_Tile_Async(PLASMA_enum uplo, PLASMA_enum trans,
                             PLASMA_sequence *sequence, PLASMA_request *request)
 {
     plasma_context_t *plasma;
-    PLASMA_desc descA = *A;
-    PLASMA_desc descC = *C;
+    PLASMA_desc descA;
+    PLASMA_desc descC;
     int N, K;
-    int Am, An, Ai, Aj, Amb, Anb;
+    int Am, An, Amb;
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
@@ -304,13 +309,17 @@ int PLASMA_zherk_Tile_Async(PLASMA_enum uplo, PLASMA_enum trans,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zherk_Tile_Async", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descC) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(C) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zherk_Tile_Async", "invalid third descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descC = *C;
     }
     /* Check input arguments */
     if ((uplo != PlasmaUpper) && (uplo != PlasmaLower)) {
@@ -326,16 +335,10 @@ int PLASMA_zherk_Tile_Async(PLASMA_enum uplo, PLASMA_enum trans,
         Am  = descA.m;
         An  = descA.n;
         Amb = descA.mb;
-        Anb = descA.nb;
-        Ai  = descA.i;
-        Aj  = descA.j;
     } else {
         Am  = descA.n;
         An  = descA.m;
         Amb = descA.nb;
-        Anb = descA.mb;
-        Ai  = descA.j;
-        Aj  = descA.i;
     }
 
     if (descC.mb != descC.nb) {
