@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Azzam Haidar
  * @author Hatem Ltaief
  * @date 2010-11-15
@@ -24,16 +24,13 @@
  *  Second stage: reduction from band to tridiagonal form (unitary
  *  Q2).  Let Q = Q1 * Q2 be the global unitary transformation; Q**H *
  *  A * Q = S.
- *  Not LAPACK compliant as A does not contain the T elements
- *  Note: Only PlasmaNoVec supported!
  *
  *******************************************************************************
  *
  * @param[in] jobz
  *          Intended usage:
- *          = PlasmaNoVec: computes eigenvalues only;
- *          = PlasmaVec: computes eigenvalues and eigenvectors.
- *          Note: Only PlasmaNoVec supported!
+ *          = PlasmaNoVec: computes tridiagonal only;
+ *          = PlasmaVec: computes tridiagonal and generate the orthogonal matrix Q.
  *
  * @param[in] uplo
  *          Specifies whether the matrix A is upper triangular or
@@ -62,22 +59,24 @@
  *          The leading dimension of the array A. LDA >= max(1,N).
  *
  * @param[out] D
- *          On exit, the diagonal elements of the tridiagonal matrix: 
+ *          On exit, the diagonal elements of the tridiagonal matrix:
  *          D(i) = A(i,i).
  *
  * @param[out] E
- *          On exit, he off-diagonal elements of the tridiagonal matrix: 
+ *          On exit, he off-diagonal elements of the tridiagonal matrix:
  *          E(i) = A(i,i+1) if uplo = PlasmaUpper, E(i) = A(i+1,i) if uplo = PlasmaLower.
  *
- * @param[in, out] descT
- *          On entry, descriptor as return by PLASMA_Alloc_Workspace_zheev
+ * @param[out] descT
+ *          On entry, descriptor as return by PLASMA_Alloc_Workspace_zhetrd
  *          On exit, contains auxiliary factorization data.
  *
  * @param[out] Q
- *          On exit, if jobz = PlasmaVec and info = 0, the eigenvectors.
+ *          On exit, if jobz = PlasmaVec, then if return value = 0, Q
+ *          contains the N-by-N unitary matrix Q.
+ *          If jobz = PlasmaNoVec, then it is not referenced.
  *
  * @param[in] LDQ
- *          The leading dimension of the array Q. LDQ >= max(1,N).
+ *          The leading dimension of the array Q. LDQ >= N.
  *
  *******************************************************************************
  *
@@ -104,12 +103,12 @@ int PLASMA_zhetrd(PLASMA_enum jobz, PLASMA_enum uplo, int N,
                  PLASMA_desc *descT,
                  PLASMA_Complex64_t *Q, int LDQ)
 {
-    int NB, IB, IBNB, NT;
+    int NB;
     int status;
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
     PLASMA_request request = PLASMA_REQUEST_INITIALIZER;
-    PLASMA_desc descA, descQ;
+    PLASMA_desc descA;
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
@@ -117,19 +116,6 @@ int PLASMA_zhetrd(PLASMA_enum jobz, PLASMA_enum uplo, int N,
         return PLASMA_ERR_NOT_INITIALIZED;
     }
 
-    /* Tune NB & IB depending on N; Set NBNB */
-    status = plasma_tune(PLASMA_FUNC_ZHETRD, N, N, 0);
-    if (status != PLASMA_SUCCESS) {
-        plasma_error("PLASMA_zhetrd", "plasma_tune() failed");
-        return status;
-    }
-
-    /* Set NT */
-    NB    = PLASMA_NB;
-    IB    = PLASMA_IB;
-    IBNB  = IB*NB;
-    NT    = (N%NB==0) ? (N/NB) : (N/NB+1);
-
     /* Check input arguments */
     if (jobz != PlasmaNoVec && jobz != PlasmaVec) {
         plasma_error("PLASMA_zhetrd", "illegal value of jobz");
@@ -137,63 +123,49 @@ int PLASMA_zhetrd(PLASMA_enum jobz, PLASMA_enum uplo, int N,
     }
     if (uplo != PlasmaLower && uplo != PlasmaUpper) {
         plasma_error("PLASMA_zhetrd", "illegal value of uplo");
-        return -2;
+        return -1;
     }
     if (N < 0) {
         plasma_error("PLASMA_zhetrd", "illegal value of N");
-        return -3;
+        return -2;
     }
     if (LDA < max(1, N)) {
         plasma_error("PLASMA_zhetrd", "illegal value of LDA");
-        return -5;
-    }
-    if ( (plasma_desc_check(descT) != PLASMA_SUCCESS) || 
-         ( descT->m != NT*IB ) || (descT->n != NT*NB) ) {
-        plasma_error("PLASMA_zhetrd", "invalid T descriptor");
-        return -8;
-    }
-    if (LDQ < max(1, N)) {
-        plasma_error("PLASMA_zhetrd", "illegal value of LDQ");
-        return -10;
+        return -4;
     }
+
     /* Quick return */
     if (N == 0)
         return PLASMA_SUCCESS;
 
-    if (jobz == PlasmaVec) {
-        plasma_error("PLASMA_zhetrd", "computing the eigenvectors is not supported in this version");
-        return -1;
+    /* Tune NB & IB depending on N; Set NBNB */
+    status = plasma_tune(PLASMA_FUNC_ZHETRD, N, N, 0);
+    if (status != PLASMA_SUCCESS) {
+        plasma_error("PLASMA_zhetrd", "plasma_tune() failed");
+        return status;
     }
+    /* Set NT */
+    NB = PLASMA_NB;
 
     plasma_sequence_create(plasma, &sequence);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   , plasma_desc_mat_free(&(descA)) );
-        if (jobz == PlasmaVec) {
-           plasma_zooplap2tile( descQ, Q, NB, NB, LDQ, N,    0, 0, N, N   , plasma_desc_mat_free(&(descQ)) );
-        }
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N, sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   );
-        if (jobz == PlasmaVec)
-           plasma_ziplap2tile( descQ, Q, NB, NB, LDQ, N,    0, 0, N, N   );
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
-    PLASMA_zhetrd_Tile_Async(jobz, uplo, &descA, D, E, descT, &descQ, sequence, &request);
+    PLASMA_zhetrd_Tile_Async(jobz, uplo, &descA, D, E, descT, Q, LDQ, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descA, A, NB, NB, LDA, N    );
-        if (jobz == PlasmaVec) {
-           plasma_zooptile2lap( descQ, Q, NB, NB, LDQ, N    );
-        }
+        plasma_zooptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
-        if (jobz == PlasmaVec)
-           plasma_desc_mat_free(&descQ);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N    );
-        if (jobz == PlasmaVec)
-           plasma_ziptile2lap( descQ, Q, NB, NB, LDQ, N    );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
         plasma_dynamic_sync();
     }
 
@@ -204,7 +176,7 @@ int PLASMA_zhetrd(PLASMA_enum jobz, PLASMA_enum uplo, int N,
 /***************************************************************************//**
  *
  * @ingroup PLASMA_Complex64_t_Tile
- * 
+ *
  *  PLASMA_zhetrd_Tile - reduces a complex Hermitian matrix A to real symmetric
  *  tridiagonal form S using a two-stage approach
  *  First stage: reduction to band tridiagonal form (unitary Q1);
@@ -220,9 +192,8 @@ int PLASMA_zhetrd(PLASMA_enum jobz, PLASMA_enum uplo, int N,
  *
  * @param[in] jobz
  *          Intended usage:
- *          = PlasmaNoVec: computes eigenvalues only;
- *          = PlasmaVec: computes eigenvalues and eigenvectors.
- *          PlasmaVec is NOT supported.
+ *          = PlasmaNoVec: computes tridiagonal only;
+ *          = PlasmaVec: computes tridiagonal and generate the orthogonal matrix Q.
  *
  * @param[in] uplo
  *          Specifies whether the matrix A is upper triangular or lower triangular:
@@ -244,19 +215,24 @@ int PLASMA_zhetrd(PLASMA_enum jobz, PLASMA_enum uplo, int N,
  *          PlasmaUpper) of A, including the diagonal, is destroyed.*
  *
  * @param[out] D
- *          On exit, the diagonal elements of the tridiagonal matrix: 
+ *          On exit, the diagonal elements of the tridiagonal matrix:
  *          D(i) = A(i,i).
  *
  * @param[out] E
- *          On exit, he off-diagonal elements of the tridiagonal matrix: 
- *          E(i) = A(i,i+1) if uplo = PlasmaUpper, 
+ *          On exit, he off-diagonal elements of the tridiagonal matrix:
+ *          E(i) = A(i,i+1) if uplo = PlasmaUpper,
  *          E(i) = A(i+1,i) if uplo = PlasmaLower.
  *
  * @param[out] T
  *          On exit, auxiliary factorization data.
  *
  * @param[out] Q
- *          On exit, if jobz = PlasmaVec and info = 0, the eigenvectors.
+ *          On exit, if jobz = PlasmaVec, then if return value = 0, Q
+ *          contains the N-by-N unitary matrix Q.
+ *          If jobz = PlasmaNoVec, then it is not referenced.
+ *
+ * @param[in] LDQ
+ *          The leading dimension of the array Q. LDQ >= N.
  *
  *******************************************************************************
  *
@@ -274,11 +250,12 @@ int PLASMA_zhetrd(PLASMA_enum jobz, PLASMA_enum uplo, int N,
  * @sa PLASMA_chetrd_Tile
  * @sa PLASMA_dsytrd_Tile
  * @sa PLASMA_ssytrd_Tile
- * @sa PLASMA_zheev_Tile
+ * @sa PLASMA_zhetrd_Tile
  *
  ******************************************************************************/
 int PLASMA_zhetrd_Tile(PLASMA_enum jobz, PLASMA_enum uplo,
-                      PLASMA_desc *A, double *D, double *E, PLASMA_desc *T, PLASMA_desc *Q)
+                       PLASMA_desc *A, double *D, double *E,
+                       PLASMA_desc *T, PLASMA_Complex64_t *Q, int LDQ)
 {
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
@@ -291,7 +268,7 @@ int PLASMA_zhetrd_Tile(PLASMA_enum jobz, PLASMA_enum uplo,
         return PLASMA_ERR_NOT_INITIALIZED;
     }
     plasma_sequence_create(plasma, &sequence);
-    PLASMA_zhetrd_Tile_Async(jobz, uplo, A, D, E, T, Q, sequence, &request);
+    PLASMA_zhetrd_Tile_Async(jobz, uplo, A, D, E, T, Q, LDQ, sequence, &request);
     plasma_dynamic_sync();
     status = sequence->status;
     plasma_sequence_destroy(plasma, sequence);
@@ -309,7 +286,7 @@ int PLASMA_zhetrd_Tile(PLASMA_enum jobz, PLASMA_enum uplo,
  *  Second stage: reduction from band to tridiagonal form.
  *
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -329,19 +306,24 @@ int PLASMA_zhetrd_Tile(PLASMA_enum jobz, PLASMA_enum uplo,
  * @sa PLASMA_ssytrd_Tile_Async
  *
  ******************************************************************************/
-int PLASMA_zhetrd_Tile_Async(PLASMA_enum jobz, PLASMA_enum uplo,
+int PLASMA_zhetrd_Tile_Async(PLASMA_enum jobz,
+                            PLASMA_enum uplo,
                             PLASMA_desc *A,
-                            double *D,
+                            double *W,
                             double *E,
                             PLASMA_desc *T,
-                            PLASMA_desc *Q,
+                            PLASMA_Complex64_t *Q, int LDQ,
                             PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    int NB, IB, IBNB, NT;
-    PLASMA_desc descA = *A;
-    PLASMA_desc descT = *T;
-
     plasma_context_t *plasma;
+    PLASMA_desc descA;
+    PLASMA_desc descT;
+    PLASMA_desc descQ;
+    PLASMA_Complex64_t *AB;
+    int N, NB, LDAB;
+    int i;
+    int status;
+
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
@@ -362,24 +344,18 @@ int PLASMA_zhetrd_Tile_Async(PLASMA_enum jobz, PLASMA_enum uplo,
     else
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
-    /* Set NT & NTRHS */
-    NB   = PLASMA_NB;
-    IB   = PLASMA_IB;
-    IBNB = IB*NB;
-    NT   = (descA.ln%NB==0) ? (descA.ln/NB) : (descA.ln/NB+1);
-
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zhetrd_Tile_Async", "invalid descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descT) != PLASMA_SUCCESS) {
-        plasma_error("PLASMA_zhetrd_Tile_Async", "invalid descriptor");
-        return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
-    }
-    if ( (jobz == PlasmaVec) && (plasma_desc_check(Q) != PLASMA_SUCCESS) ) {
+    if (plasma_desc_check(T) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zhetrd_Tile_Async", "invalid descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descT = *T;
     }
 
     /* Check input arguments */
@@ -387,80 +363,225 @@ int PLASMA_zhetrd_Tile_Async(PLASMA_enum jobz, PLASMA_enum uplo,
         plasma_error("PLASMA_zhetrd_Tile_Async", "illegal value of jobz");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
     }
+    if (uplo != PlasmaLower && uplo != PlasmaUpper) {
+        plasma_error("PLASMA_zhetrd_Tile_Async", "illegal value of uplo");
+        return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    }
     if (descA.m != descA.n) {
         plasma_error("PLASMA_zhetrd_Tile_Async", "matrix need to be square");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
     }
-   if (descA.nb != descA.mb) {
+    if (descA.nb != descA.mb) {
         plasma_error("PLASMA_zhetrd_Tile_Async", "only square tiles supported");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
     }
-    if (jobz == PlasmaVec) {
-        plasma_error("PLASMA_zhetrd_Tile_Async", "computing the eigenvectors is not supported in this version");
-        return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
-    }
-    if ( (jobz == PlasmaVec) && (Q->nb != Q->mb) ) {
-        plasma_error("PLASMA_zhetrd_Tile_Async", "only square tiles supported");
-        return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+
+    N    = descA.m;
+    NB   = min(descA.mb,descA.m);
+    LDAB = 2*NB+1;
+
+    /* Allocate workspace for band storage of the band matrix A and for the off diagonal after tridiagonalisation */
+    AB = (PLASMA_Complex64_t *)plasma_shared_alloc(plasma, LDAB*N, PlasmaComplexDouble);
+    memset( AB, 0, LDAB * N * sizeof(PLASMA_Complex64_t) );
+    if (AB == NULL) {
+        plasma_error("PLASMA_zhetrd", "plasma_shared_alloc() failed");
+        plasma_shared_free(plasma, AB);
+        return PLASMA_ERR_OUT_OF_RESOURCES;
     }
 
     /* Reduction to tridiagonal form
      * with a two-stage approach.
      */
-
-    /* Reduction to BAND tridiagonal form
-     */
-    plasma_dynamic_call_5(plasma_pzherbt,
+    /*=======================================
+     *  calling Reduction to BAND
+     *  then convert matrix to band form
+     *=======================================*/
+    plasma_dynamic_call_5(plasma_pzhetrd_he2hb,
         PLASMA_enum, uplo,
         PLASMA_desc, descA,
         PLASMA_desc, descT,
         PLASMA_sequence*, sequence,
         PLASMA_request*, request);
 
-    /* 
-     * Build the Q of the first stage
-     */
-    /* if (jobz == PlasmaVec){ */
-    /*    /\* Initialize Q to Identity *\/ */
-    /*    plasma_dynamic_call_6(plasma_pzlaset, */
-    /*        PLASMA_enum, PlasmaUpperLower, */
-    /*        PLASMA_Complex64_t, 0.0, */
-    /*        PLASMA_Complex64_t, 1.0, */
-    /*        PLASMA_desc, descQ, */
-    /*        PLASMA_sequence*, sequence, */
-    /*        PLASMA_request*, request); */
-    /*    /\* Accumulate the transformations from the first stage*\/ */
-    /*    plasma_dynamic_call_6(plasma_pzungtr, */
-    /*        PLASMA_enum, uplo, */
-    /*        PLASMA_desc, descA, */
-    /*        PLASMA_desc, descQ, */
-    /*        PLASMA_desc, descT, */
-    /*        PLASMA_sequence*, sequence, */
-    /*        PLASMA_request*, request); */
-    /* } */
-
-    /* Set the V's to zero before the 2nd stage (bulge chasing) */
-    /*
-    */
-    plasma_dynamic_call_5(plasma_pzlaset2,
-        PLASMA_enum, uplo,
-        PLASMA_Complex64_t, 0.0,
-        PLASMA_desc, uplo == PlasmaLower ? plasma_desc_submatrix(descA, descA.mb, 0, descA.m-descA.mb, descA.n-descA.nb) 
-                                         : plasma_desc_submatrix(descA, 0, descA.nb, descA.m-descA.mb, descA.n-descA.nb),
-        PLASMA_sequence*, sequence,
-        PLASMA_request*, request);
-
-    /* Reduction from BAND tridiagonal to the final condensed form
-     */
-    plasma_dynamic_call_7(plasma_pzhbrdt,
+    plasma_dynamic_call_6( plasma_pzhbcpy_t2bl,
         PLASMA_enum, uplo,
         PLASMA_desc, descA,
-        double*, D,
-        double*, E,
-        PLASMA_desc, descT,
+        PLASMA_Complex64_t*, AB,
+        int, LDAB,
         PLASMA_sequence*, sequence,
         PLASMA_request*, request);
 
+    plasma_dynamic_sync();
 
+    status = sequence->status;
+    if (status != PLASMA_SUCCESS) {
+        plasma_error("PLASMA_zhetrd","pzhetrd_he2hb+pzcopy");
+        return status;
+    }
+    /*=======================================
+     *  END of calling Reduction to BAND
+     *=======================================*/
+    /*=======================================
+     *  calling bulge chasing
+     *=======================================*/
+    PLASMA_Complex64_t *TAU2 = NULL;
+    PLASMA_Complex64_t *V2 = NULL;
+    PLASMA_Complex64_t *T2 = NULL;
+    int Vblksiz, blkcnt, LDT, LDV;
+    int WANTZ   = 0;
+    int blguplo = PlasmaLower;
+    /* int NE      = N; // for later use when a portion of the eigenvectors are requested*/
+    if( jobz == PlasmaNoVec )
+        WANTZ=0;
+    else
+        WANTZ=1;
+
+    /* Vblksiz correspond to the blocking used when applying V2 to the matrix Q
+     * it is similar to IB in LAPACK ZUNMQR.
+     * blkcnt is the number of losange or tile of Vs */
+    /* Note that in case PlamaVec requested, the V2 and T2 are stored by the
+     * bulgechasing function in a special format:
+     * for V2s: it store the V2(LDV,Vblksiz) of each losange in a tile storage meaning
+     * that V2_1 is stored then V2_2,..., V2_blkcnt.
+     * blkcnt is the number of losange.
+     * */
+    Vblksiz = min(NB,48);
+    LDT     = Vblksiz;
+    if( jobz == PlasmaVec ) {
+        findVTsiz(N, NB, Vblksiz, &blkcnt, &LDV);
+        TAU2   = (PLASMA_Complex64_t *) plasma_shared_alloc(plasma,     blkcnt*Vblksiz, PlasmaComplexDouble);
+        V2     = (PLASMA_Complex64_t *) plasma_shared_alloc(plasma, LDV*blkcnt*Vblksiz, PlasmaComplexDouble);
+        T2     = (PLASMA_Complex64_t *) plasma_shared_alloc(plasma, LDT*blkcnt*Vblksiz, PlasmaComplexDouble);
+        if ( (TAU2 == NULL) || (V2 == NULL) || (T2 == NULL) ) {
+            plasma_error("PLASMA_zhetrd", "plasma_shared_alloc() failed");
+            plasma_shared_free(plasma, TAU2);
+            plasma_shared_free(plasma, V2);
+            plasma_shared_free(plasma, T2);
+            return PLASMA_ERR_OUT_OF_RESOURCES;
+        }
+        memset(TAU2, 0,     blkcnt*Vblksiz*sizeof(PLASMA_Complex64_t));
+        memset(V2,   0, LDV*blkcnt*Vblksiz*sizeof(PLASMA_Complex64_t));
+        memset(T2,   0, LDT*blkcnt*Vblksiz*sizeof(PLASMA_Complex64_t));
+    }
+    else {
+        TAU2   = (PLASMA_Complex64_t *) plasma_shared_alloc(plasma, 2*N, PlasmaComplexDouble);
+        V2     = (PLASMA_Complex64_t *) plasma_shared_alloc(plasma, 2*N, PlasmaComplexDouble);
+        if ( (TAU2 == NULL) || (V2 == NULL) ) {
+            plasma_error("PLASMA_zhetrd", "plasma_shared_alloc() failed");
+            plasma_shared_free(plasma, TAU2);
+            plasma_shared_free(plasma, V2);
+            return PLASMA_ERR_OUT_OF_RESOURCES;
+        }
+        memset(TAU2, 0, 2*N*sizeof(PLASMA_Complex64_t));
+        memset(V2,   0, 2*N*sizeof(PLASMA_Complex64_t));
+    }
+
+    plasma_parallel_call_13(plasma_pzhetrd_hb2st_v1,
+        PLASMA_enum,         blguplo,
+        int,                 N,
+        int,                 NB,
+        int,                 Vblksiz,
+        PLASMA_Complex64_t*, AB,
+        int,                 LDAB,
+        PLASMA_Complex64_t*, V2,
+        PLASMA_Complex64_t*, TAU2,
+        double*,             W,
+        double*,             E,
+        int,                 WANTZ,
+        PLASMA_sequence*,    sequence,
+        PLASMA_request*,     request);
+    /* WARNING: If plasma_pzhetrd_hb2st is implemented through a dynamic call, don't
+     * forget to synchronize */
+    plasma_dynamic_sync();
+    /*=======================================
+     *  END of calling bulge chasing
+     *=======================================*/
+    if (jobz == PlasmaVec){
+        /*=======================================
+         *  generate Q2 from the bulge
+         *=======================================*/
+        /* Initialize Q to Identity */
+        memset(Q,   0, N*LDQ*sizeof(PLASMA_Complex64_t));
+        for(i=0; i<N; i++){
+            Q[i+i*LDQ] = 1.0;
+        }
+        /* compute T2 */
+        plasma_static_call_8(plasma_pzlarft_blgtrd,
+            int,                 N,
+            int,                 NB,
+            int,                 Vblksiz,
+            PLASMA_Complex64_t*, V2,
+            PLASMA_Complex64_t*, T2,
+            PLASMA_Complex64_t*, TAU2,
+            PLASMA_sequence*,    sequence,
+            PLASMA_request*,     request);
+        /* apply Q2 from Left */
+        plasma_static_call_14(plasma_pzunmqr_blgtrd,
+            PLASMA_enum,         PlasmaLeft,
+            PLASMA_enum,         PlasmaNoTrans,
+            int,                 N,
+            int,                 NB,
+            int,                 N,
+            int,                 Vblksiz,
+            int,                 WANTZ,
+            PLASMA_Complex64_t*, V2,
+            PLASMA_Complex64_t*, T2,
+            PLASMA_Complex64_t*, TAU2,
+            PLASMA_Complex64_t*, Q,
+            int,                 LDQ,
+            PLASMA_sequence*,    sequence,
+            PLASMA_request*,     request);
+        /*=======================================
+         *  apply Q1 from the reduction to band
+         *=======================================*/
+        /* CASE NB>N, Q1 doesn't need to be applied, only bulge chasing has been done */
+        if( NB < N ){
+            if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
+                plasma_zooplap2tile( descQ, Q, NB, NB, LDQ, N, 0, 0, N, N, sequence, request, plasma_desc_mat_free(&(descQ)) );
+            } else {
+                plasma_ziplap2tile( descQ, Q, NB, NB, LDQ, N, 0, 0, N, N, sequence, request );
+            }
+
+            /* Accumulate the transformations from the first stage */
+            if(uplo==PlasmaLower){
+                plasma_dynamic_call_7(plasma_pzunmqr,
+                    PLASMA_enum, PlasmaLeft,
+                    PLASMA_enum, PlasmaNoTrans,
+                    PLASMA_desc, plasma_desc_submatrix(descA, descA.mb, 0, descA.m-descA.mb, descA.n-descA.nb),
+                    PLASMA_desc, plasma_desc_submatrix(descQ, descQ.mb, 0, descQ.m-descQ.mb, descQ.n),
+                    PLASMA_desc, plasma_desc_submatrix(descT, descT.mb, 0, descT.m-descT.mb, descT.n-descT.nb),
+                    PLASMA_sequence*, sequence,
+                    PLASMA_request*, request);
+
+            }
+            else {
+                plasma_dynamic_call_7(plasma_pzunmlq,
+                    PLASMA_enum, PlasmaLeft,
+                    PLASMA_enum, PlasmaConjTrans,
+                    PLASMA_desc, plasma_desc_submatrix(descA, 0, descA.nb, descA.m-descA.mb, descA.n-descA.nb),
+                    PLASMA_desc, plasma_desc_submatrix(descQ, descQ.mb, 0, descQ.m-descQ.mb, descQ.n         ),
+                    PLASMA_desc, plasma_desc_submatrix(descT, 0, descT.nb, descT.m-descT.mb, descT.n-descT.nb),
+                    PLASMA_sequence*, sequence,
+                    PLASMA_request*, request);
+            }
+
+            if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
+                plasma_zooptile2lap( descQ, Q, NB, NB, LDQ, N, sequence, request );
+                plasma_dynamic_sync();
+                plasma_desc_mat_free(&descQ);
+            } else {
+                plasma_ziptile2lap( descQ, Q, NB, NB, LDQ, N, sequence, request );
+                plasma_dynamic_sync();
+            }
+        } /* END of ( NB < N ) */
+    }
+    /*=======================================
+     *  END of calling computing Q
+     *=======================================*/
+    if( jobz == PlasmaVec )
+        plasma_shared_free(plasma, T2);
+    plasma_shared_free(plasma, V2);
+    plasma_shared_free(plasma, TAU2);
+    plasma_shared_free(plasma, AB);
     return PLASMA_SUCCESS;
 }
