@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions normal z -> c
@@ -56,10 +56,6 @@
  * @param[in] LDA
  *          The leading dimension of the array A. LDA >= max(1,N).
  *
- * @param[in] work
- *          double precision array of dimension PLASMA_SIZE is
- *          PLASMA_STATIC_SCHEDULING is used, and NULL otherwise.
- *
  *******************************************************************************
  *
  * @return
@@ -75,7 +71,7 @@
  *
  ******************************************************************************/
 double PLASMA_zlanhe(PLASMA_enum norm, PLASMA_enum uplo, int N,
-                     PLASMA_Complex64_t *A, int LDA, double *work)
+                     PLASMA_Complex64_t *A, int LDA)
 {
     int NB;
     int status;
@@ -126,19 +122,21 @@ double PLASMA_zlanhe(PLASMA_enum norm, PLASMA_enum uplo, int N,
     plasma_sequence_create(plasma, &sequence);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N, plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N, sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
     } else {
-        plasma_ziplap2tile(  descA, A, NB, NB, LDA, N, 0, 0, N, N);
+        plasma_ziplap2tile(  descA, A, NB, NB, LDA, N, 0, 0, N, N,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
-    PLASMA_zlanhe_Tile_Async(norm, uplo, &descA, work, &value, sequence, &request);
+    PLASMA_zlanhe_Tile_Async(norm, uplo, &descA, &value, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
         plasma_dynamic_sync();
     }
 
@@ -174,10 +172,6 @@ double PLASMA_zlanhe(PLASMA_enum norm, PLASMA_enum uplo, int N,
  *          if UPLO = 'L', the lower triangle of A is overwritten with
  *          the lower triangle of the product L' * L.
  *
- * @param[in] work
- *          double precision array of dimension PLASMA_SIZE is
- *          PLASMA_STATIC_SCHEDULING is used, and NULL otherwise.
- *
  *******************************************************************************
  *
  * @return
@@ -192,7 +186,7 @@ double PLASMA_zlanhe(PLASMA_enum norm, PLASMA_enum uplo, int N,
  * @sa PLASMA_slanhe_Tile
  *
  ******************************************************************************/
-double PLASMA_zlanhe_Tile(PLASMA_enum norm, PLASMA_enum uplo, PLASMA_desc *A, double *work)
+double PLASMA_zlanhe_Tile(PLASMA_enum norm, PLASMA_enum uplo, PLASMA_desc *A)
 {
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
@@ -205,7 +199,7 @@ double PLASMA_zlanhe_Tile(PLASMA_enum norm, PLASMA_enum uplo, PLASMA_desc *A, do
         return PLASMA_ERR_NOT_INITIALIZED;
     }
     plasma_sequence_create(plasma, &sequence);
-    PLASMA_zlanhe_Tile_Async(norm, uplo, A, work, &value, sequence, &request);
+    PLASMA_zlanhe_Tile_Async(norm, uplo, A, &value, sequence, &request);
     plasma_dynamic_sync();
     plasma_sequence_destroy(plasma, sequence);
     return value;
@@ -217,7 +211,7 @@ double PLASMA_zlanhe_Tile(PLASMA_enum norm, PLASMA_enum uplo, PLASMA_desc *A, do
  *
  *  PLASMA_zlanhe_Tile_Async - Non-blocking equivalent of PLASMA_zlanhe_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -237,10 +231,11 @@ double PLASMA_zlanhe_Tile(PLASMA_enum norm, PLASMA_enum uplo, PLASMA_desc *A, do
  * @sa PLASMA_slanhe_Tile_Async
  *
  ******************************************************************************/
-int PLASMA_zlanhe_Tile_Async(PLASMA_enum norm, PLASMA_enum uplo, PLASMA_desc *A, double *work, double *value,
+int PLASMA_zlanhe_Tile_Async(PLASMA_enum norm, PLASMA_enum uplo, PLASMA_desc *A, double *value,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
+    PLASMA_desc descA;
+    double *work = NULL;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -263,9 +258,11 @@ int PLASMA_zlanhe_Tile_Async(PLASMA_enum norm, PLASMA_enum uplo, PLASMA_desc *A,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zlanhe_Tile", "invalid descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb) {
@@ -287,6 +284,14 @@ int PLASMA_zlanhe_Tile_Async(PLASMA_enum norm, PLASMA_enum uplo, PLASMA_desc *A,
         return PLASMA_SUCCESS;
     }
 
+    if (PLASMA_SCHEDULING == PLASMA_STATIC_SCHEDULING) {
+        if (norm == PlasmaFrobeniusNorm) {
+            work = plasma_shared_alloc(plasma, 2*PLASMA_SIZE, PlasmaRealDouble );
+        } else {
+            work = plasma_shared_alloc(plasma,   PLASMA_SIZE, PlasmaRealDouble );
+        }
+    }
+
     plasma_parallel_call_7(plasma_pzlanhe,
         PLASMA_enum, norm,
         PLASMA_enum, uplo,
@@ -296,5 +301,8 @@ int PLASMA_zlanhe_Tile_Async(PLASMA_enum norm, PLASMA_enum uplo, PLASMA_desc *A,
         PLASMA_sequence*, sequence,
         PLASMA_request*, request);
 
+    if (work != NULL)
+        plasma_shared_free( plasma, work );
+
     return PLASMA_SUCCESS;
 }
