@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -64,7 +64,7 @@
  *
  ******************************************************************************/
 int PLASMA_zlaswp(int N, PLASMA_Complex64_t *A, int LDA,
-                  int K1, int K2, int *IPIV, int INCX)
+                  int K1, int K2, const int *IPIV, int INCX)
 {
     int NB;
     int status;
@@ -105,20 +105,22 @@ int PLASMA_zlaswp(int N, PLASMA_Complex64_t *A, int LDA,
     plasma_sequence_create(plasma, &sequence);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, K2, N, plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, K2, N, sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, K2, N);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, K2, N,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
     PLASMA_zlaswp_Tile_Async(&descA, K1, K2, IPIV, INCX, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descA, A, NB, NB, LDA, N );
+        plasma_zooptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
         plasma_dynamic_sync();
     }
 
@@ -173,7 +175,7 @@ int PLASMA_zlaswp(int N, PLASMA_Complex64_t *A, int LDA,
  * @sa PLASMA_zgetrf_Tile
  *
  ******************************************************************************/
-int PLASMA_zlaswp_Tile(PLASMA_desc *A, int K1, int K2, int *IPIV, int INCX)
+int PLASMA_zlaswp_Tile(PLASMA_desc *A, int K1, int K2, const int *IPIV, int INCX)
 {
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
@@ -202,7 +204,7 @@ int PLASMA_zlaswp_Tile(PLASMA_desc *A, int K1, int K2, int *IPIV, int INCX)
  *  rows K1 through K2 of A.
  *  Non-blocking equivalent of PLASMA_zlaswp_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -223,10 +225,10 @@ int PLASMA_zlaswp_Tile(PLASMA_desc *A, int K1, int K2, int *IPIV, int INCX)
  * @sa PLASMA_zgetrf_Tile_Async
  *
  ******************************************************************************/
-int PLASMA_zlaswp_Tile_Async(PLASMA_desc *A, int K1, int K2, int *IPIV, int INCX,
+int PLASMA_zlaswp_Tile_Async(PLASMA_desc *A, int K1, int K2, const int *IPIV, int INCX,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
+    PLASMA_desc descA;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -249,9 +251,11 @@ int PLASMA_zlaswp_Tile_Async(PLASMA_desc *A, int K1, int K2, int *IPIV, int INCX
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zlaswp_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
 
     if ( (K1 != 1) || (K2 != descA.m) ) {
@@ -269,13 +273,13 @@ int PLASMA_zlaswp_Tile_Async(PLASMA_desc *A, int K1, int K2, int *IPIV, int INCX
     plasma_dynamic_call_5(
         plasma_pzlaswp,
         PLASMA_desc, descA,
-        int *,       IPIV,
+        const int *,       IPIV,
         int,         INCX,
         PLASMA_sequence*, sequence,
         PLASMA_request*,  request);
-    
+
     plasma_dynamic_call_3(
-        plasma_pzbarrier_tl2pnl,
+        plasma_pzbarrier_pnl2tl,
         PLASMA_desc, descA,
         PLASMA_sequence*, sequence,
         PLASMA_request*,  request);
