@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -105,7 +105,7 @@ int PLASMA_zplrnt( int M, int N,
     /* Call the tile interface */
     PLASMA_zplrnt_Tile_Async( &descA, seed, sequence, &request );
 
-    plasma_ziptile2lap( descA, A, NB, NB, LDA, N );
+    plasma_ziptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
     plasma_dynamic_sync();
 
     status = sequence->status;
@@ -176,7 +176,7 @@ int PLASMA_zplrnt_Tile( PLASMA_desc *A,
  *  PLASMA_zplrnt_Tile_Async - Generate a random matrix by tiles.
  *  Non-blocking equivalent of PLASMA_zplrnt_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -203,7 +203,7 @@ int PLASMA_zplrnt_Tile_Async( PLASMA_desc     *A,
                               PLASMA_sequence *sequence, 
                               PLASMA_request  *request)
 {
-    PLASMA_desc descA = *A;
+    PLASMA_desc descA;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -226,9 +226,11 @@ int PLASMA_zplrnt_Tile_Async( PLASMA_desc     *A,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zplrnt_Tile", "invalid descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb) {
