@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -134,25 +134,29 @@ int PLASMA_zposv(PLASMA_enum uplo, int N, int NRHS,
     plasma_sequence_create(plasma, &sequence);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   , plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,    sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, sequence, &request,
+                             plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   );
-        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,   
+                            sequence, &request);
+        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
     PLASMA_zposv_Tile_Async(uplo, &descA, &descB, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descA, A, NB, NB, LDA, N    );
-        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_zooptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
         plasma_desc_mat_free(&descB);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N    );
-        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
     }
     
@@ -237,7 +241,7 @@ int PLASMA_zposv_Tile(PLASMA_enum uplo, PLASMA_desc *A, PLASMA_desc *B)
  *  positive definite system of linear equations using the Cholesky factorization.
  *  Non-blocking equivalent of PLASMA_zposv_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -260,8 +264,8 @@ int PLASMA_zposv_Tile(PLASMA_enum uplo, PLASMA_desc *A, PLASMA_desc *B)
 int PLASMA_zposv_Tile_Async(PLASMA_enum uplo, PLASMA_desc *A, PLASMA_desc *B,
                             PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
-    PLASMA_desc descB = *B;
+    PLASMA_desc descA;
+    PLASMA_desc descB;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -284,13 +288,17 @@ int PLASMA_zposv_Tile_Async(PLASMA_enum uplo, PLASMA_desc *A, PLASMA_desc *B,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zposv_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zposv_Tile", "invalid second descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descB = *B;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb || descB.nb != descB.mb) {
