@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -112,20 +112,22 @@ int PLASMA_zpotrf(PLASMA_enum uplo, int N,
     plasma_sequence_create(plasma, &sequence);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N, plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N, sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
     } else {
-        plasma_ziplap2tile(  descA, A, NB, NB, LDA, N, 0, 0, N, N);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
     PLASMA_zpotrf_Tile_Async(uplo, &descA, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descA, A, NB, NB, LDA, N );
+        plasma_zooptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
         plasma_dynamic_sync();
     }
 
@@ -208,7 +210,7 @@ int PLASMA_zpotrf_Tile(PLASMA_enum uplo, PLASMA_desc *A)
  *  positive definite or Hermitian positive definite matrix.
  *  Non-blocking equivalent of PLASMA_zpotrf_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -232,20 +234,20 @@ int PLASMA_zpotrf_Tile(PLASMA_enum uplo, PLASMA_desc *A)
 int PLASMA_zpotrf_Tile_Async(PLASMA_enum uplo, PLASMA_desc *A,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
+    PLASMA_desc descA;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
-        plasma_fatal_error("PLASMA_zpotrf_Tile", "PLASMA not initialized");
+        plasma_fatal_error("PLASMA_zpotrf_Tile_Async", "PLASMA not initialized");
         return PLASMA_ERR_NOT_INITIALIZED;
     }
     if (sequence == NULL) {
-        plasma_fatal_error("PLASMA_zpotrf_Tile", "NULL sequence");
+        plasma_fatal_error("PLASMA_zpotrf_Tile_Async", "NULL sequence");
         return PLASMA_ERR_UNALLOCATED;
     }
     if (request == NULL) {
-        plasma_fatal_error("PLASMA_zpotrf_Tile", "NULL request");
+        plasma_fatal_error("PLASMA_zpotrf_Tile_Async", "NULL request");
         return PLASMA_ERR_UNALLOCATED;
     }
     /* Check sequence status */
@@ -255,17 +257,19 @@ int PLASMA_zpotrf_Tile_Async(PLASMA_enum uplo, PLASMA_desc *A,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
-        plasma_error("PLASMA_zpotrf_Tile", "invalid descriptor");
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
+        plasma_error("PLASMA_zpotrf_Tile_Async", "invalid descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb) {
-        plasma_error("PLASMA_zpotrf_Tile", "only square tiles supported");
+        plasma_error("PLASMA_zpotrf_Tile_Async", "only square tiles supported");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
     }
     if (uplo != PlasmaUpper && uplo != PlasmaLower) {
-        plasma_error("PLASMA_zpotrf_Tile", "illegal value of uplo");
+        plasma_error("PLASMA_zpotrf_Tile_Async", "illegal value of uplo");
         return plasma_request_fail(sequence, request, -1);
     }
     /* Quick return */
