@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -103,6 +103,13 @@ int PLASMA_ztrsm(PLASMA_enum side, PLASMA_enum uplo,
         plasma_fatal_error("PLASMA_ztrsm", "PLASMA not initialized");
         return PLASMA_ERR_NOT_INITIALIZED;
     }
+
+    if (side == PlasmaLeft) {
+      NA = N;
+    } else {
+      NA = NRHS;
+    }
+
     /* Check input arguments */
     if (side != PlasmaLeft && side != PlasmaRight) {
         plasma_error("PLASMA_ztrsm", "illegal value of side");
@@ -128,7 +135,7 @@ int PLASMA_ztrsm(PLASMA_enum side, PLASMA_enum uplo,
         plasma_error("PLASMA_ztrsm", "illegal value of NRHS");
         return -6;
     }
-    if (LDA < max(1, N)) {
+    if (LDA < max(1, NA)) {
         plasma_error("PLASMA_ztrsm", "illegal value of LDA");
         return -8;
     }
@@ -149,20 +156,19 @@ int PLASMA_ztrsm(PLASMA_enum side, PLASMA_enum uplo,
 
     /* Set NT & NTRHS */
     NB = PLASMA_NB;
-    if (side == PlasmaLeft) {
-      NA = N;
-    } else {
-      NA = NRHS;
-    }
 
     plasma_sequence_create(plasma, &sequence);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, NA,   0, 0, NA, NA,   plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N,  NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, NA,   0, 0, NA, NA,   sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N,  NRHS, sequence, &request,
+                             plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, NA,   0, 0, NA, NA  );
-        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N,  NRHS);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, NA,   0, 0, NA, NA,  
+                            sequence, &request);
+        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N,  NRHS,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
@@ -170,13 +176,13 @@ int PLASMA_ztrsm(PLASMA_enum side, PLASMA_enum uplo,
         side, uplo, transA, diag, alpha, &descA, &descB, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
         plasma_desc_mat_free(&descB);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, NA   );
-        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, NA,    sequence, &request);
+        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
     }
 
@@ -275,7 +281,7 @@ int PLASMA_ztrsm_Tile(PLASMA_enum side, PLASMA_enum uplo,
  *  PLASMA_ztrsm_Tile_Async - Computes triangular solve.
  *  Non-blocking equivalent of PLASMA_ztrsm_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -300,8 +306,8 @@ int PLASMA_ztrsm_Tile_Async(PLASMA_enum side, PLASMA_enum uplo,
                             PLASMA_Complex64_t alpha, PLASMA_desc *A, PLASMA_desc *B,
                             PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
-    PLASMA_desc descB = *B;
+    PLASMA_desc descA;
+    PLASMA_desc descB;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -324,13 +330,17 @@ int PLASMA_ztrsm_Tile_Async(PLASMA_enum side, PLASMA_enum uplo,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_ztrsm_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_ztrsm_Tile", "invalid second descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descB = *B;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb || descB.nb != descB.mb) {
