@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -35,7 +35,7 @@
  * @param[in] LDA
  *          The leading dimension of the array A. LDA >= max(1,N).
  *
- * @param[in] L
+ * @param[in] descL
  *          Auxiliary factorization data, related to the tile L factor, computed by PLASMA_zgetrf_incpiv.
  *
  * @param[in] IPIV
@@ -66,15 +66,15 @@
  ******************************************************************************/
 int PLASMA_ztrsmpl(int N, int NRHS,
                    PLASMA_Complex64_t *A, int LDA,
-                   PLASMA_Complex64_t *L, int *IPIV,
+                   PLASMA_desc *descL, const int *IPIV,
                    PLASMA_Complex64_t *B, int LDB)
 {
-    int NB, IB, IBNB, NT;
+    int NB;
     int status;
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
     PLASMA_request request = PLASMA_REQUEST_INITIALIZER;
-    PLASMA_desc descA, descB, descL;
+    PLASMA_desc descA, descB;
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
@@ -111,37 +111,32 @@ int PLASMA_ztrsmpl(int N, int NRHS,
 
     /* Set Mt, NT & NTRHS */
     NB    = PLASMA_NB;
-    IB    = PLASMA_IB;
-    IBNB  = IB*NB;
-    NT    = (N%NB==0) ? (N/NB) : (N/NB+1);
 
     plasma_sequence_create(plasma, &sequence);
 
-    descL = plasma_desc_init(
-        PlasmaComplexDouble,
-        IB, NB, IBNB,
-        NT*IB, NT*NB, 0, 0, NT*IB, NT*NB);
-    descL.mat = L;
-
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   , plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,    sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS, sequence, &request,
+                             plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descB)));
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N   );
-        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N,    0, 0, N, N,   
+                            sequence, &request);
+        plasma_ziplap2tile( descB, B, NB, NB, LDB, NRHS, 0, 0, N, NRHS,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
-    PLASMA_ztrsmpl_Tile_Async(&descA, &descL, IPIV, &descB, sequence, &request);
+    PLASMA_ztrsmpl_Tile_Async(&descA, descL, IPIV, &descB, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_zooptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
         plasma_desc_mat_free(&descB);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N    );
-        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,     sequence, &request);
+        plasma_ziptile2lap( descB, B, NB, NB, LDB, NRHS,  sequence, &request);
         plasma_dynamic_sync();
     }
 
@@ -188,7 +183,7 @@ int PLASMA_ztrsmpl(int N, int NRHS,
  * @sa PLASMA_zgetrf_incpiv_Tile
  *
  ******************************************************************************/
-int PLASMA_ztrsmpl_Tile(PLASMA_desc *A, PLASMA_desc *L, int *IPIV, PLASMA_desc *B)
+int PLASMA_ztrsmpl_Tile(PLASMA_desc *A, PLASMA_desc *L, const int *IPIV, PLASMA_desc *B)
 {
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
@@ -237,12 +232,12 @@ int PLASMA_ztrsmpl_Tile(PLASMA_desc *A, PLASMA_desc *L, int *IPIV, PLASMA_desc *
  * @sa PLASMA_zgetrf_incpiv_Tile_Async
  *
  ******************************************************************************/
-int PLASMA_ztrsmpl_Tile_Async(PLASMA_desc *A, PLASMA_desc *L, int *IPIV, PLASMA_desc *B,
+int PLASMA_ztrsmpl_Tile_Async(PLASMA_desc *A, PLASMA_desc *L, const int *IPIV, PLASMA_desc *B,
                               PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
-    PLASMA_desc descL = *L;
-    PLASMA_desc descB = *B;
+    PLASMA_desc descA;
+    PLASMA_desc descL;
+    PLASMA_desc descB;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -265,17 +260,23 @@ int PLASMA_ztrsmpl_Tile_Async(PLASMA_desc *A, PLASMA_desc *L, int *IPIV, PLASMA_
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_ztrsmpl_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descL) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(L) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_ztrsmpl_Tile", "invalid second descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descL = *L;
     }
-    if (plasma_desc_check(&descB) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(B) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_ztrsmpl_Tile", "invalid third descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descB = *B;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb || descB.nb != descB.mb) {
@@ -291,7 +292,7 @@ int PLASMA_ztrsmpl_Tile_Async(PLASMA_desc *A, PLASMA_desc *L, int *IPIV, PLASMA_
         PLASMA_desc, descA,
         PLASMA_desc, descB,
         PLASMA_desc, descL,
-        int*, IPIV,
+        const int*, IPIV,
         PLASMA_sequence*, sequence,
         PLASMA_request*, request);
 
