@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions normal z -> s d c
@@ -116,20 +116,22 @@ int PLASMA_ztrtri(PLASMA_enum uplo, PLASMA_enum diag, int N,
     plasma_sequence_create(plasma, &sequence);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N, plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, N, N, sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
     } else {
-        plasma_ziplap2tile(  descA, A, NB, NB, LDA, N, 0, 0, N, N);
+        plasma_ziplap2tile(  descA, A, NB, NB, LDA, N, 0, 0, N, N,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
     PLASMA_ztrtri_Tile_Async(uplo, diag, &descA, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descA, A, NB, NB, LDA, N );
+        plasma_zooptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
         plasma_dynamic_sync();
     }
 
@@ -216,7 +218,7 @@ int PLASMA_ztrtri_Tile(PLASMA_enum uplo, PLASMA_enum diag, PLASMA_desc *A)
  *  triangular matrix A.
  *  Non-blocking equivalent of PLASMA_ztrtri_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -240,7 +242,7 @@ int PLASMA_ztrtri_Tile(PLASMA_enum uplo, PLASMA_enum diag, PLASMA_desc *A)
 int PLASMA_ztrtri_Tile_Async(PLASMA_enum uplo, PLASMA_enum diag, PLASMA_desc *A,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
+    PLASMA_desc descA;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -263,9 +265,11 @@ int PLASMA_ztrtri_Tile_Async(PLASMA_enum uplo, PLASMA_enum diag, PLASMA_desc *A,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_ztrtri_Tile", "invalid descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb) {
