@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Jakub Kurzak
  * @date 2010-11-15
@@ -40,7 +40,7 @@
  * @param[in] LDA
  *          The leading dimension of the array A. LDA >= max(1,M).
  *
- * @param[in] T
+ * @param[in] descT
  *          Auxiliary factorization data, computed by PLASMA_zgelqf.
  *
  * @param[out] Q
@@ -67,21 +67,22 @@
  ******************************************************************************/
 int PLASMA_zunglq(int M, int N, int K,
                   PLASMA_Complex64_t *A, int LDA,
-                  PLASMA_Complex64_t *T,
+                  PLASMA_desc *descT,
                   PLASMA_Complex64_t *Q, int LDQ)
 {
-    int NB, IB, IBNB, KT, NT;
+    int NB;
     int status;
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
     PLASMA_request request = PLASMA_REQUEST_INITIALIZER;
-    PLASMA_desc descA, descQ, descT;
+    PLASMA_desc descA, descQ;
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
         plasma_fatal_error("PLASMA_zunglq", "PLASMA not initialized");
         return PLASMA_ERR_NOT_INITIALIZED;
     }
+    /* Check input arguments */
     if (M < 0) {
         plasma_error("PLASMA_zunglq", "illegal value of M");
         return -1;
@@ -114,49 +115,34 @@ int PLASMA_zunglq(int M, int N, int K,
         return status;
     }
 
-    /* Set MT & NT */
-    NB   = PLASMA_NB;
-    IB   = PLASMA_IB;
-    IBNB = IB*NB;
-    NT   = (N%NB==0) ? (N/NB) : (N/NB+1);
-    KT   = (K%NB==0) ? (K/NB) : (K/NB+1);
+    /* Set NT */
+    NB = PLASMA_NB;
 
     plasma_sequence_create(plasma, &sequence);
 
-    if (plasma->householder == PLASMA_FLAT_HOUSEHOLDER) {
-        descT = plasma_desc_init(
-            PlasmaComplexDouble,
-            IB, NB, IBNB,
-            KT*IB, NT*NB, 0, 0, KT*IB, NT*NB);
-    }
-    else {
-        /* Double the size of T to accomodate the tree reduction phase */
-        descT = plasma_desc_init(
-            PlasmaComplexDouble,
-            IB, NB, IBNB,
-            KT*IB, 2*NT*NB, 0, 0, KT*IB, 2*NT*NB);
-    }
-    descT.mat = T;
-
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, K, N, plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descQ, Q, NB, NB, LDQ, N, 0, 0, M, N, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descQ)));
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, K, N, sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descQ, Q, NB, NB, LDQ, N, 0, 0, M, N, sequence, &request,
+                             plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descQ)));
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, K, N);
-        plasma_ziplap2tile( descQ, Q, NB, NB, LDQ, N, 0, 0, M, N);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, K, N,
+                            sequence, &request);
+        plasma_ziplap2tile( descQ, Q, NB, NB, LDQ, N, 0, 0, M, N,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
-    PLASMA_zunglq_Tile_Async(&descA, &descT, &descQ, sequence, &request);
+    PLASMA_zunglq_Tile_Async(&descA, descT, &descQ, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descQ, Q, NB, NB, LDQ, N );
+        plasma_zooptile2lap( descQ, Q, NB, NB, LDQ, N,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
         plasma_desc_mat_free(&descQ);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, N );
-        plasma_ziptile2lap( descQ, Q, NB, NB, LDQ, N );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, N,  sequence, &request);
+        plasma_ziptile2lap( descQ, Q, NB, NB, LDQ, N,  sequence, &request);
         plasma_dynamic_sync();
     }
         
@@ -181,7 +167,7 @@ int PLASMA_zunglq(int M, int N, int K,
  * @param[in] T
  *          Auxiliary factorization data, computed by PLASMA_zgelqf.
  *
- * @param[out] B
+ * @param[out] Q
  *          On exit, the M-by-N matrix Q.
  *
  *******************************************************************************
@@ -199,7 +185,7 @@ int PLASMA_zunglq(int M, int N, int K,
  * @sa PLASMA_zgelqf_Tile
  *
  ******************************************************************************/
-int PLASMA_zunglq_Tile(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *B)
+int PLASMA_zunglq_Tile(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *Q)
 {
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
@@ -212,7 +198,7 @@ int PLASMA_zunglq_Tile(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *B)
         return PLASMA_ERR_NOT_INITIALIZED;
     }
     plasma_sequence_create(plasma, &sequence);
-    PLASMA_zunglq_Tile_Async(A, T, B, sequence, &request);
+    PLASMA_zunglq_Tile_Async(A, T, Q, sequence, &request);
     plasma_dynamic_sync();
     status = sequence->status;
     plasma_sequence_destroy(plasma, sequence);
@@ -225,7 +211,7 @@ int PLASMA_zunglq_Tile(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *B)
  *
  *  Non-blocking equivalent of PLASMA_zunglq_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -249,9 +235,9 @@ int PLASMA_zunglq_Tile(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *B)
 int PLASMA_zunglq_Tile_Async(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *Q,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
-    PLASMA_desc descT = *T;
-    PLASMA_desc descQ = *Q;
+    PLASMA_desc descA;
+    PLASMA_desc descT;
+    PLASMA_desc descQ;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -274,17 +260,23 @@ int PLASMA_zunglq_Tile_Async(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *Q,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zunglq_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descT) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(T) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zunglq_Tile", "invalid second descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descT = *T;
     }
-    if (plasma_desc_check(&descQ) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(Q) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zunglq_Tile", "invalid third descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descQ = *Q;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb || descQ.nb != descQ.mb) {
