@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Jakub Kurzak
  * @date 2010-11-15
@@ -40,7 +40,7 @@
  * @param[in] LDA
  *          The leading dimension of the array A. LDA >= max(1,M).
  *
- * @param[in] T
+ * @param[in] descT
  *          Auxiliary factorization data, computed by PLASMA_zgeqrf.
  *
  * @param[out] Q
@@ -67,21 +67,23 @@
  ******************************************************************************/
 int PLASMA_zungqr(int M, int N, int K,
                   PLASMA_Complex64_t *A, int LDA,
-                  PLASMA_Complex64_t *T,
+                  PLASMA_desc *descT,
                   PLASMA_Complex64_t *Q, int LDQ)
 {
-    int NB, IB, IBNB, MT, KT;
+    int NB;
     int status;
     plasma_context_t *plasma;
     PLASMA_sequence *sequence = NULL;
     PLASMA_request request = PLASMA_REQUEST_INITIALIZER;
-    PLASMA_desc descA, descQ, descT;
+    PLASMA_desc descA, descQ;
 
     plasma = plasma_context_self();
     if (plasma == NULL) {
         plasma_fatal_error("PLASMA_zungqr", "PLASMA not initialized");
         return PLASMA_ERR_NOT_INITIALIZED;
     }
+
+    /* Check input arguments */
     if (M < 0) {
         plasma_error("PLASMA_zungqr", "illegal value of M");
         return -1;
@@ -112,49 +114,34 @@ int PLASMA_zungqr(int M, int N, int K,
         return status;
     }
 
-    /* Set MT & KT */
-    NB   = PLASMA_NB;
-    IB   = PLASMA_IB;
-    IBNB = IB*NB;
-    MT   = (M%NB==0) ? (M/NB) : (M/NB+1);
-    KT   = (K%NB==0) ? (K/NB) : (K/NB+1);
+    /* Set NT */
+    NB = PLASMA_NB;
 
     plasma_sequence_create(plasma, &sequence);
 
-    if (plasma->householder == PLASMA_FLAT_HOUSEHOLDER) {
-        descT = plasma_desc_init(
-            PlasmaComplexDouble,
-            IB, NB, IBNB,
-            MT*IB, KT*NB, 0, 0, MT*IB, KT*NB);
-    }
-    else {
-        /* Double the size of T to accomodate the tree reduction phase */
-        descT = plasma_desc_init(
-            PlasmaComplexDouble,
-            IB, NB, IBNB,
-            MT*IB, 2*KT*NB, 0, 0, MT*IB, 2*KT*NB);
-    }
-    descT.mat = T;
-
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, M, K, plasma_desc_mat_free(&(descA)) );
-        plasma_zooplap2tile( descQ, Q, NB, NB, LDQ, N, 0, 0, M, N, plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descQ)));
+        plasma_zooplap2tile( descA, A, NB, NB, LDA, N, 0, 0, M, K, sequence, &request,
+                             plasma_desc_mat_free(&(descA)) );
+        plasma_zooplap2tile( descQ, Q, NB, NB, LDQ, N, 0, 0, M, N, sequence, &request,
+                             plasma_desc_mat_free(&(descA)); plasma_desc_mat_free(&(descQ)));
     } else {
-        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, M, K);
-        plasma_ziplap2tile( descQ, Q, NB, NB, LDQ, N, 0, 0, M, N);
+        plasma_ziplap2tile( descA, A, NB, NB, LDA, N, 0, 0, M, K,
+                            sequence, &request);
+        plasma_ziplap2tile( descQ, Q, NB, NB, LDQ, N, 0, 0, M, N,
+                            sequence, &request);
     }
 
     /* Call the tile interface */
-    PLASMA_zungqr_Tile_Async(&descA, &descT, &descQ, sequence, &request);
+    PLASMA_zungqr_Tile_Async(&descA, descT, &descQ, sequence, &request);
 
     if ( PLASMA_TRANSLATION == PLASMA_OUTOFPLACE ) {
-        plasma_zooptile2lap( descQ, Q, NB, NB, LDQ, N );
+        plasma_zooptile2lap( descQ, Q, NB, NB, LDQ, N,  sequence, &request);
         plasma_dynamic_sync();
         plasma_desc_mat_free(&descA);
         plasma_desc_mat_free(&descQ);
     } else {
-        plasma_ziptile2lap( descA, A, NB, NB, LDA, K );
-        plasma_ziptile2lap( descQ, Q, NB, NB, LDQ, N );
+        plasma_ziptile2lap( descA, A, NB, NB, LDA, K,  sequence, &request);
+        plasma_ziptile2lap( descQ, Q, NB, NB, LDQ, N,  sequence, &request);
         plasma_dynamic_sync();
     }
     
@@ -223,7 +210,7 @@ int PLASMA_zungqr_Tile(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *Q)
  *
  *  Non-blocking equivalent of PLASMA_zungqr_Tile().
  *  May return before the computation is finished.
- *  Allows for pipelining of operations ar runtime.
+ *  Allows for pipelining of operations at runtime.
  *
  *******************************************************************************
  *
@@ -247,9 +234,9 @@ int PLASMA_zungqr_Tile(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *Q)
 int PLASMA_zungqr_Tile_Async(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *Q,
                              PLASMA_sequence *sequence, PLASMA_request *request)
 {
-    PLASMA_desc descA = *A;
-    PLASMA_desc descT = *T;
-    PLASMA_desc descQ = *Q;
+    PLASMA_desc descA;
+    PLASMA_desc descT;
+    PLASMA_desc descQ;
     plasma_context_t *plasma;
 
     plasma = plasma_context_self();
@@ -272,17 +259,23 @@ int PLASMA_zungqr_Tile_Async(PLASMA_desc *A, PLASMA_desc *T, PLASMA_desc *Q,
         return plasma_request_fail(sequence, request, PLASMA_ERR_SEQUENCE_FLUSHED);
 
     /* Check descriptors for correctness */
-    if (plasma_desc_check(&descA) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(A) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zungqr_Tile", "invalid first descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descA = *A;
     }
-    if (plasma_desc_check(&descT) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(T) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zungqr_Tile", "invalid second descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descT = *T;
     }
-    if (plasma_desc_check(&descQ) != PLASMA_SUCCESS) {
+    if (plasma_desc_check(Q) != PLASMA_SUCCESS) {
         plasma_error("PLASMA_zungqr_Tile", "invalid third descriptor");
         return plasma_request_fail(sequence, request, PLASMA_ERR_ILLEGAL_VALUE);
+    } else {
+        descQ = *Q;
     }
     /* Check input arguments */
     if (descA.nb != descA.mb || descQ.nb != descQ.mb) {
