@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  *
@@ -21,20 +21,12 @@ extern "C" {
 /***************************************************************************//**
  *  Internal routines
  **/
-int plasma_request_fail(PLASMA_sequence *sequence, PLASMA_request *request, int error);
-int plasma_sequence_create(plasma_context_t *plasma, PLASMA_sequence **sequence);
-int plasma_sequence_destroy(plasma_context_t *plasma, PLASMA_sequence *sequence);
-int plasma_sequence_wait(plasma_context_t *plasma, PLASMA_sequence *sequence);
+int  plasma_request_fail(PLASMA_sequence *sequence, PLASMA_request *request, int error);
+int  plasma_sequence_create(plasma_context_t *plasma, PLASMA_sequence **sequence);
+int  plasma_sequence_destroy(plasma_context_t *plasma, PLASMA_sequence *sequence);
+int  plasma_sequence_wait(plasma_context_t *plasma, PLASMA_sequence *sequence);
 void plasma_sequence_flush(Quark *quark, PLASMA_sequence *sequence, PLASMA_request *request, int status);
 
-/***************************************************************************//**
- *  User routines
- **/
-int PLASMA_Sequence_Create(PLASMA_sequence **sequence);
-int PLASMA_Sequence_Destroy(PLASMA_sequence *sequence);
-int PLASMA_Sequence_Wait(PLASMA_sequence *sequence);
-int PLASMA_Sequence_Flush(PLASMA_sequence *sequence, PLASMA_request *request);
-
 #ifdef __cplusplus
 }
 #endif
