@@ -1,12 +1,12 @@
 /**
  *
- * @file auxiliary.c
+ * @file control/auxiliary.c
  *
  *  PLASMA auxiliary routines
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @author Piotr Luszczek
  * @author Emmanuel Agullo
@@ -227,6 +227,10 @@ int plasma_tune(PLASMA_enum func, int M, int N, int NRHS)
         case PLASMA_FUNC_CHEEV:
         case PLASMA_FUNC_DSYEV:
         case PLASMA_FUNC_SSYEV:
+        case PLASMA_FUNC_ZHEEVD:
+        case PLASMA_FUNC_CHEEVD:
+        case PLASMA_FUNC_DSYEVD:
+        case PLASMA_FUNC_SSYEVD:
         case PLASMA_FUNC_ZHETRD:
         case PLASMA_FUNC_CHETRD:
         case PLASMA_FUNC_DSYTRD:
@@ -235,6 +239,10 @@ int plasma_tune(PLASMA_enum func, int M, int N, int NRHS)
         case PLASMA_FUNC_CHEGV:
         case PLASMA_FUNC_DSYGV:
         case PLASMA_FUNC_SSYGV:
+        case PLASMA_FUNC_ZHEGVD:
+        case PLASMA_FUNC_CHEGVD:
+        case PLASMA_FUNC_DSYGVD:
+        case PLASMA_FUNC_SSYGVD:
         case PLASMA_FUNC_ZHEGST:
         case PLASMA_FUNC_CHEGST:
         case PLASMA_FUNC_DSYGST:
