@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @author Mathieu Faverge
  * @date 2010-11-15
@@ -20,12 +20,6 @@
 #ifndef _PLASMA_COMMON_H_
 #define _PLASMA_COMMON_H_
 
-#if defined( _WIN32 ) || defined( _WIN64 )
-#include <io.h>
-#else
-#include <unistd.h>
-#endif
-
 #include "global.h"
 #include "context.h"
 #include "control.h"
@@ -36,12 +30,21 @@
 #include "descriptor.h"
 #include "tile.h"
 #include "async.h"
+#include "bulge.h"
+#include "plasma_threadsetting.h"
+
+#if defined( _WIN32 ) || defined( _WIN64 )
+# include <io.h>
+#else  /* NOT WIN32 */
+# include <unistd.h>
+#endif  /* IF WIN32 */
+
 
 /** ****************************************************************************
- *  Determine if weak symbol are allowed 
+ *  Determine if weak symbols are allowed
  */
 #if defined(linux) || defined(__linux) || defined(__linux__)
-#if defined(__GNUC_EXCL__) || defined(__GNUC__) 
+#if defined(__GNUC_EXCL__) || defined(__GNUC__)
 #define PLASMA_HAVE_WEAK 1
 #endif
 #endif
@@ -51,16 +54,22 @@
  */
 #if defined(ADD_)
 #define PLASMA_FNAME(lcname, UCNAME)        plasma_##lcname##_
-#define PLASMA_TILE_FNAME(lcname, UCNAME)   plasma_tile_##lcname##_
-#define PLASMA_ASYNC_FNAME(lcname, UCNAME)  plasma_tile_async_##lcname##_
+#define PLASMA_TILE_FNAME(lcname, UCNAME)   plasma_##lcname##_tile_
+#define PLASMA_ASYNC_FNAME(lcname, UCNAME)  plasma_##lcname##_tile_async_
+#define PLASMA_WS_FNAME(lcname, UCNAME)     plasma_alloc_workspace_##lcname##_
+#define PLASMA_WST_FNAME(lcname, UCNAME)    plasma_alloc_workspace_##lcname##_tile_
 #elif defined(NOCHANGE)
 #define PLASMA_FNAME(lcname, UCNAME)        plasma_##lcname
-#define PLASMA_TILE_FNAME(lcname, UCNAME)   plasma_tile_##lcname##
-#define PLASMA_ASYNC_FNAME(lcname, UCNAME)  plasma_tile_async_##lcname##
+#define PLASMA_TILE_FNAME(lcname, UCNAME)   plasma_##lcname##_tile
+#define PLASMA_ASYNC_FNAME(lcname, UCNAME)  plasma_##lcname##_tile_async
+#define PLASMA_WS_FNAME(lcname, UCNAME)     plasma_alloc_workspace_##lcname
+#define PLASMA_WST_FNAME(lcname, UCNAME)    plasma_alloc_workspace_##lcname##_tile
 #elif defined(UPCASE)
 #define PLASMA_FNAME(lcname, UCNAME)        PLASMA_##UCNAME
-#define PLASMA_TILE_FNAME(lcname, UCNAME)   PLASMA_TILE_##UCNAME##
-#define PLASMA_ASYNC_FNAME(lcname, UCNAME)  PLASMA_TILE_ASYNC_##UCNAME##
+#define PLASMA_TILE_FNAME(lcname, UCNAME)   PLASMA_##UCNAME##_TILE
+#define PLASMA_ASYNC_FNAME(lcname, UCNAME)  PLASMA_##UCNAME##_TILE_ASYNC
+#define PLASMA_WS_FNAME(lcname, UCNAME)     PLASMA_ALLOC_WORKSPACE_##UCNAME
+#define PLASMA_WST_FNAME(lcname, UCNAME)    PLASMA_ALLOC_WORKSPACE_##UCNAME##_TILE
 #endif
 
 /***************************************************************************//**
@@ -76,6 +85,8 @@
 #define PLASMA_SCHEDULING  plasma->scheduling
 #define PLASMA_RHBLK       plasma->rhblock
 #define PLASMA_TRANSLATION plasma->translation
+#define PLASMA_TNT_MODE    plasma->tournament
+#define PLASMA_TNT_SIZE    plasma->tntsize
 
 /***************************************************************************//**
  *  IPT internal define
