@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  *
@@ -49,22 +49,24 @@ plasma_context_t *plasma_context_create()
     plasma->action = PLASMA_ACT_STAND_BY;
     plasma->parallel_func_ptr = NULL;
 
-    /* These initializations are just in case the user
-       disables autotuning and does not set nb and ib */
-    plasma->nb = 128;
-    plasma->ib = 32;
-    plasma->nbnbsize = 16384;
-    plasma->ibnbsize = 4096;
-    plasma->rhblock = 4;
+    plasma->scheduling  = PLASMA_STATIC_SCHEDULING;
+    plasma->householder = PLASMA_FLAT_HOUSEHOLDER;
+    plasma->translation = PLASMA_OUTOFPLACE;
+    plasma->tournament  = PLASMA_TOURNAMENT_LU;
 
     plasma->errors_enabled     = PLASMA_TRUE;
     plasma->warnings_enabled   = PLASMA_TRUE;
     plasma->autotuning_enabled = PLASMA_TRUE;
     plasma->dynamic_section    = PLASMA_FALSE;
 
-    plasma->scheduling  = PLASMA_STATIC_SCHEDULING;
-    plasma->householder = PLASMA_FLAT_HOUSEHOLDER;
-    plasma->translation = PLASMA_OUTOFPLACE;
+    /* These initializations are just in case the user
+       disables autotuning and does not set nb and ib */
+    plasma->nb = 160;
+    plasma->ib = 40;
+    plasma->nbnbsize = 25600;
+    plasma->ibnbsize = 6400;
+    plasma->rhblock = 4;
+    plasma->tntsize = 4;
 
     return plasma;
 }
@@ -157,7 +159,8 @@ plasma_context_t *plasma_context_self()
  *          Feature to be enabled:
  *          @arg PLASMA_WARNINGS   printing of warning messages,
  *          @arg PLASMA_ERRORS     printing of error messages,
- *          @arg PLASMA_AUTOTUNING autotuning for tile size and inner block size.
+ *          @arg PLASMA_AUTOTUNING autotuning for tile size and inner block size,
+ *          @arg PLASMA_DAG        Quark automatic DAG generation.
  *
  *******************************************************************************
  *
@@ -186,6 +189,10 @@ int PLASMA_Enable(PLASMA_enum lever)
         case PLASMA_AUTOTUNING:
             plasma->autotuning_enabled = PLASMA_TRUE;
             break;
+        case PLASMA_DAG:
+            QUARK_Barrier(plasma->quark);
+            QUARK_DOT_DAG_Enable( plasma->quark, 1);
+            break;
         default:
             plasma_error("PLASMA_Enable", "illegal parameter value");
             return PLASMA_ERR_ILLEGAL_VALUE;
@@ -205,7 +212,8 @@ int PLASMA_Enable(PLASMA_enum lever)
  *          Feature to be disabled:
  *          @arg PLASMA_WARNINGS   printing of warning messages,
  *          @arg PLASMA_ERRORS     printing of error messages,
- *          @arg PLASMA_AUTOTUNING autotuning for tile size and inner block size.
+ *          @arg PLASMA_AUTOTUNING autotuning for tile size and inner block size,
+ *          @arg PLASMA_DAG        Quark automatic DAG generation.
  *
  *******************************************************************************
  *
@@ -233,6 +241,10 @@ int PLASMA_Disable(PLASMA_enum lever)
         case PLASMA_AUTOTUNING:
             plasma->autotuning_enabled = PLASMA_FALSE;
             break;
+        case PLASMA_DAG:
+            QUARK_Barrier(plasma->quark);
+            QUARK_DOT_DAG_Enable( plasma->quark, 0);
+            break;
         default:
             plasma_error("PLASMA_Disable", "illegal parameter value");
             return PLASMA_ERR_ILLEGAL_VALUE;
@@ -255,6 +267,12 @@ int PLASMA_Disable(PLASMA_enum lever)
  *          @arg PLASMA_SCHEDULING_MODE:  type of scheduling
  *                  (PLASMA_STATIC_SCHEDULING or PLASMA_DYNAMIC_SCHEDULING).
  *          @arg PLASMA_INNER_BANDWIDTH:  matrix bandwidth size
+ *          @arg PLASMA_HOUSEHOLDER_MODE:  type of householder trees
+ *                  (PLASMA_FLAT_HOUSEHOLDER or PLASMA_TREE_HOUSEHOLDER).
+ *          @arg PLASMA_HOUSEHOLDER_SIZE: size of the groups in householder trees
+ *          @arg PLASMA_TNTPIVOTING_MODE:  type of factorization used in tournament pivoting
+ *                  (PLASMA_TOURNAMENT_LU or PLASMA_TOURNAMENT_QR).
+ *          @arg PLASMA_TNTPIVOTING_SIZE: size of the groups in tournament pivoting
  *
  * @param[in] value
  *          Value of the parameter.
@@ -347,6 +365,21 @@ int PLASMA_Set(PLASMA_enum param, int value)
             }
             plasma->translation = value;
             break;
+        case PLASMA_TNTPIVOTING_MODE:
+            if ((value != PLASMA_TOURNAMENT_LU) && 
+                (value != PLASMA_TOURNAMENT_QR) ) {
+                plasma_error("PLASMA_Set", "illegal value of PLASMA_TNTPIVOTING_MODE");
+                return PLASMA_ERR_ILLEGAL_VALUE;
+            }
+            plasma->tournament = value;
+            break;
+        case PLASMA_TNTPIVOTING_SIZE:
+            if (value <= 0) {
+                plasma_error("PLASMA_Set", "negative tournament pivoting size");
+                return PLASMA_ERR_ILLEGAL_VALUE;
+            }
+            plasma->tntsize = value;
+            break;
         default:
             plasma_error("PLASMA_Set", "unknown parameter");
             return PLASMA_ERR_ILLEGAL_VALUE;
@@ -368,6 +401,12 @@ int PLASMA_Set(PLASMA_enum param, int value)
  *          @arg PLASMA_INNER_BLOCK_SIZE: size of tile inner block,
  *          @arg PLASMA_SCHEDULING:       type of scheduling
  *                  (PLASMA_STATIC_SCHEDULING or PLASMA_DYNAMIC_SCHEDULING).
+ *          @arg PLASMA_HOUSEHOLDER_MODE:  type of householder trees
+ *                  (PLASMA_FLAT_HOUSEHOLDER or PLASMA_TREE_HOUSEHOLDER).
+ *          @arg PLASMA_HOUSEHOLDER_SIZE: size of the groups in householder trees
+ *          @arg PLASMA_TNTPIVOTING_MODE:  type of factorization used in tournament pivoting
+ *                  (PLASMA_TOURNAMENT_LU or PLASMA_TOURNAMENT_QR).
+ *          @arg PLASMA_TNTPIVOTING_SIZE: size of the groups in tournament pivoting
  *
  * @param[out] value
  *          Value of the parameter.
@@ -403,6 +442,12 @@ int PLASMA_Get(PLASMA_enum param, int *value)
         case PLASMA_HOUSEHOLDER_SIZE:
             *value = plasma->rhblock;
             return PLASMA_SUCCESS;
+        case PLASMA_TNTPIVOTING_MODE:
+            *value = plasma->tournament;
+            return PLASMA_SUCCESS;
+        case PLASMA_TNTPIVOTING_SIZE:
+            *value = plasma->tntsize;
+            return PLASMA_SUCCESS;
         case PLASMA_TRANSLATION_MODE:
             *value = plasma->translation;
             return PLASMA_SUCCESS;
