@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  *
@@ -50,6 +50,7 @@ typedef struct plasma_context_struct {
     PLASMA_enum scheduling;     // static or dynamic scheduling
     PLASMA_enum householder;    // "domino" (flat) or tree-based (reduction) Householder
     PLASMA_enum translation;    // In place or Out of place layout conversion
+    PLASMA_enum tournament;     // LU or QR rank-revealing
 
     /* Matrix tile attributes */
     int nb;
@@ -57,17 +58,18 @@ typedef struct plasma_context_struct {
     int nbnbsize;   // tile size in elements (possibly padded)
     int ibnbsize;   // T or L tile size in elements (---''---)
     int rhblock;    // block size for tree-based (reduction) Householder
+    int tntsize;    // Tournament pivoting grouping size
 
     /* Barrier implementation */
-#ifdef BUSY_WAITING
+    /* Busy waiting version */
     volatile int barrier_in[CONTEXT_THREADS_MAX];
     volatile int barrier_out[CONTEXT_THREADS_MAX];
-#else
+
+    /* Conditional version */
     int volatile    barrier_id;             /*+ ID of the barrier                     +*/
     int volatile    barrier_nblocked_thrds; /*+ Number of threads lock in the barrier +*/
     pthread_mutex_t barrier_synclock;       /*+ mutex for the barrier                 +*/
     pthread_cond_t  barrier_synccond;       /*+ condition for the barrier             +*/
-#endif
 
     /* Static scheduler implementation */
     int ss_ld;                  // static scheduler progress table leading dimension
