@@ -6,13 +6,16 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  *
  **/
 #include <stdio.h>
 #include <stdlib.h>
+#if defined(PLASMA_WITH_MKL)
+#include <omp.h>
+#endif
 #if defined( _WIN32 ) || defined( _WIN64 )
 #include "plasmawinthread.h"
 #else
@@ -27,19 +30,10 @@
  **/
 void plasma_barrier_init(plasma_context_t *plasma)
 {
-#ifdef BUSY_WAITING
-    int core;
-
-    for (core = 0; core < CONTEXT_THREADS_MAX; core++) {
-        plasma->barrier_in[core] = 0;
-        plasma->barrier_out[core] = 0;
-    }
-#else
     plasma->barrier_id = 0;
     plasma->barrier_nblocked_thrds = 0;
     pthread_mutex_init(&(plasma->barrier_synclock), NULL);
     pthread_cond_init( &(plasma->barrier_synccond), NULL);
-#endif
 }
 
 /***************************************************************************//**
@@ -47,10 +41,8 @@ void plasma_barrier_init(plasma_context_t *plasma)
  **/
 void plasma_barrier_finalize(plasma_context_t *plasma)
 {
-#ifndef BUSY_WAITING
     pthread_mutex_destroy(&(plasma->barrier_synclock));
     pthread_cond_destroy( &(plasma->barrier_synccond));
-#endif
 }
 
 /***************************************************************************//**
@@ -58,26 +50,6 @@ void plasma_barrier_finalize(plasma_context_t *plasma)
  **/
 void plasma_barrier(plasma_context_t *plasma)
 {
-#ifdef BUSY_WAITING
-    int core;
-
-    if (PLASMA_RANK == 0) {
-        for (core = 1; core < PLASMA_SIZE; core++)
-            while (plasma->barrier_in[core] == 0);
-
-        for (core = 1; core < PLASMA_SIZE; core++)
-            plasma->barrier_in[core] = 0;
-
-        for (core = 1; core < PLASMA_SIZE; core++)
-            plasma->barrier_out[core] = 1;
-    }
-    else
-    {
-        plasma->barrier_in[PLASMA_RANK] = 1;
-        while (plasma->barrier_out[PLASMA_RANK] == 0);
-        plasma->barrier_out[PLASMA_RANK] = 0;
-    }
-#else
     int id;
 
     pthread_mutex_lock(&(plasma->barrier_synclock));
@@ -91,7 +63,52 @@ void plasma_barrier(plasma_context_t *plasma)
     while (id == plasma->barrier_id)
         pthread_cond_wait(&(plasma->barrier_synccond), &(plasma->barrier_synclock));
     pthread_mutex_unlock(&(plasma->barrier_synclock));
-#endif
+}
+
+/***************************************************************************//**
+ *  Busy-waiting barrier initialization
+ **/
+void plasma_barrier_bw_init(plasma_context_t *plasma)
+{
+    int core;
+
+    for (core = 0; core < CONTEXT_THREADS_MAX; core++) {
+        plasma->barrier_in[core] = 0;
+        plasma->barrier_out[core] = 0;
+    }
+}
+
+/***************************************************************************//**
+ *  Busy-waiting barrier finalize
+ **/
+void plasma_barrier_bw_finalize(plasma_context_t *plasma)
+{
+}
+
+/***************************************************************************//**
+ *  Busy-waiting barrier
+ **/
+void plasma_barrier_bw(plasma_context_t *plasma)
+{
+    int core;
+    int size = PLASMA_SIZE;
+
+    if (PLASMA_RANK == 0) {
+        for (core = 1; core < size; core++)
+            while (plasma->barrier_in[core] == 0);
+
+        for (core = 1; core < size; core++)
+            plasma->barrier_in[core] = 0;
+
+        for (core = 1; core < size; core++)
+            plasma->barrier_out[core] = 1;
+    }
+    else
+    {
+        plasma->barrier_in[PLASMA_RANK] = 1;
+        while (plasma->barrier_out[PLASMA_RANK] == 0);
+        plasma->barrier_out[PLASMA_RANK] = 0;
+    }
 }
 
 /***************************************************************************//**
@@ -128,6 +145,8 @@ void *plasma_parallel_section(void *plasma_ptr)
         }
         plasma_barrier(plasma);
     }
+
+    plasma_unsetaffinity();
     return NULL;
 }
 
@@ -224,11 +243,12 @@ int PLASMA_Init_Affinity(int cores, int *coresbind)
 
     /* Get the size of each NUMA node */
     plasma->group_size = plasma_get_numthreads_numa();
-    while ( ((plasma->world_size)%(plasma->group_size)) != 0 ) 
+    while ( ((plasma->world_size)%(plasma->group_size)) != 0 )
         (plasma->group_size)--;
 
-    /* Initialize barrier */
+    /* Initialize barriers */
     plasma_barrier_init(plasma);
+    plasma_barrier_bw_init(plasma);
 
     /* Initialize default thread attributes */
     status = pthread_attr_init(&plasma->thread_attr);
@@ -268,8 +288,14 @@ int PLASMA_Init_Affinity(int cores, int *coresbind)
              plasma_parallel_section,
              (void*)plasma);
     }
-    /* Set thread affinity for the master */
-    plasma_setaffinity(plasma->thread_bind[0]);
+
+    /* Ensure BLAS are sequential and set thread affinity for the master */
+#if defined(PLASMA_WITH_MKL)
+#if defined(__ICC) || defined(__INTEL_COMPILER)
+    kmp_set_defaults("KMP_AFFINITY=disabled");
+#endif
+#endif
+    plasma_setlapack_sequential(plasma->thread_bind[0]);
 
     /* Initialize the dynamic scheduler */
     plasma->quark =  QUARK_Setup(plasma->world_size);
@@ -328,6 +354,10 @@ int PLASMA_Finalize()
         }
     }
     plasma_barrier_finalize(plasma);
+    plasma_barrier_bw_finalize(plasma);
+
+    /* Unbind main thread */
+    plasma_unsetaffinity();
 
     /* Destroy thread attributes */
     status = pthread_attr_destroy(&plasma->thread_attr);
@@ -342,5 +372,11 @@ int PLASMA_Finalize()
         plasma_fatal_error("PLASMA_Finalize", "plasma_context_remove() failed");
         return status;
     }
+
+    /* Restore the concurency */
+    /* actually it's really bad, we shoulde set the concurrency only
+     * if it's not already done and restore it only we had change it */
+    pthread_setconcurrency( 0 );
+
     return PLASMA_SUCCESS;
 }
