@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  *
@@ -24,15 +24,18 @@
 
 #define DAG_CORE_ASUM   DAG_SET_PROPERTIES( "ASUM"  , "white"   )
 #define DAG_CORE_AXPY   DAG_SET_PROPERTIES( "AXPY"  , "white"   )
+#define DAG_CORE_GEADD  DAG_SET_PROPERTIES( "GEADD" , "white"   )
 #define DAG_CORE_GELQT  DAG_SET_PROPERTIES( "GELQT" , "green"   )
 #define DAG_CORE_GEMM   DAG_SET_PROPERTIES( "GEMM"  , "yellow"  )
+#define DAG_CORE_GEMV   DAG_SET_PROPERTIES( "GEMV"  , "blue"    )
 #define DAG_CORE_GEQRT  DAG_SET_PROPERTIES( "GEQRT" , "green"   )
 #define DAG_CORE_GESSM  DAG_SET_PROPERTIES( "GESSM" , "cyan"    )
 #define DAG_CORE_GETRF  DAG_SET_PROPERTIES( "GETRF" , "green"   )
 #define DAG_CORE_GETRIP DAG_SET_PROPERTIES( "GETRIP", "white"   )
-#define DAG_CORE_GETRO  DAG_SET_PROPERTIES( "GETRO" , "white"   )
+#define DAG_CORE_LATRO  DAG_SET_PROPERTIES( "LATRO" , "white"   )
 #define DAG_CORE_HEMM   DAG_SET_PROPERTIES( "HEMM"  , "white"   )
 #define DAG_CORE_HER2K  DAG_SET_PROPERTIES( "HER2K" , "white"   )
+#define DAG_CORE_HERFB  DAG_SET_PROPERTIES( "HERFB" , "pink"    )
 #define DAG_CORE_HERK   DAG_SET_PROPERTIES( "HERK"  , "yellow"  )
 #define DAG_CORE_LACPY  DAG_SET_PROPERTIES( "LACPY" , "white"   )
 #define DAG_CORE_LAG2C  DAG_SET_PROPERTIES( "LAG2C" , "white"   )
@@ -40,11 +43,13 @@
 #define DAG_CORE_LANGE  DAG_SET_PROPERTIES( "LANGE" , "white"   )
 #define DAG_CORE_LANHE  DAG_SET_PROPERTIES( "LANHE" , "white"   )
 #define DAG_CORE_LANSY  DAG_SET_PROPERTIES( "LANSY" , "white"   )
+#define DAG_CORE_LASET  DAG_SET_PROPERTIES( "LASET" , "orange"  )
 #define DAG_CORE_LASWP  DAG_SET_PROPERTIES( "LASWP" , "orange"  )
 #define DAG_CORE_LAUUM  DAG_SET_PROPERTIES( "LAUUM" , "white"   )
 #define DAG_CORE_PLGHE  DAG_SET_PROPERTIES( "PLGHE" , "white"   )
 #define DAG_CORE_PLGSY  DAG_SET_PROPERTIES( "PLGSY" , "white"   )
 #define DAG_CORE_PLRNT  DAG_SET_PROPERTIES( "PLRNT" , "white"   )
+#define DAG_CORE_LASSQ  DAG_SET_PROPERTIES( "LASSQ" , "white"   )
 #define DAG_CORE_POTRF  DAG_SET_PROPERTIES( "POTRF" , "green"   )
 #define DAG_CORE_SHIFT  DAG_SET_PROPERTIES( "SHIFT" , "white"   )
 #define DAG_CORE_SHIFTW DAG_SET_PROPERTIES( "SHIFTW", "white"   )
