@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  *
@@ -14,8 +14,9 @@
 #include "common.h"
 #include "auxiliary.h"
 #include "descriptor.h"
-
 #include <stdlib.h>
+#include <assert.h>
+
 /***************************************************************************//**
  *  Internal static descriptor initializer
  **/
@@ -48,8 +49,8 @@ PLASMA_desc plasma_desc_init(PLASMA_enum dtyp, int mb, int nb, int bsiz,
     desc.m = m;
     desc.n = n;
     // Submatrix derived parameters
-    desc.mt = (i+m-1)/mb - i/mb + 1;
-    desc.nt = (j+n-1)/nb - j/nb + 1;
+    desc.mt = (m == 0) ? 0 : (i+m-1)/mb - i/mb + 1;
+    desc.nt = (n == 0) ? 0 : (j+n-1)/nb - j/nb + 1;
 
     return desc;
 }
@@ -62,17 +63,26 @@ PLASMA_desc plasma_desc_submatrix(PLASMA_desc descA, int i, int j, int m, int n)
     PLASMA_desc descB;
     int mb, nb;
 
+    if ( (descA.i + i + m) > descA.lm ) {
+        plasma_error("plasma_desc_submatrix", "The number of rows (i+m) of the submatrix doesn't fit in the parent matrix");
+        assert((descA.i + i + m) > descA.lm);
+    }
+    if ( (descA.j + j + n) > descA.ln ) {
+        plasma_error("plasma_desc_submatrix", "The number of rows (j+n) of the submatrix doesn't fit in the parent matrix");
+        assert((descA.j + j + n) > descA.ln);
+    }
+
     descB = descA;
     mb = descA.mb;
     nb = descA.nb;
     // Submatrix parameters
-    descB.i = i;
-    descB.j = j;
+    descB.i = descA.i + i;
+    descB.j = descA.j + j;
     descB.m = m;
     descB.n = n;
     // Submatrix derived parameters
-    descB.mt = (i+m-1)/mb - i/mb + 1;
-    descB.nt = (j+n-1)/nb - j/nb + 1;
+    descB.mt = (m == 0) ? 0 : (descB.i+m-1)/mb - descB.i/mb + 1;
+    descB.nt = (n == 0) ? 0 : (descB.j+n-1)/nb - descB.j/nb + 1;
     return descB;
 }
 
@@ -81,6 +91,10 @@ PLASMA_desc plasma_desc_submatrix(PLASMA_desc descA, int i, int j, int m, int n)
  **/
 int plasma_desc_check(PLASMA_desc *desc)
 {
+    if (desc == NULL) {
+        plasma_error("plasma_desc_check", "NULL descriptor");
+        return PLASMA_ERR_NOT_INITIALIZED;
+    }
     if (desc->mat == NULL) {
         plasma_error("plasma_desc_check", "NULL matrix pointer");
         return PLASMA_ERR_UNALLOCATED;
@@ -100,11 +114,15 @@ int plasma_desc_check(PLASMA_desc *desc)
         plasma_error("plasma_desc_check", "tile memory size smaller than the product of dimensions");
         return PLASMA_ERR_ILLEGAL_VALUE;
     }
-    if (desc->lm <= 0 || desc->ln <= 0) {
+    if ((desc->m < 0) || (desc->n < 0)) {
         plasma_error("plasma_desc_check", "negative matrix dimension");
         return PLASMA_ERR_ILLEGAL_VALUE;
     }
-    if (desc->i >= desc->lm || desc->j >= desc->ln) {
+    if ((desc->lm < desc->m) || (desc->ln < desc->n)) {
+        plasma_error("plasma_desc_check", "matrix dimensions larger than leading dimensions");
+        return PLASMA_ERR_ILLEGAL_VALUE;
+    }
+    if ((desc->i > 0 && desc->i >= desc->lm) || (desc->j > 0 && desc->j >= desc->ln)) {
         plasma_error("plasma_desc_check", "beginning of the matrix out of scope");
         return PLASMA_ERR_ILLEGAL_VALUE;
     }
@@ -128,7 +146,6 @@ int plasma_desc_mat_alloc(PLASMA_desc *desc)
         return PLASMA_ERR_OUT_OF_RESOURCES;
     }
 
-    memset(desc->mat, 0, size );
     return PLASMA_SUCCESS;
 }
 
@@ -172,7 +189,7 @@ int plasma_desc_mat_free(PLASMA_desc *desc)
  *          Number of columns in a tile.
  *
  * @param[in] bsiz
- *          Size in bytes including padding.
+ *          Size in elements (mb*nb).
  *
  * @param[in] lm
  *          Number of rows of the entire matrix.
@@ -184,7 +201,7 @@ int plasma_desc_mat_free(PLASMA_desc *desc)
  *          Row index to the beginning of the submatrix.
  *
  * @param[in] j
- *          Column indes to the beginning of the submatrix.
+ *          Column index to the beginning of the submatrix.
  *
  * @param[in] m
  *          Number of rows of the submatrix.
