@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Bilel Hadri
  * @date 2010-11-15
  *
@@ -92,10 +92,10 @@ void PLASMA_DESC_DESTROY(PLASMA_desc **desc, int *INFO)
  *  FORTRAN API - conversion from LAPACK F77 matrix layout to tile layout
  **/
 void PLASMA_LAPACK_TO_TILE(intptr_t *Af77, int *LDA, intptr_t *A, int *INFO)
-{   *INFO = PLASMA_Lapack_to_Tile( (void *)(*Af77), *LDA, (PLASMA_desc *)(*A)); }
+{   *INFO = PLASMA_Lapack_to_Tile( (void *)Af77, *LDA, (PLASMA_desc *)(*A)); }
 
 void PLASMA_TILE_TO_LAPACK(intptr_t *A, intptr_t *Af77, int *LDA, int *INFO)
-{   *INFO = PLASMA_Tile_to_Lapack((PLASMA_desc *)(*A), (void *)(*Af77), *LDA); }
+{   *INFO = PLASMA_Tile_to_Lapack((PLASMA_desc *)(*A), (void *)Af77, *LDA); }
 
 #ifdef __cplusplus
 }
