@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Piotr Luszczek
  * @author Mathieu Faverge
  * @date 2010-11-15
@@ -18,6 +18,13 @@
 #define _GNU_SOURCE
 #include <unistd.h>
 #include <sched.h>
+#elif defined(__FreeBSD__)
+#define PLASMA_OS_FREEBSD 1
+#include <unistd.h>
+#include <inttypes.h>
+#include <sys/param.h>
+#include <sys/cpuset.h>
+#include <sched.h>
 #elif defined( _WIN32 ) || defined( _WIN64 )
 #define PLASMA_OS_WINDOWS 1
 #include <Windows.h>
@@ -56,12 +63,12 @@ static volatile int topo_initialized = 0;
    */
 #ifdef PLASMA_HWLOC
 #include "plasmaos-hwloc.c"
-#else 
+#else
 
 void plasma_topology_init(){
     pthread_mutex_lock(&mutextopo);
     if ( !topo_initialized ) {
-#if (defined PLASMA_OS_LINUX) || (defined PLASMA_OS_AIX)
+#if (defined PLASMA_OS_LINUX) || (defined PLASMA_OS_FREEBSD) || (defined PLASMA_OS_AIX)
 
         sys_corenbr = sysconf(_SC_NPROCESSORS_ONLN);
 
@@ -70,7 +77,7 @@ void plasma_topology_init(){
         int mib[4];
         int cpu;
         size_t len = sizeof(cpu);
-        
+
         /* set the mib for hw.ncpu */
         mib[0] = CTL_HW;
         mib[1] = HW_AVAILCPU;
@@ -94,7 +101,9 @@ void plasma_topology_init(){
     pthread_mutex_unlock(&mutextopo);
 }
 
-void plasma_topology_finalize(){}
+void plasma_topology_finalize(){
+    plasma_unsetaffinity();
+}
 
 /**
  This routine will set affinity for the calling thread that has rank 'rank'.
@@ -107,16 +116,24 @@ void plasma_topology_finalize(){}
  */
 int plasma_setaffinity(int rank) {
 #ifndef PLASMA_AFFINITY_DISABLE
-#if (defined PLASMA_OS_LINUX)
+#if (defined PLASMA_OS_LINUX) || (defined PLASMA_OS_FREEBSD)
     {
+#if (defined PLASMA_OS_LINUX)
         cpu_set_t set;
+#elif (defined PLASMA_OS_FREEBSD)
+        cpuset_t set;
+#endif
         CPU_ZERO( &set );
         CPU_SET( rank, &set );
-        
+
 #if (defined HAVE_OLD_SCHED_SETAFFINITY)
         if( sched_setaffinity( 0, &set ) < 0 )
 #else /* HAVE_OLD_SCHED_SETAFFINITY */
+#if (defined PLASMA_OS_LINUX)
         if( sched_setaffinity( 0, sizeof(set), &set) < 0 )
+#elif (defined PLASMA_OS_FREEBSD)
+        if( cpuset_setaffinity(CPU_LEVEL_WHICH, CPU_WHICH_PID, 0, sizeof(set), &set) < 0 )
+#endif
 #endif /* HAVE_OLD_SCHED_SETAFFINITY */
             {
                 return PLASMA_ERR_UNEXPECTED;
@@ -128,14 +145,14 @@ int plasma_setaffinity(int rank) {
     {
         thread_affinity_policy_data_t ap;
         int                           ret;
-        
+
         ap.affinity_tag = 1; /* non-null affinity tag */
         ret = thread_policy_set( mach_thread_self(),
                                  THREAD_AFFINITY_POLICY,
                                  (integer_t*) &ap,
                                  THREAD_AFFINITY_POLICY_COUNT
             );
-        if(ret != 0) 
+        if(ret != 0)
             return PLASMA_ERR_UNEXPECTED;
 
         return PLASMA_SUCCESS;
@@ -146,7 +163,7 @@ int plasma_setaffinity(int rank) {
 
         if( SetThreadAffinityMask(GetCurrentThread(), mask) == 0)
             return PLASMA_ERR_UNEXPECTED;
-        
+
         return PLASMA_SUCCESS;
     }
 #elif (defined PLASMA_OS_AIX)
@@ -160,14 +177,100 @@ int plasma_setaffinity(int rank) {
 #endif
 #endif /* PLASMA_AFFINITY_DISABLE */
 }
+
+/**
+ This routine will set affinity for the calling thread that has rank 'rank'.
+ Ranks start with 0.
+
+ If there are multiple instances of PLASMA then affinity will be wrong: all ranks 0
+ will be pinned to core 0.
+
+ Also, affinity is not resotred when PLASMA_Finalize() is called.
+ */
+int plasma_unsetaffinity() {
+#ifndef PLASMA_AFFINITY_DISABLE
+#if (defined PLASMA_OS_LINUX) || (defined PLASMA_OS_FREEBSD)
+    {
+        int i;
+#if (defined PLASMA_OS_LINUX)
+        cpu_set_t set;
+#elif (defined PLASMA_OS_FREEBSD)
+        cpuset_t set;
+#endif
+        CPU_ZERO( &set );
+
+        for(i=0; i<sys_corenbr; i++)
+            CPU_SET( i, &set );
+
+#if (defined HAVE_OLD_SCHED_SETAFFINITY)
+        if( sched_setaffinity( 0, &set ) < 0 )
+#else /* HAVE_OLD_SCHED_SETAFFINITY */
+#if (defined PLASMA_OS_LINUX)
+        if( sched_setaffinity( 0, sizeof(set), &set) < 0 )
+#elif (defined PLASMA_OS_FREEBSD)
+        if( cpuset_setaffinity(CPU_LEVEL_WHICH, CPU_WHICH_PID, 0, sizeof(set), &set) < 0 )
+#endif
+#endif /* HAVE_OLD_SCHED_SETAFFINITY */
+            {
+                plasma_warning("plasma_unsetaffinity", "Could not unbind thread");
+                return PLASMA_ERR_UNEXPECTED;
+            }
+
+        return PLASMA_SUCCESS;
+    }
+#elif (defined PLASMA_OS_MACOS)
+    {
+        /* TODO: check how to unbind the main thread if necessary for OpenMP */
+        /* thread_affinity_policy_data_t ap; */
+        /* int                           ret; */
+
+        /* ap.affinity_tag = 1; /\* non-null affinity tag *\/ */
+        /* ret = thread_policy_set( mach_thread_self(), */
+        /*                          THREAD_AFFINITY_POLICY, */
+        /*                          (integer_t*) &ap, */
+        /*                          THREAD_AFFINITY_POLICY_COUNT */
+        /*     ); */
+        /* if(ret != 0) { */
+        /*     plasma_warning("plasma_unsetaffinity", "Could not unbind thread"); */
+        /*     return PLASMA_ERR_UNEXPECTED; */
+        /* } */
+
+        return PLASMA_SUCCESS;
+    }
+#elif (defined PLASMA_OS_WINDOWS)
+    {
+        int i;
+        DWORD mask = 0;
+
+        for(i=0; i<sys_corenbr; i++)
+            mask |= 1 << i;
+
+        if( SetThreadAffinityMask(GetCurrentThread(), mask) == 0) {
+            plasma_warning("plasma_unsetaffinity", "Could not unbind thread");
+            return PLASMA_ERR_UNEXPECTED;
+        }
+        return PLASMA_SUCCESS;
+    }
+#elif (defined PLASMA_OS_AIX)
+    {
+        /* TODO: check how to unbind the main thread if necessary for OpenMP */
+        /* tid_t self_ktid = thread_self (); */
+        /* bindprocessor(BINDTHREAD, self_ktid, rank); */
+        return PLASMA_SUCCESS;
+    }
+#else
+    return PLASMA_ERR_NOT_SUPPORTED;
+#endif
+#endif /* PLASMA_AFFINITY_DISABLE */
+}
 #endif /* PLASMA_HWLOC */
 
 /** ****************************************************************************
-   A thread can unlock the CPU if it has nothing to do to let 
+   A thread can unlock the CPU if it has nothing to do to let
    another thread of less priority running for example for I/O.
  */
 int plasma_yield() {
-#if (defined PLASMA_OS_LINUX) || (defined PLASMA_OS_MACOS) || (defined PLASMA_OS_AIX)
+#if (defined PLASMA_OS_LINUX) || (defined PLASMA_OS_FREEBSD) || (defined PLASMA_OS_MACOS) || (defined PLASMA_OS_AIX)
     return sched_yield();
 #elif PLASMA_OS_WINDOWS
     return SleepEx(0,0);
@@ -175,7 +278,7 @@ int plasma_yield() {
     return PLASMA_ERR_NOT_SUPPORTED;
 #endif
 }
-    
+
 #ifdef PLASMA_OS_WINDOWS
 #define PLASMA_GETENV(var, str) {                    \
         int len = 512;                               \
@@ -199,7 +302,7 @@ int plasma_yield() {
 
 /** ****************************************************************************
  * Check for an integer in an environment variable, returning the
- * integer value or a provided default value 
+ * integer value or a provided default value
 */
 int plasma_get_numthreads()
 {
@@ -207,7 +310,7 @@ int plasma_get_numthreads()
     char    *endptr;
     long int thrdnbr = -1;
     extern int errno;
-    
+
     /* Env variable does not exist, we search the system number of core */
     PLASMA_GETENV("PLASMA_NUM_THREADS", envstr);
     if ( envstr == NULL ) {
@@ -219,7 +322,7 @@ int plasma_get_numthreads()
             PLASMA_CLEANENV(envstr);
             return -1;
         }
-    } 
+    }
     PLASMA_CLEANENV(envstr);
     return (int)thrdnbr;
 }
@@ -230,7 +333,7 @@ int plasma_get_numthreads_numa()
     char    *endptr;
     long int thrdnbr = -1;
     extern int errno;
-    
+
     /* Env variable does not exist, we search the system number of core */
     PLASMA_GETENV("PLASMA_NUM_THREADS_NUMA", envstr);
     if ( envstr != NULL ) {
@@ -246,7 +349,7 @@ int plasma_get_numthreads_numa()
 #else
       thrdnbr = 1;
 #endif
-    } 
+    }
 
     PLASMA_CLEANENV(envstr);
     return (int)thrdnbr;
@@ -283,7 +386,7 @@ int plasma_get_affthreads(int *coresbind) {
                         coresbind[i] = 0;
                         i++;
                     }
-                    
+
                     /* there is no more values in the string                                 */
                     /* the next threads are binded with a round robin policy over this array */
                     wrap = 1;
