@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @author Hatem Ltaief
  * @author Azzam Haidar
@@ -17,6 +17,32 @@
 #include "common.h"
 #include "workspace.h"
 
+/***************************************************************************//**
+ *
+ * @ingroup Auxiliary
+ *
+ *  PLASMA_Alloc_Workspace_zgebrd - Allocates workspace for PLASMA_zgebrd or PLASMA_zgebrd_Tile routine.
+ *
+ *******************************************************************************
+ *
+ * @param[in] M
+ *          The number of rows of the matrix A. M >= 0.
+ *
+ * @param[in] N
+ *          The number of columns of the matrix A.  N >= 0.
+ *
+ * @param[out] descT
+ *          On exit, workspace handle for storage of the extra T factors required by the tile BRD.
+ *
+ *******************************************************************************
+ *
+ * @return
+ *          \retval PLASMA_SUCCESS successful exit
+ *
+ ******************************************************************************/
+int PLASMA_Alloc_Workspace_zgebrd(int M, int N, PLASMA_desc **descT) {
+    return plasma_alloc_ibnb_tile(M, N, PLASMA_FUNC_ZGESVD, PlasmaComplexDouble, descT); }
+
 /***************************************************************************//**
  *
  * @ingroup Auxiliary
@@ -28,7 +54,7 @@
  * @param[in] N
  *          The order of the matrix A.  N >= 0.
  *
- * @param[out] T
+ * @param[out] descT
  *          On exit, workspace handle for storage of the extra T factors required by the tile Hessenberg.
  *
  *******************************************************************************
@@ -51,7 +77,7 @@ int PLASMA_Alloc_Workspace_zgeev(int N, PLASMA_desc **descT) {
  * @param[in] N
  *          The number of columns of the matrix A.  N >= 0.
  *
- * @param[out] T
+ * @param[out] descT
  *          On exit, workspace handle for storage of the extra T factors required by the tile Hessenberg.
  *
  *******************************************************************************
@@ -63,32 +89,6 @@ int PLASMA_Alloc_Workspace_zgeev(int N, PLASMA_desc **descT) {
 int PLASMA_Alloc_Workspace_zgehrd(int N, PLASMA_desc **descT) {
     return plasma_alloc_ibnb_tile(N, N, PLASMA_FUNC_ZGEHRD, PlasmaComplexDouble, descT); }
 
-/***************************************************************************//**
- *
- * @ingroup Auxiliary
- *
- *  PLASMA_Alloc_Workspace_zgebrd - Allocates workspace for PLASMA_zgebrd or PLASMA_zgebrd_Tile routine.
- *
- *******************************************************************************
- *
- * @param[in] M
- *          The number of rows of the matrix A. M >= 0.
- *
- * @param[in] N
- *          The number of columns of the matrix A.  N >= 0.
- *
- * @param[out] T
- *          On exit, workspace handle for storage of the extra T factors required by the tile BRD.
- *
- *******************************************************************************
- *
- * @return
- *          \retval PLASMA_SUCCESS successful exit
- *
- ******************************************************************************/
-int PLASMA_Alloc_Workspace_zgebrd(int M, int N, PLASMA_desc **descT) {
-    return plasma_alloc_ibnb_tile(M, N, PLASMA_FUNC_ZGEBRD, PlasmaComplexDouble, descT); }
-
 /***************************************************************************//**
  *
  * @ingroup Auxiliary
@@ -103,7 +103,7 @@ int PLASMA_Alloc_Workspace_zgebrd(int M, int N, PLASMA_desc **descT) {
  * @param[in] N
  *          The number of columns of the matrix A.  N >= 0.
  *
- * @param[out] T
+ * @param[out] descT
  *          On exit, workspace handle for storage of the extra T factors required by the tile QR
  *          or the tile LQ factorization.
  *
@@ -113,14 +113,14 @@ int PLASMA_Alloc_Workspace_zgebrd(int M, int N, PLASMA_desc **descT) {
  *          \retval PLASMA_SUCCESS successful exit
  *
  ******************************************************************************/
-int PLASMA_Alloc_Workspace_zgels(int M, int N, PLASMA_Complex64_t **T) {
-    return plasma_alloc_ibnb(M, N, PLASMA_FUNC_ZGELS, PlasmaComplexDouble, (void**)T); }
+int PLASMA_Alloc_Workspace_zgels(int M, int N, PLASMA_desc **descT) {
+    return plasma_alloc_ibnb_tile(M, N, PLASMA_FUNC_ZGELS, PlasmaComplexDouble, descT); }
 
- /***************************************************************************//**
+/***************************************************************************//**
 @@ -192,33 +120,6 @@
-  *
-  * @ingroup Auxiliary
-  *
+ *
+ * @ingroup Auxiliary
+ *
  *  PLASMA_Alloc_Workspace_zgels_Tile - Allocates tile workspace for PLASMA_zgels_Tile routine.
  *
  *******************************************************************************
@@ -158,7 +158,7 @@ int PLASMA_Alloc_Workspace_zgels_Tile(int M, int N, PLASMA_desc **descT) {
  * @param[in] N
  *          The number of columns of the matrix A.  N >= 0.
  *
- * @param[out] T
+ * @param[out] descT
  *          On exit, workspace handle for storage of the extra T factors required by the tile QR
  *          factorization.
  *
@@ -168,8 +168,8 @@ int PLASMA_Alloc_Workspace_zgels_Tile(int M, int N, PLASMA_desc **descT) {
  *          \retval PLASMA_SUCCESS successful exit
  *
  ******************************************************************************/
-int PLASMA_Alloc_Workspace_zgeqrf(int M, int N, PLASMA_Complex64_t **T) {
-    return plasma_alloc_ibnb(M, N, PLASMA_FUNC_ZGELS, PlasmaComplexDouble, (void**)T); }
+int PLASMA_Alloc_Workspace_zgeqrf(int M, int N, PLASMA_desc **descT) {
+    return plasma_alloc_ibnb_tile(M, N, PLASMA_FUNC_ZGELS, PlasmaComplexDouble, descT); }
 
 /***************************************************************************//**
  *
@@ -212,7 +212,7 @@ int PLASMA_Alloc_Workspace_zgeqrf_Tile(int M, int N, PLASMA_desc **descT) {
  * @param[in] N
  *          The number of columns of the matrix A.  N >= 0.
  *
- * @param[out] T
+ * @param[out] descT
  *          On exit, workspace handle for storage of the extra T factors required by the tile LQ
  *          factorization.
  *
@@ -222,8 +222,8 @@ int PLASMA_Alloc_Workspace_zgeqrf_Tile(int M, int N, PLASMA_desc **descT) {
  *          \retval PLASMA_SUCCESS successful exit
  *
  ******************************************************************************/
-int PLASMA_Alloc_Workspace_zgelqf(int M, int N, PLASMA_Complex64_t **T) {
-    return plasma_alloc_ibnb(M, N, PLASMA_FUNC_ZGELS, PlasmaComplexDouble, (void**)T); }
+int PLASMA_Alloc_Workspace_zgelqf(int M, int N, PLASMA_desc **descT) {
+    return plasma_alloc_ibnb_tile(M, N, PLASMA_FUNC_ZGELS, PlasmaComplexDouble, descT); }
 
 /***************************************************************************//**
  *
@@ -252,6 +252,32 @@ int PLASMA_Alloc_Workspace_zgelqf(int M, int N, PLASMA_Complex64_t **T) {
 int PLASMA_Alloc_Workspace_zgelqf_Tile(int M, int N, PLASMA_desc **descT) {
     return plasma_alloc_ibnb_tile(M, N, PLASMA_FUNC_ZGELS, PlasmaComplexDouble, descT); }
 
+/***************************************************************************//**
+ *
+ * @ingroup Auxiliary
+ *
+ *  PLASMA_Alloc_Workspace_zgesdd - Allocates workspace for PLASMA_zgesdd or PLASMA_zgesdd_Tile routine.
+ *
+ *******************************************************************************
+ *
+ * @param[in] M
+ *          The number of rows of the matrix A. M >= 0.
+ *
+ * @param[in] N
+ *          The number of columns of the matrix A.  N >= 0.
+ *
+ * @param[out] descT
+ *          On exit, workspace handle for storage of the extra T factors required by the tile BRD.
+ *
+ *******************************************************************************
+ *
+ * @return
+ *          \retval PLASMA_SUCCESS successful exit
+ *
+ ******************************************************************************/
+int PLASMA_Alloc_Workspace_zgesdd(int M, int N, PLASMA_desc **descT) {
+    return plasma_alloc_ibnb_tile(M, N, PLASMA_FUNC_ZGESVD, PlasmaComplexDouble, descT); }
+
 /***************************************************************************//**
  *
  * @ingroup Auxiliary
@@ -263,7 +289,7 @@ int PLASMA_Alloc_Workspace_zgelqf_Tile(int M, int N, PLASMA_desc **descT) {
  * @param[in] N
  *          The number of linear equations, i.e., the order of the matrix A. N >= 0.
  *
- * @param[out] L
+ * @param[out] descL
  *          On exit, workspace handle for storage of the extra L factors required by the tile LU
  *          factorization.
  *
@@ -277,8 +303,8 @@ int PLASMA_Alloc_Workspace_zgelqf_Tile(int M, int N, PLASMA_desc **descT) {
  *          \retval PLASMA_SUCCESS successful exit
  *
  ******************************************************************************/
-int PLASMA_Alloc_Workspace_zgesv_incpiv(int N, PLASMA_Complex64_t **L, int **IPIV) {
-    int status = plasma_alloc_ibnb(N, N, PLASMA_FUNC_ZGESV, PlasmaComplexDouble, (void**)L);
+int PLASMA_Alloc_Workspace_zgesv_incpiv(int N, PLASMA_desc **descL, int **IPIV) {
+    int status = plasma_alloc_ibnb_tile(N, N, PLASMA_FUNC_ZGESV, PlasmaComplexDouble, descL);
     if (status != PLASMA_SUCCESS)
         return status;
     return plasma_alloc_ipiv(N, N, PLASMA_FUNC_ZGESV, (void**)IPIV); }
@@ -329,7 +355,7 @@ int PLASMA_Alloc_Workspace_zgesv_incpiv_Tile(int N, PLASMA_desc **descL, int **I
  * @param[in] N
  *          The number of columns of the matrix A.  N >= 0.
  *
- * @param[out] T
+ * @param[out] descT
  *          On exit, workspace handle for storage of the extra T factors required by the tile BRD.
  *
  *******************************************************************************
@@ -356,7 +382,7 @@ int PLASMA_Alloc_Workspace_zgesvd(int M, int N, PLASMA_desc **descT) {
  * @param[in] N
  *          The number of columns of the matrix A.  N >= 0.
  *
- * @param[out] L
+ * @param[out] descL
  *          On exit, workspace handle for storage of the extra L factors required by the tile LU
  *          factorization.
  *
@@ -376,8 +402,8 @@ int PLASMA_Alloc_Workspace_zgesvd(int M, int N, PLASMA_desc **descT) {
  * @sa PLASMA_zgetrf_incpiv_Tile_Async
  *
  ******************************************************************************/
-int PLASMA_Alloc_Workspace_zgetrf_incpiv(int M, int N, PLASMA_Complex64_t **L, int **IPIV) {
-    int status = plasma_alloc_ibnb(M, N, PLASMA_FUNC_ZGESV, PlasmaComplexDouble, (void**)L);
+int PLASMA_Alloc_Workspace_zgetrf_incpiv(int M, int N, PLASMA_desc **descL, int **IPIV) {
+    int status = plasma_alloc_ibnb_tile(M, N, PLASMA_FUNC_ZGESV, PlasmaComplexDouble, descL);
     if (status != PLASMA_SUCCESS)
         return status;
     return plasma_alloc_ipiv(M, N, PLASMA_FUNC_ZGESV, (void**)IPIV); }
@@ -429,7 +455,7 @@ int PLASMA_Alloc_Workspace_zgetrf_incpiv_Tile(int N, PLASMA_desc **descL, int **
  * @param[in] N
  *          The number of columns of the matrix A.  N >= 0.
  *
- * @param[out] T
+ * @param[out] descT
  *          On exit, workspace handle for storage of the extra T factors required by the tile TRD.
  *
  *******************************************************************************
@@ -441,6 +467,59 @@ int PLASMA_Alloc_Workspace_zgetrf_incpiv_Tile(int N, PLASMA_desc **descL, int **
 int PLASMA_Alloc_Workspace_zheev(int M, int N, PLASMA_desc **descT) {
     return plasma_alloc_ibnb_tile(M, N, PLASMA_FUNC_ZHEEV, PlasmaComplexDouble, descT); }
 
+/***************************************************************************//**
+ *
+ * @ingroup Auxiliary
+ *
+ *  PLASMA_Alloc_Workspace_zheevd - Allocates workspace for PLASMA_zheevd or PLASMA_zheevd_Tile routine.
+ *
+ *******************************************************************************
+ *
+ * @param[in] M
+ *          The number of rows of the matrix A. M >= 0.
+ *
+ * @param[in] N
+ *          The number of columns of the matrix A.  N >= 0.
+ *
+ * @param[out] descT
+ *          On exit, workspace handle for storage of the extra T factors required by the tile TRD.
+ *
+ *******************************************************************************
+ *
+ * @return
+ *          \retval PLASMA_SUCCESS successful exit
+ *
+ ******************************************************************************/
+int PLASMA_Alloc_Workspace_zheevd(int M, int N, PLASMA_desc **descT) {
+    return plasma_alloc_ibnb_tile(M, N, PLASMA_FUNC_ZHEEVD, PlasmaComplexDouble, descT); }
+
+/***************************************************************************//**
+ *
+ * @ingroup Auxiliary
+ *
+ *  PLASMA_Alloc_Workspace_zheevr - Allocates workspace for PLASMA_zheevr or PLASMA_zheevr_Tile routine.
+ *
+ *******************************************************************************
+ *
+ * @param[in] M
+ *          The number of rows of the matrix A. M >= 0.
+ *
+ * @param[in] N
+ *          The number of columns of the matrix A.  N >= 0.
+ *
+ * @param[out] descT
+ *          On exit, workspace handle for storage of the extra T factors required by the tile TRD.
+ *
+ *******************************************************************************
+ *
+ * @return
+ *          \retval PLASMA_SUCCESS successful exit
+ *
+ ******************************************************************************/
+int PLASMA_Alloc_Workspace_zheevr(int M, int N, PLASMA_desc **descT) {
+    return plasma_alloc_ibnb_tile(M, N, PLASMA_FUNC_ZHEEVD, PlasmaComplexDouble, descT); }
+
+
 /***************************************************************************//**
  *
  * @ingroup Auxiliary
@@ -455,7 +534,7 @@ int PLASMA_Alloc_Workspace_zheev(int M, int N, PLASMA_desc **descT) {
  * @param[in] N
  *          The number of columns of the matrix A.  N >= 0.
  *
- * @param[out] T
+ * @param[out] descT
  *          On exit, workspace handle for storage of the extra T factors required by the tile TRD.
  *
  *******************************************************************************
@@ -467,6 +546,32 @@ int PLASMA_Alloc_Workspace_zheev(int M, int N, PLASMA_desc **descT) {
 int PLASMA_Alloc_Workspace_zhegv(int M, int N, PLASMA_desc **descT) {
     return plasma_alloc_ibnb_tile(M, N, PLASMA_FUNC_ZHEGV, PlasmaComplexDouble, descT); }
 
+/***************************************************************************//**
+ *
+ * @ingroup Auxiliary
+ *
+ *  PLASMA_Alloc_Workspace_zhegvd - Allocates workspace for PLASMA_zhegvd or PLASMA_zhegvd_Tile routine.
+ *
+ *******************************************************************************
+ *
+ * @param[in] M
+ *          The number of rows of the matrix A. M >= 0.
+ *
+ * @param[in] N
+ *          The number of columns of the matrix A.  N >= 0.
+ *
+ * @param[out] descT
+ *          On exit, workspace handle for storage of the extra T factors required by the tile TRD.
+ *
+ *******************************************************************************
+ *
+ * @return
+ *          \retval PLASMA_SUCCESS successful exit
+ *
+ ******************************************************************************/
+int PLASMA_Alloc_Workspace_zhegvd(int M, int N, PLASMA_desc **descT) {
+    return plasma_alloc_ibnb_tile(M, N, PLASMA_FUNC_ZHEGVD, PlasmaComplexDouble, descT); }
+
  /***************************************************************************//**
  *
  * @ingroup Auxiliary
@@ -481,7 +586,7 @@ int PLASMA_Alloc_Workspace_zhegv(int M, int N, PLASMA_desc **descT) {
  * @param[in] N
  *          The number of columns of the matrix A.  N >= 0.
  *
- * @param[out] T
+ * @param[out] descT
  *          On exit, workspace handle for storage of the extra T factors required by the tile TRD.
  *
  *******************************************************************************
