@@ -1,12 +1,12 @@
 /**
  *
- * @file core_zabsum.c
+ * @file core_dzasum.c
  *
  *  PLASMA core_blas kernel
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -25,10 +25,10 @@
 #pragma weak CORE_dzasum = PCORE_dzasum
 #define CORE_dzasum PCORE_dzasum
 #endif
-void CORE_dzasum(int storev, int uplo, int M, int N,
-                 PLASMA_Complex64_t *A, int lda, double *work)
+void CORE_dzasum(PLASMA_enum storev, PLASMA_enum uplo, int M, int N,
+                 const PLASMA_Complex64_t *A, int lda, double *work)
 {
-    PLASMA_Complex64_t *tmpA;
+    const PLASMA_Complex64_t *tmpA;
     double *tmpW, sum, abs;
     int i,j;
 
@@ -94,7 +94,7 @@ void CORE_dzasum(int storev, int uplo, int M, int N,
  **/
 void QUARK_CORE_dzasum(Quark *quark, Quark_Task_Flags *task_flags,
                        PLASMA_enum storev, PLASMA_enum uplo, int M, int N,
-                       PLASMA_Complex64_t *A, int lda, int szeA,
+                       const PLASMA_Complex64_t *A, int lda, int szeA,
                        double *work, int szeW)
 {
     QUARK_Insert_Task(
@@ -118,8 +118,8 @@ void QUARK_CORE_dzasum(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_dzasum_quark(Quark *quark)
 {
-    int storev;
-    int uplo;
+    PLASMA_enum storev;
+    PLASMA_enum uplo;
     int M;
     int N;
     PLASMA_Complex64_t *A;
@@ -135,21 +135,35 @@ void CORE_dzasum_quark(Quark *quark)
  **/
 void QUARK_CORE_dzasum_f1(Quark *quark, Quark_Task_Flags *task_flags,
                           PLASMA_enum storev, PLASMA_enum uplo, int M, int N,
-                          PLASMA_Complex64_t *A, int lda, int szeA,
+                          const PLASMA_Complex64_t *A, int lda, int szeA,
                           double *work, int szeW, double *fake, int szeF)
 {
     DAG_CORE_ASUM;
-    QUARK_Insert_Task(
-        quark, CORE_dzasum_f1_quark, task_flags,
-        sizeof(PLASMA_enum),                &storev,    VALUE,
-        sizeof(PLASMA_enum),                &uplo,      VALUE,
-        sizeof(int),                        &M,         VALUE,
-        sizeof(int),                        &N,         VALUE,
-        sizeof(PLASMA_Complex64_t)*szeA,     A,                 INPUT,
-        sizeof(int),                        &lda,       VALUE,
-        sizeof(double)*szeW,                 work,              INOUT,
-        sizeof(double)*szeF,                 fake,              OUTPUT | GATHERV,
-        0);
+    if ( work == fake ) {
+        QUARK_Insert_Task(
+            quark, CORE_dzasum_f1_quark, task_flags,
+            sizeof(PLASMA_enum),                &storev,    VALUE,
+            sizeof(PLASMA_enum),                &uplo,      VALUE,
+            sizeof(int),                        &M,         VALUE,
+            sizeof(int),                        &N,         VALUE,
+            sizeof(PLASMA_Complex64_t)*szeA,     A,                 INPUT,
+            sizeof(int),                        &lda,       VALUE,
+            sizeof(double)*szeW,                 work,              INOUT | GATHERV,
+            sizeof(double)*1,                    NULL,              SCRATCH,
+            0);
+    } else {
+        QUARK_Insert_Task(
+            quark, CORE_dzasum_f1_quark, task_flags,
+            sizeof(PLASMA_enum),                &storev,    VALUE,
+            sizeof(PLASMA_enum),                &uplo,      VALUE,
+            sizeof(int),                        &M,         VALUE,
+            sizeof(int),                        &N,         VALUE,
+            sizeof(PLASMA_Complex64_t)*szeA,     A,                 INPUT,
+            sizeof(int),                        &lda,       VALUE,
+            sizeof(double)*szeW,                 work,              INOUT,
+            sizeof(double)*szeF,                 fake,              OUTPUT | GATHERV,
+            0);
+    }
 }
 
 /***************************************************************************//**
@@ -161,8 +175,8 @@ void QUARK_CORE_dzasum_f1(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_dzasum_f1_quark(Quark *quark)
 {
-    int storev;
-    int uplo;
+    PLASMA_enum storev;
+    PLASMA_enum uplo;
     int M;
     int N;
     PLASMA_Complex64_t *A;
