@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Azzam Haidar
  * @date 2011-05-15
  * @precisions normal z -> c d s
@@ -14,24 +14,30 @@
  **/
 #include <lapacke.h>
 #include "common.h"
+
+#define A(_m, _n)  (PLASMA_Complex64_t *)plasma_geteltaddr(A, ((_m)-1), ((_n)-1), eltsize)
+#define V(_m)      &(V[(_m)-1])
+#define TAU(_m)    &(TAU[(_m)-1])
+
 /***************************************************************************//**
  *
  * @ingroup CORE_PLASMA_Complex64_t
  *
- *  CORE_zgbelr is a kernel that will operate on a region (triangle) of data bounded 
- *  by st and ed. This kernel eliminate a column by an element-wise annihiliation,
- *  and for each annihiliation, it apply a left update, followed by an right 
- *  update on the diagonal 2x2 element, where it create a new nnz, eliminate it 
- *  and update V and TAU, then it continue until finishing the 
- *  the whole column. When this is done, it take advantage that data are on 
- *  cache and will apply the right on the remaining part of this region 
- *  that has not been updated by the right yet.   
+ *  CORE_zgbelr is a kernel that will operate on a region (triangle) of data
+ *  bounded by st and ed. This kernel eliminate a column by an element-wise
+ *  annihiliation, and for each annihiliation, it apply a left update, followed
+ *  by an right update on the diagonal 2x2 element, where it create a new nnz,
+ *  eliminate it and update V and TAU, then it continue until finishing the the
+ *  whole column. When this is done, it take advantage that data are on cache
+ *  and will apply the right on the remaining part of this region that has not
+ *  been updated by the right yet.
  *  Note that the column to be eliminated is located at st-1.
+ *
  *******************************************************************************
  *
  * @param[in] uplo
  *         @arg PlasmaLower:
- *         @arg PlasmaUpper: 
+ *         @arg PlasmaUpper:
  *
  * @param[in] N
  *          The order of the matrix A.
@@ -41,14 +47,14 @@
  *
  * @param[out] V
  *          PLASMA_Complex64_t array, dimension (N).
- *          The scalar elementary reflectors are written in this 
- *          array. So it is used as a workspace for V at each step 
+ *          The scalar elementary reflectors are written in this
+ *          array. So it is used as a workspace for V at each step
  *          of the bulge chasing algorithm.
  *
  * @param[out] TAU
  *          PLASMA_Complex64_t array, dimension (N).
- *          The scalar factors of the elementary reflectors are written  
- *          in thisarray. So it is used as a workspace for TAU at each step 
+ *          The scalar factors of the elementary reflectors are written
+ *          in thisarray. So it is used as a workspace for TAU at each step
  *          of the bulge chasing algorithm.
  *
  * @param[in] st
@@ -71,11 +77,8 @@
  *  TYPE 1-BDL Householder
  *  add -1 because of C
  ******************************************************************************/
-#define A(_m, _n)  (PLASMA_Complex64_t *)plasma_geteltaddr(A, ((_m)-1), ((_n)-1), eltsize)
-#define V(_m)      &(V[(_m)-1])
-#define TAU(_m)    &(TAU[(_m)-1])
-int 
-CORE_zgbelr(int uplo, int N,
+int
+CORE_zgbelr(PLASMA_enum uplo, int N,
             PLASMA_desc *A,
             PLASMA_Complex64_t *V,
             PLASMA_Complex64_t *TAU,
@@ -119,11 +122,11 @@ CORE_zgbelr(int uplo, int N,
             J2    = i-2;
             t1ed  = (J2/NB)*NB;
             t2st  = max(t1ed+1,J1);
-            len1  = t1ed-J1+1; 
+            len1  = t1ed-J1+1;
             len2  = J2-t2st+1;
             if(len1>0)CORE_zlarfx2(PlasmaLeft, len1 , *V(i), conj(*TAU(i)), A(i-1, J1  ), ELTLDD(vA, (i-1)),  A(i,  J1 ), ELTLDD(vA, i) );
             if(len2>0)CORE_zlarfx2(PlasmaLeft, len2 , *V(i), conj(*TAU(i)), A(i-1, t2st), ELTLDD(vA, (i-1)),  A(i, t2st), ELTLDD(vA, i) );
-            CORE_zlarfx2ce(PlasmaLower, V(i), TAU(i), A(i-1,i-1), A(i,i-1), A(i,i)); 
+            CORE_zlarfx2ce(PlasmaLower, V(i), TAU(i), A(i-1,i-1), A(i,i-1), A(i,i));
         }
         /* APPLY RIGHT ON THE REMAINING ELEMENT OF KERNEL 1 */
         for (i = ed; i >= st+1 ; i--){
@@ -131,7 +134,7 @@ CORE_zgbelr(int uplo, int N,
             J2    = min(ed,N);
             t1ed  = (J2/NB)*NB;
             t2st  = max(t1ed+1,J1);
-            len1  = t1ed-J1+1; 
+            len1  = t1ed-J1+1;
             len2  = J2-t2st+1;
             if(len1>0)CORE_zlarfx2(PlasmaRight, len1, conj(*V(i)), conj(*TAU(i)), A(J1,i-1),   ELTLDD(vA, J1)  , A(J1  , i), ELTLDD(vA, J1)   );
             if(len2>0)CORE_zlarfx2(PlasmaRight, len2, conj(*V(i)), conj(*TAU(i)), A(t2st,i-1), ELTLDD(vA, t2st), A(t2st, i), ELTLDD(vA, t2st) );
@@ -151,7 +154,7 @@ CORE_zgbelr(int uplo, int N,
             J2    = i-2;
             t1ed  = (J2/NB)*NB;
             t2st  = max(t1ed+1,J1);
-            len1  = t1ed-J1+1; 
+            len1  = t1ed-J1+1;
             len2  = J2-t2st+1;
             if(len1>0)CORE_zlarfx2(PlasmaRight, len1, conj(*V(i)), conj(*TAU(i)), A(J1,i-1),   ELTLDD(vA, J1)  , A(J1  , i), ELTLDD(vA, J1)   );
             if(len2>0)CORE_zlarfx2(PlasmaRight, len2, conj(*V(i)), conj(*TAU(i)), A(t2st,i-1), ELTLDD(vA, t2st), A(t2st, i), ELTLDD(vA, t2st) );
@@ -163,7 +166,7 @@ CORE_zgbelr(int uplo, int N,
             J2    = min(ed,N);
             t1ed  = (J2/NB)*NB;
             t2st  = max(t1ed+1,J1);
-            len1  = t1ed-J1+1; 
+            len1  = t1ed-J1+1;
             len2  = J2-t2st+1;
             if(len1>0)CORE_zlarfx2(PlasmaLeft, len1 , *V(i), conj(*TAU(i)), A(i-1, J1  ), ELTLDD(vA, (i-1)),  A(i,  J1 ), ELTLDD(vA, i) );
             if(len2>0)CORE_zlarfx2(PlasmaLeft, len2 , *V(i), conj(*TAU(i)), A(i-1, t2st), ELTLDD(vA, (i-1)),  A(i, t2st), ELTLDD(vA, i) );
