@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Azzam Haidar
  * @date 2011-05-15
  * @precisions normal z -> c d s
@@ -15,21 +15,26 @@
 #include <lapacke.h>
 #include "common.h"
 
+#define A(_m, _n)  (PLASMA_Complex64_t *)plasma_geteltaddr(A, ((_m)-1), ((_n)-1), eltsize)
+#define V(_m)      &(V[(_m)-1])
+#define TAU(_m)    &(TAU[(_m)-1])
+
 /***************************************************************************//**
  *
  * @ingroup CORE_PLASMA_Complex64_t
  *
- *  CORE_zgbrce is a kernel that will operate on a region (triangle) of data bounded 
- *  by st and ed. This kernel apply a right update, create a new nnz, then it 
- *  eliminate it, and move to the next right update, create a new nnz, eliminate it 
- *  and so on until finishing. When this is done, it take advantage that data are on 
- *  cache and will apply the left on the remaining part of this region 
- *  that has not been updated by the left yet.   
+ *  CORE_zgbrce is a kernel that will operate on a region (triangle) of data
+ *  bounded by st and ed. This kernel apply a right update, create a new nnz,
+ *  then it eliminate it, and move to the next right update, create a new nnz,
+ *  eliminate it and so on until finishing. When this is done, it take advantage
+ *  that data are on cache and will apply the left on the remaining part of this
+ *  region that has not been updated by the left yet.
+ *
  *******************************************************************************
  *
  * @param[in] uplo
  *         @arg PlasmaLower:
- *         @arg PlasmaUpper: 
+ *         @arg PlasmaUpper:
  *
  * @param[in] N
  *          The order of the matrix A.
@@ -39,14 +44,14 @@
  *
  * @param[out] V
  *          PLASMA_Complex64_t array, dimension (N).
- *          The scalar elementary reflectors are written in this 
- *          array. So it is used as a workspace for V at each step 
+ *          The scalar elementary reflectors are written in this
+ *          array. So it is used as a workspace for V at each step
  *          of the bulge chasing algorithm.
  *
  * @param[out] TAU
  *          PLASMA_Complex64_t array, dimension (N).
- *          The scalar factors of the elementary reflectors are written  
- *          in thisarray. So it is used as a workspace for TAU at each step 
+ *          The scalar factors of the elementary reflectors are written
+ *          in thisarray. So it is used as a workspace for TAU at each step
  *          of the bulge chasing algorithm.
  *
  * @param[in] st
@@ -69,11 +74,8 @@
  *  TYPE 1-BDL Householder
  *  add -1 because of C
  ******************************************************************************/
-#define A(_m, _n)  (PLASMA_Complex64_t *)plasma_geteltaddr(A, ((_m)-1), ((_n)-1), eltsize)
-#define V(_m)      &(V[(_m)-1])
-#define TAU(_m)    &(TAU[(_m)-1])
-int 
-CORE_zgbrce(int uplo, int N,
+int
+CORE_zgbrce(PLASMA_enum uplo, int N,
             PLASMA_desc *A,
             PLASMA_Complex64_t *V,
             PLASMA_Complex64_t *TAU,
@@ -117,11 +119,11 @@ CORE_zgbrce(int uplo, int N,
 
             t1ed  = (J3/NB)*NB;
             t2st  = max(t1ed+1,J1);
-            len1  = t1ed-J1+1; 
+            len1  = t1ed-J1+1;
             len2  = J3-t2st+1;
             if(len1>0)CORE_zlarfx2(PlasmaRight, len1, conj(*V(i)), conj(*TAU(i)), A(J1,  i-1), ELTLDD(vA, J1)  , A(J1  , i), ELTLDD(vA, J1)  );
             if(len2>0)CORE_zlarfx2(PlasmaRight, len2, conj(*V(i)), conj(*TAU(i)), A(t2st,i-1), ELTLDD(vA, t2st), A(t2st, i), ELTLDD(vA, t2st));
-            len    = J3-J2; 
+            len    = J3-J2;
             if(len>0){
                 /* generate Householder to annihilate a(j+kd,j-1) within the band */
                 *V(J3)         = *A(J3,(i-1));
@@ -140,7 +142,7 @@ CORE_zgbrce(int uplo, int N,
                 J2    = min(ed,N);
                 t1ed  = (J2/NB)*NB;
                 t2st  = max(t1ed+1,J1);
-                len1  = t1ed-J1+1; 
+                len1  = t1ed-J1+1;
                 len2  = J2-t2st+1;
                 if(len1>0)CORE_zlarfx2(PlasmaLeft, len1 , *V(J3), conj(*TAU(J3)), A(pt, i   ), ELTLDD(vA, pt),  A((pt+1),  i  ), ELTLDD(vA, pt+1) );
                 if(len2>0)CORE_zlarfx2(PlasmaLeft, len2 , *V(J3), conj(*TAU(J3)), A(pt, t2st), ELTLDD(vA, pt),  A((pt+1), t2st), ELTLDD(vA, pt+1) );
@@ -160,12 +162,12 @@ CORE_zgbrce(int uplo, int N,
 
             t1ed  = (J3/NB)*NB;
             t2st  = max(t1ed+1,J1);
-            len1  = t1ed-J1+1; 
+            len1  = t1ed-J1+1;
             len2  = J3-t2st+1;
             if(len1>0)CORE_zlarfx2(PlasmaLeft, len1 , *V(i), conj(*TAU(i)), A(i-1, J1  ), ELTLDD(vA, i-1),  A(i,  J1 ), ELTLDD(vA, i) );
             if(len2>0)CORE_zlarfx2(PlasmaLeft, len2 , *V(i), conj(*TAU(i)), A(i-1, t2st), ELTLDD(vA, i-1),  A(i, t2st), ELTLDD(vA, i) );
             /* if nonzero element a(j+kd,j-1) has been created outside the band (if index < N) then eliminate it. */
-            len    = J3-J2; 
+            len    = J3-J2;
             if(len>0){
                 /* generate Householder to annihilate a(j+kd,j-1) within the band */
                 *V(J3)         = *A(i-1, J3);
@@ -185,7 +187,7 @@ CORE_zgbrce(int uplo, int N,
                 J2    = min(ed,N);
                 t1ed  = (J2/NB)*NB;
                 t2st  = max(t1ed+1,J1);
-                len1  = t1ed-J1+1; 
+                len1  = t1ed-J1+1;
                 len2  = J2-t2st+1;
                 if(len1>0)CORE_zlarfx2(PlasmaRight, len1 , conj(*V(J3)), conj(*TAU(J3)), A(i   , pt), ELTLDD(vA, i),     A(i,    pt+1), ELTLDD(vA, i) );
                 if(len2>0)CORE_zlarfx2(PlasmaRight, len2 , conj(*V(J3)), conj(*TAU(J3)), A(t2st, pt), ELTLDD(vA, t2st),  A(t2st, pt+1), ELTLDD(vA, t2st) );
