@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -89,7 +89,6 @@ int CORE_zgelqt(int M, int N, int IB,
                 PLASMA_Complex64_t *WORK)
 {
     int i, k, sb;
-    int iinfo;
 
     /* Check input arguments */
     if (M < 0) {
@@ -122,8 +121,8 @@ int CORE_zgelqt(int M, int N, int IB,
     for(i = 0; i < k; i += IB) {
         sb = min(IB, k-i);
 
-        iinfo = LAPACKE_zgelq2_work(LAPACK_COL_MAJOR, sb, N-i, 
-                                    &A[LDA*i+i], LDA, &TAU[i], WORK);
+        LAPACKE_zgelq2_work(LAPACK_COL_MAJOR, sb, N-i,
+                            &A[LDA*i+i], LDA, &TAU[i], WORK);
 
         LAPACKE_zlarft_work(LAPACK_COL_MAJOR,
             lapack_const(PlasmaForward),
@@ -131,7 +130,7 @@ int CORE_zgelqt(int M, int N, int IB,
             N-i, sb,
             &A[LDA*i+i], LDA, &TAU[i],
             &T[LDT*i], LDT);
-        
+
         if (M > i+sb) {
             LAPACKE_zlarfb_work(
                 LAPACK_COL_MAJOR,
