@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -25,10 +25,10 @@
 #pragma weak CORE_zgemm = PCORE_zgemm
 #define CORE_zgemm PCORE_zgemm
 #endif
-void CORE_zgemm(int transA, int transB,
+void CORE_zgemm(PLASMA_enum transA, int transB,
                 int M, int N, int K,
-                PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int LDA,
-                PLASMA_Complex64_t *B, int LDB,
+                PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int LDA,
+                                          const PLASMA_Complex64_t *B, int LDB,
                 PLASMA_Complex64_t beta, PLASMA_Complex64_t *C, int LDC)
 {
     cblas_zgemm(
@@ -44,10 +44,10 @@ void CORE_zgemm(int transA, int transB,
  *
  **/
 void QUARK_CORE_zgemm(Quark *quark, Quark_Task_Flags *task_flags,
-                      int transA, int transB,
+                      PLASMA_enum transA, int transB,
                       int m, int n, int k, int nb,
-                      PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int lda,
-                      PLASMA_Complex64_t *B, int ldb,
+                      PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int lda,
+                                                const PLASMA_Complex64_t *B, int ldb,
                       PLASMA_Complex64_t beta, PLASMA_Complex64_t *C, int ldc)
 {
     DAG_CORE_GEMM;
@@ -72,10 +72,10 @@ void QUARK_CORE_zgemm(Quark *quark, Quark_Task_Flags *task_flags,
  *
  **/
 void QUARK_CORE_zgemm2( Quark *quark, Quark_Task_Flags *task_flags,
-                        int transA, int transB,
+                        PLASMA_enum transA, int transB,
                         int m, int n, int k, int nb,
-                        PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int lda,
-                        PLASMA_Complex64_t *B, int ldb,
+                        PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int lda,
+                        const PLASMA_Complex64_t *B, int ldb,
                         PLASMA_Complex64_t beta, PLASMA_Complex64_t *C, int ldc)
 {
     DAG_CORE_GEMM;
@@ -105,8 +105,8 @@ void QUARK_CORE_zgemm2( Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zgemm_quark(Quark *quark)
 {
-    int transA;
-    int transB;
+    PLASMA_enum transA;
+    PLASMA_enum transB;
     int m;
     int n;
     int k;
@@ -133,10 +133,10 @@ void CORE_zgemm_quark(Quark *quark)
  *
  **/
 void QUARK_CORE_zgemm_f2(Quark *quark, Quark_Task_Flags *task_flags,
-                         int transA, int transB,
+                         PLASMA_enum transA, int transB,
                          int m, int n, int k, int nb,
-                         PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int lda,
-                         PLASMA_Complex64_t *B, int ldb,
+                         PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int lda,
+                                                   const PLASMA_Complex64_t *B, int ldb,
                          PLASMA_Complex64_t beta, PLASMA_Complex64_t *C, int ldc,
                          PLASMA_Complex64_t *fake1, int szefake1, int flag1,
                          PLASMA_Complex64_t *fake2, int szefake2, int flag2)
@@ -154,7 +154,7 @@ void QUARK_CORE_zgemm_f2(Quark *quark, Quark_Task_Flags *task_flags,
         sizeof(PLASMA_Complex64_t)*nb*nb,    B,                 INPUT,
         sizeof(int),                        &ldb,       VALUE,
         sizeof(PLASMA_Complex64_t),         &beta,      VALUE,
-        sizeof(PLASMA_Complex64_t)*nb*nb,    C,                 INOUT,
+        sizeof(PLASMA_Complex64_t)*nb*nb,    C,                 INOUT | LOCALITY,
         sizeof(int),                        &ldc,       VALUE,
         sizeof(PLASMA_Complex64_t)*szefake1, fake1,             flag1,
         sizeof(PLASMA_Complex64_t)*szefake2, fake2,             flag2,
@@ -170,8 +170,8 @@ void QUARK_CORE_zgemm_f2(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zgemm_f2_quark(Quark* quark)
 {
-    int transA;
-    int transB;
+    PLASMA_enum transA;
+    PLASMA_enum transB;
     int M;
     int N;
     int K;
@@ -184,8 +184,8 @@ void CORE_zgemm_f2_quark(Quark* quark)
     PLASMA_Complex64_t *C;
     int LDC;
     void *fake1, *fake2;
-    
-    quark_unpack_args_15(quark, transA, transB, M, N, K, alpha, 
+
+    quark_unpack_args_15(quark, transA, transB, M, N, K, alpha,
                          A, LDA, B, LDB, beta, C, LDC, fake1, fake2);
     cblas_zgemm(
         CblasColMajor,
@@ -200,11 +200,11 @@ void CORE_zgemm_f2_quark(Quark* quark)
  *
  **/
 void QUARK_CORE_zgemm_p2(Quark *quark, Quark_Task_Flags *task_flags,
-                           int transA, int transB,
-                           int m, int n, int k, int nb,
-                           PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int lda,
-                           PLASMA_Complex64_t **B, int ldb,
-                           PLASMA_Complex64_t beta, PLASMA_Complex64_t *C, int ldc)
+                         PLASMA_enum transA, int transB,
+                         int m, int n, int k, int nb,
+                         PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int lda,
+                         const PLASMA_Complex64_t **B, int ldb,
+                         PLASMA_Complex64_t beta, PLASMA_Complex64_t *C, int ldc)
 {
     DAG_CORE_GEMM;
     QUARK_Insert_Task(quark, CORE_zgemm_p2_quark, task_flags,
@@ -219,7 +219,7 @@ void QUARK_CORE_zgemm_p2(Quark *quark, Quark_Task_Flags *task_flags,
         sizeof(PLASMA_Complex64_t*),         B,                 INPUT,
         sizeof(int),                        &ldb,       VALUE,
         sizeof(PLASMA_Complex64_t),         &beta,      VALUE,
-        sizeof(PLASMA_Complex64_t)*ldc*nb,    C,                 INOUT,
+        sizeof(PLASMA_Complex64_t)*ldc*nb,    C,                 INOUT | LOCALITY,
         sizeof(int),                        &ldc,       VALUE,
         0);
 }
@@ -233,8 +233,8 @@ void QUARK_CORE_zgemm_p2(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zgemm_p2_quark(Quark* quark)
 {
-    int transA;
-    int transB;
+    PLASMA_enum transA;
+    PLASMA_enum transB;
     int M;
     int N;
     int K;
@@ -246,8 +246,8 @@ void CORE_zgemm_p2_quark(Quark* quark)
     PLASMA_Complex64_t beta;
     PLASMA_Complex64_t *C;
     int LDC;
-    
-    quark_unpack_args_13(quark, transA, transB, M, N, K, alpha, 
+
+    quark_unpack_args_13(quark, transA, transB, M, N, K, alpha,
                          A, LDA, B, LDB, beta, C, LDC);
     cblas_zgemm(
         CblasColMajor,
@@ -262,10 +262,10 @@ void CORE_zgemm_p2_quark(Quark* quark)
  *
  **/
 void QUARK_CORE_zgemm_p3(Quark *quark, Quark_Task_Flags *task_flags,
-                           int transA, int transB,
+                           PLASMA_enum transA, int transB,
                            int m, int n, int k, int nb,
-                           PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int lda,
-                           PLASMA_Complex64_t *B, int ldb,
+                           PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int lda,
+                           const PLASMA_Complex64_t *B, int ldb,
                            PLASMA_Complex64_t beta, PLASMA_Complex64_t **C, int ldc)
 {
     DAG_CORE_GEMM;
@@ -281,7 +281,7 @@ void QUARK_CORE_zgemm_p3(Quark *quark, Quark_Task_Flags *task_flags,
         sizeof(PLASMA_Complex64_t)*ldb*nb,   B,                 INPUT,
         sizeof(int),                        &ldb,       VALUE,
         sizeof(PLASMA_Complex64_t),         &beta,      VALUE,
-        sizeof(PLASMA_Complex64_t*),         C,                 INOUT,
+        sizeof(PLASMA_Complex64_t*),         C,                 INOUT | LOCALITY,
         sizeof(int),                        &ldc,       VALUE,
         0);
 }
@@ -295,8 +295,8 @@ void QUARK_CORE_zgemm_p3(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zgemm_p3_quark(Quark* quark)
 {
-    int transA;
-    int transB;
+    PLASMA_enum transA;
+    PLASMA_enum transB;
     int M;
     int N;
     int K;
@@ -308,8 +308,8 @@ void CORE_zgemm_p3_quark(Quark* quark)
     PLASMA_Complex64_t beta;
     PLASMA_Complex64_t **C;
     int LDC;
-    
-    quark_unpack_args_13(quark, transA, transB, M, N, K, alpha, 
+
+    quark_unpack_args_13(quark, transA, transB, M, N, K, alpha,
                          A, LDA, B, LDB, beta, C, LDC);
     cblas_zgemm(
         CblasColMajor,
@@ -324,10 +324,10 @@ void CORE_zgemm_p3_quark(Quark* quark)
  *
  **/
 void QUARK_CORE_zgemm_p2f1(Quark *quark, Quark_Task_Flags *task_flags,
-                           int transA, int transB,
+                           PLASMA_enum transA, int transB,
                            int m, int n, int k, int nb,
-                           PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int lda,
-                           PLASMA_Complex64_t **B, int ldb,
+                           PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int lda,
+                           const PLASMA_Complex64_t **B, int ldb,
                            PLASMA_Complex64_t beta, PLASMA_Complex64_t *C, int ldc,
                            PLASMA_Complex64_t *fake1, int szefake1, int flag1)
 {
@@ -344,7 +344,7 @@ void QUARK_CORE_zgemm_p2f1(Quark *quark, Quark_Task_Flags *task_flags,
         sizeof(PLASMA_Complex64_t*),         B,                 INPUT,
         sizeof(int),                        &ldb,       VALUE,
         sizeof(PLASMA_Complex64_t),         &beta,      VALUE,
-        sizeof(PLASMA_Complex64_t)*ldc*nb,    C,                 INOUT,
+        sizeof(PLASMA_Complex64_t)*ldc*nb,    C,                 INOUT | LOCALITY,
         sizeof(int),                        &ldc,       VALUE,
         sizeof(PLASMA_Complex64_t)*szefake1, fake1,             flag1,
         0);
@@ -359,8 +359,8 @@ void QUARK_CORE_zgemm_p2f1(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zgemm_p2f1_quark(Quark* quark)
 {
-    int transA;
-    int transB;
+    PLASMA_enum transA;
+    PLASMA_enum transB;
     int M;
     int N;
     int K;
@@ -373,8 +373,8 @@ void CORE_zgemm_p2f1_quark(Quark* quark)
     PLASMA_Complex64_t *C;
     int LDC;
     void *fake1;
-    
-    quark_unpack_args_14(quark, transA, transB, M, N, K, alpha, 
+
+    quark_unpack_args_14(quark, transA, transB, M, N, K, alpha,
                          A, LDA, B, LDB, beta, C, LDC, fake1);
     cblas_zgemm(
         CblasColMajor,
