@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -90,7 +90,6 @@ int CORE_zgeqrt(int M, int N, int IB,
                 PLASMA_Complex64_t *WORK)
 {
     int i, k, sb;
-    int iinfo = 0;
 
     /* Check input arguments */
     if (M < 0) {
@@ -123,8 +122,8 @@ int CORE_zgeqrt(int M, int N, int IB,
     for(i = 0; i < k; i += IB) {
         sb = min(IB, k-i);
 
-        iinfo = LAPACKE_zgeqr2_work(LAPACK_COL_MAJOR, M-i, sb, 
-                                    &A[LDA*i+i], LDA, &TAU[i], WORK);
+        LAPACKE_zgeqr2_work(LAPACK_COL_MAJOR, M-i, sb,
+                            &A[LDA*i+i], LDA, &TAU[i], WORK);
 
         LAPACKE_zlarft_work(LAPACK_COL_MAJOR,
             lapack_const(PlasmaForward),
