@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -22,7 +22,7 @@
  *
  * @ingroup CORE_PLASMA_Complex64_t
  *
- *  CORE_zgessm applies the factor L computed by CORE_zgetrf_incpiv to
+ *  CORE_zgessm applies the factors L computed by CORE_zgetrf_incpiv to
  *  a complex M-by-N tile A.
  *
  *******************************************************************************
@@ -34,18 +34,20 @@
  *         The number of columns of the tile A.  N >= 0.
  *
  * @param[in] K
+ *         The number of columns of the tile L. K >= 0.
  *
  * @param[in] IB
  *         The inner-blocking size.  IB >= 0.
  *
  * @param[in] IPIV
- *         as returned by CORE_zgetrf_incpiv.
+ *         The pivot indices array of size K as returned by
+ *         CORE_zgetrf_incpiv.
  *
  * @param[in] L
- *         The NB-by-NB lower triangular tile.
+ *         The M-by-K lower triangular tile.
  *
  * @param[in] LDL
- *         The leading dimension of the array L.  LDL >= max(1,NB).
+ *         The leading dimension of the array L.  LDL >= max(1,M).
  *
  * @param[in,out] A
  *         On entry, the M-by-N tile A.
@@ -66,8 +68,8 @@
 #define CORE_zgessm PCORE_zgessm
 #endif
 int CORE_zgessm(int M, int N, int K, int IB,
-                int *IPIV,
-                PLASMA_Complex64_t *L, int LDL,
+                const int *IPIV,
+                const PLASMA_Complex64_t *L, int LDL,
                 PLASMA_Complex64_t *A, int LDA)
 {
     static PLASMA_Complex64_t zone  =  1.0;
@@ -144,8 +146,8 @@ int CORE_zgessm(int M, int N, int K, int IB,
  **/
 void QUARK_CORE_zgessm(Quark *quark, Quark_Task_Flags *task_flags,
                        int m, int n, int k, int ib, int nb,
-                       int *IPIV,
-                       PLASMA_Complex64_t *L, int ldl,
+                       const int *IPIV,
+                       const PLASMA_Complex64_t *L, int ldl,
                        PLASMA_Complex64_t *A, int lda)
 {
     DAG_CORE_GESSM;
