@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -73,6 +73,13 @@ void CORE_zgetrf_quark(Quark *quark)
 
     quark_unpack_args_9(quark, m, n, A, lda, IPIV, sequence, request, check_info, iinfo);
     info = LAPACKE_zgetrf_work(LAPACK_COL_MAJOR, m, n, A, lda, IPIV );
-    if (info != PLASMA_SUCCESS && check_info)
-        plasma_sequence_flush(quark, sequence, request, iinfo+info);
+    if (info != PLASMA_SUCCESS) {
+        if (check_info || info < 0)
+            plasma_sequence_flush(quark, sequence, request, iinfo+info);
+        else{
+            int i;
+            for(i=info-1; i<min(m,n); i++)
+                IPIV[i] = i+1;
+        }
+    }
 }
