@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -75,10 +75,14 @@
 #pragma weak CORE_zgetrf_incpiv = PCORE_zgetrf_incpiv
 #define CORE_zgetrf_incpiv PCORE_zgetrf_incpiv
 #define CORE_zgessm PCORE_zgessm
+int  CORE_zgessm(int M, int N, int K, int IB,
+                 int *IPIV,
+                 const PLASMA_Complex64_t *L, int LDL,
+                 PLASMA_Complex64_t *A, int LDA);
 #endif
 int CORE_zgetrf_incpiv(int M, int N, int IB,
-                PLASMA_Complex64_t *A, int LDA,
-                int *IPIV, int *INFO)
+                       PLASMA_Complex64_t *A, int LDA,
+                       int *IPIV, int *INFO)
 {
     int i, j, k, sb;
     int iinfo;
@@ -139,11 +143,11 @@ int CORE_zgetrf_incpiv(int M, int N, int IB,
  *
  **/
 void QUARK_CORE_zgetrf_incpiv(Quark *quark, Quark_Task_Flags *task_flags,
-                       int m, int n, int ib, int nb,
-                       PLASMA_Complex64_t *A, int lda,
-                       int *IPIV,
-                       PLASMA_sequence *sequence, PLASMA_request *request,
-                       PLASMA_bool check_info, int iinfo)
+                              int m, int n, int ib, int nb,
+                              PLASMA_Complex64_t *A, int lda,
+                              int *IPIV,
+                              PLASMA_sequence *sequence, PLASMA_request *request,
+                              PLASMA_bool check_info, int iinfo)
 {
     DAG_CORE_GETRF;
     QUARK_Insert_Task(quark, CORE_zgetrf_incpiv_quark, task_flags,
