@@ -6,11 +6,11 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- *  This work is the implementation of an inplace transformation 
- *  based on the GKK algorithm by Gustavson, Karlsson, Kagstrom 
+ *  This work is the implementation of an inplace transformation
+ *  based on the GKK algorithm by Gustavson, Karlsson, Kagstrom
  *  and its fortran implementation.
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  *
@@ -28,7 +28,7 @@
  * @ingroup CORE_PLASMA_Complex64_t
  *
  *  CORE_zgetrip transposes a m-by-n matrix in place using an extra
- *      workspace of size m-by-n.  
+ *      workspace of size m-by-n.
  *      Note : For square tile, workspace is not used.
  *
  *******************************************************************************
@@ -54,7 +54,7 @@
 void CORE_zgetrip(int m, int n, PLASMA_Complex64_t *A, PLASMA_Complex64_t *W) {
     PLASMA_Complex64_t t;
     int i, j;
-    
+
     if( m != n ) {
         /* rectangular transposition (use workspace) */
         for (i=0; i<m; i++) {
@@ -91,6 +91,76 @@ void QUARK_CORE_zgetrip(Quark *quark, Quark_Task_Flags *task_flags,
         0);
 }
 
+/***************************************************************************//**
+ *
+ **/
+void QUARK_CORE_zgetrip_f1(Quark *quark, Quark_Task_Flags *task_flags,
+                           int m, int n,
+                           PLASMA_Complex64_t *A,    int szeA,
+                           PLASMA_Complex64_t *fake, int szeF, int paramF)
+{
+    DAG_CORE_GETRIP;
+    if ( (fake == A) && (paramF & GATHERV) ) {
+        QUARK_Insert_Task(
+            quark, CORE_zgetrip_quark, task_flags,
+            sizeof(int),                     &m,   VALUE,
+            sizeof(int),                     &n,   VALUE,
+            sizeof(PLASMA_Complex64_t)*szeA, A,        INOUT | paramF,
+            sizeof(PLASMA_Complex64_t)*szeA, NULL,     SCRATCH,
+            0);
+    } else {
+        QUARK_Insert_Task(
+            quark, CORE_zgetrip_f1_quark, task_flags,
+            sizeof(int),                     &m,   VALUE,
+            sizeof(int),                     &n,   VALUE,
+            sizeof(PLASMA_Complex64_t)*szeA, A,        INOUT,
+            sizeof(PLASMA_Complex64_t)*szeA, NULL,     SCRATCH,
+            sizeof(PLASMA_Complex64_t)*szeF, fake,     paramF,
+            0);
+    }
+}
+
+/***************************************************************************//**
+ *
+ **/
+void QUARK_CORE_zgetrip_f2(Quark *quark, Quark_Task_Flags *task_flags,
+                           int m, int n,
+                           PLASMA_Complex64_t *A,    int szeA,
+                           PLASMA_Complex64_t *fake1, int szeF1, int paramF1,
+                           PLASMA_Complex64_t *fake2, int szeF2, int paramF2)
+{
+    DAG_CORE_GETRIP;
+    if ( (fake2 == A) && (paramF2 & GATHERV) ) {
+        QUARK_Insert_Task(
+            quark, CORE_zgetrip_f1_quark, task_flags,
+            sizeof(int),                     &m,   VALUE,
+            sizeof(int),                     &n,   VALUE,
+            sizeof(PLASMA_Complex64_t)*szeA, A,        INOUT | paramF2,
+            sizeof(PLASMA_Complex64_t)*szeA, NULL,     SCRATCH,
+            sizeof(PLASMA_Complex64_t)*szeF1, fake1,     paramF1,
+            0);
+    } else if ( (fake1 == A) && (paramF1 & GATHERV) ) {
+        QUARK_Insert_Task(
+            quark, CORE_zgetrip_f1_quark, task_flags,
+            sizeof(int),                     &m,   VALUE,
+            sizeof(int),                     &n,   VALUE,
+            sizeof(PLASMA_Complex64_t)*szeA, A,        INOUT | paramF1,
+            sizeof(PLASMA_Complex64_t)*szeA, NULL,     SCRATCH,
+            sizeof(PLASMA_Complex64_t)*szeF2, fake2,     paramF2,
+            0);
+    } else {
+        QUARK_Insert_Task(
+            quark, CORE_zgetrip_f2_quark, task_flags,
+            sizeof(int),                     &m,   VALUE,
+            sizeof(int),                     &n,   VALUE,
+            sizeof(PLASMA_Complex64_t)*szeA, A,        INOUT,
+            sizeof(PLASMA_Complex64_t)*szeA, NULL,     SCRATCH,
+            sizeof(PLASMA_Complex64_t)*szeF1, fake1,     paramF1,
+            sizeof(PLASMA_Complex64_t)*szeF2, fake2,     paramF2,
+            0);
+    }
+}
+
 /***************************************************************************//**
  *
  **/
@@ -109,25 +179,6 @@ void CORE_zgetrip_quark(Quark *quark)
     CORE_zgetrip(m, n, A, W);
 }
 
-/***************************************************************************//**
- *
- **/
-void QUARK_CORE_zgetrip_f1(Quark *quark, Quark_Task_Flags *task_flags,
-                           int m, int n, 
-                           PLASMA_Complex64_t *A,    int szeA,
-                           PLASMA_Complex64_t *fake, int szeF, int paramF)
-{
-    DAG_CORE_GETRIP;
-    QUARK_Insert_Task(
-        quark, CORE_zgetrip_f1_quark, task_flags,
-        sizeof(int),                     &m,   VALUE,
-        sizeof(int),                     &n,   VALUE,
-        sizeof(PLASMA_Complex64_t)*szeA, A,        INOUT,
-        sizeof(PLASMA_Complex64_t)*szeA, NULL,     SCRATCH,
-        sizeof(PLASMA_Complex64_t)*szeF, fake,     paramF,
-        0);
-}
-
 /***************************************************************************//**
  *
  **/
@@ -147,27 +198,6 @@ void CORE_zgetrip_f1_quark(Quark *quark)
     CORE_zgetrip(m, n, A, W);
 }
 
-/***************************************************************************//**
- *
- **/
-void QUARK_CORE_zgetrip_f2(Quark *quark, Quark_Task_Flags *task_flags,
-                           int m, int n, 
-                           PLASMA_Complex64_t *A,    int szeA,
-                           PLASMA_Complex64_t *fake1, int szeF1, int paramF1,
-                           PLASMA_Complex64_t *fake2, int szeF2, int paramF2)
-{
-    DAG_CORE_GETRIP;
-    QUARK_Insert_Task(
-        quark, CORE_zgetrip_f2_quark, task_flags,
-        sizeof(int),                     &m,   VALUE,
-        sizeof(int),                     &n,   VALUE,
-        sizeof(PLASMA_Complex64_t)*szeA, A,        INOUT,
-        sizeof(PLASMA_Complex64_t)*szeA, NULL,     SCRATCH,
-        sizeof(PLASMA_Complex64_t)*szeF1, fake1,     paramF1,
-        sizeof(PLASMA_Complex64_t)*szeF2, fake2,     paramF2,
-        0);
-}
-
 /***************************************************************************//**
  *
  **/
