@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Azzam Haidar
  * @date 2011-05-15
  * @precisions normal z -> c d s
@@ -15,21 +15,26 @@
 #include <lapacke.h>
 #include "common.h"
 
+#define A(_m, _n)  (PLASMA_Complex64_t *)plasma_geteltaddr(A, ((_m)-1), ((_n)-1), eltsize)
+#define V(_m)      &(V[(_m)-1])
+#define TAU(_m)    &(TAU[(_m)-1])
+
 /***************************************************************************//**
  *
  * @ingroup CORE_PLASMA_Complex64_t
  *
- *  CORE_zhbrce is a kernel that will operate on a region (triangle) of data bounded 
- *  by st and ed. This kernel apply a right update, create a new nnz, then it 
- *  eliminate it, and move to the next right update, create a new nnz, eliminate it 
- *  and so on until finishing. When this is done, it take advantage that data are on 
- *  cache and will apply the left on the remaining part of this region 
- *  that has not been updated by the left yet.   
+ *  CORE_zhbrce is a kernel that will operate on a region (triangle) of data
+ *  bounded by st and ed. This kernel apply a right update, create a new nnz,
+ *  then it eliminate it, and move to the next right update, create a new nnz,
+ *  eliminate it and so on until finishing. When this is done, it take advantage
+ *  that data are on cache and will apply the left on the remaining part of this
+ *  region that has not been updated by the left yet.
+ *
  *******************************************************************************
  *
  * @param[in] uplo
  *         @arg PlasmaLower:
- *         @arg PlasmaUpper: 
+ *         @arg PlasmaUpper:
  *
  * @param[in] N
  *          The order of the matrix A.
@@ -39,14 +44,14 @@
  *
  * @param[out] V
  *          PLASMA_Complex64_t array, dimension (N).
- *          The scalar elementary reflectors are written in this 
- *          array. So it is used as a workspace for V at each step 
+ *          The scalar elementary reflectors are written in this
+ *          array. So it is used as a workspace for V at each step
  *          of the bulge chasing algorithm.
  *
  * @param[out] TAU
  *          PLASMA_Complex64_t array, dimension (N).
- *          The scalar factors of the elementary reflectors are written  
- *          in thisarray. So it is used as a workspace for TAU at each step 
+ *          The scalar factors of the elementary reflectors are written
+ *          in thisarray. So it is used as a workspace for TAU at each step
  *          of the bulge chasing algorithm.
  *
  * @param[in] st
@@ -69,11 +74,8 @@
  *  TYPE 1-BDL Householder
  *  add -1 because of C
  ******************************************************************************/
-#define A(_m, _n)  (PLASMA_Complex64_t *)plasma_geteltaddr(A, ((_m)-1), ((_n)-1), eltsize)
-#define V(_m)      &(V[(_m)-1])
-#define TAU(_m)    &(TAU[(_m)-1])
-int 
-CORE_zhbrce(int uplo, int N,
+int
+CORE_zhbrce(PLASMA_enum uplo, int N,
             PLASMA_desc *A,
             PLASMA_Complex64_t *V,
             PLASMA_Complex64_t *TAU,
