@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -15,6 +15,9 @@
 #include <lapacke.h>
 #include "common.h"
 
+#undef REAL
+#define COMPLEX
+
 /***************************************************************************//**
  *
  * @ingroup CORE_PLASMA_Complex64_t
@@ -24,7 +27,9 @@
 #pragma weak CORE_zhegst = PCORE_zhegst
 #define CORE_zhegst PCORE_zhegst
 #endif
-void CORE_zhegst(int itype, PLASMA_enum uplo, int N, PLASMA_Complex64_t *A, int LDA, PLASMA_Complex64_t *B, int LDB, int *INFO)
+void CORE_zhegst(int itype, PLASMA_enum uplo, int N,
+                 PLASMA_Complex64_t *A, int LDA,
+                 PLASMA_Complex64_t *B, int LDB, int *INFO)
 {
     *INFO = LAPACKE_zhegst_work(
         LAPACK_COL_MAJOR,
@@ -47,9 +52,13 @@ void QUARK_CORE_zhegst(Quark *quark, Quark_Task_Flags *task_flags,
         sizeof(int),                        &itype,      VALUE,
         sizeof(PLASMA_enum),                &uplo,      VALUE,
         sizeof(int),                        &n,         VALUE,
-        sizeof(PLASMA_Complex64_t)*n*n,    A,                 INOUT,
+        sizeof(PLASMA_Complex64_t)*lda*n,    A,             INOUT,
         sizeof(int),                        &lda,       VALUE,
-        sizeof(PLASMA_Complex64_t)*n*n,    B,                 INOUT,
+#ifdef COMPLEX
+        sizeof(PLASMA_Complex64_t)*ldb*n,    B,             INOUT,
+#else
+        sizeof(PLASMA_Complex64_t)*ldb*n,    B,             INPUT,
+#endif
         sizeof(int),                        &ldb,       VALUE,
         sizeof(PLASMA_sequence*),           &sequence,  VALUE,
         sizeof(PLASMA_request*),            &request,   VALUE,
