@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -28,10 +28,10 @@
 #pragma weak CORE_zhemm = PCORE_zhemm
 #define CORE_zhemm PCORE_zhemm
 #endif
-void CORE_zhemm(int side, int uplo,
+void CORE_zhemm(PLASMA_enum side, PLASMA_enum uplo,
                 int M, int N,
-                PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int LDA,
-                PLASMA_Complex64_t *B, int LDB,
+                PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int LDA,
+                const PLASMA_Complex64_t *B, int LDB,
                 PLASMA_Complex64_t beta, PLASMA_Complex64_t *C, int LDC)
 {
     cblas_zhemm(
@@ -47,10 +47,10 @@ void CORE_zhemm(int side, int uplo,
  *
  **/
 void QUARK_CORE_zhemm(Quark *quark, Quark_Task_Flags *task_flags,
-                      int side, int uplo,
+                      PLASMA_enum side, PLASMA_enum uplo,
                       int m, int n, int nb,
-                      PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int lda,
-                      PLASMA_Complex64_t *B, int ldb,
+                      PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int lda,
+                      const PLASMA_Complex64_t *B, int ldb,
                       PLASMA_Complex64_t beta, PLASMA_Complex64_t *C, int ldc)
 {
     DAG_CORE_HEMM;
@@ -79,8 +79,8 @@ void QUARK_CORE_zhemm(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zhemm_quark(Quark *quark)
 {
-    int side;
-    int uplo;
+    PLASMA_enum side;
+    PLASMA_enum uplo;
     int M;
     int N;
     PLASMA_Complex64_t alpha;
