@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -28,10 +28,10 @@
 #pragma weak CORE_zher2k = PCORE_zher2k
 #define CORE_zher2k PCORE_zher2k
 #endif
-void CORE_zher2k(int uplo, int trans,
+void CORE_zher2k(PLASMA_enum uplo, PLASMA_enum trans,
                  int N, int K,
-                 PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int LDA,
-                 PLASMA_Complex64_t *B, int LDB,
+                 PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int LDA,
+                 const PLASMA_Complex64_t *B, int LDB,
                  double beta, PLASMA_Complex64_t *C, int LDC)
 {
     cblas_zher2k(
@@ -46,10 +46,10 @@ void CORE_zher2k(int uplo, int trans,
  *
  **/
 void QUARK_CORE_zher2k(Quark *quark, Quark_Task_Flags *task_flags,
-                       int uplo, int trans,
+                       PLASMA_enum uplo, PLASMA_enum trans,
                        int n, int k, int nb,
-                       PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int lda,
-                       PLASMA_Complex64_t *B, int ldb,
+                       PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int lda,
+                       const PLASMA_Complex64_t *B, int ldb,
                        double beta, PLASMA_Complex64_t *C, int ldc)
 {
     DAG_CORE_HER2K;
@@ -78,8 +78,8 @@ void QUARK_CORE_zher2k(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zher2k_quark(Quark *quark)
 {
-    int uplo;
-    int trans;
+    PLASMA_enum uplo;
+    PLASMA_enum trans;
     int n;
     int k;
     PLASMA_Complex64_t alpha;
@@ -92,6 +92,7 @@ void CORE_zher2k_quark(Quark *quark)
     int ldc;
 
     quark_unpack_args_12(quark, uplo, trans, n, k, alpha, A, lda, B, ldb, beta, C, ldc);
-    CORE_zher2k(uplo, trans, n, k, alpha, A, lda, B, ldb, beta, C, ldc);
+    cblas_zher2k(CblasColMajor, (CBLAS_UPLO)uplo, (CBLAS_TRANSPOSE)trans,
+                 n, k, CBLAS_SADDR(alpha), A, lda, B, ldb, beta, C, ldc);
 }
 #endif
