@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -35,9 +35,9 @@
  *******************************************************************************
  *
  * @param[in] uplo
- *         @arg PlasmaLower : the upper part of the symmetric matrix C 
+ *         @arg PlasmaLower : the upper part of the symmetric matrix C
  *                            is not referenced.
- *         @arg PlasmaUpper : the lower part of the symmetric matrix C 
+ *         @arg PlasmaUpper : the lower part of the symmetric matrix C
  *                            is not referenced (not supported).
  *
  * @param[in] n
@@ -61,7 +61,7 @@
  * @param[in] lda
  *         The leading dimension of the array A. LDA >= max(1,N).
  *
- * @param[out] T
+ * @param[in] T
  *         The IB-by-K triangular factor T of the block reflector.
  *         T is upper triangular by block (economic storage);
  *         The rest of the array is not referenced.
@@ -92,71 +92,110 @@
 #if defined(PLASMA_HAVE_WEAK)
 #pragma weak CORE_zherfb = PCORE_zherfb
 #define CORE_zherfb PCORE_zherfb
+#define CORE_zunmlq PCORE_zunmlq
+#define CORE_zunmqr PCORE_zunmqr
+int  CORE_zunmlq(PLASMA_enum side, PLASMA_enum trans,
+                 int M, int N, int IB, int K,
+                 const PLASMA_Complex64_t *V, int LDV,
+                 const PLASMA_Complex64_t *T, int LDT,
+                 PLASMA_Complex64_t *C, int LDC,
+                 PLASMA_Complex64_t *WORK, int LDWORK);
+int  CORE_zunmqr(PLASMA_enum side, PLASMA_enum trans,
+                 int M, int N, int K, int IB,
+                 const PLASMA_Complex64_t *V, int LDV,
+                 const PLASMA_Complex64_t *T, int LDT,
+                 PLASMA_Complex64_t *C, int LDC,
+                 PLASMA_Complex64_t *WORK, int LDWORK);
 #endif
 int CORE_zherfb( PLASMA_enum uplo, int n,
                  int k, int ib, int nb,
-                 PLASMA_Complex64_t *A, int lda,
-                 PLASMA_Complex64_t *T, int ldt,
+                 const PLASMA_Complex64_t *A, int lda,
+                 const PLASMA_Complex64_t *T, int ldt,
                  PLASMA_Complex64_t *C, int ldc,
                  PLASMA_Complex64_t *WORK, int ldwork )
 {
+    PLASMA_Complex64_t tmp;
     int i, j;
 
+    /* Check input arguments */
+    if ((uplo != PlasmaUpper) && (uplo != PlasmaLower)) {
+        coreblas_error(1, "Illegal value of uplo");
+        return -1;
+    }
+    if (n < 0) {
+        coreblas_error(2, "Illegal value of n");
+        return -2;
+    }
+    if (k < 0) {
+        coreblas_error(3, "Illegal value of k");
+        return -3;
+    }
+    if (ib < 0) {
+        coreblas_error(4, "Illegal value of ib");
+        return -4;
+    }
+    if (nb < 0) {
+        coreblas_error(5, "Illegal value of nb");
+        return -5;
+    }
+    if ( (lda < max(1,n)) && (n > 0) ) {
+        coreblas_error(7, "Illegal value of lda");
+        return -7;
+    }
+    if ( (ldt < max(1,ib)) && (ib > 0) ) {
+        coreblas_error(9, "Illegal value of ldt");
+        return -9;
+    }
+    if ( (ldc < max(1,n)) && (n > 0) ) {
+        coreblas_error(11, "Illegal value of ldc");
+        return -11;
+    }
+
     if (uplo == PlasmaLower) {
         /* Rebuild the symmetric block: WORK <- C */
-        for (j = 0; j < n; j++)
-            for (i = j; i < n; i++){
-                *(WORK + i + j * ldwork) = *(C + i + j*ldc);
-                if (i > j){
-                    *(WORK + j + i * ldwork) =  *(WORK + i + j * ldwork);
-#ifdef COMPLEX
-                    LAPACKE_zlacgv_work(1, WORK + j + i * ldwork, ldwork);
-#endif
-                }
+        for (j = 0; j < n; j++) {
+            *(WORK + j + j * ldwork) =  *(C + ldc*j + j);
+            for (i = j+1; i < n; i++){
+                tmp = *(C + i + j*ldc);
+                *(WORK + i + j * ldwork) = tmp;
+                *(WORK + j + i * ldwork) = conj( tmp );
             }
-        
+        }
+            
         /* Left */
-        CORE_zunmqr(PlasmaLeft, PlasmaConjTrans, n, n, k, ib, 
+        CORE_zunmqr(PlasmaLeft, PlasmaConjTrans, n, n, k, ib,
                     A, lda, T, ldt, WORK, ldwork, WORK+nb*ldwork, ldwork);
         /* Right */
-        CORE_zunmqr(PlasmaRight, PlasmaNoTrans, n, n, k, ib, 
+        CORE_zunmqr(PlasmaRight, PlasmaNoTrans, n, n, k, ib,
                     A, lda, T, ldt, WORK, ldwork, WORK+nb*ldwork, ldwork);
-        
-        /* 
-         * Copy back the final result to the lower part of C 
+
+        /*
+         * Copy back the final result to the lower part of C
          */
-        /* C = WORK */
-        for (j = 0; j < n; j++)
-            for (i = j; i < n; i++)
-                *(C + i + j*ldc) = *(WORK + i + j * ldwork);
+        LAPACKE_zlacpy_work( LAPACK_COL_MAJOR, lapack_const(PlasmaLower), n, n, WORK, ldwork, C, ldc );
     }
     else {
         /* Rebuild the symmetric block: WORK <- C */
-        for (i = 0; i < n; i++)
-            for (j = i; j < n; j++){
-                *(WORK + i + j * ldwork) = *(C + i + j*ldc);
-                if (j > i){
-                    *(WORK + j + i * ldwork) =  *(WORK + i + j * ldwork);
-#ifdef COMPLEX
-                    LAPACKE_zlacgv_work(1, WORK + j + i * ldwork, ldwork);
-#endif
-                }
+        for (j = 0; j < n; j++) {
+            for (i = 0; i < j; i++){
+                tmp = *(C + i + j*ldc);
+                *(WORK + i + j * ldwork) = tmp;
+                *(WORK + j + i * ldwork) = conj( tmp );
             }
-        
+            *(WORK + j + j * ldwork) =  *(C + ldc*j + j);
+        }
+
         /* Right */
-        CORE_zunmlq(PlasmaRight, PlasmaConjTrans, n, n, k, ib, 
+        CORE_zunmlq(PlasmaRight, PlasmaConjTrans, n, n, k, ib,
                     A, lda, T, ldt, WORK, ldwork, WORK+nb*ldwork, ldwork);
         /* Left */
-        CORE_zunmlq(PlasmaLeft, PlasmaNoTrans, n, n, k, ib, 
+        CORE_zunmlq(PlasmaLeft, PlasmaNoTrans, n, n, k, ib,
                     A, lda, T, ldt, WORK, ldwork, WORK+nb*ldwork, ldwork);
-        
-        /* 
-         * Copy back the final result to the upper part of C 
+
+        /*
+         * Copy back the final result to the upper part of C
          */
-        /* C = WORK */
-        for (i = 0; i < n; i++)
-            for (j = i; j < n; j++)
-                *(C + i + j*ldc) = *(WORK + i + j * ldwork);
+        LAPACKE_zlacpy_work( LAPACK_COL_MAJOR, lapack_const(PlasmaUpper), n, n, WORK, ldwork, C, ldc );
     }
     return 0;
 }
@@ -169,10 +208,12 @@ int CORE_zherfb( PLASMA_enum uplo, int n,
 void QUARK_CORE_zherfb(Quark *quark, Quark_Task_Flags *task_flags,
                        PLASMA_enum uplo,
                        int n, int k, int ib, int nb,
-                       PLASMA_Complex64_t *A, int lda,
-                       PLASMA_Complex64_t *T, int ldt,
+                       const PLASMA_Complex64_t *A, int lda,
+                       const PLASMA_Complex64_t *T, int ldt,
                        PLASMA_Complex64_t *C, int ldc)
 {
+    /* TODO: Understand why A needs to be INOUT and not INPUT */
+    DAG_CORE_HERFB;
     QUARK_Insert_Task(
         quark, CORE_zherfb_quark, task_flags,
         sizeof(PLASMA_enum),                     &uplo,  VALUE,
