@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -28,9 +28,9 @@
 #pragma weak CORE_zherk = PCORE_zherk
 #define CORE_zherk PCORE_zherk
 #endif
-void CORE_zherk(int uplo, int trans,
+void CORE_zherk(PLASMA_enum uplo, PLASMA_enum trans,
                 int N, int K,
-                double alpha, PLASMA_Complex64_t *A, int LDA,
+                double alpha, const PLASMA_Complex64_t *A, int LDA,
                 double beta, PLASMA_Complex64_t *C, int LDC)
 {
     cblas_zherk(
@@ -45,9 +45,9 @@ void CORE_zherk(int uplo, int trans,
  *
  **/
 void QUARK_CORE_zherk(Quark *quark, Quark_Task_Flags *task_flags,
-                      int uplo, int trans,
+                      PLASMA_enum uplo, PLASMA_enum trans,
                       int n, int k, int nb,
-                      double alpha, PLASMA_Complex64_t *A, int lda,
+                      double alpha, const PLASMA_Complex64_t *A, int lda,
                       double beta, PLASMA_Complex64_t *C, int ldc)
 {
     DAG_CORE_HERK;
@@ -74,8 +74,8 @@ void QUARK_CORE_zherk(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zherk_quark(Quark *quark)
 {
-    int uplo;
-    int trans;
+    PLASMA_enum uplo;
+    PLASMA_enum trans;
     int n;
     int k;
     double alpha;
