@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions mixed zc -> ds
@@ -25,7 +25,7 @@
 #define CORE_zlag2c PCORE_zlag2c
 #endif
 void CORE_zlag2c(int m, int n,
-                 PLASMA_Complex64_t *A, int lda,
+                 const PLASMA_Complex64_t *A, int lda,
                  PLASMA_Complex32_t *B, int ldb, int *info)
 {
     *info = LAPACKE_zlag2c_work(LAPACK_COL_MAJOR, m, n, A, lda, B, ldb);
@@ -35,7 +35,7 @@ void CORE_zlag2c(int m, int n,
  **/
 void QUARK_CORE_zlag2c(Quark *quark, Quark_Task_Flags *task_flags,
                        int m, int n, int nb,
-                       PLASMA_Complex64_t *A, int lda,
+                       const PLASMA_Complex64_t *A, int lda,
                        PLASMA_Complex32_t *B, int ldb,
                        PLASMA_sequence *sequence, PLASMA_request *request)
 {
@@ -88,20 +88,19 @@ void CORE_zlag2c_quark(Quark *quark)
 #define CORE_clag2z PCORE_clag2z
 #endif
 void CORE_clag2z(int m, int n,
-                 PLASMA_Complex32_t *A, int lda,
+                 const PLASMA_Complex32_t *A, int lda,
                  PLASMA_Complex64_t *B, int ldb)
 {
-    int info;
-    info = LAPACKE_clag2z_work(LAPACK_COL_MAJOR, m, n, A, lda, B, ldb);
+    LAPACKE_clag2z_work(LAPACK_COL_MAJOR, m, n, A, lda, B, ldb);
 }
 
 /***************************************************************************//**
  *
  **/
 void QUARK_CORE_clag2z(Quark *quark, Quark_Task_Flags *task_flags,
-                      int m, int n, int nb,
-                      PLASMA_Complex32_t *A, int lda,
-                      PLASMA_Complex64_t *B, int ldb)
+                       int m, int n, int nb,
+                       const PLASMA_Complex32_t *A, int lda,
+                       PLASMA_Complex64_t *B, int ldb)
 {
     QUARK_Insert_Task(quark, CORE_clag2z_quark, task_flags,
         sizeof(int),                        &m,     VALUE,
@@ -128,9 +127,8 @@ void CORE_clag2z_quark(Quark *quark)
     int lda;
     PLASMA_Complex64_t *B;
     int ldb;
-    int info;
 
     quark_unpack_args_6(quark, m, n, A, lda, B, ldb);
-    info = LAPACKE_clag2z_work(LAPACK_COL_MAJOR, m, n, A, lda, B, ldb);
+    LAPACKE_clag2z_work(LAPACK_COL_MAJOR, m, n, A, lda, B, ldb);
 }
 
