@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Julien Langou
  * @author Henricus Bouwmeester
  * @author Mathieu Faverge
@@ -27,7 +27,7 @@
 #define CORE_zlange PCORE_zlange
 #endif
 void CORE_zlange(int norm, int M, int N,
-                 PLASMA_Complex64_t *A, int LDA,
+                 const PLASMA_Complex64_t *A, int LDA,
                  double *work, double *normA)
 {
     *normA = LAPACKE_zlange_work(
@@ -41,7 +41,7 @@ void CORE_zlange(int norm, int M, int N,
  **/
 void QUARK_CORE_zlange(Quark *quark, Quark_Task_Flags *task_flags,
                        int norm, int M, int N,
-                       PLASMA_Complex64_t *A, int LDA, int szeA,
+                       const PLASMA_Complex64_t *A, int LDA, int szeA,
                        int szeW, double *result)
 {
     szeW = max(1, szeW);
@@ -57,6 +57,42 @@ void QUARK_CORE_zlange(Quark *quark, Quark_Task_Flags *task_flags,
         0);
 }
 
+/***************************************************************************//**
+ *
+ **/
+void QUARK_CORE_zlange_f1(Quark *quark, Quark_Task_Flags *task_flags,
+                          PLASMA_enum norm, int M, int N,
+                          const PLASMA_Complex64_t *A, int LDA, int szeA,
+                          int szeW, double *result,
+                          double *fake, int szeF)
+{
+    szeW = max(1, szeW);
+    DAG_CORE_LANGE;
+
+    if ( result == fake ) {
+        QUARK_Insert_Task(quark, CORE_zlange_quark, task_flags,
+            sizeof(PLASMA_enum),                &norm,  VALUE,
+            sizeof(int),                        &M,     VALUE,
+            sizeof(int),                        &N,     VALUE,
+            sizeof(PLASMA_Complex64_t)*szeA,     A,             INPUT,
+            sizeof(int),                        &LDA,   VALUE,
+            sizeof(double)*szeW,                 NULL,          SCRATCH,
+            sizeof(double),                      result,        OUTPUT | GATHERV,
+            0);
+    } else {
+        QUARK_Insert_Task(quark, CORE_zlange_f1_quark, task_flags,
+            sizeof(PLASMA_enum),                &norm,  VALUE,
+            sizeof(int),                        &M,     VALUE,
+            sizeof(int),                        &N,     VALUE,
+            sizeof(PLASMA_Complex64_t)*szeA,     A,             INPUT,
+            sizeof(int),                        &LDA,   VALUE,
+            sizeof(double)*szeW,                 NULL,          SCRATCH,
+            sizeof(double),                      result,        OUTPUT,
+            sizeof(double)*szeF,                 fake,          OUTPUT | GATHERV,
+            0);
+    }
+}
+
 /***************************************************************************//**
  *
  **/
@@ -81,29 +117,6 @@ void CORE_zlange_quark(Quark *quark)
         M, N, A, LDA, work);
 }
 
-/***************************************************************************//**
- *
- **/
-void QUARK_CORE_zlange_f1(Quark *quark, Quark_Task_Flags *task_flags,
-                          PLASMA_enum norm, int M, int N,
-                          PLASMA_Complex64_t *A, int LDA, int szeA,
-                          int szeW, double *result,
-                          double *fake, int szeF)
-{
-    szeW = max(1, szeW);
-    DAG_CORE_LANGE;
-    QUARK_Insert_Task(quark, CORE_zlange_f1_quark, task_flags,
-        sizeof(PLASMA_enum),                &norm,  VALUE,
-        sizeof(int),                        &M,     VALUE,
-        sizeof(int),                        &N,     VALUE,
-        sizeof(PLASMA_Complex64_t)*szeA,     A,             INPUT,
-        sizeof(int),                        &LDA,   VALUE,
-        sizeof(double)*szeW,                 NULL,          SCRATCH,
-        sizeof(double),                      result,        OUTPUT,
-        sizeof(double)*szeF,                 fake,          OUTPUT | GATHERV,
-        0);
-}
-
 /***************************************************************************//**
  *
  **/
