@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Julien Langou
  * @author Henricus Bouwmeester
  * @author Mathieu Faverge
@@ -26,8 +26,8 @@
 #pragma weak CORE_zlanhe = PCORE_zlanhe
 #define CORE_zlanhe PCORE_zlanhe
 #endif
-void CORE_zlanhe(int norm, int uplo, int N,
-                 PLASMA_Complex64_t *A, int LDA,
+void CORE_zlanhe(int norm, PLASMA_enum uplo, int N,
+                 const PLASMA_Complex64_t *A, int LDA,
                  double *work, double *normA)
 {
     *normA = LAPACKE_zlanhe_work(
@@ -40,8 +40,8 @@ void CORE_zlanhe(int norm, int uplo, int N,
  *
  **/
 void QUARK_CORE_zlanhe(Quark *quark, Quark_Task_Flags *task_flags,
-                       int norm, int uplo, int N,
-                       PLASMA_Complex64_t *A, int LDA, int szeA,
+                       int norm, PLASMA_enum uplo, int N,
+                       const PLASMA_Complex64_t *A, int LDA, int szeA,
                        int szeW, double *result)
 {
     szeW = max(1, szeW);
@@ -57,6 +57,42 @@ void QUARK_CORE_zlanhe(Quark *quark, Quark_Task_Flags *task_flags,
         0);
 }
 
+/***************************************************************************//**
+ *
+ **/
+void QUARK_CORE_zlanhe_f1(Quark *quark, Quark_Task_Flags *task_flags,
+                          PLASMA_enum norm, PLASMA_enum uplo, int N,
+                          const PLASMA_Complex64_t *A, int LDA, int szeA,
+                          int szeW, double *result,
+                          double *fake, int szeF)
+{
+    szeW = max(1, szeW);
+    DAG_CORE_LANHE;
+
+    if ( result == fake ) {
+        QUARK_Insert_Task(quark, CORE_zlanhe_quark, task_flags,
+            sizeof(PLASMA_enum),                &norm,  VALUE,
+            sizeof(PLASMA_enum),                &uplo,  VALUE,
+            sizeof(int),                        &N,     VALUE,
+            sizeof(PLASMA_Complex64_t)*szeA,     A,             INPUT,
+            sizeof(int),                        &LDA,   VALUE,
+            sizeof(double)*szeW,                 NULL,          SCRATCH,
+            sizeof(double)*szeF,                 result,        OUTPUT | GATHERV,
+            0);
+    } else {
+        QUARK_Insert_Task(quark, CORE_zlanhe_f1_quark, task_flags,
+            sizeof(PLASMA_enum),                &norm, VALUE,
+            sizeof(PLASMA_enum),                &uplo, VALUE,
+            sizeof(int),                        &N,    VALUE,
+            sizeof(PLASMA_Complex64_t)*szeA,     A,             INPUT,
+            sizeof(int),                        &LDA,  VALUE,
+            sizeof(double)*szeW,                 NULL,          SCRATCH,
+            sizeof(double),                      result,        OUTPUT,
+            sizeof(double)*szeF,                 fake,          OUTPUT | GATHERV,
+            0);
+    }
+}
+
 /***************************************************************************//**
  *
  **/
@@ -68,41 +104,19 @@ void CORE_zlanhe_quark(Quark *quark)
 {
     double *normA;
     int norm;
-    int uplo;
+    PLASMA_enum uplo;
     int N;
     PLASMA_Complex64_t *A;
     int LDA;
     double *work;
 
-    quark_unpack_args_7(quark, normA, norm, uplo, N, A, LDA, work);
+    quark_unpack_args_7(quark, norm, uplo, N, A, LDA, work, normA);
     *normA = LAPACKE_zlanhe_work(
         LAPACK_COL_MAJOR,
         lapack_const(norm), lapack_const(uplo),
         N, A, LDA, work);
 }
 
-/***************************************************************************//**
- *
- **/
-void QUARK_CORE_zlanhe_f1(Quark *quark, Quark_Task_Flags *task_flags,
-                          PLASMA_enum norm, PLASMA_enum uplo, int N,
-                          PLASMA_Complex64_t *A, int LDA, int szeA,
-                          int szeW, double *result,
-                          double *fake, int szeF)
-{
-    szeW = max(1, szeW);
-    DAG_CORE_LANHE;
-    QUARK_Insert_Task(quark, CORE_zlanhe_f1_quark, task_flags,
-        sizeof(PLASMA_enum),                &norm, VALUE,
-        sizeof(PLASMA_enum),                &uplo, VALUE,
-        sizeof(int),                        &N,    VALUE,
-        sizeof(PLASMA_Complex64_t)*szeA,     A,             INPUT,
-        sizeof(int),                        &LDA,  VALUE,
-        sizeof(double)*szeW,                 NULL,          SCRATCH,
-        sizeof(double),                      result,        OUTPUT,
-        sizeof(double)*szeF,                 fake,          OUTPUT | GATHERV,
-        0);
-}
 /***************************************************************************//**
  *
  **/
@@ -114,7 +128,7 @@ void CORE_zlanhe_f1_quark(Quark *quark)
 {
     double *normA;
     int norm;
-    int uplo;
+    PLASMA_enum uplo;
     int N;
     PLASMA_Complex64_t *A;
     int LDA;
