@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -26,7 +26,7 @@
  *******************************************************************************
  *
  * @param[in] uplo
- *          Specifies which elements of the matrix are to be set 
+ *          Specifies which elements of the matrix are to be set
  *          = PlasmaUpper: Upper part of A is set;
  *          = PlasmaLower: Lower part of A is set;
  *          = PlasmaUpperLower: ALL elements of A are set.
@@ -56,7 +56,7 @@
 #define CORE_zlaset PCORE_zlaset
 #endif
 void CORE_zlaset(PLASMA_enum uplo, int M, int N,
-                 PLASMA_Complex64_t alpha, PLASMA_Complex64_t beta, 
+                 PLASMA_Complex64_t alpha, PLASMA_Complex64_t beta,
                  PLASMA_Complex64_t *A, int LDA)
 {
     LAPACKE_zlaset_work(
@@ -70,16 +70,17 @@ void CORE_zlaset(PLASMA_enum uplo, int M, int N,
  **/
 void QUARK_CORE_zlaset(Quark *quark, Quark_Task_Flags *task_flags,
                        PLASMA_enum uplo, int M, int N,
-                       PLASMA_Complex64_t alpha, PLASMA_Complex64_t beta, 
+                       PLASMA_Complex64_t alpha, PLASMA_Complex64_t beta,
                        PLASMA_Complex64_t *A, int LDA)
 {
+    DAG_CORE_LASET;
     QUARK_Insert_Task(quark, CORE_zlaset_quark, task_flags,
         sizeof(PLASMA_enum),                &uplo,  VALUE,
         sizeof(int),                        &M,     VALUE,
         sizeof(int),                        &N,     VALUE,
         sizeof(PLASMA_Complex64_t),         &alpha, VALUE,
         sizeof(PLASMA_Complex64_t),         &beta,  VALUE,
-        sizeof(PLASMA_Complex64_t)*M*N,     A,      OUTPUT,
+        sizeof(PLASMA_Complex64_t)*LDA*N,    A,      OUTPUT,
         sizeof(int),                        &LDA,   VALUE,
         0);
 }
@@ -93,7 +94,7 @@ void QUARK_CORE_zlaset(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zlaset_quark(Quark *quark)
 {
-    int uplo;
+    PLASMA_enum uplo;
     int M;
     int N;
     PLASMA_Complex64_t alpha;
