@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -26,7 +26,7 @@
  *******************************************************************************
  *
  * @param[in] uplo
- *          Specifies which elements of the matrix are to be set 
+ *          Specifies which elements of the matrix are to be set
  *          = PlasmaUpper: STRICT Upper part of A is set to alpha;
  *          = PlasmaLower: STRICT Lower part of A is set to alpha;
  *          = PlasmaUpperLower: ALL elements of A are set to alpha.
@@ -83,6 +83,7 @@ void QUARK_CORE_zlaset2(Quark *quark, Quark_Task_Flags *task_flags,
                        PLASMA_enum uplo, int M, int N,
                        PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int LDA)
 {
+    DAG_CORE_LASET;
     QUARK_Insert_Task(quark, CORE_zlaset2_quark, task_flags,
         sizeof(PLASMA_enum),                &uplo,  VALUE,
         sizeof(int),                        &M,     VALUE,
@@ -102,7 +103,7 @@ void QUARK_CORE_zlaset2(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zlaset2_quark(Quark *quark)
 {
-    int uplo;
+    PLASMA_enum uplo;
     int M;
     int N;
     PLASMA_Complex64_t alpha;
