@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Julien Langou
  * @author Henricus Bouwmeester
  * @author Mathieu Faverge
@@ -26,17 +26,16 @@
 #pragma weak CORE_zlauum = PCORE_zlauum
 #define CORE_zlauum PCORE_zlauum
 #endif
-void CORE_zlauum(int uplo, int N, PLASMA_Complex64_t *A, int LDA)
+void CORE_zlauum(PLASMA_enum uplo, int N, PLASMA_Complex64_t *A, int LDA)
 {
-    int info;
-    info = LAPACKE_zlauum_work(LAPACK_COL_MAJOR, lapack_const(uplo), N, A, LDA );
+    LAPACKE_zlauum_work(LAPACK_COL_MAJOR, lapack_const(uplo), N, A, LDA );
 }
 
 /***************************************************************************//**
  *
  **/
 void QUARK_CORE_zlauum(Quark *quark, Quark_Task_Flags *task_flags,
-                       int uplo, int n, int nb,
+                       PLASMA_enum uplo, int n, int nb,
                        PLASMA_Complex64_t *A, int lda)
 {
     DAG_CORE_LAUUM;
@@ -57,12 +56,11 @@ void QUARK_CORE_zlauum(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zlauum_quark(Quark *quark)
 {
-    int uplo;
+    PLASMA_enum uplo;
     int N;
     PLASMA_Complex64_t *A;
     int LDA;
-    int info;
 
     quark_unpack_args_4(quark, uplo, N, A, LDA);
-    info = LAPACKE_zlauum_work(LAPACK_COL_MAJOR, lapack_const(uplo), N, A, LDA);
+    LAPACKE_zlauum_work(LAPACK_COL_MAJOR, lapack_const(uplo), N, A, LDA);
 }
