@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Piotr Luszczek
  * @author Pierre Lemarinier
  * @author Mathieu Faverge
@@ -65,10 +65,10 @@ void CORE_zplgsy( PLASMA_Complex64_t bump, int m, int n, PLASMA_Complex64_t *A,
                   int bigM, int m0, int n0, unsigned long long int seed )
 {
     PLASMA_Complex64_t *tmp = A;
-    int i, j;
+    int64_t i, j;
     unsigned long long int ran, jump;
 
-    jump = m0 + n0 * bigM;
+    jump = (unsigned long long int)m0 + (unsigned long long int)n0 * (unsigned long long int)bigM;
 
     /*
      * Tile diagonal
@@ -87,7 +87,7 @@ void CORE_zplgsy( PLASMA_Complex64_t bump, int m, int n, PLASMA_Complex64_t *A,
                 tmp++;
             }
             tmp  += (lda - i + j + 1);
-            jump += bigM + j;
+            jump += bigM + 1;
         }
 
         for (j = 0; j < n; j++) {
@@ -97,7 +97,7 @@ void CORE_zplgsy( PLASMA_Complex64_t bump, int m, int n, PLASMA_Complex64_t *A,
                 A[lda*j+i] = A[lda*i+j];
             }
         }
-    } 
+    }
     /*
      * Lower part
      */
