@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Piotr Luszczek
  * @author Pierre Lemarinier
  * @author Mathieu Faverge
@@ -65,10 +65,10 @@ void CORE_zplrnt( int m, int n, PLASMA_Complex64_t *A, int lda,
                   int bigM, int m0, int n0, unsigned long long int seed )
 {
     PLASMA_Complex64_t *tmp = A;
-    int i, j;
+    int64_t i, j;
     unsigned long long int ran, jump;
 
-    jump = m0 + n0 * bigM;
+    jump = (unsigned long long int)m0 + (unsigned long long int)n0 * (unsigned long long int)bigM;
 
     for (j=0; j<n; ++j ) {
         ran = Rnd64_jump( NBELEM*jump, seed );
