@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -26,7 +26,7 @@
 #pragma weak CORE_zpotrf = PCORE_zpotrf
 #define CORE_zpotrf PCORE_zpotrf
 #endif
-void CORE_zpotrf(int uplo, int N, PLASMA_Complex64_t *A, int LDA, int *INFO)
+void CORE_zpotrf(PLASMA_enum uplo, int N, PLASMA_Complex64_t *A, int LDA, int *INFO)
 {
     *INFO = LAPACKE_zpotrf_work(
         LAPACK_COL_MAJOR,
@@ -38,7 +38,7 @@ void CORE_zpotrf(int uplo, int N, PLASMA_Complex64_t *A, int LDA, int *INFO)
  *
  **/
 void QUARK_CORE_zpotrf(Quark *quark, Quark_Task_Flags *task_flags,
-                       int uplo, int n, int nb,
+                       PLASMA_enum uplo, int n, int nb,
                        PLASMA_Complex64_t *A, int lda,
                        PLASMA_sequence *sequence, PLASMA_request *request,
                        int iinfo)
@@ -64,7 +64,7 @@ void QUARK_CORE_zpotrf(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zpotrf_quark(Quark *quark)
 {
-    int uplo;
+    PLASMA_enum uplo;
     int n;
     PLASMA_Complex64_t *A;
     int lda;
@@ -80,5 +80,5 @@ void CORE_zpotrf_quark(Quark *quark)
         lapack_const(uplo),
         n, A, lda);
     if (sequence->status == PLASMA_SUCCESS && info != 0)
-      plasma_sequence_flush(quark, sequence, request, iinfo+info);
+        plasma_sequence_flush(quark, sequence, request, iinfo+info);
 }
