@@ -6,11 +6,11 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- *  This work is the implementation of an inplace transformation 
- *  based on the GKK algorithm by Gustavson, Karlsson, Kagstrom 
+ *  This work is the implementation of an inplace transformation
+ *  based on the GKK algorithm by Gustavson, Karlsson, Kagstrom
  *  and its fortran implementation.
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  *
@@ -65,7 +65,8 @@
 #endif
 void CORE_zshiftw(int s, int cl, int m, int n, int L, PLASMA_Complex64_t *A, PLASMA_Complex64_t *W) {
     int64_t k, k1;
-    int     i, j, q, kL, k1L;
+    int     i, q, kL, k1L;
+    size_t  chunk_size = L*sizeof(PLASMA_Complex64_t);
 
     q = m * n - 1;
     k = s;
@@ -73,33 +74,30 @@ void CORE_zshiftw(int s, int cl, int m, int n, int L, PLASMA_Complex64_t *A, PLA
     if( cl != 0 ) {
         for (i=1; i<cl; i++) {
             k1 = (k * m) % (int64_t)q;
-            
+
             /* A(k*L:k*L+L-1) = A(k1*L:k1*L+L-1) */
             kL  = k *L;
             k1L = k1*L;
 
-            for(j=0; j<L; j++) {
-                A[kL+j] = A[k1L+j];
-            }
-            k = k1;
-        }
-    } 
-    else {
-        while (1) {
-            k1 = (k * m) % (int64_t)q;
-            if( k1 == s ) 
-                break;
-            
-            /* A(k*L:k*L+L-1) = A(k1*L:k1*L+L-1) */
-            kL  = k *L;
-            k1L = k1*L;
-            for (j=0; j<L; j++) {
-                A[kL+j] = A[k1L+j];
-            }
+            memcpy( A+kL, A+k1L, chunk_size);
             k = k1;
         }
     }
-    memcpy(&(A[k*L]), W, L*sizeof(PLASMA_Complex64_t));
+    else {
+        while (1) {
+            k1 = (k * m) % (int64_t)q;
+            if( k1 == s )
+                break;
+
+            /* A(k*L:k*L+L-1) = A(k1*L:k1*L+L-1) */
+            kL  = k *L;
+            k1L = k1*L;
+            memcpy( A+kL, A+k1L, chunk_size);
+            k = k1;
+        }
+    }
+    kL = k*L;
+    memcpy(A+kL, W, chunk_size);
 }
 
 /***************************************************************************//**
