@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -21,9 +21,10 @@
  *
  * @ingroup CORE_PLASMA_Complex64_t
  *
- *  CORE_ztstrf computes an LU factorization of a complex matrix formed
- *  by an upper triangular M1-by-N1 tile U on top of a M2-by-N2 tile A
- *  (N1 == N2) using partial pivoting with row interchanges.
+ *  CORE_zssssm applies the LU factorization update from a complex
+ *  matrix formed by a lower triangular IB-by-K tile L1 on top of a
+ *  M2-by-K tile L2 to a second complex matrix formed by a M1-by-N1
+ *  tile A1 on top of a M2-by-N2 tile A2 (N1 == N2).
  *
  *  This is the right-looking Level 2.5 BLAS version of the algorithm.
  *
@@ -36,7 +37,8 @@
  *         The number of columns of the tile A1.  N1 >= 0.
  *
  * @param[in] M2
- *         The number of rows of the tile A2.  M2 >= 0.
+ *         The number of rows of the tile A2 and of the tile L2.
+ *         M2 >= 0.
  *
  * @param[in] N2
  *         The number of columns of the tile A2.  N2 >= 0.
@@ -49,32 +51,34 @@
  *
  * @param[in,out] A1
  *         On entry, the M1-by-N1 tile A1.
- *         On exit, A1 is overwritten by the application of L.
+ *         On exit, A1 is updated by the application of L (L1 L2).
  *
  * @param[in] LDA1
  *         The leading dimension of the array A1.  LDA1 >= max(1,M1).
  *
  * @param[in,out] A2
  *         On entry, the M2-by-N2 tile A2.
- *         On exit, A2 is overwritten by the application of L.
+ *         On exit, A2 is updated by the application of L (L1 L2).
  *
  * @param[in] LDA2
  *         The leading dimension of the array A2.  LDA2 >= max(1,M2).
  *
  * @param[in] L1
- *         The IB-by-K lower triangular tile as returned by CORE_ztstrf.
+ *         The IB-by-K lower triangular tile as returned by
+ *         CORE_ztstrf.
  *
  * @param[in] LDL1
  *         The leading dimension of the array L1.  LDL1 >= max(1,IB).
  *
  * @param[in] L2
- *         The M2-by-N2 tile as returned by CORE_ztstrf.
+ *         The M2-by-K tile as returned by CORE_ztstrf.
  *
  * @param[in] LDL2
  *         The leading dimension of the array L2.  LDL2 >= max(1,M2).
  *
  * @param[in] IPIV
- *         as returned by CORE_ztstrf.
+ *         The pivot indices array of size K as returned by
+ *         CORE_ztstrf.
  *
  *******************************************************************************
  *
@@ -90,9 +94,9 @@
 int CORE_zssssm(int M1, int N1, int M2, int N2, int K, int IB,
                 PLASMA_Complex64_t *A1, int LDA1,
                 PLASMA_Complex64_t *A2, int LDA2,
-                PLASMA_Complex64_t *L1, int LDL1,
-                PLASMA_Complex64_t *L2, int LDL2,
-                int *IPIV)
+                const PLASMA_Complex64_t *L1, int LDL1,
+                const PLASMA_Complex64_t *L2, int LDL2,
+                const int *IPIV)
 {
     static PLASMA_Complex64_t zone  = 1.0;
     static PLASMA_Complex64_t mzone =-1.0;
@@ -185,9 +189,9 @@ void QUARK_CORE_zssssm(Quark *quark, Quark_Task_Flags *task_flags,
                        int m1, int n1, int m2, int n2, int k, int ib, int nb,
                        PLASMA_Complex64_t *A1, int lda1,
                        PLASMA_Complex64_t *A2, int lda2,
-                       PLASMA_Complex64_t *L1, int ldl1,
-                       PLASMA_Complex64_t *L2, int ldl2,
-                       int *IPIV)
+                       const PLASMA_Complex64_t *L1, int ldl1,
+                       const PLASMA_Complex64_t *L2, int ldl2,
+                       const int *IPIV)
 {
     DAG_CORE_SSSSM;
     QUARK_Insert_Task(quark, CORE_zssssm_quark, task_flags,
