@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -25,9 +25,9 @@
 #pragma weak CORE_zsyrk = PCORE_zsyrk
 #define CORE_zsyrk PCORE_zsyrk
 #endif
-void CORE_zsyrk(int uplo, int trans,
+void CORE_zsyrk(PLASMA_enum uplo, PLASMA_enum trans,
                 int N, int K,
-                PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int LDA,
+                PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int LDA,
                 PLASMA_Complex64_t beta, PLASMA_Complex64_t *C, int LDC)
 {
     cblas_zsyrk(
@@ -42,9 +42,9 @@ void CORE_zsyrk(int uplo, int trans,
  *
  **/
 void QUARK_CORE_zsyrk(Quark *quark, Quark_Task_Flags *task_flags,
-                      int uplo, int trans,
+                      PLASMA_enum uplo, PLASMA_enum trans,
                       int n, int k, int nb,
-                      PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int lda,
+                      PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int lda,
                       PLASMA_Complex64_t beta, PLASMA_Complex64_t *C, int ldc)
 {
     DAG_CORE_SYRK;
@@ -71,8 +71,8 @@ void QUARK_CORE_zsyrk(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zsyrk_quark(Quark *quark)
 {
-    int uplo;
-    int trans;
+    PLASMA_enum uplo;
+    PLASMA_enum trans;
     int n;
     int k;
     PLASMA_Complex64_t alpha;
