@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Julien Langou
  * @author Henricus Bouwmeester
  * @author Mathieu Faverge
@@ -25,11 +25,11 @@
 #pragma weak CORE_ztrmm = PCORE_ztrmm
 #define CORE_ztrmm PCORE_ztrmm
 #endif
-void CORE_ztrmm(int side, int uplo,
-                int transA, int diag,
-                int M, int N, 
-                PLASMA_Complex64_t alpha, 
-                PLASMA_Complex64_t *A, int LDA,
+void CORE_ztrmm(PLASMA_enum side, PLASMA_enum uplo,
+                PLASMA_enum transA, PLASMA_enum diag,
+                int M, int N,
+                PLASMA_Complex64_t alpha,
+                const PLASMA_Complex64_t *A, int LDA,
                 PLASMA_Complex64_t *B, int LDB)
 {
     cblas_ztrmm(
@@ -45,9 +45,9 @@ void CORE_ztrmm(int side, int uplo,
  *
  **/
 void QUARK_CORE_ztrmm(Quark *quark, Quark_Task_Flags *task_flags,
-                      int side, int uplo, int transA, int diag,
+                      PLASMA_enum side, PLASMA_enum uplo, PLASMA_enum transA, PLASMA_enum diag,
                       int m, int n, int nb,
-                      PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int lda,
+                      PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int lda,
                       PLASMA_Complex64_t *B, int ldb)
 {
     DAG_CORE_TRMM;
@@ -75,10 +75,10 @@ void QUARK_CORE_ztrmm(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_ztrmm_quark(Quark *quark)
 {
-    int side;
-    int uplo;
-    int transA;
-    int diag;
+    PLASMA_enum side;
+    PLASMA_enum uplo;
+    PLASMA_enum transA;
+    PLASMA_enum diag;
     int M;
     int N;
     PLASMA_Complex64_t alpha;
@@ -101,9 +101,9 @@ void CORE_ztrmm_quark(Quark *quark)
  *
  **/
 void QUARK_CORE_ztrmm_p2(Quark *quark, Quark_Task_Flags *task_flags,
-                         int side, int uplo, int transA, int diag,
+                         PLASMA_enum side, PLASMA_enum uplo, PLASMA_enum transA, PLASMA_enum diag,
                          int m, int n, int nb,
-                         PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int lda,
+                         PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int lda,
                          PLASMA_Complex64_t **B, int ldb)
 {
     DAG_CORE_TRMM;
@@ -131,10 +131,10 @@ void QUARK_CORE_ztrmm_p2(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_ztrmm_p2_quark(Quark *quark)
 {
-    int side;
-    int uplo;
-    int transA;
-    int diag;
+    PLASMA_enum side;
+    PLASMA_enum uplo;
+    PLASMA_enum transA;
+    PLASMA_enum diag;
     int M;
     int N;
     PLASMA_Complex64_t alpha;
