@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -25,10 +25,10 @@
 #pragma weak CORE_ztrsm = PCORE_ztrsm
 #define CORE_ztrsm PCORE_ztrsm
 #endif
-void CORE_ztrsm(int side, int uplo,
-                int transA, int diag,
+void CORE_ztrsm(PLASMA_enum side, PLASMA_enum uplo,
+                PLASMA_enum transA, PLASMA_enum diag,
                 int M, int N,
-                PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int LDA,
+                PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int LDA,
                 PLASMA_Complex64_t *B, int LDB)
 {
     cblas_ztrsm(
@@ -44,9 +44,9 @@ void CORE_ztrsm(int side, int uplo,
  *
  **/
 void QUARK_CORE_ztrsm(Quark *quark, Quark_Task_Flags *task_flags,
-                      int side, int uplo, int transA, int diag,
+                      PLASMA_enum side, PLASMA_enum uplo, PLASMA_enum transA, PLASMA_enum diag,
                       int m, int n, int nb,
-                      PLASMA_Complex64_t alpha, PLASMA_Complex64_t *A, int lda,
+                      PLASMA_Complex64_t alpha, const PLASMA_Complex64_t *A, int lda,
                       PLASMA_Complex64_t *B, int ldb)
 {
     DAG_CORE_TRSM;
@@ -60,7 +60,7 @@ void QUARK_CORE_ztrsm(Quark *quark, Quark_Task_Flags *task_flags,
         sizeof(PLASMA_Complex64_t),         &alpha,     VALUE,
         sizeof(PLASMA_Complex64_t)*nb*nb,    A,                 INPUT,
         sizeof(int),                        &lda,       VALUE,
-        sizeof(PLASMA_Complex64_t)*nb*nb,    B,                 INOUT | LOCALITY,
+        sizeof(PLASMA_Complex64_t)*nb*nb,    B,                 INOUT,
         sizeof(int),                        &ldb,       VALUE,
         0);
 }
@@ -74,10 +74,10 @@ void QUARK_CORE_ztrsm(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_ztrsm_quark(Quark *quark)
 {
-    int side;
-    int uplo;
-    int transA;
-    int diag;
+    PLASMA_enum side;
+    PLASMA_enum uplo;
+    PLASMA_enum transA;
+    PLASMA_enum diag;
     int m;
     int n;
     PLASMA_Complex64_t alpha;
