@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Julien Langou
  * @author Henricus Bouwmeester
  * @author Mathieu Faverge
@@ -26,7 +26,7 @@
 #pragma weak CORE_ztrtri = PCORE_ztrtri
 #define CORE_ztrtri PCORE_ztrtri
 #endif
-void CORE_ztrtri(int uplo, int diag, int N, PLASMA_Complex64_t *A, int LDA, int *info)
+void CORE_ztrtri(PLASMA_enum uplo, PLASMA_enum diag, int N, PLASMA_Complex64_t *A, int LDA, int *info)
 {
     *info = LAPACKE_ztrtri_work(
         LAPACK_COL_MAJOR,
@@ -38,7 +38,7 @@ void CORE_ztrtri(int uplo, int diag, int N, PLASMA_Complex64_t *A, int LDA, int
  *
  **/
 void QUARK_CORE_ztrtri(Quark *quark, Quark_Task_Flags *task_flags,
-                       int uplo, int diag,
+                       PLASMA_enum uplo, PLASMA_enum diag,
                        int n, int nb,
                        PLASMA_Complex64_t *A, int lda,
                        PLASMA_sequence *sequence, PLASMA_request *request,
@@ -66,8 +66,8 @@ void QUARK_CORE_ztrtri(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_ztrtri_quark(Quark *quark)
 {
-    int uplo;
-    int diag;
+    PLASMA_enum uplo;
+    PLASMA_enum diag;
     int N;
     PLASMA_Complex64_t *A;
     int LDA;
@@ -82,6 +82,6 @@ void CORE_ztrtri_quark(Quark *quark)
         LAPACK_COL_MAJOR,
         lapack_const(uplo), lapack_const(diag),
         N, A, LDA);
-    if ((sequence->status == PLASMA_SUCCESS) && (info != 0))
+    if ((sequence->status == PLASMA_SUCCESS) && (info > 0))
         plasma_sequence_flush(quark, sequence, request, iinfo + info);
 }
