@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -95,6 +95,14 @@
 #if defined(PLASMA_HAVE_WEAK)
 #pragma weak CORE_ztslqt = PCORE_ztslqt
 #define CORE_ztslqt PCORE_ztslqt
+#define CORE_ztsmlq PCORE_ztsmlq
+int  CORE_ztsmlq(PLASMA_enum side, PLASMA_enum trans,
+                 int M1, int N1, int M2, int N2, int K, int IB,
+                 PLASMA_Complex64_t *A1, int LDA1,
+                 PLASMA_Complex64_t *A2, int LDA2,
+                 const PLASMA_Complex64_t *V, int LDV,
+                 const PLASMA_Complex64_t *T, int LDT,
+                 PLASMA_Complex64_t *WORK, int LDWORK);
 #endif
 int CORE_ztslqt(int M, int N, int IB,
                 PLASMA_Complex64_t *A1, int LDA1,
@@ -214,11 +222,12 @@ void QUARK_CORE_ztslqt(Quark *quark, Quark_Task_Flags *task_flags,
                        PLASMA_Complex64_t *T, int ldt)
 {
     DAG_CORE_TSLQT;
+    /* TODO: Regions are commented for 2-sided factorizations that have a dependency problem with it */
     QUARK_Insert_Task(quark, CORE_ztslqt_quark, task_flags,
         sizeof(int),                        &m,     VALUE,
         sizeof(int),                        &n,     VALUE,
         sizeof(int),                        &ib,    VALUE,
-        sizeof(PLASMA_Complex64_t)*nb*nb,    A1,            INOUT | QUARK_REGION_D | QUARK_REGION_L,
+        sizeof(PLASMA_Complex64_t)*nb*nb,    A1,            INOUT /*| QUARK_REGION_D | QUARK_REGION_L */,
         sizeof(int),                        &lda1,  VALUE,
         sizeof(PLASMA_Complex64_t)*nb*nb,    A2,            INOUT | LOCALITY,
         sizeof(int),                        &lda2,  VALUE,
