@@ -6,11 +6,12 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
  * @author Azzam Haidar
+ * @author Dulceneia Becker
  * @date 2010-11-15
  * @precisions normal z -> c d s
  *
@@ -26,7 +27,7 @@
  *
  *                        SIDE = 'L'        SIDE = 'R'
  *    TRANS = 'N':         Q * | A1 |     | A1 A2 | * Q
- *                             | A2 |     
+ *                             | A2 |
  *
  *    TRANS = 'C':      Q**H * | A1 |     | A1 A2 | * Q**H
  *                             | A2 |
@@ -91,7 +92,7 @@
  * @param[in] LDV
  *         The leading dimension of the array V. LDV >= max(1,K).
  *
- * @param[out] T
+ * @param[in] T
  *         The IB-by-N1 triangular factor T of the block reflector.
  *         T is upper triangular by block (economic storage);
  *         The rest of the array is not referenced.
@@ -100,12 +101,12 @@
  *         The leading dimension of the array T. LDT >= IB.
  *
  * @param[out] WORK
- *         Workspace array of size 
+ *         Workspace array of size
  *             LDWORK-by-M1 if side == PlasmaLeft
  *             LDWORK-by-IB if side == PlasmaRight
  *
  * @param[in] LDWORK
- *         The leading dimension of the array WORK. 
+ *         The leading dimension of the array WORK.
  *             LDWORK >= max(1,IB) if side == PlasmaLeft
  *             LDWORK >= max(1,N1) if side == PlasmaRight
  *
@@ -119,14 +120,13 @@
 #if defined(PLASMA_HAVE_WEAK)
 #pragma weak CORE_ztsmlq = PCORE_ztsmlq
 #define CORE_ztsmlq PCORE_ztsmlq
-#define CORE_ztsrfb PCORE_ztsrfb
 #endif
-int CORE_ztsmlq(int side, int trans,
+int CORE_ztsmlq(PLASMA_enum side, PLASMA_enum trans,
                 int M1, int N1, int M2, int N2, int K, int IB,
                 PLASMA_Complex64_t *A1, int LDA1,
                 PLASMA_Complex64_t *A2, int LDA2,
-                PLASMA_Complex64_t *V, int LDV,
-                PLASMA_Complex64_t *T, int LDT,
+                const PLASMA_Complex64_t *V, int LDV,
+                const PLASMA_Complex64_t *T, int LDT,
                 PLASMA_Complex64_t *WORK, int LDWORK)
 {
     int i, i1, i3;
@@ -163,17 +163,17 @@ int CORE_ztsmlq(int side, int trans,
         coreblas_error(4, "Illegal value of N1");
         return -4;
     }
-    if ( (M2 < 0) || 
+    if ( (M2 < 0) ||
          ( (M2 != M1) && (side == PlasmaRight) ) ){
         coreblas_error(5, "Illegal value of M2");
         return -5;
     }
-    if ( (N2 < 0) || 
+    if ( (N2 < 0) ||
          ( (N2 != N1) && (side == PlasmaLeft) ) ){
         coreblas_error(6, "Illegal value of N2");
         return -6;
     }
-    if ((K < 0) || 
+    if ((K < 0) ||
         ( (side == PlasmaLeft)  && (K > M1) ) ||
         ( (side == PlasmaRight) && (K > N1) ) ) {
         coreblas_error(7, "Illegal value of K");
@@ -243,11 +243,11 @@ int CORE_ztsmlq(int side, int trans,
             jc = i;
         }
         /*
-         * Apply H or H'
+         * Apply H or H' (NOTE: CORE_zparfb used to be CORE_ztsrfb)
          */
-        CORE_ztsrfb(
+        CORE_zparfb(
             side, trans, PlasmaForward, PlasmaRowwise,
-            mi, ni, M2, N2, kb,
+            mi, ni, M2, N2, kb, 0,
             &A1[LDA1*jc+ic], LDA1,
             A2, LDA2,
             &V[i], LDV,
@@ -261,12 +261,12 @@ int CORE_ztsmlq(int side, int trans,
  *
  **/
 void QUARK_CORE_ztsmlq(Quark *quark, Quark_Task_Flags *task_flags,
-                       int side, int trans,
+                       PLASMA_enum side, PLASMA_enum trans,
                        int m1, int n1, int m2, int n2, int k, int ib, int nb,
                        PLASMA_Complex64_t *A1, int lda1,
                        PLASMA_Complex64_t *A2, int lda2,
-                       PLASMA_Complex64_t *V, int ldv,
-                       PLASMA_Complex64_t *T, int ldt)
+                       const PLASMA_Complex64_t *V, int ldv,
+                       const PLASMA_Complex64_t *T, int ldt)
 {
     int ldwork = side == PlasmaLeft ? ib : nb;
 
@@ -302,8 +302,8 @@ void QUARK_CORE_ztsmlq(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_ztsmlq_quark(Quark *quark)
 {
-    int side;
-    int trans;
+    PLASMA_enum side;
+    PLASMA_enum trans;
     int m1;
     int n1;
     int m2;
@@ -321,8 +321,8 @@ void CORE_ztsmlq_quark(Quark *quark)
     PLASMA_Complex64_t *WORK;
     int ldwork;
 
-    quark_unpack_args_18(quark, side, trans, m1, n1, m2, n2, k, ib, 
+    quark_unpack_args_18(quark, side, trans, m1, n1, m2, n2, k, ib,
                          A1, lda1, A2, lda2, V, ldv, T, ldt, WORK, ldwork);
-    CORE_ztsmlq(side, trans, m1, n1, m2, n2, k, ib, 
+    CORE_ztsmlq(side, trans, m1, n1, m2, n2, k, ib,
                 A1, lda1, A2, lda2, V, ldv, T, ldt, WORK, ldwork);
 }
