@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Azzam Haidar
@@ -34,74 +34,79 @@
  *
  *******************************************************************************
  *
- * @param[in] side
- *         @arg PlasmaLeft  : apply Q or Q**H from the Left;
- *         @arg PlasmaRight : apply Q or Q**H from the Right.
+ * @param[in] m1
+ *         The number of rows of the tile A1. m1 >= 0.
  *
- * @param[in] trans
- *         @arg PlasmaNoTrans   :  No transpose, apply Q;
- *         @arg PlasmaConjTrans :  ConjTranspose, apply Q**H.
+ * @param[in] n1
+ *         The number of columns of the tile A1. n1 >= 0.
  *
- * @param[in] M1
- *         The number of rows of the tile A1. M1 >= 0.
+ * @param[in] m2
+ *         The number of rows of the tile A2. m2 >= 0.
  *
- * @param[in] N1
- *         The number of columns of the tile A1. N1 >= 0.
+ * @param[in] n2
+ *         The number of columns of the tile A2. n2 >= 0.
  *
- * @param[in] M2
- *         The number of rows of the tile A2. M2 >= 0.
- *         M2 = M1 if side == PlasmaRight.
+ * @param[in] m3
+ *         The number of rows of the tile A3. m3 >= 0.
  *
- * @param[in] N2
- *         The number of columns of the tile A2. N2 >= 0.
- *         N2 = N1 if side == PlasmaLeft.
+ * @param[in] n3
+ *         The number of columns of the tile A3. n3 >= 0.
  *
- * @param[in] K
+ * @param[in] k
  *         The number of elementary reflectors whose product defines
  *         the matrix Q.
  *
- * @param[in] IB
- *         The inner-blocking size.  IB >= 0.
+ * @param[in] ib
+ *         The inner-blocking size.  ib >= 0.
+ *
+ * @param[in] nb
+ *         The blocking size.  nb >= 0.
  *
  * @param[in,out] A1
- *         On entry, the M1-by-N1 tile A1.
+ *         On entry, the m1-by-n1 tile A1.
  *         On exit, A1 is overwritten by the application of Q.
  *
- * @param[in] LDA1
- *         The leading dimension of the array A1. LDA1 >= max(1,M1).
+ * @param[in] lda1
+ *         The leading dimension of the array A1. lda1 >= max(1,m1).
  *
  * @param[in,out] A2
- *         On entry, the M2-by-N2 tile A2.
+ *         On entry, the m2-by-n2 tile A2.
  *         On exit, A2 is overwritten by the application of Q.
  *
- * @param[in] LDA2
- *         The leading dimension of the tile A2. LDA2 >= max(1,M2).
+ * @param[in] lda2
+ *         The leading dimension of the tile A2. lda2 >= max(1,m2).
+ *
+ * @param[in,out] A3
+ *         On entry, the m3-by-n3 tile A3.
+ *
+ * @param[in] lda3
+ *         The leading dimension of the tile A3. lda3 >= max(1,m3).
  *
  * @param[in] V
  *         The i-th row must contain the vector which defines the
  *         elementary reflector H(i), for i = 1,2,...,k, as returned by
  *         CORE_ZTSLQT in the first k rows of its array argument V.
  *
- * @param[in] LDV
- *         The leading dimension of the array V. LDV >= max(1,K).
+ * @param[in] ldv
+ *         The leading dimension of the array V. ldv >= max(1,K).
  *
- * @param[out] T
- *         The IB-by-N1 triangular factor T of the block reflector.
+ * @param[in] T
+ *         The IB-by-n1 triangular factor T of the block reflector.
  *         T is upper triangular by block (economic storage);
  *         The rest of the array is not referenced.
  *
- * @param[in] LDT
- *         The leading dimension of the array T. LDT >= IB.
+ * @param[in] ldt
+ *         The leading dimension of the array T. ldt >= IB.
  *
  * @param[out] WORK
- *         Workspace array of size 
- *             LDWORK-by-M1 if side == PlasmaLeft
+ *         Workspace array of size
+ *             LDWORK-by-m1 if side == PlasmaLeft
  *             LDWORK-by-IB if side == PlasmaRight
  *
- * @param[in] LDWORK
- *         The leading dimension of the array WORK. 
+ * @param[in] ldwork
+ *         The leading dimension of the array WORK.
  *             LDWORK >= max(1,IB) if side == PlasmaLeft
- *             LDWORK >= max(1,N1) if side == PlasmaRight
+ *             LDWORK >= max(1,n1) if side == PlasmaRight
  *
  *******************************************************************************
  *
@@ -113,14 +118,22 @@
 #if defined(PLASMA_HAVE_WEAK)
 #pragma weak CORE_ztsmlq_corner = PCORE_ztsmlq_corner
 #define CORE_ztsmlq_corner PCORE_ztsmlq_corner
+#define CORE_ztsmlq PCORE_ztsmlq
+int  CORE_ztsmlq(PLASMA_enum side, PLASMA_enum trans,
+                 int m1, int n1, int m2, int n2, int K, int IB,
+                 PLASMA_Complex64_t *A1, int lda1,
+                 PLASMA_Complex64_t *A2, int lda2,
+                 const PLASMA_Complex64_t *V, int ldv,
+                 const PLASMA_Complex64_t *T, int ldt,
+                 PLASMA_Complex64_t *WORK, int LDWORK);
 #endif
 int CORE_ztsmlq_corner( int m1, int n1, int m2, int n2, int m3, int n3,
                         int k, int ib, int nb,
                         PLASMA_Complex64_t *A1, int lda1,
                         PLASMA_Complex64_t *A2, int lda2,
                         PLASMA_Complex64_t *A3, int lda3,
-                        PLASMA_Complex64_t *V, int ldv,
-                        PLASMA_Complex64_t *T, int ldt,
+                        const PLASMA_Complex64_t *V, int ldv,
+                        const PLASMA_Complex64_t *T, int ldt,
                         PLASMA_Complex64_t *WORK, int ldwork)
 {
     PLASMA_enum side;
@@ -151,9 +164,9 @@ int CORE_ztsmlq_corner( int m1, int n1, int m2, int n2, int m3, int n3,
     trans = PlasmaConjTrans;
 
     /*  Right application on |A1 A2| */
-    CORE_ztsmlq(side, trans, m1, n1, m2, n2, k, ib, 
-                WORK, ldwork, A2, lda2, 
-                V, ldv, T, ldt, 
+    CORE_ztsmlq(side, trans, m1, n1, m2, n2, k, ib,
+                WORK, ldwork, A2, lda2,
+                V, ldv, T, ldt,
                 WORK+3*nb*ldwork, ldwork);
 
     /*  Rebuild the symmetric block: WORK+2*nb*ldwork <- A3 */
@@ -166,9 +179,9 @@ int CORE_ztsmlq_corner( int m1, int n1, int m2, int n2, int m3, int n3,
         }
 
     /*  Right application on | A2' A3 | */
-    CORE_ztsmlq(side, trans, n2, m2, m3, n3, k, ib, 
-                WORK+nb*ldwork, ldwork, WORK+2*nb*ldwork, ldwork, 
-                V, ldv, T, ldt, 
+    CORE_ztsmlq(side, trans, n2, m2, m3, n3, k, ib,
+                WORK+nb*ldwork, ldwork, WORK+2*nb*ldwork, ldwork,
+                V, ldv, T, ldt,
                 WORK + 3*nb*ldwork, ldwork);
 
     side = PlasmaLeft;
@@ -176,9 +189,9 @@ int CORE_ztsmlq_corner( int m1, int n1, int m2, int n2, int m3, int n3,
 
     /*  Left application on | A1  | */
     /*                      | A2' | */
-    CORE_ztsmlq(side, trans, m1, n1, n2, m2, k, ib, 
-                WORK, ldwork, WORK+nb*ldwork, ldwork, 
-                V, ldv, T, ldt, 
+    CORE_ztsmlq(side, trans, m1, n1, n2, m2, k, ib,
+                WORK, ldwork, WORK+nb*ldwork, ldwork,
+                V, ldv, T, ldt,
                 WORK + 3*nb*ldwork, ldwork);
 
     /*  Copy back the final result to the upper part of A1 */
@@ -189,9 +202,9 @@ int CORE_ztsmlq_corner( int m1, int n1, int m2, int n2, int m3, int n3,
 
     /*  Left application on | A2 | */
     /*                      | A3 | */
-    CORE_ztsmlq(side, trans, m2, n2, m3, n3, k, ib, 
-                A2, lda2, WORK+2*nb*ldwork, ldwork, 
-                V, ldv, T, ldt, 
+    CORE_ztsmlq(side, trans, m2, n2, m3, n3, k, ib,
+                A2, lda2, WORK+2*nb*ldwork, ldwork,
+                V, ldv, T, ldt,
                 WORK + 3*nb*ldwork, ldwork);
 
     /*  Copy back the final result to the upper part of A3 */
@@ -209,15 +222,16 @@ int CORE_ztsmlq_corner( int m1, int n1, int m2, int n2, int m3, int n3,
  *
  **/
 void QUARK_CORE_ztsmlq_corner(Quark *quark, Quark_Task_Flags *task_flags,
-                         int m1, int n1, int m2, int n2, int m3, int n3, int k, int ib, int nb,
-                         PLASMA_Complex64_t *A1, int lda1,
-                         PLASMA_Complex64_t *A2, int lda2,
-                         PLASMA_Complex64_t *A3, int lda3,
-                         PLASMA_Complex64_t *V, int ldv,
-                         PLASMA_Complex64_t *T, int ldt)
+                              int m1, int n1, int m2, int n2, int m3, int n3, int k, int ib, int nb,
+                              PLASMA_Complex64_t *A1, int lda1,
+                              PLASMA_Complex64_t *A2, int lda2,
+                              PLASMA_Complex64_t *A3, int lda3,
+                              const PLASMA_Complex64_t *V, int ldv,
+                              const PLASMA_Complex64_t *T, int ldt)
 {
     int ldwork = nb;
 
+    DAG_CORE_TSMLQ;
     QUARK_Insert_Task(quark, CORE_ztsmlq_corner_quark, task_flags,
         sizeof(int),                        &m1,    VALUE,
         sizeof(int),                        &n1,    VALUE,
@@ -279,10 +293,10 @@ void CORE_ztsmlq_corner_quark(Quark *quark)
     PLASMA_Complex64_t *WORK;
     int ldwork;
 
-    quark_unpack_args_21(quark, m1, n1, m2, n2, m3, n3, k, ib, nb, 
+    quark_unpack_args_21(quark, m1, n1, m2, n2, m3, n3, k, ib, nb,
                          A1, lda1, A2, lda2, A3, lda3, V, ldv, T, ldt, WORK, ldwork);
 
-    CORE_ztsmlq_corner(m1, n1, m2, n2, m3, n3, k, ib, nb, 
+    CORE_ztsmlq_corner(m1, n1, m2, n2, m3, n3, k, ib, nb,
                        A1, lda1, A2, lda2, A3, lda3, V, ldv, T, ldt, WORK, ldwork);
 
 }
