@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Azzam Haidar
@@ -42,66 +42,66 @@
  *         @arg PlasmaNoTrans   :  No transpose, apply Q;
  *         @arg PlasmaConjTrans :  ConjTranspose, apply Q**H.
  *
- * @param[in] M1
- *         The number of rows of the tile A1. M1 >= 0.
+ * @param[in] m1
+ *         The number of rows of the tile A1. m1 >= 0.
  *
- * @param[in] N1
- *         The number of columns of the tile A1. N1 >= 0.
+ * @param[in] n1
+ *         The number of columns of the tile A1. n1 >= 0.
  *
- * @param[in] M2
- *         The number of rows of the tile A2. M2 >= 0.
- *         M2 = M1 if side == PlasmaRight.
+ * @param[in] m2
+ *         The number of rows of the tile A2. m2 >= 0.
+ *         m2 = m1 if side == PlasmaRight.
  *
- * @param[in] N2
- *         The number of columns of the tile A2. N2 >= 0.
- *         N2 = N1 if side == PlasmaLeft.
+ * @param[in] n2
+ *         The number of columns of the tile A2. n2 >= 0.
+ *         n2 = n1 if side == PlasmaLeft.
  *
- * @param[in] K
+ * @param[in] k
  *         The number of elementary reflectors whose product defines
  *         the matrix Q.
  *
- * @param[in] IB
- *         The inner-blocking size.  IB >= 0.
+ * @param[in] ib
+ *         The inner-blocking size.  ib >= 0.
  *
  * @param[in,out] A1
- *         On entry, the M1-by-N1 tile A1.
+ *         On entry, the m1-by-n1 tile A1.
  *         On exit, A1 is overwritten by the application of Q.
  *
- * @param[in] LDA1
- *         The leading dimension of the array A1. LDA1 >= max(1,M1).
+ * @param[in] lda1
+ *         The leading dimension of the array A1. lda1 >= max(1,m1).
  *
  * @param[in,out] A2
- *         On entry, the M2-by-N2 tile A2.
+ *         On entry, the m2-by-n2 tile A2.
  *         On exit, A2 is overwritten by the application of Q.
  *
- * @param[in] LDA2
- *         The leading dimension of the tile A2. LDA2 >= max(1,M2).
+ * @param[in] lda2
+ *         The leading dimension of the tile A2. lda2 >= max(1,m2).
  *
  * @param[in] V
  *         The i-th row must contain the vector which defines the
  *         elementary reflector H(i), for i = 1,2,...,k, as returned by
  *         CORE_ZTSLQT in the first k rows of its array argument V.
+ *!
+ * @param[in] ldv
+ *         The leading dimension of the array V. ldv >= max(1,K).
  *
- * @param[in] LDV
- *         The leading dimension of the array V. LDV >= max(1,K).
- *
- * @param[out] T
- *         The IB-by-N1 triangular factor T of the block reflector.
+ * @param[in] T
+ *         The IB-by-n1 triangular factor T of the block reflector.
  *         T is upper triangular by block (economic storage);
  *         The rest of the array is not referenced.
  *
- * @param[in] LDT
- *         The leading dimension of the array T. LDT >= IB.
+ * @param[in] ldt
+ *         The leading dimension of the array T. ldt >= IB.
  *
  * @param[out] WORK
- *         Workspace array of size 
- *             LDWORK-by-M1 if side == PlasmaLeft
+ *         Workspace array of size
+ *             LDWORK-by-m1 if side == PlasmaLeft
  *             LDWORK-by-IB if side == PlasmaRight
  *
- * @param[in] LDWORK
- *         The leading dimension of the array WORK. 
+ * @param[in] ldwork
+ *         The leading dimension of the array WORK.
  *             LDWORK >= max(1,IB) if side == PlasmaLeft
- *             LDWORK >= max(1,N1) if side == PlasmaRight
+ *             LDWORK >= max(1,n1) if side == PlasmaRight
  *
  *******************************************************************************
  *
@@ -113,14 +113,22 @@
 #if defined(PLASMA_HAVE_WEAK)
 #pragma weak CORE_ztsmlq_hetra1 = PCORE_ztsmlq_hetra1
 #define CORE_ztsmlq_hetra1 PCORE_ztsmlq_hetra1
+#define CORE_ztsmlq PCORE_ztsmlq
+int  CORE_ztsmlq(PLASMA_enum side, PLASMA_enum trans,
+                 int m1, int n1, int m2, int n2, int K, int IB,
+                 PLASMA_Complex64_t *A1, int lda1,
+                 PLASMA_Complex64_t *A2, int lda2,
+                 const PLASMA_Complex64_t *V, int ldv,
+                 const PLASMA_Complex64_t *T, int ldt,
+                 PLASMA_Complex64_t *WORK, int LDWORK);
 #endif
-int CORE_ztsmlq_hetra1( int side, int trans,
+int CORE_ztsmlq_hetra1( PLASMA_enum side, PLASMA_enum trans,
                         int m1, int n1, int m2, int n2,
                         int k, int ib,
                         PLASMA_Complex64_t *A1, int lda1,
                         PLASMA_Complex64_t *A2, int lda2,
-                        PLASMA_Complex64_t *V, int ldv,
-                        PLASMA_Complex64_t *T, int ldt,
+                        const PLASMA_Complex64_t *V, int ldv,
+                        const PLASMA_Complex64_t *T, int ldt,
                         PLASMA_Complex64_t *WORK, int ldwork)
 {
     int i, j;
@@ -141,9 +149,9 @@ int CORE_ztsmlq_hetra1( int side, int trans,
         }
     }
 
-    CORE_ztsmlq(side, trans, m1, n1, m2, n2, k, ib, 
-                A1, lda1, A2, lda2, 
-                V,  ldv,  T,  ldt, 
+    CORE_ztsmlq(side, trans, m1, n1, m2, n2, k, ib,
+                A1, lda1, A2, lda2,
+                V,  ldv,  T,  ldt,
                 WORK, ldwork);
 
     /* in-place transposition of A1 */
@@ -156,7 +164,7 @@ int CORE_ztsmlq_hetra1( int side, int trans,
             *(A1 + j + i*lda1) = conj(*WORK);
         }
     }
-    
+
     return PLASMA_SUCCESS;
 }
 
@@ -164,15 +172,16 @@ int CORE_ztsmlq_hetra1( int side, int trans,
  *
  **/
 void QUARK_CORE_ztsmlq_hetra1(Quark *quark, Quark_Task_Flags *task_flags,
-                              int side, int trans,
+                              PLASMA_enum side, PLASMA_enum trans,
                               int m1, int n1, int m2, int n2, int k, int ib, int nb,
                               PLASMA_Complex64_t *A1, int lda1,
                               PLASMA_Complex64_t *A2, int lda2,
-                              PLASMA_Complex64_t *V, int ldv,
-                              PLASMA_Complex64_t *T, int ldt)
+                              const PLASMA_Complex64_t *V, int ldv,
+                              const PLASMA_Complex64_t *T, int ldt)
 {
     int ldwork = side == PlasmaLeft ? ib : nb;
 
+    DAG_CORE_TSMLQ;
     QUARK_Insert_Task(quark, CORE_ztsmlq_hetra1_quark, task_flags,
         sizeof(PLASMA_enum),                &side,  VALUE,
         sizeof(PLASMA_enum),                &trans, VALUE,
@@ -209,8 +218,8 @@ void QUARK_CORE_ztsmlq_hetra1(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_ztsmlq_hetra1_quark(Quark *quark)
 {
-    int side;
-    int trans;
+    PLASMA_enum side;
+    PLASMA_enum trans;
     int m1;
     int n1;
     int m2;
@@ -228,9 +237,9 @@ void CORE_ztsmlq_hetra1_quark(Quark *quark)
     PLASMA_Complex64_t *WORK;
     int ldwork;
 
-    quark_unpack_args_18(quark, side, trans, m1, n1, m2, n2, k, ib, 
+    quark_unpack_args_18(quark, side, trans, m1, n1, m2, n2, k, ib,
                          A1, lda1, A2, lda2, V, ldv, T, ldt, WORK, ldwork);
-    CORE_ztsmlq_hetra1(side, trans, m1, n1, m2, n2, k, ib, 
+    CORE_ztsmlq_hetra1(side, trans, m1, n1, m2, n2, k, ib,
                        A1, lda1, A2, lda2, V, ldv, T, ldt, WORK, ldwork);
 }
 
