@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -87,7 +87,7 @@
  * @param[in] ldv
  *         The leading dimension of the array V. LDV >= max(1,K).
  *
- * @param[out] T
+ * @param[in] T
  *         The IB-by-N1 triangular factor T of the block reflector.
  *         T is upper triangular by block (economic storage);
  *         The rest of the array is not referenced.
@@ -96,12 +96,12 @@
  *         The leading dimension of the array T. LDT >= IB.
  *
  * @param[out] WORK
- *         Workspace array of size 
+ *         Workspace array of size
  *             LDWORK-by-N1 if side == PlasmaLeft
  *             LDWORK-by-IB if side == PlasmaRight
  *
  * @param[in] ldwork
- *         The leading dimension of the array WORK. 
+ *         The leading dimension of the array WORK.
  *             LDWORK >= max(1,IB) if side == PlasmaLeft
  *             LDWORK >= max(1,M1) if side == PlasmaRight
  *
@@ -115,14 +115,22 @@
 #if defined(PLASMA_HAVE_WEAK)
 #pragma weak CORE_ztsmqr_hetra1 = PCORE_ztsmqr_hetra1
 #define CORE_ztsmqr_hetra1 PCORE_ztsmqr_hetra1
+#define CORE_ztsmqr PCORE_ztsmqr
+int  CORE_ztsmqr(PLASMA_enum side, PLASMA_enum trans,
+                 int M1, int N1, int M2, int N2, int K, int IB,
+                 PLASMA_Complex64_t *A1, int LDA1,
+                 PLASMA_Complex64_t *A2, int LDA2,
+                 const PLASMA_Complex64_t *V, int LDV,
+                 const PLASMA_Complex64_t *T, int LDT,
+                 PLASMA_Complex64_t *WORK, int LDWORK);
 #endif
-int CORE_ztsmqr_hetra1( int side, int trans,
+int CORE_ztsmqr_hetra1( PLASMA_enum side, PLASMA_enum trans,
                         int m1, int n1, int m2, int n2,
                         int k, int ib,
                         PLASMA_Complex64_t *A1, int lda1,
                         PLASMA_Complex64_t *A2, int lda2,
-                        PLASMA_Complex64_t *V, int ldv,
-                        PLASMA_Complex64_t *T, int ldt,
+                        const PLASMA_Complex64_t *V, int ldv,
+                        const PLASMA_Complex64_t *T, int ldt,
                         PLASMA_Complex64_t *WORK, int ldwork)
 {
     int i, j;
@@ -163,15 +171,16 @@ int CORE_ztsmqr_hetra1( int side, int trans,
  *
  **/
 void QUARK_CORE_ztsmqr_hetra1(Quark *quark, Quark_Task_Flags *task_flags,
-                        int side, int trans,
-                        int m1, int n1, int m2, int n2, int k, int ib, int nb,
-                        PLASMA_Complex64_t *A1, int lda1,
-                        PLASMA_Complex64_t *A2, int lda2,
-                        PLASMA_Complex64_t *V, int ldv,
-                        PLASMA_Complex64_t *T, int ldt)
+                              PLASMA_enum side, PLASMA_enum trans,
+                              int m1, int n1, int m2, int n2, int k, int ib, int nb,
+                              PLASMA_Complex64_t *A1, int lda1,
+                              PLASMA_Complex64_t *A2, int lda2,
+                              const PLASMA_Complex64_t *V, int ldv,
+                              const PLASMA_Complex64_t *T, int ldt)
 {
     int ldwork = side == PlasmaLeft ? ib : nb;
 
+    DAG_CORE_TSMQR;
     QUARK_Insert_Task(quark, CORE_ztsmqr_hetra1_quark, task_flags,
         sizeof(PLASMA_enum),                &side,  VALUE,
         sizeof(PLASMA_enum),                &trans, VALUE,
@@ -203,8 +212,8 @@ void QUARK_CORE_ztsmqr_hetra1(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_ztsmqr_hetra1_quark(Quark *quark)
 {
-    int side;
-    int trans;
+    PLASMA_enum side;
+    PLASMA_enum trans;
     int m1;
     int n1;
     int m2;
