@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -85,7 +85,15 @@
 #pragma weak CORE_ztsqrt = PCORE_ztsqrt
 #define CORE_ztsqrt PCORE_ztsqrt
 #define CORE_ztsmqr PCORE_ztsmqr
+int  CORE_ztsmqr(PLASMA_enum side, PLASMA_enum trans,
+                 int M1, int N1, int M2, int N2, int K, int IB,
+                 PLASMA_Complex64_t *A1, int LDA1,
+                 PLASMA_Complex64_t *A2, int LDA2,
+                 const PLASMA_Complex64_t *V, int LDV,
+                 const PLASMA_Complex64_t *T, int LDT,
+                 PLASMA_Complex64_t *WORK, int LDWORK);
 #endif
+
 int CORE_ztsqrt(int M, int N, int IB,
                 PLASMA_Complex64_t *A1, int LDA1,
                 PLASMA_Complex64_t *A2, int LDA2,
@@ -205,13 +213,14 @@ void QUARK_CORE_ztsqrt(Quark *quark, Quark_Task_Flags *task_flags,
                        PLASMA_Complex64_t *T, int ldt)
 {
     DAG_CORE_TSQRT;
+    /* TODO: Regions are commented for 2-sided factorizations that have a dependency problem with it */
     QUARK_Insert_Task(quark, CORE_ztsqrt_quark, task_flags,
         sizeof(int),                        &m,     VALUE,
         sizeof(int),                        &n,     VALUE,
         sizeof(int),                        &ib,    VALUE,
-        sizeof(PLASMA_Complex64_t)*nb*nb,    A1,            INOUT | QUARK_REGION_D | QUARK_REGION_U,
+        sizeof(PLASMA_Complex64_t)*nb*nb,    A1,            INOUT /*| QUARK_REGION_D | QUARK_REGION_U */| LOCALITY,
         sizeof(int),                        &lda1,  VALUE,
-        sizeof(PLASMA_Complex64_t)*nb*nb,    A2,            INOUT | LOCALITY,
+        sizeof(PLASMA_Complex64_t)*nb*nb,    A2,            INOUT,
         sizeof(int),                        &lda2,  VALUE,
         sizeof(PLASMA_Complex64_t)*ib*nb,    T,             OUTPUT,
         sizeof(int),                        &ldt,   VALUE,
