@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Azzam Haidar
@@ -115,13 +115,14 @@
 #pragma weak CORE_ztsrfb = PCORE_ztsrfb
 #define CORE_ztsrfb PCORE_ztsrfb
 #endif
-int CORE_ztsrfb(int side, int trans, int direct, int storev,
-                int M1, int N1, int M2, int N2, int K,
-                PLASMA_Complex64_t *A1, int LDA1,
-                PLASMA_Complex64_t *A2, int LDA2,
-                PLASMA_Complex64_t *V, int LDV,
-                PLASMA_Complex64_t *T, int LDT,
-                PLASMA_Complex64_t *WORK, int LDWORK)
+int
+CORE_ztsrfb(PLASMA_enum side, PLASMA_enum trans, int direct, int storev,
+            int M1, int N1, int M2, int N2, int K,
+            PLASMA_Complex64_t *A1, int LDA1,
+            PLASMA_Complex64_t *A2, int LDA2,
+            const PLASMA_Complex64_t *V, int LDV,
+            const PLASMA_Complex64_t *T, int LDT,
+            PLASMA_Complex64_t *WORK, int LDWORK)
 {
     static PLASMA_Complex64_t zone  =  1.0;
     static PLASMA_Complex64_t mzone = -1.0;
@@ -137,12 +138,12 @@ int CORE_ztsrfb(int side, int trans, int direct, int storev,
         coreblas_error(6, "Illegal value of N1");
         return -6;
     }
-    if ( (M2 < 0) || 
+    if ( (M2 < 0) ||
          ( (M2 != M1) && (side == PlasmaRight) ) ){
         coreblas_error(7, "Illegal value of M2");
         return -7;
     }
-    if ( (N2 < 0) || 
+    if ( (N2 < 0) ||
          ( (N2 != N1) && (side == PlasmaLeft) ) ){
         coreblas_error(8, "Illegal value of N2");
         return -8;
