@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -69,7 +69,7 @@
  * @param[in,out] WORK
  *
  * @param[in] LDWORK
- *         The dimension of the array WORK.
+ *         The leading dimension of the array WORK.
  *
  * @param[out] INFO
  *
@@ -89,6 +89,12 @@
 #pragma weak CORE_ztstrf = PCORE_ztstrf
 #define CORE_ztstrf PCORE_ztstrf
 #define CORE_zssssm PCORE_zssssm
+int  CORE_zssssm(int M1, int N1, int M2, int N2, int K, int IB,
+                 PLASMA_Complex64_t *A1, int LDA1,
+                 PLASMA_Complex64_t *A2, int LDA2,
+                 const PLASMA_Complex64_t *L1, int LDL1,
+                 const PLASMA_Complex64_t *L2, int LDL2,
+                 const int *IPIV);
 #endif
 int CORE_ztstrf(int M, int N, int IB, int NB,
                 PLASMA_Complex64_t *U, int LDU,
