@@ -6,9 +6,10 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
+ * @author Dulceneia Becker
  * @date 2010-11-15
  * @precisions normal z -> c d s
  *
@@ -95,6 +96,22 @@
 #if defined(PLASMA_HAVE_WEAK)
 #pragma weak CORE_zttlqt = PCORE_zttlqt
 #define CORE_zttlqt PCORE_zttlqt
+/* Trick to call the version without tracing */
+#define CORE_zlaset PCORE_zlaset
+void
+CORE_zlaset(PLASMA_enum uplo, int n1, int n2,
+            PLASMA_Complex64_t alpha, PLASMA_Complex64_t beta,
+            PLASMA_Complex64_t *tileA, int ldtilea);
+#define CORE_zpemv PCORE_zpemv
+int
+CORE_zpemv(PLASMA_enum trans, int storev,
+           int M, int N, int L,
+           PLASMA_Complex64_t ALPHA,
+           const PLASMA_Complex64_t *A, int LDA,
+           const PLASMA_Complex64_t *X, int INCX,
+           PLASMA_Complex64_t BETA,
+           PLASMA_Complex64_t *Y, int INCY,
+           PLASMA_Complex64_t *WORK);
 #endif
 int CORE_zttlqt(int M, int N, int IB,
                 PLASMA_Complex64_t *A1, int LDA1,
@@ -104,10 +121,12 @@ int CORE_zttlqt(int M, int N, int IB,
 {
     static PLASMA_Complex64_t zone  = 1.0;
     static PLASMA_Complex64_t zzero = 0.0;
+#ifdef COMPLEX
     static int                ione  = 1;
+#endif
 
     PLASMA_Complex64_t alpha;
-    int i, j, ii, sb, mi, ni;
+    int i, j, l, ii, sb, mi, ni;
 
     /* Check input arguments */
     if (M < 0) {
@@ -131,113 +150,107 @@ int CORE_zttlqt(int M, int N, int IB,
     if ((M == 0) || (N == 0) || (IB == 0))
         return PLASMA_SUCCESS;
 
+    /* TODO: Need to check why some cases require
+     *  this to not have uninitialized values */
+    CORE_zlaset( PlasmaUpperLower, IB, N,
+                 0., 0., T, LDT);
+
     for(ii = 0; ii < M; ii += IB) {
         sb = min(M-ii, IB);
         for(i = 0; i < sb; i++) {
+            j  = ii + i;
+            mi = sb-i-1;
+            ni = min( j + 1, N);
             /*
              * Generate elementary reflector H( II*IB+I ) to annihilate A( II*IB+I, II*IB+I:M ).
              */
-            ni = ii+i+1;
 #ifdef COMPLEX
-            LAPACKE_zlacgv_work(ni, &A2[ii+i], LDA2);
-            LAPACKE_zlacgv_work(ione, &A1[LDA1*(ii+i)+ii+i], LDA1);
+            LAPACKE_zlacgv_work(ni, &A2[j], LDA2);
+            LAPACKE_zlacgv_work(ione, &A1[LDA1*j+j], LDA1);
 #endif
-            //tmp = ii+i+2;
-            LAPACKE_zlarfg_work(ni+1, &A1[LDA1*(ii+i)+ii+i], &A2[ii+i], LDA2, &TAU[ii+i]);
+            LAPACKE_zlarfg_work(ni+1, &A1[LDA1*j+j], &A2[j], LDA2, &TAU[j]);
 
-            if (sb-i-1 > 0) { //*DB
-            //if (ii+i+1 < M) {
+            if (mi > 0) {
                 /*
-                 * Apply H( II+I-1 ) to A( II+I:II+IB-1, II+I-1:M  ) from the right.
+                 * Apply H( j-1 ) to A( j:II+IB-1, j-1:M  ) from the right.
                  */
-                mi = sb-i-1;
                 cblas_zcopy(
                     mi,
-                    &A1[LDA1*(ii+i)+(ii+i+1)], 1,
+                    &A1[LDA1*j+(j+1)], 1,
                     WORK, 1);
 
                 cblas_zgemv(
                     CblasColMajor, (CBLAS_TRANSPOSE)PlasmaNoTrans,
                     mi, ni,
-                    CBLAS_SADDR(zone), &A2[ii+i+1], LDA2,
-                    &A2[ii+i], LDA2,
+                    CBLAS_SADDR(zone), &A2[j+1], LDA2,
+                    &A2[j], LDA2,
                     CBLAS_SADDR(zone), WORK, 1);
 
-                alpha = -(TAU[ii+i]);
+                alpha = -(TAU[j]);
                 cblas_zaxpy(
                     mi, CBLAS_SADDR(alpha),
                     WORK, 1,
-                    &A1[LDA1*(ii+i)+ii+i+1], 1);
+                    &A1[LDA1*j+j+1], 1);
 
                 cblas_zgerc(
                     CblasColMajor, mi, ni,
                     CBLAS_SADDR(alpha), WORK, 1,
-                    &A2[ii+i], LDA2,
-                    &A2[ii+i+1], LDA2);
+                    &A2[j], LDA2,
+                    &A2[j+1], LDA2);
             }
+
             /*
              * Calculate T.
              */
 
             if (i > 0 ) {
 
-                cblas_zcopy(i, &A2[LDA2*ii+ii+i], LDA2, &WORK[ii], 1);
+                l = min(i, max(0, N-ii));
+                alpha = -(TAU[j]);
 
+                CORE_zpemv(
+                        PlasmaNoTrans, PlasmaRowwise,
+                        i , min(j, N), l,
+                        alpha, &A2[ii], LDA2,
+                        &A2[j], LDA2,
+                        zzero, &T[LDT*j], 1,
+                        WORK);
+
+                /* T(0:i-1, j) = T(0:i-1, ii:j-1) * T(0:i-1, j) */
                 cblas_ztrmv(
-                    CblasColMajor, (CBLAS_UPLO)PlasmaLower,
-                    (CBLAS_TRANSPOSE)PlasmaNoTrans, (CBLAS_DIAG)PlasmaNonUnit,
-                    i, &A2[LDA2*ii+ii], LDA2,
-                    &WORK[ii], 1);
-
-                alpha = -(TAU[ii+i]);
-
-                for(j = 0; j < i; j++) {
-                    WORK[ii+j] = alpha * WORK[ii+j];
-                }
-
-                if (ii > 0) {
-                    cblas_zgemv(
-                        CblasColMajor, (CBLAS_TRANSPOSE)PlasmaNoTrans, i, ii,
-                        CBLAS_SADDR(alpha), &A2[ii], LDA2,
-                        &A2[ii+i], LDA2,
-                        CBLAS_SADDR(zzero), WORK, 1);
-
-                    cblas_zaxpy(i, CBLAS_SADDR(zone), &WORK[ii], 1, WORK, 1);
-                }
-
-                cblas_zcopy(i, WORK, 1, &T[LDT*(ii+i)], 1);
-
-                cblas_ztrmv(
-                    CblasColMajor, (CBLAS_UPLO)PlasmaUpper,
-                    (CBLAS_TRANSPOSE)PlasmaNoTrans, (CBLAS_DIAG)PlasmaNonUnit,
-                    i, &T[LDT*ii], LDT,
-                    &T[LDT*(ii+i)], 1);
+                        CblasColMajor, (CBLAS_UPLO)PlasmaUpper,
+                        (CBLAS_TRANSPOSE)PlasmaNoTrans,
+                        (CBLAS_DIAG)PlasmaNonUnit,
+                        i, &T[LDT*ii], LDT,
+                        &T[LDT*j], 1);
 
             }
 
 #ifdef COMPLEX
-            LAPACKE_zlacgv_work(ni, &A2[ii+i], LDA2 );
-            LAPACKE_zlacgv_work(ione, &A1[LDA1*(ii+i)+ii+i], LDA1 );
+            LAPACKE_zlacgv_work(ni, &A2[j], LDA2 );
+            LAPACKE_zlacgv_work(ione, &A1[LDA1*j+j], LDA1 );
 #endif
 
-            T[LDT*(ii+i)+i] = TAU[ii+i];
+            T[LDT*j+i] = TAU[j];
         }
 
-        /* Apply Q' to the rest of the matrix to the right */
+        /* Apply Q to the rest of the matrix to the right */
         if (M > ii+sb) {
-            CORE_zttrfb(
+            mi = M-(ii+sb);
+            ni = min(ii+sb, N);
+            l  = min(sb, max(0, ni-ii));
+            CORE_zparfb(
                 PlasmaRight, PlasmaNoTrans,
                 PlasmaForward, PlasmaRowwise,
-                M-(ii+sb), sb, M-(ii+sb), ii+sb, sb,
+                mi, IB, mi, ni, sb, l,
                 &A1[LDA1*ii+ii+sb], LDA1,
                 &A2[ii+sb], LDA2,
                 &A2[ii], LDA2,
                 &T[LDT*ii], LDT,
-                WORK, N);
+                WORK, M);
 
         }
     }
-
     return PLASMA_SUCCESS;
 }
 
@@ -255,9 +268,9 @@ void QUARK_CORE_zttlqt(Quark *quark, Quark_Task_Flags *task_flags,
         sizeof(int),                        &m,     VALUE,
         sizeof(int),                        &n,     VALUE,
         sizeof(int),                        &ib,    VALUE,
-        sizeof(PLASMA_Complex64_t)*nb*nb,    A1,            INOUT|QUARK_REGION_D|QUARK_REGION_L,
+        sizeof(PLASMA_Complex64_t)*nb*nb,    A1,            INOUT/*|QUARK_REGION_D|QUARK_REGION_L*/,
         sizeof(int),                        &lda1,  VALUE,
-        sizeof(PLASMA_Complex64_t)*nb*nb,    A2,            INOUT|QUARK_REGION_D|QUARK_REGION_L,
+        sizeof(PLASMA_Complex64_t)*nb*nb,    A2,            INOUT/*|QUARK_REGION_D|QUARK_REGION_L*/|LOCALITY,
         sizeof(int),                        &lda2,  VALUE,
         sizeof(PLASMA_Complex64_t)*ib*nb,    T,             OUTPUT,
         sizeof(int),                        &ldt,   VALUE,
