@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Dulceneia Becker
@@ -37,8 +37,6 @@
  *
  *  as returned by CORE_zttqrt.
  *
- *  Only LEFT is supported !!!
- *
  *******************************************************************************
  *
  * @param[in] side
@@ -90,7 +88,7 @@
  * @param[in] LDV
  *         The leading dimension of the array V. LDV >= max(1,K).
  *
- * @param[out] T
+ * @param[in] T
  *         The IB-by-N1 triangular factor T of the block reflector.
  *         T is upper triangular by block (economic storage);
  *         The rest of the array is not referenced.
@@ -114,19 +112,18 @@
 #if defined(PLASMA_HAVE_WEAK)
 #pragma weak CORE_zttmqr = PCORE_zttmqr
 #define CORE_zttmqr PCORE_zttmqr
-#define CORE_zttrfb PCORE_zttrfb
 #endif
-int CORE_zttmqr(int side, int trans,
+int CORE_zttmqr(PLASMA_enum side, PLASMA_enum trans,
                 int M1, int N1, int M2, int N2, int K, int IB,
                 PLASMA_Complex64_t *A1, int LDA1,
                 PLASMA_Complex64_t *A2, int LDA2,
-                PLASMA_Complex64_t *V, int LDV,
-                PLASMA_Complex64_t *T, int LDT,
+                const PLASMA_Complex64_t *V, int LDV,
+                const PLASMA_Complex64_t *T, int LDT,
                 PLASMA_Complex64_t *WORK, int LDWORK)
 {
     int i,  i1, i3;
     int NQ, NW;
-    int kb;
+    int kb, l;
     int ic = 0;
     int jc = 0;
     int mi1 = M1;
@@ -162,17 +159,17 @@ int CORE_zttmqr(int side, int trans,
         coreblas_error(4, "Illegal value of N1");
         return -4;
     }
-    if ( (M2 < 0) || 
+    if ( (M2 < 0) ||
          ( (M2 != M1) && (side == PlasmaRight) ) ){
         coreblas_error(5, "Illegal value of M2");
         return -5;
     }
-    if ( (N2 < 0) || 
+    if ( (N2 < 0) ||
          ( (N2 != N1) && (side == PlasmaLeft) ) ){
         coreblas_error(6, "Illegal value of N2");
         return -6;
     }
-    if ((K < 0) || 
+    if ((K < 0) ||
         ( (side == PlasmaLeft)  && (K > M1) ) ||
         ( (side == PlasmaRight) && (K > N1) ) ) {
         coreblas_error(7, "Illegal value of K");
@@ -221,24 +218,24 @@ int CORE_zttmqr(int side, int trans,
         kb = min(IB, K-i);
 
         if (side == PlasmaLeft) {
-            mi1 = M1 - i;
-            mi2 = i + kb;
-            /*if ((K == IB) && (NeqIB > 0)) { mi2 = M2 - i; }*/
-            ic = i;
+            mi1 = kb;
+            mi2 = min(i+kb, M2);
+            l   = min(kb, max(0, M2-i));
+            ic  = i;
         }
         else {
-            /* ni2 must be the number of rows of V */
-            ni1 = kb; 
-            ni2 = i + kb;
-            jc = i;
+            ni1 = kb;
+            ni2 = min(i+kb, N2);
+            l   = min(kb, max(0, N2-i));
+            jc  = i;
         }
 
         /*
-         * H or H' is applied to C(i:m,1:n)
+         * Apply H or H' (NOTE: CORE_zparfb used to be CORE_zttrfb)
          */
-        CORE_zttrfb(
+        CORE_zparfb(
             side, trans, PlasmaForward, PlasmaColumnwise,
-            mi1, ni1, mi2, ni2, kb,
+            mi1, ni1, mi2, ni2, kb, l,
             &A1[LDA1*jc+ic], LDA1,
             A2, LDA2,
             &V[LDV*i], LDV,
@@ -252,12 +249,12 @@ int CORE_zttmqr(int side, int trans,
  *
  **/
 void QUARK_CORE_zttmqr(Quark *quark, Quark_Task_Flags *task_flags,
-                       int side, int trans,
+                       PLASMA_enum side, PLASMA_enum trans,
                        int m1, int n1, int m2, int n2, int k, int ib, int nb,
                        PLASMA_Complex64_t *A1, int lda1,
                        PLASMA_Complex64_t *A2, int lda2,
-                       PLASMA_Complex64_t *V, int ldv,
-                       PLASMA_Complex64_t *T, int ldt)
+                       const PLASMA_Complex64_t *V, int ldv,
+                       const PLASMA_Complex64_t *T, int ldt)
 {
     int ldwork = side == PlasmaLeft ? ib : nb;
 
@@ -293,8 +290,8 @@ void QUARK_CORE_zttmqr(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zttmqr_quark(Quark *quark)
 {
-    int side;
-    int trans;
+    PLASMA_enum side;
+    PLASMA_enum trans;
     int m1;
     int n1;
     int m2;
@@ -312,8 +309,8 @@ void CORE_zttmqr_quark(Quark *quark)
     PLASMA_Complex64_t *WORK;
     int ldwork;
 
-    quark_unpack_args_18(quark, side, trans, m1, n1, m2, n2, k, ib, 
+    quark_unpack_args_18(quark, side, trans, m1, n1, m2, n2, k, ib,
                          A1, lda1, A2, lda2, V, ldv, T, ldt, WORK, ldwork);
-    CORE_zttmqr(side, trans, m1, n1, m2, n2, k, ib, 
+    CORE_zttmqr(side, trans, m1, n1, m2, n2, k, ib,
                 A1, lda1, A2, lda2, V, ldv, T, ldt, WORK, ldwork);
 }
