@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Dulceneia Becker
@@ -25,7 +25,7 @@
  *
  *  CORE_zttqrt computes a QR factorization of a rectangular matrix
  *  formed by coupling a complex N-by-N upper triangular tile A1
- *  on top of a complex M-by-N upper triangular tile A2:
+ *  on top of a complex M-by-N upper trapezoidal tile A2:
  *
  *    | A1 | = Q * R
  *    | A2 |
@@ -96,6 +96,22 @@
 #if defined(PLASMA_HAVE_WEAK)
 #pragma weak CORE_zttqrt = PCORE_zttqrt
 #define CORE_zttqrt PCORE_zttqrt
+/* Trick to call the version without tracing */
+#define CORE_zlaset PCORE_zlaset
+void
+CORE_zlaset(PLASMA_enum uplo, int n1, int n2,
+            PLASMA_Complex64_t alpha, PLASMA_Complex64_t beta,
+            PLASMA_Complex64_t *tileA, int ldtilea);
+#define CORE_zpemv PCORE_zpemv
+int
+CORE_zpemv(PLASMA_enum trans, int storev,
+           int M, int N, int L,
+           PLASMA_Complex64_t ALPHA,
+           const PLASMA_Complex64_t *A, int LDA,
+           const PLASMA_Complex64_t *X, int INCX,
+           PLASMA_Complex64_t BETA,
+           PLASMA_Complex64_t *Y, int INCY,
+           PLASMA_Complex64_t *WORK);
 #endif
 int CORE_zttqrt(int M, int N, int IB,
                 PLASMA_Complex64_t *A1, int LDA1,
@@ -103,12 +119,12 @@ int CORE_zttqrt(int M, int N, int IB,
                 PLASMA_Complex64_t *T, int LDT,
                 PLASMA_Complex64_t *TAU, PLASMA_Complex64_t *WORK)
 {
+    static int                ione  = 1;
     static PLASMA_Complex64_t zone  = 1.0;
     static PLASMA_Complex64_t zzero = 0.0;
-    static int                ione  = 1;
 
     PLASMA_Complex64_t alpha;
-    int i, j, ii, sb, mi, ni;
+    int i, j, l, ii, sb, mi, ni;
 
     /* Check input arguments */
     if (M < 0) {
@@ -132,101 +148,102 @@ int CORE_zttqrt(int M, int N, int IB,
     if ((M == 0) || (N == 0) || (IB == 0))
         return PLASMA_SUCCESS;
 
-    for(ii = 0; ii < N; ii += IB) {
+    /* TODO: Need to check why some cases require
+     *  this to not have uninitialized values */
+    CORE_zlaset( PlasmaUpperLower, IB, N,
+                 0., 0., T, LDT);
+
+    for (ii = 0; ii < N; ii += IB) {
         sb = min(N-ii, IB);
-        for(i = 0; i < sb; i++) {
+        for (i = 0; i < sb; i++) {
+            j  = ii + i;
+            mi = min( j + 1, M );
+            ni = sb-i-1;
+
             /*
              * Generate elementary reflector H( II*IB+I ) to annihilate
              * A( II*IB+I:mi, II*IB+I ).
              */
-            mi = ii + i + 1;
-            LAPACKE_zlarfg_work(mi+1, &A1[LDA1*(ii+i)+ii+i], &A2[LDA2*(ii+i)], ione, &TAU[ii+i]);
+            LAPACKE_zlarfg_work(
+                    mi+1, &A1[LDA1*j+j], &A2[LDA2*j], ione, &TAU[j]);
 
-            if (sb-i-1>0) {
+            if (ni > 0) {
                 /*
                  * Apply H( II*IB+I ) to A( II*IB+I:M, II*IB+I+1:II*IB+IB ) from the left.
                  */
-                ni = sb-i-1;
                 cblas_zcopy(
                     ni,
-                    &A1[LDA1*(ii+i+1)+(ii+i)], LDA1,
+                    &A1[LDA1*(j+1)+j], LDA1,
                     WORK, 1);
 
 #ifdef COMPLEX
-                LAPACKE_zlacgv_work(ni, WORK, ione);
+                LAPACKE_zlacgv_work(ni, WORK, 1);
 #endif
                 cblas_zgemv(
                     CblasColMajor, (CBLAS_TRANSPOSE)PlasmaConjTrans,
                     mi, ni,
-                    CBLAS_SADDR(zone), &A2[LDA2*(ii+i+1)], LDA2,
-                    &A2[LDA2*(ii+i)], 1,
-                    CBLAS_SADDR(zone), WORK, 1);
+                    CBLAS_SADDR(zone), &A2[LDA2*(j+1)], LDA2,
+                                       &A2[LDA2*j],     1,
+                    CBLAS_SADDR(zone), WORK,            1);
 #ifdef COMPLEX
-                LAPACKE_zlacgv_work(ni, WORK, ione);
+                LAPACKE_zlacgv_work(ni, WORK, 1);
 #endif
-                alpha = -conj(TAU[ii+i]);
+                alpha = -conj(TAU[j]);
                 cblas_zaxpy(
                     ni, CBLAS_SADDR(alpha),
                     WORK, 1,
-                    &A1[LDA1*(ii+i+1)+ii+i], LDA1);
+                    &A1[LDA1*(j+1)+j], LDA1);
 #ifdef COMPLEX
-                LAPACKE_zlacgv_work(ni, WORK, ione);
+                LAPACKE_zlacgv_work(ni, WORK, 1);
 #endif
                 cblas_zgerc(
                     CblasColMajor, mi, ni,
-                    CBLAS_SADDR(alpha), &A2[LDA2*(ii+i)], 1,
+                    CBLAS_SADDR(alpha), &A2[LDA2*j], 1,
                     WORK, 1,
-                    &A2[LDA2*(ii+i+1)], LDA2);
+                    &A2[LDA2*(j+1)], LDA2);
             }
 
             /*
-             * Calculate T.
+             * Calculate T
+             *
+             * T(0:i-1, j) = alpha * A2(0:M-1, ii:j-1)' * A2(0:M-1, j)
              */
 
-            if (i > 0 ) {
+            if ( i > 0 ) {
 
-                cblas_zcopy(i, &A2[LDA2*(ii+i)+ii], 1, &WORK[ii], 1);
-    
+                l = min(i, max(0, M-ii));
+                alpha = -(TAU[j]);
+
+                CORE_zpemv(
+                        PlasmaConjTrans, PlasmaColumnwise,
+                        min(j, M), i, l,
+                        alpha, &A2[LDA2*ii], LDA2,
+                               &A2[LDA2*j],  1,
+                        zzero, &T[LDT*j],    1,
+                        WORK);
+
+                /* T(0:i-1, j) = T(0:i-1, ii:j-1) * T(0:i-1, j) */
                 cblas_ztrmv(
-                    CblasColMajor, (CBLAS_UPLO)PlasmaUpper,
-                    (CBLAS_TRANSPOSE)PlasmaConjTrans, (CBLAS_DIAG)PlasmaNonUnit,
-                    i, &A2[LDA2*ii+ii], LDA2,
-                    &WORK[ii], 1);
-    
-                alpha = -(TAU[ii+i]);
-                for(j = 0; j < i; j++) {
-                    WORK[ii+j] = alpha * WORK[ii+j];
-                }
-    
-                if (ii > 0) {
-                    cblas_zgemv(
-                        CblasColMajor, (CBLAS_TRANSPOSE)PlasmaConjTrans, ii, i,
-                        CBLAS_SADDR(alpha), &A2[LDA2*ii], LDA2,
-                        &A2[LDA2*(ii+i)], 1,
-                        CBLAS_SADDR(zzero), WORK, 1);
-    
-                    cblas_zaxpy(i, CBLAS_SADDR(zone), &WORK[ii], 1, WORK, 1);
-                }
-    
-                cblas_zcopy(i, WORK, 1, &T[LDT*(ii+i)], 1);
-    
-                cblas_ztrmv(
-                    CblasColMajor, (CBLAS_UPLO)PlasmaUpper,
-                    (CBLAS_TRANSPOSE)PlasmaNoTrans, (CBLAS_DIAG)PlasmaNonUnit,
-                    i, &T[LDT*ii], LDT,
-                    &T[LDT*(ii+i)], 1);
+                        CblasColMajor, (CBLAS_UPLO)PlasmaUpper,
+                        (CBLAS_TRANSPOSE)PlasmaNoTrans,
+                        (CBLAS_DIAG)PlasmaNonUnit,
+                        i, &T[LDT*ii], LDT,
+                           &T[LDT*j], 1);
 
             }
 
-            T[LDT*(ii+i)+i] = TAU[ii+i];
+            T[LDT*j+i] = TAU[j];
         }
 
         /* Apply Q' to the rest of the matrix to the left  */
         if (N > ii+sb) {
-            CORE_zttrfb(
+            mi = min(ii+sb, M);
+            ni = N-(ii+sb);
+            l  = min(sb, max(0, mi-ii));
+            CORE_zparfb(
                 PlasmaLeft, PlasmaConjTrans,
                 PlasmaForward, PlasmaColumnwise,
-                sb, N-(ii+sb), ii+sb, N-(ii+sb), sb,
+                IB, ni, mi, ni, sb, l,             //replaced sb by IB
                 &A1[LDA1*(ii+sb)+ii], LDA1,
                 &A2[LDA2*(ii+sb)], LDA2,
                 &A2[LDA2*ii], LDA2,
@@ -251,9 +268,9 @@ void QUARK_CORE_zttqrt(Quark *quark, Quark_Task_Flags *task_flags,
         sizeof(int),                        &m,     VALUE,
         sizeof(int),                        &n,     VALUE,
         sizeof(int),                        &ib,    VALUE,
-        sizeof(PLASMA_Complex64_t)*nb*nb,    A1,            INOUT|QUARK_REGION_D|QUARK_REGION_U,
+        sizeof(PLASMA_Complex64_t)*nb*nb,    A1,            INOUT/*|QUARK_REGION_D|QUARK_REGION_U*/,
         sizeof(int),                        &lda1,  VALUE,
-        sizeof(PLASMA_Complex64_t)*nb*nb,    A2,            INOUT|QUARK_REGION_D|QUARK_REGION_U,
+        sizeof(PLASMA_Complex64_t)*nb*nb,    A2,            INOUT/*|QUARK_REGION_D|QUARK_REGION_U*/|LOCALITY,
         sizeof(int),                        &lda2,  VALUE,
         sizeof(PLASMA_Complex64_t)*ib*nb,    T,             OUTPUT,
         sizeof(int),                        &ldt,   VALUE,
