@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -71,7 +71,7 @@
  * @param[in] LDA
  *         The leading dimension of the array A.  LDA >= max(1,K).
  *
- * @param[out] T
+ * @param[in] T
  *         The IB-by-K triangular factor T of the block reflector.
  *         T is upper triangular by block (economic storage);
  *         The rest of the array is not referenced.
@@ -105,10 +105,10 @@
 #pragma weak CORE_zunmlq = PCORE_zunmlq
 #define CORE_zunmlq PCORE_zunmlq
 #endif
-int CORE_zunmlq(int side, int trans,
+int CORE_zunmlq(PLASMA_enum side, PLASMA_enum trans,
                 int M, int N, int K, int IB,
-                PLASMA_Complex64_t *A, int LDA,
-                PLASMA_Complex64_t *T, int LDT,
+                const PLASMA_Complex64_t *A, int LDA,
+                const PLASMA_Complex64_t *T, int LDT,
                 PLASMA_Complex64_t *C, int LDC,
                 PLASMA_Complex64_t *WORK, int LDWORK)
 {
@@ -229,10 +229,10 @@ int CORE_zunmlq(int side, int trans,
  *
  **/
 void QUARK_CORE_zunmlq(Quark *quark, Quark_Task_Flags *task_flags,
-                       int side, int trans,
+                       PLASMA_enum side, PLASMA_enum trans,
                        int m, int n, int k, int ib, int nb,
-                       PLASMA_Complex64_t *A, int lda,
-                       PLASMA_Complex64_t *T, int ldt,
+                       const PLASMA_Complex64_t *A, int lda,
+                       const PLASMA_Complex64_t *T, int ldt,
                        PLASMA_Complex64_t *C, int ldc)
 {
     DAG_CORE_UNMLQ;
@@ -263,8 +263,8 @@ void QUARK_CORE_zunmlq(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zunmlq_quark(Quark *quark)
 {
-    int side;
-    int trans;
+    PLASMA_enum side;
+    PLASMA_enum trans;
     int m;
     int n;
     int k;
@@ -280,6 +280,6 @@ void CORE_zunmlq_quark(Quark *quark)
 
     quark_unpack_args_14(quark, side, trans, m, n, k, ib,
                          A, lda, T, ldt, C, ldc, WORK, ldwork);
-    CORE_zunmlq(side, trans, m, n, k, ib, 
+    CORE_zunmlq(side, trans, m, n, k, ib,
                 A, lda, T, ldt, C, ldc, WORK, ldwork);
 }
