@@ -1,4 +1,4 @@
-/**
+ /**
  *
  * @file core_zunmqr.c
  *
@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
  * @author Mathieu Faverge
  * @author Jakub Kurzak
@@ -71,7 +71,7 @@
  *         If SIDE = PlasmaLeft,  LDA >= max(1,M);
  *         if SIDE = PlasmaRight, LDA >= max(1,N).
  *
- * @param[out] T
+ * @param[in] T
  *         The IB-by-K triangular factor T of the block reflector.
  *         T is upper triangular by block (economic storage);
  *         The rest of the array is not referenced.
@@ -105,10 +105,10 @@
 #pragma weak CORE_zunmqr = PCORE_zunmqr
 #define CORE_zunmqr PCORE_zunmqr
 #endif
-int CORE_zunmqr(int side, int trans,
+int CORE_zunmqr(PLASMA_enum side, PLASMA_enum trans,
                 int M, int N, int K, int IB,
-                PLASMA_Complex64_t *A, int LDA,
-                PLASMA_Complex64_t *T, int LDT,
+                const PLASMA_Complex64_t *A, int LDA,
+                const PLASMA_Complex64_t *T, int LDT,
                 PLASMA_Complex64_t *C, int LDC,
                 PLASMA_Complex64_t *WORK, int LDWORK)
 {
@@ -222,10 +222,10 @@ int CORE_zunmqr(int side, int trans,
  *
  **/
 void QUARK_CORE_zunmqr(Quark *quark, Quark_Task_Flags *task_flags,
-                       int side, int trans,
+                       PLASMA_enum side, PLASMA_enum trans,
                        int m, int n, int k, int ib, int nb,
-                       PLASMA_Complex64_t *A, int lda,
-                       PLASMA_Complex64_t *T, int ldt,
+                       const PLASMA_Complex64_t *A, int lda,
+                       const PLASMA_Complex64_t *T, int ldt,
                        PLASMA_Complex64_t *C, int ldc)
 {
     DAG_CORE_UNMQR;
@@ -236,13 +236,13 @@ void QUARK_CORE_zunmqr(Quark *quark, Quark_Task_Flags *task_flags,
         sizeof(int),                        &n,     VALUE,
         sizeof(int),                        &k,     VALUE,
         sizeof(int),                        &ib,    VALUE,
-        sizeof(PLASMA_Complex64_t)*nb*nb,    A,             INPUT | QUARK_REGION_L,
+        sizeof(PLASMA_Complex64_t)*nb*nb,   A,      INPUT | QUARK_REGION_L,
         sizeof(int),                        &lda,   VALUE,
-        sizeof(PLASMA_Complex64_t)*ib*nb,    T,             INPUT,
+        sizeof(PLASMA_Complex64_t)*ib*nb,   T,      INPUT,
         sizeof(int),                        &ldt,   VALUE,
-        sizeof(PLASMA_Complex64_t)*nb*nb,    C,             INOUT,
+        sizeof(PLASMA_Complex64_t)*nb*nb,   C,      INOUT,
         sizeof(int),                        &ldc,   VALUE,
-        sizeof(PLASMA_Complex64_t)*ib*nb,    NULL,          SCRATCH,
+        sizeof(PLASMA_Complex64_t)*ib*nb,   NULL,   SCRATCH,
         sizeof(int),                        &nb,    VALUE,
         0);
 }
@@ -256,8 +256,8 @@ void QUARK_CORE_zunmqr(Quark *quark, Quark_Task_Flags *task_flags,
 #endif
 void CORE_zunmqr_quark(Quark *quark)
 {
-    int side;
-    int trans;
+    PLASMA_enum side;
+    PLASMA_enum trans;
     int m;
     int n;
     int k;
@@ -273,6 +273,7 @@ void CORE_zunmqr_quark(Quark *quark)
 
     quark_unpack_args_14(quark, side, trans, m, n, k, ib,
                          A, lda, T, ldt, C, ldc, WORK, ldwork);
-    CORE_zunmqr(side, trans, m, n, k, ib, 
+    CORE_zunmqr(side, trans, m, n, k, ib,
                 A, lda, T, ldt, C, ldc, WORK, ldwork);
 }
+
