@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @author Piotr Luszczek
  * @date 2010-11-15
@@ -32,7 +32,11 @@ char *plasma_lapack_constants[] =
     "",                     // 100
     "Row",                  // 101: PlasmaRowMajor
     "Column",               // 102: PlasmaColMajor
-    "", "", "", "", "", "", "", "",
+    "CCRB",                 // 103: PlasmaCCRB
+    "CRRB",                 // 104: PlasmaCRRB
+    "RCRB",                 // 105: PlasmaRCRB
+    "RRRB",                 // 106: PlasmaRRRB
+    "", "", "", "",
     "No transpose",         // 111: PlasmaNoTrans
     "Transpose",            // 112: PlasmaTrans
     "Conjugate transpose",  // 113: PlasmaConjTrans
@@ -101,8 +105,9 @@ char *plasma_lapack_constants[] =
     "",                     // 300
     "No vectors",           // 301 PlasmaNoVec
     "Vectors needed",       // 302 PlasmaVec
-    "",                     // 303
-    "", "", "", "", "", "",
+    "I",                    // 303 PlasmaIvec
+    "A",                    // 304 PlasmaAllVec 
+    "", "", "", "", "",
     "", "", "", "", "", "", "", "", "", "",
     "", "", "", "", "", "", "", "", "", "",
     "", "", "", "", "", "", "", "", "", "",
@@ -117,5 +122,5 @@ char *plasma_lapack_constants[] =
     "", "", "", "", "", "", "", "",
     "Columnwise",           // 401
     "Rowwise",              // 402
-    "", "", "", "", "", "", "", ""  // Remember to add a coma!
+    "", "", "", "", "", "", "", ""  // Remember to add a comma!
 };
