@@ -8,7 +8,7 @@
  *
  * @brief Example using LQ factorization
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Bilel Hadri
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -53,12 +53,12 @@ int main ()
     PLASMA_Complex64_t *A1 = (PLASMA_Complex64_t *)malloc(LDA*N*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *A2 = (PLASMA_Complex64_t *)malloc(LDA*N*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *Q  = (PLASMA_Complex64_t *)malloc(LDA*N*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *T;
+    PLASMA_desc *T;
 
     /* Check if unable to allocate memory */
     if ((!A1)||(!A2)||(!Q)){
         printf("Out of Memory \n ");
-        exit(0);
+        return EXIT_SUCCESS;
     }
 
     /* Plasma Initialization */
@@ -97,7 +97,7 @@ int main ()
 
     PLASMA_Finalize();
 
-    exit(0);
+    return EXIT_SUCCESS;
 }
 
 /*-------------------------------------------------------------------
