@@ -8,7 +8,7 @@
  *
  * @brief Example for solving overdetermined or underdetemined linear systems
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Bilel Hadri
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -56,12 +56,12 @@ int main ()
     PLASMA_Complex64_t *A2 = (PLASMA_Complex64_t *)malloc(LDA*N*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *B1 = (PLASMA_Complex64_t *)malloc(LDB*NRHS*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *B2 = (PLASMA_Complex64_t *)malloc(LDB*NRHS*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *T;
+    PLASMA_desc *T;
 
     /* Check if unable to allocate memory */
     if ((!A1)||(!A2)||(!B1)||(!B2)){
         printf("Out of Memory \n ");
-        exit(0);
+        return EXIT_SUCCESS;
     }
 
     /* Plasma Initialization */
@@ -98,7 +98,7 @@ int main ()
 
     PLASMA_Finalize();
 
-    exit(0);
+    return EXIT_SUCCESS;
 }
 
 /*--------------------------------------------------------------
