@@ -1,7 +1,7 @@
         PROGRAM EXAMPLE_ZGESV_F
 *
 *********************************************************************
-*     PLASMA example routine (version 2.4.0)                        
+*     PLASMA example routine (version 2.5.1)                        
 *     Author: Bilel Hadri                                           
 *     Release Date: November, 15th 2010                             
 *     PLASMA is a software package provided by Univ. of Tennessee,  
@@ -41,7 +41,7 @@
 *     ..
 *     .. External Subroutines ..
       EXTERNAL          ZLARNV, DLAMCH, ZLANGE
-      EXTERNAL          PLASMA_INIT, PLASMA_ALLOC_WORKSPACE_ZGESV
+      EXTERNAL          PLASMA_INIT, PLASMA_ALLOC_WORKSPACE_ZGESV_INCPIV
       EXTERNAL          PLASMA_ZGESV, PLASMA_FINALIZE
       EXTERNAL          PLASMA_DEALLOC_HANDLE
       EXTERNAL          ZGEMM
