@@ -8,7 +8,7 @@
  *
  * @brief Example of solving a linear system with a Cholesky factorization
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Bilel Hadri
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -22,7 +22,6 @@
 #include <plasma.h>
 #include <cblas.h>
 #include <lapacke.h>
-#include <plasma_tmg.h>
 #include <core_blas.h>
 
 int check_solution(int, int, PLASMA_Complex64_t*, int, PLASMA_Complex64_t*, PLASMA_Complex64_t*, int);
@@ -40,21 +39,16 @@ int main ()
     int LDB   = 10;
     int info;
     int info_solution;
-    int i,j;
-    int NminusOne = N-1;
-    int LDBxNRHS = LDB*NRHS;
 
     PLASMA_Complex64_t *A1   = (PLASMA_Complex64_t *)malloc(LDA*N*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *A2   = (PLASMA_Complex64_t *)malloc(LDA*N*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *B1   = (PLASMA_Complex64_t *)malloc(LDB*NRHS*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *B2   = (PLASMA_Complex64_t *)malloc(LDB*NRHS*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *WORK = (PLASMA_Complex64_t *)malloc(2*LDA*sizeof(PLASMA_Complex64_t));
-    double *D                = (double *)malloc(LDA*sizeof(double));
 
     /* Check if unable to allocate memory */
     if ((!A1)||(!A2)||(!B1)||(!B2)){
         printf("Out of Memory \n ");
-        exit(0);
+        return EXIT_SUCCESS;
     }
 
     /* Plasma Initialize */
@@ -65,23 +59,13 @@ int main ()
     *  TESTING ZPOSV
     */
 
-    /* Initialize A1 and A2 for Symmetric Positif Matrix */
-    LAPACKE_dlarnv_work(IONE, ISEED, LDA, D);
-    zlaghe(&N, &NminusOne, D, A1, &LDA, ISEED, WORK, &info);
-    for ( i = 0; i < N; i++)
-        for (  j = 0; j < N; j++)
-            A2[LDA*j+i] = A1[LDA*j+i];
-
-    for ( i = 0; i < N; i++){
-        A1[LDA*i+i] = A1[LDA*i+i] + (PLASMA_Complex64_t)N ;
-        A2[LDA*i+i] = A1[LDA*i+i];
-    }
+    /* Initialize A1 and A2 for Symmetric Positif Matrix (Hessenberg in the complex case) */
+    PLASMA_zplghe( (double)N, N, A1, LDA, 51 );
+    PLASMA_zlacpy( PlasmaUpperLower, N, N, A1, LDA, A2, LDA );
 
     /* Initialize B1 and B2 */
-    LAPACKE_zlarnv_work(IONE, ISEED, LDBxNRHS, B1);
-    for ( i = 0; i < N; i++)
-        for ( j = 0; j < NRHS; j++)
-            B2[LDB*j+i] = B1[LDB*j+i];
+    PLASMA_zplrnt( N, NRHS, B1, LDB, 371 );
+    PLASMA_zlacpy( PlasmaUpperLower, N, NRHS, B1, LDB, B2, LDB );
 
     /* PLASMA ZPOSV */
     info = PLASMA_zposv(PlasmaUpper, N, NRHS, A2, LDA, B2, LDB);
@@ -94,11 +78,11 @@ int main ()
     else
        printf("-- Run of ZPOSV example successful ! \n");
 
-    free(A1); free(A2); free(B1); free(B2); free(WORK); free(D);
+    free(A1); free(A2); free(B1); free(B2);
 
     PLASMA_Finalize();
 
-    exit(0);
+    return EXIT_SUCCESS;
 }
 
 /*------------------------------------------------------------------------
