@@ -8,7 +8,7 @@
  *
  * @brief Example of Cholesky factorization
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Bilel Hadri
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -22,7 +22,6 @@
 #include <plasma.h>
 #include <cblas.h>
 #include <lapacke.h>
-#include <plasma_tmg.h>
 #include <core_blas.h>
 
 int check_factorization(int, PLASMA_Complex64_t*, PLASMA_Complex64_t*, int, int);
@@ -36,20 +35,15 @@ int main ()
     int cores = 2;
     int N     = 10 ;
     int LDA   = 10 ;
-    int info;
     int info_factorization;
-    int i,j;
-    int NminusOne = N-1;
 
     PLASMA_Complex64_t *A1   = (PLASMA_Complex64_t *)malloc(LDA*N*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *A2   = (PLASMA_Complex64_t *)malloc(LDA*N*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *WORK = (PLASMA_Complex64_t *)malloc(2*LDA*sizeof(PLASMA_Complex64_t));
-    double *D                = (double *)malloc(LDA*sizeof(double));
 
     /* Check if unable to allocate memory */
     if ((!A1)||(!A2)){
         printf("Out of Memory \n ");
-        exit(0);
+        return EXIT_SUCCESS;
     }
 
     /* Plasma Initialize */
@@ -57,16 +51,8 @@ int main ()
     printf("-- PLASMA is initialized to run on %d cores. \n",cores);
 
     /* Initialize A1 and A2 for Symmetric Positive Matrix */
-    LAPACKE_dlarnv_work(IONE, ISEED, LDA, D);
-    zlaghe(&N, &NminusOne, D, A1, &LDA, ISEED, WORK, &info);
-    for ( i = 0; i < N; i++)
-        for (  j = 0; j < N; j++)
-            A2[LDA*j+i] = A1[LDA*j+i];
-
-    for ( i = 0; i < N; i++){
-        A1[LDA*i+i] = A1[LDA*i+i]+ (PLASMA_Complex64_t)N ;
-        A2[LDA*i+i] = A1[LDA*i+i];
-    }
+    PLASMA_zplghe( (double)N, N, A1, LDA, 51 );
+    PLASMA_zlacpy( PlasmaUpperLower, N, N, A1, LDA, A2, LDA );
 
     /* Plasma routines */
     PLASMA_zpotrf(PlasmaUpper, N, A2, LDA);
@@ -74,16 +60,16 @@ int main ()
     /* Check the factorization */
     info_factorization = check_factorization( N, A1, A2, LDA, PlasmaUpper);
 
-    if ((info_factorization != 0)|(info != 0))
+    if ( info_factorization != 0 )
        printf("-- Error in ZPOTRF example ! \n");
     else
        printf("-- Run of ZPOTRF example successful ! \n");
 
-    free(A1); free(A2); free(WORK); free(D);
+    free(A1); free(A2);
 
     PLASMA_Finalize();
 
-    exit(0);
+    return EXIT_SUCCESS;
 }
 
 
