@@ -9,7 +9,7 @@
  * @brief Example for solving a system of linear equations using 
  *        Cholesky factorization and PLASMA_ztrsm routine
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Bilel Hadri
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -23,7 +23,6 @@
 #include <plasma.h>
 #include <cblas.h>
 #include <lapacke.h>
-#include <plasma_tmg.h>
 #include <core_blas.h>
 
 int check_solution(int, int, PLASMA_Complex64_t*, int, PLASMA_Complex64_t*, PLASMA_Complex64_t*, int);
@@ -39,23 +38,17 @@ int main ()
     int LDA   = 10 ;
     int NRHS  = 5 ;
     int LDB   = 10 ;
-    int info;
     int info_solution;
-    int i,j;
-    int NminusOne = N-1;
-    int LDBxNRHS = LDB*NRHS;
 
     PLASMA_Complex64_t *A1   = (PLASMA_Complex64_t *)malloc(LDA*N*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *A2   = (PLASMA_Complex64_t *)malloc(LDA*N*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *B1   = (PLASMA_Complex64_t *)malloc(LDB*NRHS*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *B2   = (PLASMA_Complex64_t *)malloc(LDB*NRHS*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *WORK = (PLASMA_Complex64_t *)malloc(2*LDA*sizeof(PLASMA_Complex64_t));
-    double *D                = (double *)malloc(LDA*sizeof(double));
 
     /* Check if unable to allocate memory */
     if ((!A1)||(!A2)||(!B1)||(!B2)){
         printf("Out of Memory \n ");
-        exit(0);
+        return EXIT_SUCCESS;
     }
 
     /* Plasma Initialize */
@@ -63,43 +56,33 @@ int main ()
     printf("-- PLASMA is initialized to run on %d cores. \n",cores);
 
     /* Initialize A1 and A2 for Symmetric Positive Matrix */
-    LAPACKE_dlarnv_work(IONE, ISEED, LDA, D);
-    zlaghe(&N, &NminusOne, D, A1, &LDA, ISEED, WORK, &info);
-    for ( i = 0; i < N; i++)
-        for (  j = 0; j < N; j++)
-            A2[LDA*j+i] = A1[LDA*j+i];
-
-    for ( i = 0; i < N; i++){
-      A1[LDA*i+i] = A1[LDA*i+i]+ (PLASMA_Complex64_t)N ;
-       A2[LDA*i+i] = A1[LDA*i+i];
-    }
+    PLASMA_zplghe( (double)N, N, A1, LDA, 51 );
+    PLASMA_zlacpy( PlasmaUpperLower, N, N, A1, LDA, A2, LDA );
 
     /* Initialize B1 and B2 */
-    LAPACKE_zlarnv_work(IONE, ISEED, LDBxNRHS, B1);
-    for ( i = 0; i < N; i++)
-        for ( j = 0; j < NRHS; j++)
-            B2[LDB*j+i] = B1[LDB*j+i];
+    PLASMA_zplrnt( N, NRHS, B1, LDB, 371 );
+    PLASMA_zlacpy( PlasmaUpperLower, N, NRHS, B1, LDB, B2, LDB );
 
     /* PLASMA routines */
-    info = PLASMA_zpotrf(PlasmaLower, N, A2, LDA);
-    info = PLASMA_ztrsm(PlasmaLeft, PlasmaLower, PlasmaNoTrans, PlasmaNonUnit, 
-                        N, NRHS, (PLASMA_Complex64_t)1.0, A2, LDA, B2, LDB);
-    info = PLASMA_ztrsm(PlasmaLeft, PlasmaLower, PlasmaConjTrans, PlasmaNonUnit,
-                        N, NRHS, (PLASMA_Complex64_t)1.0, A2, LDA, B2, LDB);
+    PLASMA_zpotrf(PlasmaLower, N, A2, LDA);
+    PLASMA_ztrsm(PlasmaLeft, PlasmaLower, PlasmaNoTrans, PlasmaNonUnit, 
+                 N, NRHS, (PLASMA_Complex64_t)1.0, A2, LDA, B2, LDB);
+    PLASMA_ztrsm(PlasmaLeft, PlasmaLower, PlasmaConjTrans, PlasmaNonUnit,
+                 N, NRHS, (PLASMA_Complex64_t)1.0, A2, LDA, B2, LDB);
 
     /* Check the solution */
     info_solution = check_solution(N, NRHS, A1, LDA, B1, B2, LDB);
 
-    if ((info_solution != 0)|(info != 0))
+    if ( info_solution != 0 )
        printf("-- Error in ZTRSM example ! \n");
     else
        printf("-- Run of ZTRSM example successful ! \n");
 
-    free(A1); free(A2); free(B1); free(B2); free(WORK); free(D);
+    free(A1); free(A2); free(B1); free(B2);
 
     PLASMA_Finalize();
 
-    exit(0);
+    return EXIT_SUCCESS;
 }
 
 
