@@ -9,7 +9,7 @@
  * @brief Example for solving a system of linear equations using 
  *        LU factorization and PLASMA_ztrsmpl routine
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Bilel Hadri
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -48,13 +48,13 @@ int main ()
     PLASMA_Complex64_t *A2 = (PLASMA_Complex64_t *)malloc(LDA*N*(sizeof*A2));
     PLASMA_Complex64_t *B1 = (PLASMA_Complex64_t *)malloc(LDB*NRHS*(sizeof*B1));
     PLASMA_Complex64_t *B2 = (PLASMA_Complex64_t *)malloc(LDB*NRHS*(sizeof*B2));
-    PLASMA_Complex64_t *L;
+    PLASMA_desc *L;
     int *IPIV;
 
     /* Check if unable to allocate memory */
     if ((!A1)||(!A2)||(!B1)||(!B2)){
         printf("Out of Memory \n ");
-        exit(0);
+        return EXIT_SUCCESS;
     }
 
     /*Plasma Initialize*/
@@ -97,7 +97,7 @@ int main ()
 
     PLASMA_Finalize();
 
-    exit(0);
+    return EXIT_SUCCESS;
 }
 
 /*------------------------------------------------------------------------
