@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @author Hatem Ltaief
  * @date 2010-11-15
@@ -29,13 +29,13 @@
 extern "C" {
 #endif
 
-  /* 
-   * Coreblas Error 
+  /*
+   * Coreblas Error
    */
-#define coreblas_error(k, str) fprintf(stderr, "%s: Parameter %d / %s\n", __func__, k, str)
+#define coreblas_error(k, str) fprintf(stderr, "%s: Parameter %d / %s\n", __func__, k, str);
 
-  /* 
-   * CBlas enum 
+  /*
+   * CBlas enum
    */
 #define CBLAS_TRANSPOSE enum CBLAS_TRANSPOSE
 #define CBLAS_UPLO      enum CBLAS_UPLO
@@ -47,8 +47,8 @@ extern "C" {
  **/
 extern char *plasma_lapack_constants[];
 #define lapack_const(plasma_const) plasma_lapack_constants[plasma_const][0]
-  
-  /* 
+
+  /*
    * Functions which don't depend on precision
    */
 void CORE_free_quark(Quark *quark);
@@ -58,6 +58,12 @@ void CORE_foo2_quark(Quark *quark);
 void QUARK_CORE_free(Quark *quark, Quark_Task_Flags *task_flags,
                      void *A, int szeA);
 
+void CORE_pivot_update(int m, int n, int *ipiv, int *indices,
+                       int offset, int init);
+void CORE_pivot_update_quark(Quark *quark);
+void QUARK_CORE_pivot_update(Quark *quark, Quark_Task_Flags *task_flags,
+                             int m, int n, int *ipiv, int *indices,
+                             int offset, int init);
 #ifdef __cplusplus
 }
 #endif
