@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @author Hatem Ltaief
  * @author Mathieu Faverge
@@ -16,7 +16,6 @@
  **/
 #ifndef _PLASMA_CORE_ZCBLAS_H_
 #define _PLASMA_CORE_ZCBLAS_H_
-#define COMPLEX
 
 #ifdef __cplusplus
 extern "C" {
@@ -25,23 +24,23 @@ extern "C" {
 /** ****************************************************************************
  *  Declarations of serial kernels - alphabetical order
  **/
-void CORE_clag2z(int m, int n, 
-                 PLASMA_Complex32_t *A, int lda, 
+void CORE_clag2z(int m, int n,
+                 const PLASMA_Complex32_t *A, int lda,
                  PLASMA_Complex64_t *B, int ldb);
-void CORE_zlag2c(int m, int n, 
-                 PLASMA_Complex64_t *A, int lda, 
+void CORE_zlag2c(int m, int n,
+                 const PLASMA_Complex64_t *A, int lda,
                  PLASMA_Complex32_t *B, int ldb, int *info);
 
 /** ****************************************************************************
  *  Declarations of QUARK wrappers (called by PLASMA) - alphabetical order
  **/
 void QUARK_CORE_clag2z(Quark *quark, Quark_Task_Flags *task_flags,
-                      int m, int n, int nb,
-                      PLASMA_Complex32_t *A, int lda,
+                       int m, int n, int nb,
+                       const PLASMA_Complex32_t *A, int lda,
                        PLASMA_Complex64_t *B, int ldb);
 void QUARK_CORE_zlag2c(Quark *quark, Quark_Task_Flags *task_flags,
                        int m, int n, int nb,
-                       PLASMA_Complex64_t *A, int lda,
+                       const PLASMA_Complex64_t *A, int lda,
                        PLASMA_Complex32_t *B, int ldb,
                        PLASMA_sequence *sequence, PLASMA_request *request);
 
