@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Jakub Kurzak
  * @date 2010-11-15
  *
@@ -15,14 +15,12 @@
 #define _PLASMA_H_
 
 #define PLASMA_VERSION_MAJOR 2
-#define PLASMA_VERSION_MINOR 4
-#define PLASMA_VERSION_MICRO 0
-
-#include <quark.h>
+#define PLASMA_VERSION_MINOR 5
+#define PLASMA_VERSION_MICRO 1
 
 /* CBLAS requires for scalar arguments to be passed by address rather than by value */
 #ifndef CBLAS_SADDR
-#define CBLAS_SADDR &
+#define CBLAS_SADDR( _val_ ) &(_val_)
 #endif
 
 /** ****************************************************************************
@@ -42,24 +40,25 @@ typedef int  PLASMA_enum;
 typedef int  PLASMA_bool;
 typedef long PLASMA_index;
 typedef long PLASMA_size;
+typedef double PLASMA_Double_t;
 
 /** ****************************************************************************
  * PLASMA Complex numbers
  **/
 #define PLASMA_HAS_COMPLEX_H 1
 
-#if defined(_WIN32) 
+#if defined(_WIN32)
 # include <float.h>
-# if defined(__INTEL_COMPILER) 
+# if defined(__INTEL_COMPILER)
     /* Fix name conflict within the cabs prototype (_Complex) that
      * conflicts with a C99 keyword.  */
     #define _Complex __ConflictingComplex
     #include <math.h>
-    #undef _Complex 
+    #undef _Complex
     #undef complex
     typedef float  _Complex PLASMA_Complex32_t;
     typedef double _Complex PLASMA_Complex64_t;
-# else 
+# else
     /* Use MS VC complex class */
     #include <complex>
     typedef std::complex<float> PLASMA_Complex32_t;
@@ -68,7 +67,7 @@ typedef long PLASMA_size;
     #define LAPACK_COMPLEX_CUSTOM
     #define lapack_complex_float std::complex<float>
     #define lapack_complex_double std::complex<double>
-    #undef PLASMA_HAS_COMPLEX_H 
+    #undef PLASMA_HAS_COMPLEX_H
 # endif
 # define isnan _isnan
 # define isinf !_finite
@@ -78,13 +77,19 @@ typedef long PLASMA_size;
     typedef float  _Complex PLASMA_Complex32_t;
     typedef double _Complex PLASMA_Complex64_t;
 
-#endif 
+#endif
 
 /* Sun doesn't ship the complex.h header. Sun Studio doesn't have it and older GCC compilers don't have it either. */
 #if defined(__SUNPRO_C) || defined(__SUNPRO_CC) || defined(sun) || defined(__sun)
 #undef PLASMA_HAS_COMPLEX_H
 #endif
 
+#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1))
+#define PLASMA_DEPRECATED  __attribute__((__deprecated__))
+#else
+#define PLASMA_DEPRECATED
+#endif /* __GNUC__ */
+
 #ifdef PLASMA_HAS_COMPLEX_H
 #include <complex.h>
 #else
@@ -94,7 +99,7 @@ extern "C" {
 #endif
 
 /* These declarations will not clash with what C++ provides because the names in C++ are name-mangled. */
-#if !defined(_WIN32) 
+#if !defined(_WIN32)
 extern double cabs(PLASMA_Complex64_t z);
 extern PLASMA_Complex64_t conj(PLASMA_Complex64_t z);
 #endif
@@ -108,17 +113,19 @@ extern double creal(PLASMA_Complex64_t z);
 
 #endif
 
+#include <quark.h>
+
 /** ****************************************************************************
  *  Tile matrix descriptor
  *
- *  Matrices are stored in a contiguous data chunk containning in order
+ *  Matrices are stored in a contiguous data chunk containing in order
  *  A11, A21, A12, A22 with :
  *
  *           n1      n2
  *      +----------+---+
  *      |          |   |    With m1 = lm - (lm%mb)
  *      |          |   |         m2 = lm%mb
- *  m1  |    A11   |A12|         n1 = ln - (ln%mb)
+ *  m1  |    A11   |A12|         n1 = ln - (ln%nb)
  *      |          |   |         n2 = ln%nb
  *      |          |   |
  *      +----------+---+
@@ -127,33 +134,33 @@ extern double creal(PLASMA_Complex64_t z);
  *
  */
 typedef struct plasma_desc_t {
-    void *mat;          // pointer to the beginning of the matrix
-    size_t A21;        // pointer to the beginning of the matrix A21
-    size_t A12;        // pointer to the beginning of the matrix A12
-    size_t A22;        // pointer to the beginning of the matrix A22
-    PLASMA_enum dtyp;   // precision of the matrix
-    int mb;             // number of rows in a tile
-    int nb;             // number of columns in a tile
-    int bsiz;           // size in elements including padding
-    int lm;             // number of rows of the entire matrix
-    int ln;             // number of columns of the entire matrix
-    int lm1;            // number of tile rows of the A11 matrix - derived parameter
-    int ln1;            // number of tile columns of the A11 matrix - derived parameter
-    int lmt;            // number of tile rows of the entire matrix - derived parameter
-    int lnt;            // number of tile columns of the entire matrix - derived parameter
-    int i;              // row index to the beginning of the submatrix
-    int j;              // column index to the beginning of the submatrix
-    int m;              // number of rows of the submatrix
-    int n;              // number of columns of the submatrix
-    int mt;             // number of tile rows of the submatrix - derived parameter
-    int nt;             // number of tile columns of the submatrix - derived parameter
+    void *mat;          /**< pointer to the beginning of the matrix                           */
+    size_t A21;         /**< pointer to the beginning of the matrix A21                       */
+    size_t A12;         /**< pointer to the beginning of the matrix A12                       */
+    size_t A22;         /**< pointer to the beginning of the matrix A22                       */
+    PLASMA_enum dtyp;   /**< precision of the matrix                                          */
+    int mb;             /**< number of rows in a tile                                         */
+    int nb;             /**< number of columns in a tile                                      */
+    int bsiz;           /**< size in elements including padding                               */
+    int lm;             /**< number of rows of the entire matrix                              */
+    int ln;             /**< number of columns of the entire matrix                           */
+    int lm1;            /**< number of tile rows of the A11 matrix - derived parameter        */
+    int ln1;            /**< number of tile columns of the A11 matrix - derived parameter     */
+    int lmt;            /**< number of tile rows of the entire matrix - derived parameter     */
+    int lnt;            /**< number of tile columns of the entire matrix - derived parameter  */
+    int i;              /**< row index to the beginning of the submatrix                      */
+    int j;              /**< column index to the beginning of the submatrix                   */
+    int m;              /**< number of rows of the submatrix                                  */
+    int n;              /**< number of columns of the submatrix                               */
+    int mt;             /**< number of tile rows of the submatrix - derived parameter         */
+    int nt;             /**< number of tile columns of the submatrix - derived parameter      */
 } PLASMA_desc;
 
 /** ****************************************************************************
  *  PLASMA request uniquely identifies each asynchronous function call.
  **/
 typedef struct plasma_request_t {
-    PLASMA_enum status; // PLASMA_SUCCESS or appropriate error code
+    PLASMA_enum status; /**< PLASMA_SUCCESS or appropriate error code */
 } PLASMA_request;
 
 #define PLASMA_REQUEST_INITIALIZER {PLASMA_SUCCESS}
@@ -163,22 +170,22 @@ typedef struct plasma_request_t {
  *  sharing common exception handling.
  **/
 typedef struct plasma_sequence_t {
-    Quark_Sequence *quark_sequence; // QUARK sequence associated with PLASMA sequence
-    PLASMA_bool     status;         // PLASMA_SUCCESS or appropriate error code
-    PLASMA_request *request;        // failed request
+    Quark_Sequence *quark_sequence; /**< QUARK sequence associated with PLASMA sequence */
+    PLASMA_bool     status;         /**< PLASMA_SUCCESS or appropriate error code       */
+    PLASMA_request *request;        /**< failed request                                 */
 } PLASMA_sequence;
 
 /** ****************************************************************************
  *
  *  PLASMA constants - CBLAS & LAPACK
- *  The naming and numbering is consisten with:
+ *  The naming and numbering is consistent with:
  *
  *    1) CBLAS from Netlib (http://www.netlib.org/blas/blast-forum/cblas.tgz),
  *    2) C Interface to LAPACK from Netlib (http://www.netlib.org/lapack/lapwrapc/).
  *
  **/
-#define PlasmaCM            101
-#define PlasmaRM            102
+#define PlasmaRM            101
+#define PlasmaCM            102
 #define PlasmaCCRB          103
 #define PlasmaCRRB          104
 #define PlasmaRCRB          105
@@ -227,6 +234,8 @@ typedef struct plasma_sequence_t {
 
 #define PlasmaNoVec         301
 #define PlasmaVec           302
+#define PlasmaIvec          303
+#define PlasmaAllVec        304
 
 #define PlasmaForward       391
 #define PlasmaBackward      392
@@ -236,6 +245,8 @@ typedef struct plasma_sequence_t {
 #define PlasmaTrd          1001
 #define PlasmaBrd          1002
 
+#define PlasmaW             501
+#define PlasmaA2            502
 
 #define plasma_const_neg(const) (((const-1)^0x01)+1)
 
@@ -251,6 +262,7 @@ typedef struct plasma_sequence_t {
 #define PLASMA_WARNINGS   1
 #define PLASMA_ERRORS     2
 #define PLASMA_AUTOTUNING 3
+#define PLASMA_DAG        4
 
 /** ****************************************************************************
  *  PLASMA constants - configuration parameters
@@ -262,6 +274,8 @@ typedef struct plasma_sequence_t {
 #define PLASMA_HOUSEHOLDER_MODE 5
 #define PLASMA_HOUSEHOLDER_SIZE 6
 #define PLASMA_TRANSLATION_MODE 7
+#define PLASMA_TNTPIVOTING_MODE 8
+#define PLASMA_TNTPIVOTING_SIZE 9
 
 #define PLASMA_STATIC_SCHEDULING  1
 #define PLASMA_DYNAMIC_SCHEDULING 2
@@ -269,6 +283,9 @@ typedef struct plasma_sequence_t {
 #define PLASMA_FLAT_HOUSEHOLDER 1
 #define PLASMA_TREE_HOUSEHOLDER 2
 
+#define PLASMA_TOURNAMENT_LU 1
+#define PLASMA_TOURNAMENT_QR 2
+
 #define PLASMA_INPLACE    1
 #define PLASMA_OUTOFPLACE 2
 
