@@ -1,21 +1,21 @@
-***
-*
-* @file plasmaf.h
-*
-*  PLASMA FORTRAN header
-*  PLASMA is a software package provided by Univ. of Tennessee,
-*  Univ. of California Berkeley and Univ. of Colorado Denver
-*
-* @version 2.4.0
-* @author Bilel Hadri
-* @author Mathieu Faverge
-* @date 2010-11-15
-*
-***
+!**
+!
+! @file plasmaf.h
+!
+!  PLASMA FORTRAN header
+!  PLASMA is a software package provided by Univ. of Tennessee,
+!  Univ. of California Berkeley and Univ. of Colorado Denver
+!
+! @version 2.5.1
+! @author Bilel Hadri
+! @author Mathieu Faverge
+! @date 2010-11-15
+!
+!**
 
-*********************************************************************
-*   PLASMA constants - precisions
-*
+!********************************************************************
+!   PLASMA constants - precisions
+!
       integer  PlasmaByte, PlasmaInteger, PlasmaRealFloat
       integer  PlasmaRealDouble, PlasmaComplexFloat, PlasmaComplexDouble
       parameter ( PlasmaByte          = 0 )
@@ -25,13 +25,13 @@
       parameter ( PlasmaComplexFloat  = 4 )
       parameter ( PlasmaComplexDouble = 5 )
 
-*********************************************************************
-*   PLASMA constants - CBLAS & LAPACK
-*
-      integer PlasmaCM, PlasmaRM, PlasmaCCRB
+!********************************************************************
+!   PLASMA constants - CBLAS & LAPACK
+!
+      integer PlasmaRM, PlasmaCM, PlasmaCCRB
       integer PlasmaCRRB, PlasmaRCRB, PlasmaRRRB
-      parameter ( PlasmaCM         = 101 )
-      parameter ( PlasmaRM         = 102 )
+      parameter ( PlasmaRM         = 101 )
+      parameter ( PlasmaCM         = 102 )
       parameter ( PlasmaCCRB       = 103 )
       parameter ( PlasmaCRRB       = 104 )
       parameter ( PlasmaRCRB       = 105 )
@@ -102,9 +102,11 @@
       parameter ( PlasmaPackUpeprBand = 297 )
       parameter ( PlasmaPackAll       = 298 )
 
-      integer PlasmaNoVec,PlasmaVec
-      parameter ( PlasmaNoVec = 301 )
-      parameter ( PlasmaVec   = 302 )
+      integer PlasmaNoVec,PlasmaVec,PlasmaIvec, PlasmaAllVec
+      parameter ( PlasmaNoVec  = 301 )
+      parameter ( PlasmaVec    = 302 )
+      parameter ( PlasmaIvec   = 303 )
+      parameter ( PlasmaAllVec = 304 )
 
       integer PlasmaForward, PlasmaBackward
       parameter ( PlasmaForward    = 391 )
@@ -114,24 +116,26 @@
       parameter ( PlasmaColumnwise = 401 )
       parameter ( PlasmaRowwise    = 402 )
 
-*********************************************************************
-*   PLASMA constants - boolean
-*
+!********************************************************************
+!   PLASMA constants - boolean
+!
       integer PLASMA_FALSE, PLASMA_TRUE
       parameter ( PLASMA_FALSE = 0 )
       parameter ( PLASMA_TRUE  = 1 )
 
-*********************************************************************
-*   State machine switches
-*
+!********************************************************************
+!   State machine switches
+!
       integer PLASMA_WARNINGS, PLASMA_ERRORS, PLASMA_AUTOTUNING
+      integer PLASMA_DAG
       parameter ( PLASMA_WARNINGS   = 1 )
       parameter ( PLASMA_ERRORS     = 2 )
       parameter ( PLASMA_AUTOTUNING = 3 )
+      parameter ( PLASMA_DAG        = 4 )
 
-*********************************************************************
-*   PLASMA constants - configuration  parameters
-*
+!********************************************************************
+!   PLASMA constants - configuration  parameters
+!
       integer PLASMA_CONCURRENCY, PLASMA_TILE_SIZE
       integer PLASMA_INNER_BLOCK_SIZE, PLASMA_SCHEDULING_MODE
       integer PLASMA_HOUSEHOLDER_MODE, PLASMA_HOUSEHOLDER_SIZE
@@ -144,30 +148,30 @@
       parameter ( PLASMA_HOUSEHOLDER_SIZE = 6 )
       parameter ( PLASMA_TRANSLATION_MODE = 7 )
 
-*********************************************************************
-*   PLASMA constants - scheduling mode
-*
+!********************************************************************
+!   PLASMA constants - scheduling mode
+!
       integer PLASMA_STATIC_SCHEDULING, PLASMA_DYNAMIC_SCHEDULING
       parameter ( PLASMA_STATIC_SCHEDULING   = 1 )
       parameter ( PLASMA_DYNAMIC_SCHEDULING  = 2 )
 
-*********************************************************************
-*   PLASMA constants - householder mode
-*
+!********************************************************************
+!   PLASMA constants - householder mode
+!
       integer PLASMA_FLAT_HOUSEHOLDER, PLASMA_TREE_HOUSEHOLDER
       parameter ( PLASMA_FLAT_HOUSEHOLDER  = 1 )
       parameter ( PLASMA_TREE_HOUSEHOLDER  = 2 )
 
-**********************************************************************
-*   PLASMA constants - translation mode
-*
+!*********************************************************************
+!   PLASMA constants - translation mode
+!
       integer PLASMA_INPLACE, PLASMA_OUTOFPLACE
       parameter ( PLASMA_INPLACE     = 1 )
       parameter ( PLASMA_OUTOFPLACE  = 2 )
 
-*********************************************************************
-*   PLASMA constants - success & error codes
-*
+!********************************************************************
+!   PLASMA constants - success & error codes
+!
       integer PLASMA_SUCCESS, PLASMA_ERR_NOT_INITIALIZED
       integer PLASMA_ERR_REINITIALIZED, PLASMA_ERR_NOT_SUPPORTED
       integer PLASMA_ERR_ILLEGAL_VALUE, PLASMA_ERR_NOT_FOUND
