@@ -7,7 +7,7 @@
  * PLASMA is a software package provided by Univ. of Tennessee,
  * Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Asim YarKhan
  * @date 2010-11-15
  *
@@ -34,12 +34,11 @@
 extern "C" {
 #endif
 
-#undef DBGQUARK
 #ifdef DBGQUARK
-/* #define DBGPRINTF(args...) { fprintf(stderr,"%s:%d: [%s] ",__FILE__,__LINE__,__FUNCTION__); fprintf(stderr, args); } */
-#define DBGPRINTF(args...) { fprintf(stderr, args); }
+/* #define DBGPRINTF(str, ...) { fprintf(stderr, "%s:%d: [%s] " str, __FILE__, __LINE__, __FUNCTION__, __VA_ARGS__); } */
+#define DBGPRINTF(...) { fprintf(stderr, __VA_ARGS__); }
 #else
-#define DBGPRINTF(args...) if (0) {};
+#define DBGPRINTF(...) if (0) {};
 #endif
 
 #define QUARK_SUCCESS 0
@@ -97,7 +96,7 @@ typedef enum { QINPUT=0x100, OUTPUT=0x200, INOUT=0x300, VALUE=0x400, NODEP=0x500
 /* The thread that runs this task should have manual scheduling enabled (1) or disabled (0) */
 #define THREAD_SET_TO_MANUAL_SCHEDULING ( 1 << 21 )
 /* Lock the task to a thead mask (0 ... NTHREADS-1) bits long, provided as a character array (byte array) */
-// #define TASK_LOCK_TO_THREAD_MASK ( 1 << 16 )
+#define TASK_LOCK_TO_THREAD_MASK ( 1 << 22 )
 
 /* The range for priority values */
 #define QUARK_TASK_MIN_PRIORITY 0
@@ -118,12 +117,12 @@ struct quark_task_flags_s {
     void *task_sequence;
     int task_thread_count;
     int thread_set_to_manual_scheduling;
-//    char *task_lock_to_thread_mask;
+    unsigned char *task_lock_to_thread_mask;
 };
 
 typedef struct quark_task_flags_s Quark_Task_Flags;
 /* Static initializer for Quark_Task_Flags_t */
-#define Quark_Task_Flags_Initializer { (int)0, (int)-1, (char *)NULL, (char *)NULL, (void *)NULL, (int)1, (int)-1 }
+#define Quark_Task_Flags_Initializer { (int)0, (int)-1, (char *)NULL, (char *)NULL, (void *)NULL, (int)1, (int)-1, (unsigned char *)NULL }
 
 /* Setup scheduler data structures, assumes threads are managed seperately */
 Quark *QUARK_Setup(int num_threads);
@@ -184,7 +183,10 @@ void QUARK_Task_Pack_Arg( Quark *quark, Quark_Task *task, int arg_size, void *ar
 unsigned long long QUARK_Insert_Task_Packed(Quark * quark, Quark_Task *task );
 
 /* Unsupported function for debugging purposes; execute task AT ONCE */
-unsigned long long QUARK_Execute_Task(Quark * quark, void (*function) (Quark *), Quark_Task_Flags *task_flags, ...);
+unsigned long long QUARK_Execute_Task_Packed( Quark *quark, Quark_Task *task );
+
+/* Unsupported function for debugging purposes; execute task AT ONCE */
+unsigned long long QUARK_Execute_Task( Quark *quark, void (*function) (Quark *), Quark_Task_Flags *task_flags, ...);
 
 /* Get the label (if any) associated with the current task; used for printing and debugging  */
 char *QUARK_Get_Task_Label(Quark *quark);
@@ -222,6 +224,35 @@ intptr_t QUARK_Task_Flag_Get( Quark *quark, int flag );
  * a sync, such as QUARK_Barrier. */
 void QUARK_DOT_DAG_Enable( Quark *quark, int boolean_value );
 
+/* Get the number_th bit in a bitset (unsigned char *); useful for QUARK_LOCK_TO_THREAD_MASK flag */
+static inline int QUARK_Bit_Get(unsigned char *set, int number)
+{
+    set += number / 8;
+    return (*set & (1 << (7-(number % 8)))) != 0; /* 0 or 1       */
+}
+
+/* Set the number_th bit in a bitset (unsigned char *) to value (0 or 1); useful for QUARK_LOCK_TO_THREAD_MASK flag */
+static inline void QUARK_Bit_Set(unsigned char *set, int number, int value)
+{
+    set += number / 8;
+    if (value)
+        *set =  (unsigned char)(*set | (1 << (7-(number % 8)))); /* set bit      */
+    else    *set = (unsigned char)(*set & ( ~(1 << (7-(number % 8))))); /* clear bit    */
+}
+
+/* The following are QUARK internal functions, and will be moved to a
+ * separate header file in the future.  Please do not use these
+ * functions outside of the QUARK source code.  */
+void quark_warning(const char *func_name, char* msg_text);
+void quark_topology_init();
+void quark_topology_finalize();
+int quark_setaffinity(int rank);
+int quark_unsetaffinity();
+int quark_yield();
+int quark_get_numthreads();
+int *quark_get_affthreads();
+int quark_getenv_int(char* name, int defval);
+
 #if defined(c_plusplus) || defined(__cplusplus)
 }
 #endif
