@@ -8,7 +8,7 @@
  *
  *  Note : this file is a copy of plasmaos-hwloc.c for use of Quark alone.
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Piotr Luszczek
  * @author Mathieu Faverge
  * @date 2010-11-15
@@ -85,7 +85,7 @@ int quark_setaffinity(int rank) {
 
     /* Get a copy of its cpuset that we may modify.  */
     /* Get only one logical processor (in case the core is SMT/hyperthreaded).  */
-#if !defined(HAVE_HWLOC_BITMAP)
+#if !defined(HWLOC_BITMAP_H)
     cpuset = hwloc_cpuset_dup(obj->cpuset);
     hwloc_cpuset_singlify(cpuset);
 #else
@@ -96,7 +96,7 @@ int quark_setaffinity(int rank) {
     /* And try to bind ourself there.  */
     if (hwloc_set_cpubind(quark_topology, cpuset, HWLOC_CPUBIND_THREAD)) {
         char *str = NULL;
-#if !defined(HAVE_HWLOC_BITMAP)
+#if !defined(HWLOC_BITMAP_H)
         hwloc_cpuset_asprintf(&str, obj->cpuset);
 #else
         hwloc_bitmap_asprintf(&str, obj->cpuset);
@@ -110,7 +110,7 @@ int quark_setaffinity(int rank) {
     rank = obj->children[0]->os_index;
 
     /* Free our cpuset copy */
-#if !defined(HAVE_HWLOC_BITMAP)
+#if !defined(HWLOC_BITMAP_H)
     hwloc_cpuset_free(cpuset);
 #else
     hwloc_bitmap_free(cpuset);
