@@ -129,6 +129,7 @@
 *     .. External Subroutines ..
       EXTERNAL           ALAREQ, CCHKGE, 
      $                   CCHKLQ, CCHKPO, 
+     $                   CCHKQ3,
      $                   CCHKQR, 
      $                   CDRVLS,
      $                   CDRVPO,
@@ -405,7 +406,6 @@
       ENDIF      
 *
       CALL PLASMA_DISABLE( PLASMA_AUTOTUNING, INFO )
-*
 *
    80 CONTINUE
 *
@@ -536,6 +536,22 @@
          ELSE
             WRITE( NOUT, FMT = 9989 )PATH
          END IF
+*
+      ELSE IF( LSAMEN( 2, C2, 'QP' ) ) THEN
+*
+*        QP:  QR factorization with pivoting
+*
+         NTYPES = 3
+         CALL ALAREQ( PATH, NMATS, DOTYPE, NTYPES, NIN, NOUT )
+*
+         IF( TSTCHK ) THEN
+            CALL CCHKQ3( DOTYPE, NM, MVAL, NN, NVAL, NNB, NBVAL, NXVAL,
+     $                   THRESH, A( 1, 1 ), A( 1, 2 ), S( 1 ),
+     $                   B( 1, 1 ), WORK, RWORK, IWORK, NOUT )
+         ELSE
+            WRITE( NOUT, FMT = 9989 )PATH
+         END IF
+
 *
       ELSE IF( LSAMEN( 2, C2, 'LS' ) ) THEN
 *
