@@ -61,6 +61,11 @@
       NOUT = NUNIT
       WRITE( NOUT, FMT = * )
 *
+*     Disable PLASMA warnings/errors
+* 
+      CALL PLASMA_DISABLE( PLASMA_WARNINGS, INFO )
+      CALL PLASMA_DISABLE( PLASMA_ERRORS,   INFO )
+*
 *     Set the variables to innocuous values.
 *
       DO 20 J = 1, NMAX
@@ -143,11 +148,11 @@
 *     PLASMA_DORMLQ
 *
       SRNAMT = 'DORMLQ'
-      INFOT = 103
+      INFOT = 1
       CALL PLASMA_DORMLQ( '/', PLASMATRANS, 0, 0, 0, A, 1, HT, AF, 1,
      $                   INFO )
       CALL CHKXER( 'DORMLQ', INFOT, NOUT, INFO, OK )
-      INFOT = 103
+      INFOT = 2
       CALL PLASMA_DORMLQ( PLASMALEFT, '/', 0, 0, 0, A, 1, HT, AF, 1,
      $                   INFO )
       CALL CHKXER( 'DORMLQ', INFOT, NOUT, INFO, OK )
@@ -200,6 +205,11 @@
 *     Deallocate HT
 *
       CALL PLASMA_DEALLOC_HANDLE( HT, INFO )
+*
+*     Enable PLASMA warnings/errors
+* 
+      CALL PLASMA_ENABLE( PLASMA_WARNINGS, INFO )
+      CALL PLASMA_ENABLE( PLASMA_ERRORS,   INFO )
 *
       RETURN
 *
