@@ -253,6 +253,9 @@
                DO 90 INB = 1, NNB
                   NB = NBVAL( INB )
                   CALL XLAENV( 1, NB )
+                  IF ( (N / 25) .GT. NB ) THEN
+                     GOTO 90
+                  END IF
                   CALL PLASMA_SET( PLASMA_TILE_SIZE, NB, INFO )
 *
 *                 Compute the L*L' or U'*U factorization of the matrix.
@@ -393,8 +396,8 @@
 *
                   ANORM = SLANSY( '1', UPLO, N, A, LDA, RWORK )
                   SRNAMT = 'SPOCON'
-                  CALL SPOCON( UPLO, N, AFAC, LDA, ANORM, RCOND, WORK,
-     $                         IWORK, INFO )
+                  CALL PLASMA_SPOCON( PLASMA_UPLO, N, AFAC, LDA, ANORM,
+     $                 RCOND, INFO )
 *
 *                 Check error code from SPOCON.
 *
