@@ -130,6 +130,7 @@
 *     .. External Subroutines ..
       EXTERNAL           ALAREQ, ZCHKGE, 
      $                   ZCHKLQ, ZCHKPO, 
+     $                   ZCHKQ3,
      $                   ZCHKQR, 
      $                   ZDRVLS,
      $                   ZDRVPO,
@@ -537,6 +538,22 @@
          ELSE
             WRITE( NOUT, FMT = 9989 )PATH
          END IF
+*
+      ELSE IF( LSAMEN( 2, C2, 'QP' ) ) THEN
+*
+*        QP:  QR factorization with pivoting
+*
+         NTYPES = 3
+         CALL ALAREQ( PATH, NMATS, DOTYPE, NTYPES, NIN, NOUT )
+*
+         IF( TSTCHK ) THEN
+            CALL ZCHKQ3( DOTYPE, NM, MVAL, NN, NVAL, NNB, NBVAL, NXVAL,
+     $                   THRESH, A( 1, 1 ), A( 1, 2 ), S( 1 ),
+     $                   B( 1, 1 ), WORK, RWORK, IWORK,
+     $                   NOUT )
+         ELSE
+            WRITE( NOUT, FMT = 9989 )PATH
+         END IF
 *
       ELSE IF( LSAMEN( 2, C2, 'LS' ) ) THEN
 *
