@@ -165,9 +165,9 @@
 *
 *     Set the block size and minimum block size for testing.
 *
-      NB = 1
-      IB = 1
-      NBMIN = 2
+      NB = 128
+      IB = 32
+      NBMIN = 32
       CALL XLAENV( 1, NB )
       CALL XLAENV( 2, NBMIN )
       CALL PLASMA_SET( PLASMA_TILE_SIZE, NB, INFO )
@@ -185,7 +185,8 @@
 *
 *        ALLOCATE L and IPIV
 *
-         CALL PLASMA_ALLOC_WORKSPACE_ZGETRF( N, N, HL, HPIV, INFO )
+c$$$         CALL PLASMA_ALLOC_WORKSPACE_ZGETRF_INCPIV(
+c$$$     $        N, N, HL, HPIV, INFO )
 *
 *
          DO 80 IMAT = 1, NIMAT
@@ -318,8 +319,10 @@
 *
 *                    Factor the matrix A.
 *
+c$$$                     CALL PLASMA_ZGETRF_INCPIV( N, N, AFAC, LDA, 
+c$$$     $                                   HL, HPIV, INFO )
                      CALL PLASMA_ZGETRF( N, N, AFAC, LDA, 
-     $                                   HL, HPIV, INFO )
+     $                                   IWORK, INFO )
 *
                   END IF
 *
@@ -359,8 +362,10 @@
                         CALL ZLACPY( 'Full', N, NRHS, B, LDA, X, LDA )
 *
                         SRNAMT = 'ZGESV '
-                        CALL PLASMA_ZGESV( N, NRHS, AFAC, LDA, HL, HPIV,
-     $                                     X, LDA, INFO )
+c$$$                        CALL PLASMA_ZGESV_INCPIV( N, NRHS, AFAC, LDA, 
+c$$$     $                       HL, HPIV, X, LDA, INFO )
+                        CALL PLASMA_ZGESV( N, NRHS, AFAC, LDA, 
+     $                       IWORK, X, LDA, INFO )
 *
 *                       Check error code from ZGESV .
 *
@@ -408,8 +413,8 @@
 *
 *        DEALLOCATE HL and HPIV
 *
-         CALL PLASMA_DEALLOC_HANDLE( HL, INFO )
-         CALL PLASMA_DEALLOC_HANDLE( HPIV, INFO )
+c$$$         CALL PLASMA_DEALLOC_HANDLE( HL, INFO )
+c$$$         CALL PLASMA_DEALLOC_HANDLE( HPIV, INFO )
    90 CONTINUE
 *
 *     Print a summary of the results.
