@@ -8,7 +8,7 @@
  *
  * @brief Test mixed precision solution with iterative refinement routine PLASMA_zcposv
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Emmanuel Agullo
  * @author Mathieu Faverge
  * @date 2010-11-15
@@ -23,7 +23,6 @@
 #include <plasma.h>
 #include <cblas.h>
 #include <lapacke.h>
-#include <plasma_tmg.h>
 #include <core_blas.h>
 #include "testing_zmain.h"
 
@@ -50,16 +49,11 @@ int testing_zcposv(int argc, char **argv)
     int uplo;
     int info;
     int info_solution = 0; /*, info_factorization;*/
-    int i,j;
-    int NminusOne = N-1;
-    int LDBxNRHS = LDB*NRHS;
 
     PLASMA_Complex64_t *A1   = (PLASMA_Complex64_t *)malloc(LDA*N   *sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *A2   = (PLASMA_Complex64_t *)malloc(LDA*N   *sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *B1   = (PLASMA_Complex64_t *)malloc(LDB*NRHS*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *B2   = (PLASMA_Complex64_t *)malloc(LDB*NRHS*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *WORK = (PLASMA_Complex64_t *)malloc(2*LDA   *sizeof(PLASMA_Complex64_t));
-    double *D                = (double *)malloc(LDA*sizeof(double));
 
     /* Check if unable to allocate memory */
     if ( (!A1) || (!A2) || (!B1) || (!B2) ){
@@ -74,24 +68,12 @@ int testing_zcposv(int argc, char **argv)
     */
 
     /* Initialize A1 and A2 for Symmetric Positif Matrix (Hessenberg in the complex case) */
-    LAPACKE_dlarnv_work(IONE, ISEED, LDA, D);
-    zlaghe(&N, &NminusOne, D, A1, &LDA, ISEED, WORK, &info);
-    free(D);
-
-    for ( i = 0; i < N; i++)
-        for (  j = 0; j < N; j++)
-            A2[LDA*j+i] = A1[LDA*j+i];
-
-    for ( i = 0; i < N; i++){
-        A1[LDA*i+i] = A1[LDA*i+i] + N ;
-        A2[LDA*i+i] = A1[LDA*i+i];
-    }
+    PLASMA_zplghe( (double)N, N, A1, LDA, 51 );
+    PLASMA_zlacpy( PlasmaUpperLower, N, N, A1, LDA, A2, LDA );
 
     /* Initialize B1 and B2 */
-    LAPACKE_zlarnv_work(IONE, ISEED, LDBxNRHS, B1);
-    for ( i = 0; i < N; i++)
-        for ( j = 0; j < NRHS; j++)
-            B2[LDB*j+i] = B1[LDB*j+i];
+    PLASMA_zplrnt( N, NRHS, B1, LDB, 371 );
+    PLASMA_zlacpy( PlasmaUpperLower, N, NRHS, B1, LDB, B2, LDB );
 
     printf("\n");
     printf("------ TESTS FOR PLASMA ZCPOSV ROUTINE ------  \n");
@@ -127,7 +109,7 @@ int testing_zcposv(int argc, char **argv)
         printf("***************************************************\n");
     }
 
-    free(A1); free(A2); free(B1); free(B2); free(WORK);
+    free(A1); free(A2); free(B1); free(B2);
     
     return 0;
 }
