@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Bilel Hadri
  * @author Hatem Ltaief
  * @date 2010-11-15
@@ -239,7 +239,7 @@ int testing_zgels(int argc, char **argv)
     PLASMA_Complex64_t *B1 = (PLASMA_Complex64_t *)malloc(LDB*NRHS*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *B2 = (PLASMA_Complex64_t *)malloc(LDB*NRHS*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *Q  = (PLASMA_Complex64_t *)malloc(LDA*N*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *T;
+    PLASMA_desc *T;
 
     /* Check if unable to allocate memory */
     if ((!A1)||(!A2)||(!B1)||(!B2)||(!Q)){
@@ -479,7 +479,8 @@ int testing_zgels(int argc, char **argv)
         }
     }
 
-    free(A1); free(A2); free(B1); free(B2); free(Q); free(T);
+    free(A1); free(A2); free(B1); free(B2); free(Q);
+    PLASMA_Dealloc_Handle_Tile( &T );
 
     return 0;
 }
