@@ -6,7 +6,8 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
+ * @author Azzam Haidar
  * @author Hatem Ltaief
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -20,7 +21,6 @@
 #include <plasma.h>
 #include <cblas.h>
 #include <lapacke.h>
-#include <plasma_tmg.h>
 #include <core_blas.h>
 #include "testing_zmain.h"
 
@@ -28,7 +28,7 @@
 #define COMPLEX
 
 static int check_orthogonality(int, int, int, PLASMA_Complex64_t*, int, double);
-static int check_reduction(int, int, PLASMA_Complex64_t*, PLASMA_Complex64_t*, int, PLASMA_Complex64_t*, int, PLASMA_Complex64_t*, int, double);
+static int check_reduction(int, int, double*, PLASMA_Complex64_t*, int, PLASMA_Complex64_t*, int, PLASMA_Complex64_t*, int, double);
 static int check_solution(int, double*, double*, double);
 
 int testing_zgesvd(int argc, char **argv)
@@ -71,8 +71,8 @@ int testing_zgesvd(int argc, char **argv)
 
     double eps  = LAPACKE_dlamch_work('e');
     double dmax = 1.0;
-    PLASMA_enum vecu  = PlasmaNoVec;
-    PLASMA_enum vecvt = PlasmaNoVec;
+    PLASMA_enum jobu  = PlasmaVec;
+    PLASMA_enum jobvt = PlasmaVec;
     int info_orthou    = 0;
     int info_orthovt   = 0;
     int info_solution  = 0;
@@ -80,6 +80,7 @@ int testing_zgesvd(int argc, char **argv)
     int minMN = min(M, N);
     int mode  = 4;
     double rcond = (double) minMN;
+    int INFO=-1;
 
     PLASMA_Complex64_t *A1   = (PLASMA_Complex64_t *)malloc(LDA*N*sizeof(PLASMA_Complex64_t));
     double *S1               = (double *)            malloc(minMN*sizeof(double));
@@ -96,11 +97,9 @@ int testing_zgesvd(int argc, char **argv)
         return -2;
     }
 
-    /*
     PLASMA_Disable(PLASMA_AUTOTUNING);
     PLASMA_Set(PLASMA_TILE_SIZE, 120);
-    PLASMA_Set(PLASMA_INNER_BLOCK_SIZE, 40);
-    */
+    PLASMA_Set(PLASMA_INNER_BLOCK_SIZE, 20);
 
     PLASMA_Enable(PLASMA_WARNINGS);
     PLASMA_Enable(PLASMA_ERRORS);
@@ -118,36 +117,24 @@ int testing_zgesvd(int argc, char **argv)
     free(work);
 
     /* Copy A1 for check */
-    if ( (vecu == PlasmaVec) && (vecvt == PlasmaVec) ) {
+    if ( (jobu == PlasmaVec) && (jobvt == PlasmaVec) ) {
         A2 = (PLASMA_Complex64_t *)malloc(LDA*N*sizeof(PLASMA_Complex64_t));
         LAPACKE_zlacpy_work(LAPACK_COL_MAJOR, 'A', M, N, A1, LDA, A2, LDA);
     }
-    if ( vecu == PlasmaVec ) {
+    if ( jobu == PlasmaVec ) {
         U = (PLASMA_Complex64_t *)malloc(M*M*sizeof(PLASMA_Complex64_t));
         LAPACKE_zlaset_work(LAPACK_COL_MAJOR, 'A', M, M, 0., 1., U, M);
     }
-    if ( vecvt == PlasmaVec ) {
+    if ( jobvt == PlasmaVec ) {
         VT = (PLASMA_Complex64_t *)malloc(N*N*sizeof(PLASMA_Complex64_t));
         LAPACKE_zlaset_work(LAPACK_COL_MAJOR, 'A', N, N, 0., 1., VT, N);
     }
- 
+
     /* PLASMA ZGESVD */
-    PLASMA_zgesvd(vecu, vecvt, M, N, A1, LDA, S2, U, M, VT, N, T);
-
-    if (getenv("PLASMA_TESTING_VERBOSE"))
-    {
-        int i;
-        printf("Eigenvalues original\n");
-        for (i = 0; i < min(N,25); i++){
-            printf("%f ", S1[i]);
-        }
-        printf("\n");
-
-        printf("Eigenvalues computed\n");
-        for (i = 0; i < min(N,25); i++){
-            printf("%f ", S2[i]);
-        }
-        printf("\n");
+    INFO = PLASMA_zgesvd(jobu, jobvt, M, N, A1, LDA, S2, T, U, M, VT, N);
+    if(INFO!=0){
+            printf(" ERROR OCCURED INFO %d\n",INFO);
+            goto fin;
     }
 
     printf("\n");
@@ -161,37 +148,53 @@ int testing_zgesvd(int argc, char **argv)
 
 
     /* Check the orthogonality, reduction and the singular values */
-    if ( vecu == PlasmaVec )
+    if ( jobu == PlasmaVec )
         info_orthou = check_orthogonality(PlasmaLeft, M, M, U, M, eps);
 
-    if ( vecvt == PlasmaVec )
+    if ( jobvt == PlasmaVec )
         info_orthovt = check_orthogonality(PlasmaRight, N, N, VT, N, eps);
 
-    /* 
-     * WARNING: For now, Q is associated to Band tridiagonal reduction and 
+    /*
+     * WARNING: For now, Q is associated to Band tridiagonal reduction and
      * not to the final tridiagonal reduction, so we can not call the check
      * Need to accumulate the orthogonal transformations
      * during the bulge chasing to be able to perform the next test!
      */
-    if ( (vecu == PlasmaVec) && (vecvt == PlasmaVec) && 0 )
-        info_reduction = check_reduction(M, N, A2, A1, LDA, U, M, VT, N, eps);
+    if ( (jobu == PlasmaVec) && (jobvt == PlasmaVec) )
+        info_reduction = check_reduction(M, N, S2, A2, LDA, U, M, VT, N, eps);
 
     info_solution = check_solution(minMN, S1, S2, eps);
 
-    if ( (info_solution == 0) & (info_orthou == 0) & (info_orthovt == 0) ) {
-        printf("***************************************************\n");
-        printf(" ---- TESTING ZGESVD ..................... PASSED !\n");
-        printf("***************************************************\n");
+    if ( (info_solution == 0) & (info_orthou == 0) &
+         (info_orthovt == 0) & (info_reduction == 0) ) {
+        if (M >= N) {
+           printf("***************************************************\n");
+           printf(" ---- TESTING ZGESVD .. M >= N ........... PASSED !\n");
+           printf("***************************************************\n");
+        }
+        else {
+           printf("***************************************************\n");
+           printf(" ---- TESTING ZGESVD .. M < N ............ PASSED !\n");
+           printf("***************************************************\n");
+        }
     }
     else {
-        printf("************************************************\n");
-        printf(" - TESTING ZGESVD ... FAILED !\n");
-        printf("************************************************\n");
+        if (M >= N) {
+           printf("************************************************\n");
+           printf(" - TESTING ZGESVD .. M >= N .. FAILED !\n");
+           printf("************************************************\n");
+        }
+        else {
+           printf("************************************************\n");
+           printf(" - TESTING ZGESVD .. M < N .. FAILED !\n");
+           printf("************************************************\n");
+        }
     }
 
-    if ( A2 != NULL ) free(A2); 
-    if ( U  != NULL ) free(U); 
-    if ( VT != NULL ) free(VT); 
+fin:
+    if ( A2 != NULL ) free(A2);
+    if ( U  != NULL ) free(U);
+    if ( VT != NULL ) free(VT);
     free(A1); free(S1); free(S2);
     PLASMA_Dealloc_Handle_Tile(&T);
 
@@ -199,12 +202,12 @@ int testing_zgesvd(int argc, char **argv)
 }
 
 /*-------------------------------------------------------------------
- * Check the orthogonality of Q
+ * Check the orthogonality of U VT
  */
 static int check_orthogonality(int side, int M, int N, PLASMA_Complex64_t *Q, int LDQ, double eps)
 {
-    double  alpha =  1.0;
-    double  beta  = -1.0;
+    double  done =  1.0;
+    double  mdone  = -1.0;
     double  normQ, result;
     int     info_ortho;
     int     minMN = min(M, N);
@@ -216,26 +219,27 @@ static int check_orthogonality(int side, int M, int N, PLASMA_Complex64_t *Q, in
 
     /* Perform Id - Q'Q */
     if (M >= N)
-        cblas_zherk(CblasColMajor, CblasUpper, CblasConjTrans, N, M, alpha, Q, LDQ, beta, Id, N);
+        cblas_zherk(CblasColMajor, CblasUpper, CblasConjTrans, N, M, done, Q, LDQ, mdone, Id, N);
     else
-        cblas_zherk(CblasColMajor, CblasUpper, CblasNoTrans,   M, N, alpha, Q, LDQ, beta, Id, M);
+        cblas_zherk(CblasColMajor, CblasUpper, CblasNoTrans,   M, N, done, Q, LDQ, mdone, Id, M);
 
     normQ = LAPACKE_zlansy_work(LAPACK_COL_MAJOR, lapack_const(PlasmaInfNorm), 'U', minMN, Id, minMN, work);
 
     if (getenv("PLASMA_TESTING_VERBOSE"))
         printf( "||Q||_oo=%f\n", normQ );
-    
-    printf("================================\n");
+
     result = normQ / (M * eps);
     if (side == PlasmaLeft)
     {
-        printf("Checking the orthogonality of U (Left singular vectors)\n");
-        printf("||Id-U'*U||_oo / (N*eps) = %e \n", result);
+        printf(" ======================================================\n");
+        printf(" ||Id-U'*U||_oo / (M*eps)          : %15.3E \n",  result );
+        printf(" ======================================================\n");
     }
-    else 
+    else
     {
-        printf("Checking the orthogonality of VT (Right singular vectors)\n");
-        printf("||Id-VT'*VT||_oo / (N*eps) = %e \n", result);
+        printf(" ======================================================\n");
+        printf(" ||Id-VT'*VT||_oo / (M*eps)          : %15.3E \n",  result );
+        printf(" ======================================================\n");
     }
 
     if ( isnan(result) || isinf(result) || (result > 60.0) ) {
@@ -246,55 +250,60 @@ static int check_orthogonality(int side, int M, int N, PLASMA_Complex64_t *Q, in
         printf("-- Orthogonality is CORRECT ! \n");
         info_ortho = 0;
     }
-    
+
     free(work); free(Id);
-    
+
     return info_ortho;
 }
 
 /*------------------------------------------------------------
  *  Check the bidiagonal reduction
  */
-static int check_reduction(int M, int N, PLASMA_Complex64_t *A1, PLASMA_Complex64_t *A2, int LDA, 
+static int check_reduction(int M, int N, double *S, PLASMA_Complex64_t *A, int LDA,
                            PLASMA_Complex64_t *U, int LDU, PLASMA_Complex64_t *VT, int LDVT, double eps )
 {
-    PLASMA_Complex64_t alpha = 1.0;
-    PLASMA_Complex64_t beta  = 0.0;
+    PLASMA_Complex64_t zone = 1.0;
+    PLASMA_Complex64_t mzone  = -1.0;
     double Anorm, Rnorm, result;
     int info_reduction;
-    int i, j;
+    int i;
     int maxMN = max(M, N);
+    int minMN = min(M, N);
 
-    PLASMA_Complex64_t *Aorig    = (PLASMA_Complex64_t *)malloc(M*N*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *Residual = (PLASMA_Complex64_t *)malloc(M*N*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *B        = (PLASMA_Complex64_t *)malloc(M*N*sizeof(PLASMA_Complex64_t));
-    double *work = (double *)malloc(N*sizeof(double));
+    PLASMA_Complex64_t *TEMP     = (PLASMA_Complex64_t *)malloc(maxMN*maxMN*sizeof(PLASMA_Complex64_t));
+    PLASMA_Complex64_t *Residual = (PLASMA_Complex64_t *)malloc(maxMN*maxMN*sizeof(PLASMA_Complex64_t));
+    double *work = (double *)malloc(maxMN*sizeof(double));
 
-    memset((void*)B, 0, M*N*sizeof(PLASMA_Complex64_t));
+    memset((void*)TEMP, 0, maxMN*maxMN*sizeof(PLASMA_Complex64_t));
 
-    /* Rebuild the B */
-    LAPACKE_zlacpy_work(LAPACK_COL_MAJOR, PlasmaUpperLower, M, N, A2, LDA, B, M);
-    memset((void*)Aorig, 0, M*N*sizeof(PLASMA_Complex64_t));
+    /* Compute TEMP =  U * SIGMA */
+    /*
+     * U is of size (M,M),
+     * SIGMA is of size (M,N) but contains only MINMN singular values.
+     * Thus TEMP is of size (M,N)
+     */
+    LAPACKE_zlacpy_work(LAPACK_COL_MAJOR, lapack_const(PlasmaUpperLower), M, M, U, LDU, TEMP, M);
+    for (i = 0; i < minMN; i++){
+            cblas_zdscal(M, S[i], &(TEMP[i*M]),1);
+    }
+    /* Compute Residual = A - U * SIGMA * VT */
+    /*
+     * VT is of size (N,N)
+     */
+    LAPACKE_zlacpy_work(LAPACK_COL_MAJOR, lapack_const(PlasmaUpperLower), M, N, A, LDA, Residual, M);
+    cblas_zgemm(CblasColMajor, CblasNoTrans, CblasNoTrans, M, N, N, CBLAS_SADDR(mzone), TEMP, M,  VT, LDVT, CBLAS_SADDR(zone), Residual,     M);
 
-    /* Compute Aorig = U * B * VT */
-    cblas_zgemm(CblasColMajor, CblasNoTrans, CblasNoTrans, M, N, M, CBLAS_SADDR(alpha), U,     LDU, B,  M,    CBLAS_SADDR(beta), Aorig, M);
-    cblas_zgemm(CblasColMajor, CblasNoTrans, CblasNoTrans, M, N, N, CBLAS_SADDR(alpha), Aorig, M,   VT, LDVT, CBLAS_SADDR(beta), B,     M);
-    
-    /* Compute the Residual */
-    for (i = 0; i < M; i++)
-        for (j = 0 ; j < N; j++)
-            Residual[j*M+i] = A1[j*LDA+i] - B[j*M+i];
-    
+    /* Compute the norms */
     Rnorm = LAPACKE_zlange_work(LAPACK_COL_MAJOR, lapack_const(PlasmaInfNorm), M, N, Residual, M,   work);
-    Anorm = LAPACKE_zlange_work(LAPACK_COL_MAJOR, lapack_const(PlasmaInfNorm), M, N, A2,       LDA, work);
-    
-   if (getenv("PLASMA_TESTING_VERBOSE"))
+    Anorm = LAPACKE_zlange_work(LAPACK_COL_MAJOR, lapack_const(PlasmaInfNorm), M, N, A,       LDA, work);
+
+    if (getenv("PLASMA_TESTING_VERBOSE"))
         printf( "||A||_oo=%f\n||A - U*B*VT||_oo=%e\n", Anorm, Rnorm );
 
     result = Rnorm / ( Anorm * maxMN * eps);
-    printf("============\n");
-    printf("Checking the bidiagonal reduction \n");
-    printf("-- ||A-U*B*VT||_oo/(||A||_oo.max(M,N).eps) = %e \n", result);
+    printf(" ======================================================\n");
+    printf(" ||A-U*SIGMA*VT'||_oo/(||A||_oo.N.eps) : %15.3E \n",  result );
+    printf(" ======================================================\n");
 
     if ( isnan(result) || isinf(result) || (result > 60.0) ) {
         printf("-- Reduction is suspicious ! \n");
@@ -305,11 +314,16 @@ static int check_reduction(int M, int N, PLASMA_Complex64_t *A1, PLASMA_Complex6
         info_reduction = 0;
     }
 
-    free(Aorig); free(Residual); free(B); free(work);
+    free(TEMP);
+    free(Residual);
+    free(work);
 
     return info_reduction;
 }
 
+/*------------------------------------------------------------
+ *  Check the eigenvalues
+ */
 static int check_solution(int N, double *E1, double *E2, double eps)
 {
     int info_solution, i;
@@ -328,8 +342,8 @@ static int check_solution(int N, double *E1, double *E2, double eps)
 
     maxel = maxel / (maxeig * N * eps);
     printf(" ======================================================\n");
-    printf(" Checking the singular values of A\n");
-    printf("    | D - eigcomputed | / (|D| * N * eps) : %15.3E \n",  maxel );
+    printf(" | S - singularcomputed | / (|S| * N * eps) : %15.3E \n",  maxel );
+    printf(" ======================================================\n");
 
     if ( isnan(maxel) || isinf(maxel) || (maxel > 100) ) {
         printf("-- The singular values are suspicious ! \n");
@@ -339,6 +353,5 @@ static int check_solution(int N, double *E1, double *E2, double eps)
         printf("-- The singular values are CORRECT ! \n");
         info_solution = 0;
     }
-
     return info_solution;
 }
