@@ -6,8 +6,9 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Hatem Ltaief
+ * @author Azzam Haidar
  * @date 2010-11-15
  * @precisions normal z -> c d s
  *
@@ -20,7 +21,6 @@
 #include <plasma.h>
 #include <cblas.h>
 #include <lapacke.h>
-#include <plasma_tmg.h>
 #include <core_blas.h>
 #include "testing_zmain.h"
 
@@ -28,9 +28,7 @@
 #define COMPLEX
 
 static int check_orthogonality(int, int, PLASMA_Complex64_t*, int, double);
-#if 0
-static int check_reduction(int, int, int, PLASMA_Complex64_t*, PLASMA_Complex64_t*, int, PLASMA_Complex64_t*, double);
-#endif
+static int check_reduction(int, int, int, PLASMA_Complex64_t*, double*, int, PLASMA_Complex64_t*, double);
 static int check_solution(int, double*, double*, double);
 
 int testing_zheevd(int argc, char **argv)
@@ -50,47 +48,47 @@ int testing_zheevd(int argc, char **argv)
     double eps   = LAPACKE_dlamch_work('e');
     double dmax  = 1.0;
     double rcond = 1.0e6;
+    int INFO=-1;
 
     PLASMA_enum uplo = PlasmaLower;
     PLASMA_enum vec  = PlasmaVec;
     int info_ortho     = 0;
     int info_solution  = 0;
     int info_reduction = 0;
-    int i;
     int LDAxN = LDA*N;
     int LDQxN = LDQ*N;
 
-    PLASMA_Complex64_t *A1 = NULL;
-    PLASMA_Complex64_t *A2 = (PLASMA_Complex64_t *)malloc(LDAxN*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *Q  = NULL;
-    double *W1             = (double *)malloc(N*sizeof(double));
-    double *W2             = (double *)malloc(N*sizeof(double));
+    PLASMA_Complex64_t *A1   = NULL;
+    PLASMA_Complex64_t *A2   = (PLASMA_Complex64_t *)malloc(LDAxN*sizeof(PLASMA_Complex64_t));
+    PLASMA_Complex64_t *Q    = NULL;
+    double             *W1   = (double *)malloc(N*sizeof(double));
+    double             *W2   = (double *)malloc(N*sizeof(double));
     PLASMA_Complex64_t *work = (PLASMA_Complex64_t *)malloc(3*N* sizeof(PLASMA_Complex64_t));
     PLASMA_desc *T;
 
     /* Check if unable to allocate memory */
-    if ( !A2 ){
+    if ( (!A2) || (!W1) || (!W2) ){
         printf("Out of Memory \n ");
         return -2;
     }
 
-    /*
+    
     PLASMA_Disable(PLASMA_AUTOTUNING);
-    PLASMA_Set(PLASMA_TILE_SIZE, 5);
-    PLASMA_Set(PLASMA_INNER_BLOCK_SIZE, 5);
-    */
-
-    PLASMA_Alloc_Workspace_zheev(N, N, &T);
+    PLASMA_Set(PLASMA_TILE_SIZE, 120);
+    PLASMA_Set(PLASMA_INNER_BLOCK_SIZE, 20);
+    
+    PLASMA_Enable(PLASMA_WARNINGS);
+    PLASMA_Enable(PLASMA_ERRORS);
+    PLASMA_Alloc_Workspace_zheevd(N, N, &T);
 
     /*----------------------------------------------------------
     *  TESTING ZHEEVD
     */
-
     /* Initialize A1 */
     LAPACKE_zlatms_work( LAPACK_COL_MAJOR, N, N,
                          lapack_const(PlasmaDistSymmetric), ISEED,
                          lapack_const(PlasmaHermGeev), W1, mode, rcond,
-                         dmax, N-1, N-1,
+                         dmax, N, N,
                          lapack_const(PlasmaNoPacking), A2, LDA, work );
     
     /*
@@ -100,15 +98,6 @@ int testing_zheevd(int argc, char **argv)
     */
     LAPACKE_dlasrt_work( 'I', N, W1 );
 
-    if (getenv("PLASMA_TESTING_VERBOSE"))
-    {
-        printf("Eigenvalues original\n");
-        for (i = 0; i < min(N,25); i++){
-            printf("%f \n", W1[i]);
-        }
-        printf("\n");
-    }
-
     if ( vec == PlasmaVec ) {
         Q  = (PLASMA_Complex64_t *)malloc(LDQxN*sizeof(PLASMA_Complex64_t));
         A1 = (PLASMA_Complex64_t *)malloc(LDAxN*sizeof(PLASMA_Complex64_t));    
@@ -116,17 +105,13 @@ int testing_zheevd(int argc, char **argv)
         /* Copy A2 into A1 */
         LAPACKE_zlacpy_work(LAPACK_COL_MAJOR, 'A', N, N, A2, LDA, A1, LDA);
     }
-
-    /* PLASMA ZHEEVD */
-    PLASMA_zheevd(vec, uplo, N, A2, LDA, W2, T, Q, LDQ);
-
-    if (getenv("PLASMA_TESTING_VERBOSE"))
-    {
-        printf("Eigenvalues computed\n");
-        for (i = 0; i < min(N,25); i++){
-            printf("%f \n", W2[i]);
-        }
-        printf("\n");
+    /*
+     * PLASMA ZHEEVD 
+     */
+    INFO = PLASMA_zheevd(vec, uplo, N, A2, LDA, W2, T, Q, LDQ);
+    if(INFO!=0){
+            printf(" ERROR OCCURED INFO %d\n",INFO);
+            goto fin;
     }
 
     printf("\n");
@@ -141,11 +126,7 @@ int testing_zheevd(int argc, char **argv)
     /* Check the orthogonality, reduction and the eigen solutions */
     if (vec == PlasmaVec) {
         info_ortho = check_orthogonality(N, N, Q, LDQ, eps);
-        /* 
-         * WARNING: For now, Q is associated to Band tridiagonal reduction and 
-         * not to the final tridiagonal reduction, so we can not call the check
-         */
-        /*info_reduction = check_reduction(uplo, N, 1, A1, A2, LDA, Q, eps);*/
+        info_reduction = check_reduction(uplo, N, 1, A1, W2, LDA, Q, eps);
     }
     info_solution = check_solution(N, W1, W2, eps);
 
@@ -160,11 +141,12 @@ int testing_zheevd(int argc, char **argv)
         printf("************************************************\n");
     }
 
+fin:
+    PLASMA_Dealloc_Handle_Tile(&T);
     free(A2);
     free(W1);
     free(W2);
     free(work);
-    PLASMA_Dealloc_Handle_Tile(&T);
     if (Q  != NULL) free(Q);
     if (A1 != NULL) free(A1);
 
@@ -174,11 +156,10 @@ int testing_zheevd(int argc, char **argv)
 /*-------------------------------------------------------------------
  * Check the orthogonality of Q
  */
-
 static int check_orthogonality(int M, int N, PLASMA_Complex64_t *Q, int LDQ, double eps)
 {
-    double  alpha =  1.0;
-    double  beta  = -1.0;
+    double  done  =  1.0;
+    double  mdone = -1.0;
     double  normQ, result;
     int     info_ortho;
     int     minMN = min(M, N);
@@ -190,16 +171,16 @@ static int check_orthogonality(int M, int N, PLASMA_Complex64_t *Q, int LDQ, dou
 
     /* Perform Id - Q'Q */
     if (M >= N)
-        cblas_zherk(CblasColMajor, CblasUpper, CblasConjTrans, N, M, alpha, Q, LDQ, beta, Id, N);
+        cblas_zherk(CblasColMajor, CblasUpper, CblasConjTrans, N, M, done, Q, LDQ, mdone, Id, N);
     else
-        cblas_zherk(CblasColMajor, CblasUpper, CblasNoTrans,   M, N, alpha, Q, LDQ, beta, Id, M);
+        cblas_zherk(CblasColMajor, CblasUpper, CblasNoTrans,   M, N, done, Q, LDQ, mdone, Id, M);
 
-    normQ = LAPACKE_zlansy_work(LAPACK_COL_MAJOR, lapack_const(PlasmaInfNorm), 'U', minMN, Id, minMN, work);
+    normQ = LAPACKE_zlanhe_work(LAPACK_COL_MAJOR, lapack_const(PlasmaInfNorm), 'U', minMN, Id, minMN, work);
 
     result = normQ / (minMN * eps);
-    printf("============\n");
-    printf("Checking the orthogonality of Q \n");
-    printf("||Id-Q'*Q||_oo / (minMN*eps) = %e \n", result);
+    printf(" ======================================================\n");
+    printf(" ||Id-Q'*Q||_oo / (minMN*eps)          : %15.3E \n",  result );
+    printf(" ======================================================\n");
 
     if ( isnan(result) || isinf(result) || (result > 60.0) ) {
         printf("-- Orthogonality is suspicious ! \n");
@@ -209,75 +190,51 @@ static int check_orthogonality(int M, int N, PLASMA_Complex64_t *Q, int LDQ, dou
         printf("-- Orthogonality is CORRECT ! \n");
         info_ortho=0;
     }
-
     free(work); free(Id);
-
     return info_ortho;
 }
 
 /*------------------------------------------------------------
  *  Check the reduction 
  */
-#if 0
-static int check_reduction(int uplo, int N, int bw, PLASMA_Complex64_t *A1, PLASMA_Complex64_t *A2, int LDA, PLASMA_Complex64_t *Q, double eps )
+static int check_reduction(int uplo, int N, int bw, PLASMA_Complex64_t *A, double *D, int LDA, PLASMA_Complex64_t *Q, double eps )
 {
-    PLASMA_Complex64_t alpha = 1.0;
-    PLASMA_Complex64_t beta  = 0.0;
+    PLASMA_Complex64_t zone  =  1.0;
+    PLASMA_Complex64_t mzone = -1.0;
+    PLASMA_Complex64_t *TEMP     = (PLASMA_Complex64_t *)malloc(N*N*sizeof(PLASMA_Complex64_t));
+    PLASMA_Complex64_t *Residual = (PLASMA_Complex64_t *)malloc(N*N*sizeof(PLASMA_Complex64_t));
+    double *work = (double *)malloc(N*sizeof(double));
     double Anorm, Rnorm, result;
     int info_reduction;
-    int i, j;
+    int i;
 
-    PLASMA_Complex64_t *Aorig    = (PLASMA_Complex64_t *)malloc(N*N*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *Residual = (PLASMA_Complex64_t *)malloc(N*N*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *T        = (PLASMA_Complex64_t *)malloc(N*N*sizeof(PLASMA_Complex64_t));
-    double *work = (double *)malloc(N*sizeof(double));
-
-    memset((void*)T, 0, N*N*sizeof(PLASMA_Complex64_t));
-
-    /* Rebuild the T */
-    LAPACKE_zlacpy_work(LAPACK_COL_MAJOR, lapack_const(uplo), N, N, A2, LDA, T, N);
-
-    printf("============\n");
-    printf("Checking the tridiagonal reduction \n");
-
-    if (uplo == PlasmaLower) 
-    {
-        /* Copy the lower part to the upper part to rebuild the hermitian/symmetry */
-        for (i = 0; i < N; i++)
-            for (j = max(0, i-bw); j < i; j++)
-                T[i*N+j] = conj(T[j*N+i]);
-        
-        /* Compute Aorig = Q * T * Q^H */
-        memset((void*)Aorig, 0, N*N*sizeof(PLASMA_Complex64_t));
-        cblas_zgemm(CblasColMajor, CblasNoTrans, CblasNoTrans,   N, N, N, CBLAS_SADDR(alpha), Q,    LDA, T, N,   CBLAS_SADDR(beta), Aorig, N);
-        cblas_zgemm(CblasColMajor, CblasNoTrans, CblasConjTrans, N, N, N, CBLAS_SADDR(alpha), Aorig, N,  Q, LDA, CBLAS_SADDR(beta), T,     N);
-    } 
-    else
-    {
-        /* Copy the upper part to the lower part to rebuild the symmetry */
-        for (i = 0; i < N; i++)
-            for (j = i+1 ; j < min(i+bw, N); j++)
-                T[i*N+j] = conj(T[j*N+i]);
-
-        /* Compute Aorig = Q^H * T * Q */
-        memset((void*)Aorig, 0, N*N*sizeof(PLASMA_Complex64_t));
-        cblas_zgemm(CblasColMajor, CblasConjTrans, CblasNoTrans, N, N, N, CBLAS_SADDR(alpha), Q,     LDA, T, N,   CBLAS_SADDR(beta), Aorig, N);
-        cblas_zgemm(CblasColMajor, CblasNoTrans,   CblasNoTrans, N, N, N, CBLAS_SADDR(alpha), Aorig, N,   Q, LDA, CBLAS_SADDR(beta), T,     N);
+    /* Compute TEMP =  Q * LAMBDA */
+    LAPACKE_zlacpy_work(LAPACK_COL_MAJOR, lapack_const(PlasmaUpperLower), N, N, Q, LDA, TEMP, N);        
+    for (i = 0; i < N; i++){
+            cblas_zdscal(N, D[i], &(TEMP[i*N]),1);
     }
+    /* Compute Residual = A - Q * LAMBDA * Q^H */
+    /* A is Hermetian but both upper and lower 
+     * are assumed valable here for checking 
+     * otherwise it need to be symetrized before 
+     * checking.
+     */ 
+    LAPACKE_zlacpy_work(LAPACK_COL_MAJOR, lapack_const(PlasmaUpperLower), N, N, A, LDA, Residual, N);        
+    cblas_zgemm(CblasColMajor, CblasNoTrans, CblasConjTrans, N, N, N, CBLAS_SADDR(mzone), TEMP, N,  Q, LDA, CBLAS_SADDR(zone), Residual,     N);
+
+    Rnorm = LAPACKE_zlange_work(LAPACK_COL_MAJOR, lapack_const(PlasmaOneNorm), N, N, Residual, N,   work);
+    Anorm = LAPACKE_zlanhe_work(LAPACK_COL_MAJOR, lapack_const(PlasmaOneNorm), lapack_const(uplo), N, A,       LDA, work);
 
-    /* Compute the Residual */
-    for (i = 0; i < N; i++)
-        for (j = 0 ; j < N; j++)
-            Residual[j*N+i] = A1[j*LDA+i] - T[j*N+i];
-    
-    Rnorm = LAPACKE_zlange_work(LAPACK_COL_MAJOR, lapack_const(PlasmaInfNorm), N, N, Residual, N,   work);
-    Anorm = LAPACKE_zlange_work(LAPACK_COL_MAJOR, lapack_const(PlasmaInfNorm), N, N, A2,       LDA, work);
-    
     result = Rnorm / ( Anorm * N * eps);
-    if ( uplo == PlasmaLower )
-        printf("-- ||A-Q*T*Q'||_oo/(||A||_oo.N.eps) = %e \n", result);
-    else 
-        printf("-- ||A-Q'*T*Q||_oo/(||A||_oo.N.eps) = %e \n", result);
+    if ( uplo == PlasmaLower ){
+        printf(" ======================================================\n");
+        printf(" ||A-Q*LAMBDA*Q'||_oo/(||A||_oo.N.eps) : %15.3E \n",  result );
+        printf(" ======================================================\n");
+    }else{ 
+        printf(" ======================================================\n");
+        printf(" ||A-Q'*LAMBDA*Q||_oo/(||A||_oo.N.eps) : %15.3E \n",  result );
+        printf(" ======================================================\n");
+    }
 
     if ( isnan(result) || isinf(result) || (result > 60.0) ) {
         printf("-- Reduction is suspicious ! \n");
@@ -288,12 +245,14 @@ static int check_reduction(int uplo, int N, int bw, PLASMA_Complex64_t *A1, PLAS
         info_reduction = 0;
     }
 
-    free(Aorig); free(Residual); free(T);
+    free(TEMP); free(Residual);
+    free(work);
 
     return info_reduction;
 }
-#endif
-
+/*------------------------------------------------------------
+ *  Check the eigenvalues 
+ */
 static int check_solution(int N, double *E1, double *E2, double eps)
 {
     int info_solution, i;
@@ -315,8 +274,6 @@ static int check_solution(int N, double *E1, double *E2, double eps)
     printf(" | D - eigcomputed | / (|D| * N * eps) : %15.3E \n",  maxel );
     printf(" ======================================================\n");
 
-    printf("============\n");
-    printf("Checking the eigenvalues of A\n");
     if ( isnan(maxel) || isinf(maxel) || (maxel > 100) ) {
         printf("-- The eigenvalues are suspicious ! \n");
         info_solution = 1;
@@ -325,6 +282,5 @@ static int check_solution(int N, double *E1, double *E2, double eps)
         printf("-- The eigenvalues are CORRECT ! \n");
         info_solution = 0;
     }
-
     return info_solution;
 }
