@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions normal z -> c
@@ -21,7 +21,6 @@
 #include <cblas.h>
 #include <lapacke.h>
 #include <core_blas.h>
-#include <plasma_tmg.h>
 #include "testing_zmain.h"
 
 static int check_solution(PLASMA_enum transA, PLASMA_enum transB, int M, int N,
@@ -52,12 +51,11 @@ int testing_zhemm(int argc, char **argv)
     int LDB   = atoi(argv[5]);
     int LDC   = atoi(argv[6]);
     int MNmax = max(M, N);
-    int MminusOne = MNmax - 1;
 
     double eps;
     int info_solution;
-    int i, j, s, u, info;
-    int LDAxM = LDA*max(M, N);
+    int i, j, s, u;
+    int LDAxM = LDA*MNmax;
     int LDBxN = LDB*N;
     int LDCxN = LDC*N;
 
@@ -66,8 +64,6 @@ int testing_zhemm(int argc, char **argv)
     PLASMA_Complex64_t *C      = (PLASMA_Complex64_t *)malloc(LDCxN*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *Cinit  = (PLASMA_Complex64_t *)malloc(LDCxN*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *Cfinal = (PLASMA_Complex64_t *)malloc(LDCxN*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *WORK   = (PLASMA_Complex64_t *)malloc(2*LDC*sizeof(PLASMA_Complex64_t));
-    double             *D      = (double *)            malloc(LDC  *sizeof(double));
 
     /* Check if unable to allocate memory */
     if ((!A)||(!B)||(!Cinit)||(!Cfinal)){
@@ -91,9 +87,7 @@ int testing_zhemm(int argc, char **argv)
     */
 
     /* Initialize A */
-    LAPACKE_dlarnv_work(IONE, ISEED, LDC, D);
-    zlaghe(&MNmax, &MminusOne, D, A, &LDA, ISEED, WORK, &info);
-    free(D); free(WORK);
+    PLASMA_zplghe( (double)0., MNmax, A, LDA, 51 );
 
     /* Initialize B */
     LAPACKE_zlarnv_work(IONE, ISEED, LDBxN, B);
