@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions normal z -> c
@@ -20,7 +20,6 @@
 #include <plasma.h>
 #include <cblas.h>
 #include <lapacke.h>
-#include <plasma_tmg.h>
 #include <core_blas.h>
 #include "testing_zmain.h"
 
@@ -53,11 +52,10 @@ int testing_zher2k(int argc, char **argv)
     int LDB   = atoi(argv[5]);
     int LDC   = atoi(argv[6]);
     int NKmax = max(N, K);
-    int NminusOne = N - 1;
 
     double eps;
     int info_solution;
-    int info, u, t;
+    int u, t;
     size_t LDAxK = LDA*NKmax;
     size_t LDBxK = LDB*NKmax;
     size_t LDCxN = LDC*N;
@@ -67,11 +65,9 @@ int testing_zher2k(int argc, char **argv)
     PLASMA_Complex64_t *C      = (PLASMA_Complex64_t *)malloc(LDCxN*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *Cinit  = (PLASMA_Complex64_t *)malloc(LDCxN*sizeof(PLASMA_Complex64_t));
     PLASMA_Complex64_t *Cfinal = (PLASMA_Complex64_t *)malloc(LDCxN*sizeof(PLASMA_Complex64_t));
-    PLASMA_Complex64_t *WORK   = (PLASMA_Complex64_t *)malloc(2*LDC*sizeof(PLASMA_Complex64_t));
-    double             *D      = (double *)            malloc(LDC  *sizeof(double));
 
     /* Check if unable to allocate memory */
-    if ( (!A) || (!B) || (!Cinit) || (!Cfinal) || (!D) ){
+    if ( (!A) || (!B) || (!Cinit) || (!Cfinal) ){
         printf("Out of Memory \n ");
         return -2;
     }
@@ -96,9 +92,7 @@ int testing_zher2k(int argc, char **argv)
     LAPACKE_zlarnv_work(IONE, ISEED, LDBxK, B);
 
     /* Initialize C */
-    LAPACKE_dlarnv_work(IONE, ISEED, LDC, D);
-    zlaghe(&N, &NminusOne, D, C, &LDC, ISEED, WORK, &info);
-    free(D); free(WORK);
+    PLASMA_zplghe( (double)0., N, C, LDC, 51 );
 
     for (u=0; u<2; u++) {
         for (t=0; t<3; t++) {
