@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -25,19 +25,24 @@
 int   IONE     = 1;
 int   ISEED[4] = {0,0,0,1};   /* initial seed for zlarnv() */
 
-int   format[6]= { PlasmaCM, PlasmaRM, PlasmaCCRB, PlasmaCRRB, PlasmaRCRB, PlasmaRRRB };
-int   side[2]  = { PlasmaLeft,    PlasmaRight };
-int   uplo[2]  = { PlasmaUpper,   PlasmaLower };
-int   diag[2]  = { PlasmaNonUnit, PlasmaUnit  };
-int   trans[3] = { PlasmaNoTrans, PlasmaTrans, PlasmaConjTrans };
+int   format[6] = { PlasmaRM, PlasmaCM, PlasmaCCRB, PlasmaCRRB, PlasmaRCRB, PlasmaRRRB };
+int   side[2]   = { PlasmaLeft,    PlasmaRight };
+int   uplo[2]   = { PlasmaUpper,   PlasmaLower };
+int   diag[2]   = { PlasmaNonUnit, PlasmaUnit  };
+int   trans[3]  = { PlasmaNoTrans, PlasmaTrans, PlasmaConjTrans };
 int   itype[3]  = { 1, 2, 3 };
+int   storev[2] = { PlasmaRowwise, PlasmaColumnwise };
+int   norm[4]   = { PlasmaMaxNorm, PlasmaOneNorm, PlasmaInfNorm, PlasmaFrobeniusNorm };
 
-char *formatstr[6]= { "CM", "RM", "CCRB", "CRRB", "RCRB", "RRRB"};
+
+char *formatstr[6]= { "RM", "CM", "CCRB", "CRRB", "RCRB", "RRRB"};
 char *sidestr[2]  = { "Left ", "Right" };
 char *uplostr[2]  = { "Upper", "Lower" };
 char *diagstr[2]  = { "NonUnit", "Unit   " };
 char *transstr[3] = { "N", "T", "H" };
-char *itypestr[3]  = { "inv(U')xAxinv(U) or inv(L)xAxinv(L')", "UxAxU' or L'xAxL", "UxAxU' or L'xAxL" };
+char *itypestr[3] = { "inv(U')xAxinv(U) or inv(L)xAxinv(L')", "UxAxU' or L'xAxL", "UxAxU' or L'xAxL" };
+char *storevstr[2]= { "Rowwise", "Columnwise" };
+char *normstr[4]  = { "Max", "One", "Inf", "Fro" };
 
 #define map_cm(m, n, i, j)   ((i) + (j) * (m))
 #define map_rm(m, n, i, j)   ((i) * (n) + (j))
@@ -120,7 +125,7 @@ int map_RRRB(int m, int n, int mb, int nb, int i, int j) {
             return ( m*n0  + (n-n0)*m0                      + map_rm( m%mb, n%nb, i%mb, j%nb) );
 }
 
-void *formatmap[6] = {  map_CM, map_RM, map_CCRB, map_CRRB, map_RCRB, map_RRRB };
+void *formatmap[6] = {  map_RM, map_CM, map_CCRB, map_CRRB, map_RCRB, map_RRRB };
 
 int main (int argc, char **argv)
 {
@@ -179,6 +184,8 @@ int main (int argc, char **argv)
         info = testing_ztrmm( argc, argv );
     } else if ( strcmp(func, "TRSM") == 0 ) {
         info = testing_ztrsm( argc, argv );
+    } else if ( strcmp(func, "PEMV") == 0 ) {
+        info = testing_zpemv( argc, argv );
     /*
      * Linear system
      */
@@ -186,22 +193,43 @@ int main (int argc, char **argv)
         info = testing_zposv( argc, argv );
     } else if ( strcmp(func, "GELS") == 0 ) {
         info = testing_zgels( argc, argv );
+    } else if ( strcmp(func, "GESV_INCPIV") == 0 ) {
+        info = testing_zgesv_incpiv( argc, argv );
     } else if ( strcmp(func, "GESV") == 0 ) {
         info = testing_zgesv( argc, argv );
+    /*
+     * Matrix inversion
+     */
+    } else if ( strcmp(func, "POTRI") == 0 ) {
+        info = testing_zpotri( argc, argv );
+    } else if ( strcmp(func, "GETRI") == 0 ) {
+        info = testing_zgetri( argc, argv );
     /*
      * Eigenvalue Problems
      */
+    } else if ( strcmp(func, "HETRD") == 0 ) {
+      info = testing_zhetrd( argc, argv );
     } else if ( strcmp(func, "HEEV") == 0 ) {
       info = testing_zheev( argc, argv );
+    } else if ( strcmp(func, "HEEVD") == 0 ) {
+      info = testing_zheevd( argc, argv );
+    } else if ( strcmp(func, "HEEVR") == 0 ) {
+      info = testing_zheevr( argc, argv );
     } else if ( strcmp(func, "HEGV") == 0 ) {
       info = testing_zhegv( argc, argv );
+    } else if ( strcmp(func, "HEGVD") == 0 ) {
+      info = testing_zhegv( argc, argv );
     } else if ( strcmp(func, "HEGST") == 0 ) {
       info = testing_zhegst( argc, argv );
     /*
      * Singular Value Decomposition
      */
+    } else if ( strcmp(func, "GEBRD") == 0 ) {
+      info = testing_zgebrd( argc, argv );
     } else if ( strcmp(func, "GESVD") == 0 ) {
       info = testing_zgesvd( argc, argv );
+    } else if ( strcmp(func, "GESDD") == 0 ) {
+      info = testing_zgesdd( argc, argv );
 #ifdef DOUBLE
     /*
      * Mixed precision
