@@ -6,7 +6,7 @@
  *  PLASMA is a software package provided by Univ. of Tennessee,
  *  Univ. of California Berkeley and Univ. of Colorado Denver
  *
- * @version 2.4.0
+ * @version 2.5.1
  * @author Mathieu Faverge
  * @date 2010-11-15
  * @precisions normal z -> c d s
@@ -43,12 +43,17 @@ extern int uplo[2];
 extern int side[2];
 extern int diag[2];
 extern int itype[3];
+extern int storev[2];
+extern int norm[4];
+
 extern char *formatstr[6];
 extern char *transstr[3];
 extern char *uplostr[2];
 extern char *sidestr[2];
 extern char *diagstr[2];
 extern char *itypestr[3];
+extern char *storevstr[2];
+extern char *normstr[4];
 
 extern void *formatmap[6];
 
@@ -68,14 +73,24 @@ int testing_zher2k(int argc, char **argv);
 int testing_zsyr2k(int argc, char **argv);
 int testing_ztrmm(int argc, char **argv);
 int testing_ztrsm(int argc, char **argv);
+int testing_zpemv(int argc, char **argv);
 
 int testing_zposv(int argc, char **argv);
 int testing_zgels(int argc, char **argv);
 int testing_zgesv(int argc, char **argv);
+int testing_zgesv_incpiv(int argc, char **argv);
 
+int testing_zpotri(int argc, char **argv);
+int testing_zgetri(int argc, char **argv);
+
+int testing_zgebrd(int argc, char **argv);
 int testing_zgeev(int argc, char **argv);
 int testing_zgesvd(int argc, char **argv);
+int testing_zgesdd(int argc, char **argv);
+int testing_zhetrd(int argc, char **argv);
 int testing_zheev(int argc, char **argv);
+int testing_zheevd(int argc, char **argv);
+int testing_zheevr(int argc, char **argv);
 int testing_zhegv(int argc, char **argv);
 int testing_zhegst(int argc, char **argv);
 
